//===- LoopVectorize.cpp : Loop-Mix-----------------------------------------------------===//
//===-------------------------------------------------------------------------------===//
//===-------------------------------------------------------------------------------===//
//
//                     Loop-Mix: A Loop Vectorizer that exploits mixed SIMD parallelism
//
// Copyright (C) <2014-2016>  <Hao Zhou>
// Copyright (C) <2014-2016>  <Jingling Xue>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//===----------------------------------------------------------------------===//



// Reference:
//  Hao Zhou and Jingling Xue. Exploiting mixed SIMD parallelism by reducing data reorganization overhead. CGO 2016.
//  http://www.cse.unsw.edu.au/~haozhou/loopmix/
//  http://www.cse.unsw.edu.au/~corg/loopmix/



/*
 * LoopVectorize.cpp
 *
 *  Created on: Sep 12, 2014
 *      Author: Hao Zhou
 */



#include "llvm/Transforms/Vectorize.h"
#include "llvm/ADT/DenseMap.h"
#include "llvm/ADT/EquivalenceClasses.h"
#include "llvm/ADT/Hashing.h"
#include "llvm/ADT/MapVector.h"
#include "llvm/ADT/SetVector.h"
#include "llvm/ADT/SmallPtrSet.h"
#include "llvm/ADT/SmallSet.h"
#include "llvm/ADT/SmallVector.h"
#include "llvm/ADT/Statistic.h"
#include "llvm/ADT/StringExtras.h"
#include "llvm/Analysis/AliasAnalysis.h"
#include "llvm/Analysis/AliasSetTracker.h"
#include "llvm/Analysis/BlockFrequencyInfo.h"
#include "llvm/Analysis/LoopInfo.h"
#include "llvm/Analysis/LoopIterator.h"
#include "llvm/Analysis/LoopPass.h"
#include "llvm/Analysis/ScalarEvolution.h"
#include "llvm/Analysis/ScalarEvolutionExpander.h"
#include "llvm/Analysis/ScalarEvolutionExpressions.h"
#include "llvm/Analysis/TargetTransformInfo.h"
#include "llvm/Analysis/ValueTracking.h"
#include "llvm/IR/Constants.h"
#include "llvm/IR/DataLayout.h"
#include "llvm/IR/DebugInfo.h"
#include "llvm/IR/DerivedTypes.h"
#include "llvm/IR/DiagnosticInfo.h"
#include "llvm/IR/Dominators.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/IRBuilder.h"
#include "llvm/IR/Instructions.h"
#include "llvm/IR/IntrinsicInst.h"
#include "llvm/IR/LLVMContext.h"
#include "llvm/IR/Module.h"
#include "llvm/IR/PatternMatch.h"
#include "llvm/IR/Type.h"
#include "llvm/IR/Value.h"
#include "llvm/IR/ValueHandle.h"
#include "llvm/IR/Verifier.h"
#include "llvm/Pass.h"
#include "llvm/Support/BranchProbability.h"
#include "llvm/Support/CommandLine.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/Transforms/Scalar.h"
#include "llvm/Transforms/Utils/BasicBlockUtils.h"
#include "llvm/Transforms/Utils/Local.h"
#include "llvm/Transforms/Utils/VectorUtils.h"
#include <algorithm>
#include <map>
#include <tuple>

using namespace llvm;
using namespace llvm::PatternMatch;

#define LV_NAME "loop-vectorize"
#define DEBUG_TYPE LV_NAME

STATISTIC(LoopsVectorized, "Number of loops vectorized");
STATISTIC(LoopsAnalyzed, "Number of loops analyzed for vectorization");

static cl::opt<unsigned>
VectorizationFactor("force-vector-width", cl::init(0), cl::Hidden,
                    cl::desc("Sets the SIMD width. Zero is autoselect."));

static cl::opt<unsigned>
VectorizationUnroll("force-vector-unroll", cl::init(0), cl::Hidden,
                    cl::desc("Sets the vectorization unroll count. "
                             "Zero is autoselect."));

static cl::opt<bool>
EnableIfConversion("enable-if-conversion", cl::init(true), cl::Hidden,
                   cl::desc("Enable if-conversion during vectorization."));

/// We don't vectorize loops with a known constant trip count below this number.
static cl::opt<unsigned>
TinyTripCountVectorThreshold("vectorizer-min-trip-count", cl::init(16),
                             cl::Hidden,
                             cl::desc("Don't vectorize loops with a constant "
                                      "trip count that is smaller than this "
                                      "value."));

/// This enables versioning on the strides of symbolically striding memory
/// accesses in code like the following.
///   for (i = 0; i < N; ++i)
///     A[i * Stride1] += B[i * Stride2] ...
///
/// Will be roughly translated to
///    if (Stride1 == 1 && Stride2 == 1) {
///      for (i = 0; i < N; i+=4)
///       A[i:i+3] += ...
///    } else
///      ...
static cl::opt<bool> EnableMemAccessVersioning(
    "enable-mem-access-versioning", cl::init(true), cl::Hidden,
    cl::desc("Enable symblic stride memory access versioning"));

/// We don't unroll loops with a known constant trip count below this number.
static const unsigned TinyTripCountUnrollThreshold = 128;

/// When performing memory disambiguation checks at runtime do not make more
/// than this number of comparisons.
static const unsigned RuntimeMemoryCheckThreshold = 100;//8;

/// Maximum simd width.
static const unsigned MaxVectorWidth = 64;

static cl::opt<unsigned> ForceTargetNumScalarRegs(
    "force-target-num-scalar-regs", cl::init(0), cl::Hidden,
    cl::desc("A flag that overrides the target's number of scalar registers."));

static cl::opt<unsigned> ForceTargetNumVectorRegs(
    "force-target-num-vector-regs", cl::init(0), cl::Hidden,
    cl::desc("A flag that overrides the target's number of vector registers."));

/// Maximum vectorization unroll count.
static const unsigned MaxUnrollFactor = 16;

static cl::opt<unsigned> ForceTargetMaxScalarUnrollFactor(
    "force-target-max-scalar-unroll", cl::init(0), cl::Hidden,
    cl::desc("A flag that overrides the target's max unroll factor for scalar "
             "loops."));

static cl::opt<unsigned> ForceTargetMaxVectorUnrollFactor(
    "force-target-max-vector-unroll", cl::init(0), cl::Hidden,
    cl::desc("A flag that overrides the target's max unroll factor for "
             "vectorized loops."));

static cl::opt<unsigned> ForceTargetInstructionCost(
    "force-target-instruction-cost", cl::init(0), cl::Hidden,
    cl::desc("A flag that overrides the target's expected cost for "
             "an instruction to a single constant value. Mostly "
             "useful for getting consistent testing."));

static cl::opt<unsigned> SmallLoopCost(
    "small-loop-cost", cl::init(20), cl::Hidden,
    cl::desc("The cost of a loop that is considered 'small' by the unroller."));

static cl::opt<bool> LoopVectorizeWithBlockFrequency(
    "loop-vectorize-with-block-frequency", cl::init(false), cl::Hidden,
    cl::desc("Enable the use of the block frequency analysis to access PGO "
             "heuristics minimizing code growth in cold regions and being more "
             "aggressive in hot regions."));

/// Runtime unroll loops for load/store throughput.
static cl::opt<bool> EnableLoadStoreRuntimeUnroll(
    "enable-loadstore-runtime-unroll", cl::init(true), cl::Hidden,
    cl::desc("Enable runtime unrolling until load/store ports are saturated"));

/// The number of stores in a loop that are allowed to need predication.
static cl::opt<unsigned> NumberOfStoresToPredicate(
    "vectorize-num-stores-pred", cl::init(1), cl::Hidden,
    cl::desc("Max number of stores to be predicated behind an if."));

static cl::opt<bool> EnableIndVarRegisterHeur(
    "enable-ind-var-reg-heur", cl::init(true), cl::Hidden,
    cl::desc("Count the induction variable only once when unrolling"));

static cl::opt<bool> EnableCondStoresVectorization(
    "enable-cond-stores-vec", cl::init(false), cl::Hidden,
    cl::desc("Enable if predication of stores during vectorization."));





static cl::opt<unsigned>
VectorCostThreshold("vectorize-cost-threshold", cl::init(6),
                             cl::Hidden,
                             cl::desc("Don't vectorize loops with if the benefits "
                                      "is smaller than this "
                                      "value."));

static cl::opt<unsigned>
maxVecRegWidthInbit("max-vector-width-in-bit", cl::init(256),
                             cl::Hidden,
                             cl::desc("the vector length used for this platform."));



static bool hasComputableBounds(ScalarEvolution *SE, ValueToValueMap &Strides,
                                Value *Ptr);
static int isStridedPtr(ScalarEvolution *SE, const DataLayout *DL, Value *Ptr,
                        const Loop *Lp, ValueToValueMap &StridesMap);
const SCEV * getStride(ScalarEvolution *SE, const SCEV *PtrScev, const Loop *Lp);
static bool canCombineAsAltInst(unsigned Op);
static unsigned isAltInst(ArrayRef<Value *> VL);
static BasicBlock *getSameBlock(ArrayRef<Value *> VL);
static Value *stripIntegerCast(Value *V) {
  if (CastInst *CI = dyn_cast<CastInst>(V))
    if (CI->getOperand(0)->getType()->isIntegerTy())
      return CI->getOperand(0);
  return V;
}

///\brief Replaces the symbolic stride in a pointer SCEV expression by one.
/// If \p OrigPtr is not null, use it to look up the stride value instead of
/// \p Ptr.
static const SCEV *replaceSymbolicStrideSCEV(ScalarEvolution *SE,
                                             ValueToValueMap &PtrToStride,
                                             Value *Ptr, Value *OrigPtr = nullptr) {
  const SCEV *OrigSCEV = SE->getSCEV(Ptr);

  // If there is an entry in the map return the SCEV of the pointer with the
  // symbolic stride replaced by one.
  ValueToValueMap::iterator SI = PtrToStride.find(OrigPtr ? OrigPtr : Ptr);
  if (SI != PtrToStride.end()) {
    Value *StrideVal = SI->second;

    // Strip casts.
    StrideVal = stripIntegerCast(StrideVal);

    // Replace symbolic stride by one.
    Value *One = ConstantInt::get(StrideVal->getType(), 1);
    ValueToValueMap RewriteMap;
    RewriteMap[StrideVal] = One;

    const SCEV *ByOne =
        SCEVParameterRewriter::rewrite(OrigSCEV, *SE, RewriteMap, true);
    return ByOne;
  }

  // Otherwise, just return the SCEV of the original pointer.
  return SE->getSCEV(Ptr);
}

static void addInnerLoop(Loop &L, SmallVectorImpl<Loop *> &V) {
  if (L.empty())
    return V.push_back(&L);

  for (Loop *InnerL : L)
    addInnerLoop(*InnerL, V);
}



namespace {

// Forward declarations.
class LoopVectorizationLegality;
class LoopVectorizationCostModel;

/// Optimization analysis message produced during vectorization. Messages inform
/// the user why vectorization did not occur.
class Report {
  std::string Message;
  raw_string_ostream Out;
  Instruction *Instr;

public:
  Report(Instruction *I = nullptr) : Out(Message), Instr(I) {
    Out << "loop not vectorized: ";
  }

  template <typename A> Report &operator<<(const A &Value) {
    Out << Value;
    return *this;
  }

  Instruction *getInstr() { return Instr; }

  std::string &str() { return Out.str(); }
  operator Twine() { return Out.str(); }
};

/// \brief Look for a meaningful debug location on the instruction or it's
/// operands.
static Instruction *getDebugLocFromInstOrOperands(Instruction *I) {
  if (!I)
    return I;

  DebugLoc Empty;
  if (I->getDebugLoc() != Empty)
    return I;

  for (User::op_iterator OI = I->op_begin(), OE = I->op_end(); OI != OE; ++OI) {
    if (Instruction *OpInst = dyn_cast<Instruction>(*OI))
      if (OpInst->getDebugLoc() != Empty)
        return OpInst;
  }

  return I;
}

/// \brief Set the debug location in the builder using the debug location in the
/// instruction.
static void setDebugLocFromInst(IRBuilder<> &B, const Value *Ptr) {
  if (const Instruction *Inst = dyn_cast_or_null<Instruction>(Ptr))
    B.SetCurrentDebugLocation(Inst->getDebugLoc());
  else
    B.SetCurrentDebugLocation(DebugLoc());
}

#ifndef NDEBUG
/// \return string containing a file name and a line # for the given loop.
static std::string getDebugLocString(const Loop *L) {
  std::string Result;
  if (L) {
    raw_string_ostream OS(Result);
    const DebugLoc LoopDbgLoc = L->getStartLoc();
    if (!LoopDbgLoc.isUnknown())
      LoopDbgLoc.print(L->getHeader()->getContext(), OS);
    else
      // Just print the module name.
      OS << L->getHeader()->getParent()->getParent()->getModuleIdentifier();
    OS.flush();
  }
  return Result;
}
#endif

/// \brief Propagate known metadata from one instruction to another.
static void propagateMetadata(Instruction *To, const Instruction *From) {
  SmallVector<std::pair<unsigned, MDNode *>, 4> Metadata;
  From->getAllMetadataOtherThanDebugLoc(Metadata);

  for (auto M : Metadata) {
    unsigned Kind = M.first;

    // These are safe to transfer (this is safe for TBAA, even when we
    // if-convert, because should that metadata have had a control dependency
    // on the condition, and thus actually aliased with some other
    // non-speculated memory access when the condition was false, this would be
    // caught by the runtime overlap checks).
    if (Kind != LLVMContext::MD_tbaa &&
        Kind != LLVMContext::MD_fpmath)
      continue;

    To->setMetadata(Kind, M.second);
  }
}

/// \brief Propagate known metadata from one instruction to a vector of others.
static void propagateMetadata(SmallVectorImpl<Value *> &To, const Instruction *From) {
  for (Value *V : To)
    if (Instruction *I = dyn_cast<Instruction>(V))
      propagateMetadata(I, From);
}


/// A helper class for numbering instructions in multiple blocks.
/// Numbers start at zero for each basic block.
struct BlockNumbering {

  BlockNumbering(BasicBlock *Bb) : BB(Bb), Valid(false) {}

  void numberInstructions() {
	unsigned Loc = 0;
	InstrIdx.clear();
	InstrVec.clear();
	// Number the instructions in the block.
	for (BasicBlock::iterator it = BB->begin(), e = BB->end(); it != e; ++it) {
	  InstrIdx[it] = Loc++;
	  InstrVec.push_back(it);
	  assert(InstrVec[InstrIdx[it]] == it && "Invalid allocation");
	}
	Valid = true;
  }

  int getIndex(Instruction *I) {
	assert(I->getParent() == BB && "Invalid instruction");
	if (!Valid)
	  numberInstructions();
	assert(InstrIdx.count(I) && "Unknown instruction");
	return InstrIdx[I];
  }

  Instruction *getInstruction(unsigned loc) {
	if (!Valid)
	  numberInstructions();
	assert(InstrVec.size() > loc && "Invalid Index");
	return InstrVec[loc];
  }

  void forget() { Valid = false; }

private:
  /// The block we are numbering.
  BasicBlock *BB;
  /// Is the block numbered.
  bool Valid;
  /// Maps instructions to numbers and back.
  SmallDenseMap<Instruction *, int> InstrIdx;
  /// Maps integers to Instructions.
  SmallVector<Instruction *, 32> InstrVec;
};



/// Utility class for getting and setting loop vectorizer hints in the form
/// of loop metadata.
class LoopVectorizeHints {
public:
  enum ForceKind {
    FK_Undefined = -1, ///< Not selected.
    FK_Disabled = 0,   ///< Forcing disabled.
    FK_Enabled = 1,    ///< Forcing enabled.
  };

  LoopVectorizeHints(const Loop *L, bool DisableUnrolling)
      : Width(VectorizationFactor),
        Unroll(DisableUnrolling),
        Force(FK_Undefined),
        LoopID(L->getLoopID()) {
    getHints(L);
    // force-vector-unroll overrides DisableUnrolling.
    if (VectorizationUnroll.getNumOccurrences() > 0)
      Unroll = VectorizationUnroll;

    // DEBUG(if (DisableUnrolling && Unroll == 1)  dbgs() << "LV: Unrolling disabled by the pass manager\n");
  }

  /// Return the loop metadata prefix.
  static StringRef Prefix() { return "llvm.loop."; }

  MDNode *createHint(LLVMContext &Context, StringRef Name, unsigned V) const {
    SmallVector<Value*, 2> Vals;
    Vals.push_back(MDString::get(Context, Name));
    Vals.push_back(ConstantInt::get(Type::getInt32Ty(Context), V));
    return MDNode::get(Context, Vals);
  }

  /// Mark the loop L as already vectorized by setting the width to 1.
  void setAlreadyVectorized(Loop *L) {
    LLVMContext &Context = L->getHeader()->getContext();

    Width = 1;

    // Create a new loop id with one more operand for the already_vectorized
    // hint. If the loop already has a loop id then copy the existing operands.
    SmallVector<Value*, 4> Vals(1);
    if (LoopID)
      for (unsigned i = 1, ie = LoopID->getNumOperands(); i < ie; ++i)
        Vals.push_back(LoopID->getOperand(i));

    Vals.push_back(
        createHint(Context, Twine(Prefix(), "vectorize.width").str(), Width));
    Vals.push_back(
        createHint(Context, Twine(Prefix(), "interleave.count").str(), 1));

    MDNode *NewLoopID = MDNode::get(Context, Vals);
    // Set operand 0 to refer to the loop id itself.
    NewLoopID->replaceOperandWith(0, NewLoopID);

    L->setLoopID(NewLoopID);
    if (LoopID)
      LoopID->replaceAllUsesWith(NewLoopID);

    LoopID = NewLoopID;
  }

  std::string emitRemark() const {
    Report R;
    R << "vectorization ";
    switch (Force) {
    case LoopVectorizeHints::FK_Disabled:
      R << "is explicitly disabled";
      break;
    case LoopVectorizeHints::FK_Enabled:
      R << "is explicitly enabled";
      if (Width != 0 && Unroll != 0)
        R << " with width " << Width << " and interleave count " << Unroll;
      else if (Width != 0)
        R << " with width " << Width;
      else if (Unroll != 0)
        R << " with interleave count " << Unroll;
      break;
    case LoopVectorizeHints::FK_Undefined:
      R << "was not specified";
      break;
    }
    return R.str();
  }

  unsigned getWidth() const { return Width; }
  unsigned getUnroll() const { return Unroll; }
  enum ForceKind getForce() const { return Force; }
  MDNode *getLoopID() const { return LoopID; }

private:
  /// Find hints specified in the loop metadata.
  void getHints(const Loop *L) {
    if (!LoopID)
      return;

    // First operand should refer to the loop id itself.
    assert(LoopID->getNumOperands() > 0 && "requires at least one operand");
    assert(LoopID->getOperand(0) == LoopID && "invalid loop id");

    for (unsigned i = 1, ie = LoopID->getNumOperands(); i < ie; ++i) {
      const MDString *S = nullptr;
      SmallVector<Value*, 4> Args;

      // The expected hint is either a MDString or a MDNode with the first
      // operand a MDString.
      if (const MDNode *MD = dyn_cast<MDNode>(LoopID->getOperand(i))) {
        if (!MD || MD->getNumOperands() == 0)
          continue;
        S = dyn_cast<MDString>(MD->getOperand(0));
        for (unsigned i = 1, ie = MD->getNumOperands(); i < ie; ++i)
          Args.push_back(MD->getOperand(i));
      } else {
        S = dyn_cast<MDString>(LoopID->getOperand(i));
        assert(Args.size() == 0 && "too many arguments for MDString");
      }

      if (!S)
        continue;

      // Check if the hint starts with the loop metadata prefix.
      StringRef Hint = S->getString();
      if (!Hint.startswith(Prefix()))
        continue;
      // Remove the prefix.
      Hint = Hint.substr(Prefix().size(), StringRef::npos);

      if (Args.size() == 1)
        getHint(Hint, Args[0]);
    }
  }

  // Check string hint with one operand.
  void getHint(StringRef Hint, Value *Arg) {
    const ConstantInt *C = dyn_cast<ConstantInt>(Arg);
    if (!C) return;
    unsigned Val = C->getZExtValue();

    if (Hint == "vectorize.width") {
      if (isPowerOf2_32(Val) && Val <= MaxVectorWidth)
        Width = Val;
      else{
        //DEBUG(dbgs() << "LV: ignoring invalid width hint metadata\n");
	  }
    } else if (Hint == "vectorize.enable") {
      if (C->getBitWidth() == 1)
        Force = Val == 1 ? LoopVectorizeHints::FK_Enabled
                         : LoopVectorizeHints::FK_Disabled;
      else{
        //DEBUG(dbgs() << "LV: ignoring invalid enable hint metadata\n");
	  }
    } else if (Hint == "interleave.count") {
      if (isPowerOf2_32(Val) && Val <= MaxUnrollFactor)
        Unroll = Val;
      else{
        //DEBUG(dbgs() << "LV: ignoring invalid unroll hint metadata\n");
	  }
    } else {
      //DEBUG(dbgs() << "LV: ignoring unknown hint " << Hint << '\n');
    }
  }

  /// Vectorization width.
  unsigned Width;
  /// Vectorization unroll factor.
  unsigned Unroll;
  /// Vectorization forced
  enum ForceKind Force;

  MDNode *LoopID;
};

//static void emitMissedWarning(Function *F, Loop *L, const LoopVectorizeHints &LH);

static void emitMissedWarning(Function *F, Loop *L,
                              const LoopVectorizeHints &LH) {
  emitOptimizationRemarkMissed(F->getContext(), DEBUG_TYPE, *F,
                               L->getStartLoc(), LH.emitRemark());

  if (LH.getForce() == LoopVectorizeHints::FK_Enabled) {
    if (LH.getWidth() != 1)
      emitLoopVectorizeWarning(
          F->getContext(), *F, L->getStartLoc(),
          "failed explicitly specified loop vectorization");
    else if (LH.getUnroll() != 1)
      emitLoopInterleaveWarning(
          F->getContext(), *F, L->getStartLoc(),
          "failed explicitly specified loop interleaving");
  }
}


/// InnerLoopVectorizer vectorizes loops which contain only one basic
/// block to a specified vectorization factor (VF).
/// This class performs the widening of scalars into vectors, or multiple
/// scalars. This class also implements the following features:
/// * It inserts an epilogue loop for handling loops that don't have iteration
///   counts that are known to be a multiple of the vectorization factor.
/// * It handles the code generation for reduction variables.
/// * Scalarization (implementation using scalars) of un-vectorizable
///   instructions.
/// InnerLoopVectorizer does not perform any vectorization-legality
/// checks, and relies on the caller to check for the different legality
/// aspects. The InnerLoopVectorizer relies on the
/// LoopVectorizationLegality class to provide information about the induction
/// and reduction variables that were found to a given vectorization factor.
class InnerLoopVectorizer {

typedef SmallVector<Value*, 4> ValuePack;
typedef SmallVector<ValuePack, 4> SLPVarPackList; 
typedef SmallVector<ValuePack, 4> MemAccessChainList;
typedef std::map<Value*, ValuePack> ExternalUserMap;

public:
  InnerLoopVectorizer(Loop *OrigLoop, ScalarEvolution *SE, LoopInfo *LI,
                      DominatorTree *DT, const DataLayout *DL, const TargetTransformInfo &TTI,
                      const TargetLibraryInfo *TLI, unsigned VecWidth,
                      unsigned UnrollFactor)
      : OrigLoop(OrigLoop), SE(SE), LI(LI), DT(DT), DL(DL),
        TLI(TLI), TTI(TTI), VF(VecWidth), UF(UnrollFactor), Builder(SE->getContext()),
        Induction(nullptr), OldInduction(nullptr), WidenMap(UnrollFactor),
        SLPScaMap(UnrollFactor*VecWidth),
        Legal(nullptr) {}

  // Perform the actual loop widening (vectorization).
  void vectorize(LoopVectorizationLegality *L) {
    Legal = L;
    // Create a new empty loop. Unlink the old loop and connect the new one.
    createEmptyLoop();
	// Widen each instruction in the old loop to a new one in the new loop.
	// Use the Legality module to find the induction and reduction variables.		
	vectorizeLoop();
    // Register the new loop and update the analysis passes.		
	updateAnalysis();
  }

  virtual ~InnerLoopVectorizer() {}

protected:
  /// A small list of PHINodes.
  typedef SmallVector<PHINode*, 4> PhiVector;
  /// When we unroll loops we have multiple vector values for each scalar.
  /// This data structure holds the unrolled and vectorized values that
  /// originated from one scalar instruction.
  typedef SmallVector<Value*, 2> VectorParts;
  // When we if-convert we need create edge masks. We have to cache values so
  // that we don't end up with exponential recursion/IR.
  typedef DenseMap<std::pair<BasicBlock*, BasicBlock*>,
                   VectorParts> EdgeMaskCache;

  /// \brief The aollowing two fucntion add code that checks at runtime if the accessed arrays overlap.
  /// Returns a pair of instructions where the first element is the first
  /// instruction generated in possibly a sequence of instructions and the
  /// second value is the final comparator value or NULL if no check is needed.

  ///check a special case : if |offset|<|stride| when both offset and stride are loop invarivant, than accessed 
  ///array will not overlap
  std::pair<Instruction *, Instruction *> addOffsetRemStrideCheck(Instruction *Loc);
  ///check the general case : check the overlap of the whole rang of the accessed arrays
  std::pair<Instruction *, Instruction *> addArrayRangeCheck(Instruction *Loc);

  /// Create an empty loop, based on the loop ranges of the old loop.
  void createEmptyLoop();
  /// Copy and widen the instructions from the old loop.
  virtual void vectorizeLoop();
  /// \brief The Loop exit block may have single value PHI nodes where the
  /// incoming value is 'Undef'. While vectorizing we only handled real values
  /// that were defined inside the loop. Here we fix the 'undef case'.
  /// See PR14725.
  void fixLCSSAPHIs();
  /// A helper function that computes the predicate of the block BB, assuming
  /// that the header block of the loop is set to True. It returns the *entry*
  /// mask for the block BB.
  VectorParts createBlockInMask(BasicBlock *BB);
  /// A helper function that computes the predicate of the edge between SRC
  /// and DST.
  VectorParts createEdgeMask(BasicBlock *Src, BasicBlock *Dst);
  /// A helper function to vectorize a single BB within the innermost loop.
  void vectorizeBlockInLoop(BasicBlock *BB, PhiVector *PV);
  /// Vectorize a single PHINode in a block. This method handles the induction
  /// variable canonicalization. It supports both VF = 1 for unrolled loops and
  /// arbitrary length vectors.
  void widenPHIInstruction(Instruction *PN, VectorParts &Entry,
                           unsigned UF, unsigned VF, PhiVector *PV);
  /// Insert the new loop to the loop hierarchy and pass manager
  /// and update the analysis passes.
  void updateAnalysis();
  /// This instruction is un-vectorizable. Implement it as a sequence
  /// of scalars. If \p IfPredicateStore is true we need to 'hide' each
  /// scalarized instruction behind an if block predicated on the control
  /// dependence of the instruction.
  virtual void scalarizeInstruction(Instruction *Instr,
                                    bool IfPredicateStore=false);
  /// Vectorize Load and Store instructions,
  virtual void vectorizeMemoryInstruction(Instruction *Instr);
  /// Create a broadcast instruction. This method generates a broadcast
  /// instruction (shuffle) for loop invariant values and for the induction
  /// value. If this is the induction variable then we extend it to N, N+1, ...
  /// this is needed because each iteration in the loop corresponds to a SIMD
  /// element.
  virtual Value *getBroadcastInstrs(Value *V);
  /// This function adds 0, 1, 2 ... to each vector element, starting at zero.
  /// If Negate is set then negative numbers are added e.g. (0, -1, -2, ...).
  /// The sequence starts at StartIndex.
  virtual Value *getConsecutiveVector(Value* Val, int StartIdx, bool Negate, int stride = 1);
  /// When we go over instructions in the basic block we rely on previous
  /// values within the current basic block or on loop invariant values.
  /// When we widen (vectorize) values we place them in the map. If the values
  /// are not within the map, they have to be loop invariant, so we simply
  /// broadcast them into a vector.
  VectorParts &getVectorValue(Value *V);
  /// Generate a shuffle sequence that will reverse the vector Vec.
  virtual Value *reverseVector(Value *Vec);

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////begin: newly added functions for slp vectorization/////////////////////////////
  ////the following function handle the slp variables
  ////////////////////////////////////////////////////////////////////////////////////////////////////
  ////for a scalar value, generate its instances from consecutive iterations in the vectoried loop body
  ////note: the slp variables are not vectoried here yet  
  void vectorizeSLPValuePack(Value * sca);
  ////specially handle load/store instructions, called by vectorizeSLPValuePack.
  void vectorizeSLPMemAccPack(ValuePack bundle);
  
  ////pack the scalar values together for some users vectorized at the loop level
  void switchSLPToLoopLevel(Value* sca);
  ////unpack the vector values for some users vectorized at the slp level
  void switchLoopLevelToSLP(ValuePack bundle);

  ////generate shuffle for vector loads/store, and optimise the generated code
  void generateShuffleForVecLoads (unsigned subgroup_sz, ValuePack & vecLoads);
  void generateShuffleForVecStores (ValuePack vecStores);
  Value* optimizeShuffles(Value * shuf);

  //// check if the vectorized value (before extraction) can be reused
  bool isExtractedVectorReusable(ValuePack bundle){
  	   unsigned size = bundle.size();
	   Value * V = cast<ExtractElementInst>(bundle[0])->getVectorOperand();

	   unsigned NumElts = V->getType()->getVectorNumElements();	   
	   if (NumElts != size)
		 return false;
	   
       for(unsigned ii = 0; ii < size; ii++){
	   	   if(dyn_cast<ExtractElementInst>(bundle[ii])){
				if(cast<ExtractElementInst>(bundle[ii])->getVectorOperand() != V)
					return false;
				else{ 
					Value * indx = cast<ExtractElementInst>(bundle[ii])->getIndexOperand();
					ConstantInt *CI = dyn_cast<ConstantInt>(indx);
					if (!CI || CI->getZExtValue() != ii)
		 				return false;
				}					
		   }
		   else
		   	return false;
	   }  	  
	   return true;
  }

  ////check if the slp bundle has been vectorized 
  bool isSlpBundleVectorized(ValuePack root);
  ////reuse the vecotrs already generated
  Value * reusePackedValue(ValuePack root, Instruction * ori_insert_pos); 
  ////apply data packing on the slp bundle
  Value * dataPacking(ValuePack root, unsigned v_sz);
  ////apply data splating on the slp bundle
  Value * dataSplating(ValuePack root, unsigned v_sz);

  ////check consecutiveness of LDs/STs for SLP variables
  void getConsecutivenessSLPMemAccPack();
  ////vectorize slp variables/bundles, start with consective memory access, 
  void vectorizeSLPBundles(); 
  ////do bottom-up slp vectirozation
  Value * vectorizeSLPBundles(ValuePack bundle, unsigned sz);
  ////specially handle store slp bundles
  void vectorizeStoreSLPBundle(ValuePack root, Value * v);
  ////specially handle load slp bundles  
  Value * vectorizeLoadSLPBundle(ValuePack root, unsigned sz);
  ////extract the proper value for the external user after slp vectorization
  void HandleExternalUsers();

  ////check if bundle bd_b is the subbundle of bd_a  
  bool isSubBundle(ValuePack bd_a, ValuePack bd_b);
  ////the following two functions check if the bundle to be packed already exists
  int isExistingPackedValue(ValuePack bundle);  
  bool isExactExistingPackedValue(ValuePack bundle, int bd_idx);

  ////generate shuffle mask to genrate bundle based on another bundle 
  Value* getShuffleMaskForBundles(ValuePack from, ValuePack to);
  //////////////////////////////end: newly added functions/////////////////////////////////////////////

  
  /// This is a helper class that holds the vectorizer state. It maps scalar
  /// instructions to vector instructions. When the code is 'unrolled' then
  /// then a single scalar value is mapped to multiple vector parts. The parts
  /// are stored in the VectorPart type.
  struct ValueMap {
    /// C'tor.  UnrollFactor controls the number of vectors ('parts') that
    /// are mapped.
    ValueMap(unsigned UnrollFactor) : UF(UnrollFactor) {}
    /// \return True if 'Key' is saved in the Value Map.
    bool has(Value *Key) const { return MapStorage.count(Key); }
    /// Initializes a new entry in the map. Sets all of the vector parts to the
    /// save value in 'Val'.
    /// \return A reference to a vector with splat values.
    VectorParts &splat(Value *Key, Value *Val) {
      VectorParts &Entry = MapStorage[Key];
      Entry.assign(UF, Val);
      return Entry;
    }
    ///\return A reference to the value that is stored at 'Key'.
    VectorParts &get(Value *Key) {
      VectorParts &Entry = MapStorage[Key];
      if (Entry.empty())
        Entry.resize(UF);
      assert(Entry.size() == UF && "new entry should be size of UF!");
      return Entry;
    }

    private:
    /// The unroll factor. Each entry in the map stores this number of vector
    /// elements.
    unsigned UF;
    /// Map storage. We use std::map and not DenseMap because insertions to a
    /// dense map invalidates its iterators.
    std::map<Value *, VectorParts> MapStorage;
  };

  /// The original loop.
  Loop *OrigLoop;
  /// Scev analysis to use.
  ScalarEvolution *SE;
  /// Loop Info.
  LoopInfo *LI;
  /// Dominator Tree.
  DominatorTree *DT;
  /// Alias Analysis.
  AliasAnalysis *AA;
  /// Data Layout.
  const DataLayout *DL;
  /// Target Library Info.
  const TargetLibraryInfo *TLI;
  /// Target Transform Info.
  const TargetTransformInfo &TTI;
  /// The vectorization SIMD factor to use. Each vector will have this many
  /// vector elements.
  unsigned VF;

protected:
  /// The vectorization unroll factor to use. Each scalar is vectorized to this
  /// many different vector instructions.
  unsigned UF;
  /// The builder that we use
  IRBuilder<> Builder;

  // --- Vectorization state ---
  /// The vector-loop preheader.
  BasicBlock *LoopVectorPreHeader;
  /// The scalar-loop preheader.
  BasicBlock *LoopScalarPreHeader;
  /// Middle Block between the vector and the scalar.
  BasicBlock *LoopMiddleBlock;
  ///The ExitBlock of the scalar loop.
  BasicBlock *LoopExitBlock;
  ///The vector loop body.
  SmallVector<BasicBlock *, 4> LoopVectorBody;
  ///The scalar loop body.
  BasicBlock *LoopScalarBody;
  /// A list of all bypass blocks. The first block is the entry of the loop.
  SmallVector<BasicBlock *, 4> LoopBypassBlocks;

  /// The new Induction variable which was added to the new block.
  PHINode *Induction;
  /// The induction variable of the old basic block.
  PHINode *OldInduction;
  /// Holds the extended (to the widest induction type) start index.
  Value *ExtendedIdx;
  /// Cache masks for if-conversion
  EdgeMaskCache MaskCache;

  /// Maps scalars to widened vectors.
  ValueMap WidenMap;
  /// Maps slp scalars to their unrolled but not vectorized values
  ValueMap SLPScaMap;  
  /// Maps flatten/unrolled sca values to their vectoried value
  ////map of slp sca value to its slp bundle id
  std::map<Value*, unsigned> SLPVarScatoVec;
  ////vector of slp sca value bundles
  SLPVarPackList SLPScaValues;
  ////vector of slp vec values
  ValuePack SLPVecValues;

  ////vector of sca value bundles that are packed together (insertion/broadcast)
  SLPVarPackList PackedValueSca;
  ////the vector value of the bundles that are packed together (insertion/broadcast)  
  ValuePack PackedValueVec;  

  ///map of the slp sca value and its external users
  ExternalUserMap externalUserMap;

  ///map of the consecutive memory access values
  std::map<Value*, Value*> consecutiveMemAccs;

  LoopVectorizationLegality *Legal;
};



class InnerLoopUnroller : public InnerLoopVectorizer {
public:
  InnerLoopUnroller(Loop *OrigLoop, ScalarEvolution *SE, LoopInfo *LI,
                    DominatorTree *DT, const DataLayout *DL,
                    const TargetLibraryInfo *TLI, unsigned UnrollFactor) :
    InnerLoopVectorizer(OrigLoop, SE, LI, DT, DL, TTI, TLI, 1, UnrollFactor) { }

private:
  void scalarizeInstruction(Instruction *Instr,
                            bool IfPredicateStore = false) override;
  void vectorizeMemoryInstruction(Instruction *Instr) override;
  Value *getBroadcastInstrs(Value *V) override;
  Value *getConsecutiveVector(Value* Val, int StartIdx, bool Negate, int stride) override;
  Value *reverseVector(Value *Vec) override;
};



/// LoopVectorizationLegality checks if it is legal to vectorize a loop, and
/// to what vectorization factor.
/// This class does not look at the profitability of vectorization, only the
/// legality. This class has two main kinds of checks:
/// * Memory checks - The code in canVectorizeMemory checks if vectorization
///   will change the order of memory accesses in a way that will change the
///   correctness of the program.
/// * Scalars checks - The code in canVectorizeInstrs and canVectorizeMemory
/// checks for a number of different conditions, such as the availability of a
/// single induction variable, that all types are supported and vectorize-able,
/// etc. This code reflects the capabilities of InnerLoopVectorizer.
/// This class is also used by InnerLoopVectorizer for identifying
/// induction variable and the different reduction variables.
class LoopVectorizationLegality {
public:
  unsigned NumLoads;
  unsigned NumStores;
  unsigned NumPredStores;

  LoopVectorizationLegality(Loop *L, ScalarEvolution *SE, const DataLayout *DL,
                            DominatorTree *DT, TargetLibraryInfo *TLI,
                            AliasAnalysis *AA, Function *F)
      : NumLoads(0), NumStores(0), NumPredStores(0), TheLoop(L), SE(SE), DL(DL),
        DT(DT), TLI(TLI), AA(AA), TheFunction(F), Induction(nullptr),
        WidestIndTy(nullptr), HasFunNoNaNAttr(false), MaxSafeDepDistBytes(-1U),
        PtrRtCheck(RuntimePointerCheck(SE)), SwitchTimes(0), SLPMemAccTimes(0){
  }

  /// This enum represents the kinds of reductions that we support.
  enum ReductionKind {
    RK_NoReduction, ///< Not a reduction.
    RK_IntegerAdd,  ///< Sum of integers.
    RK_IntegerMult, ///< Product of integers.
    RK_IntegerOr,   ///< Bitwise or logical OR of numbers.
    RK_IntegerAnd,  ///< Bitwise or logical AND of numbers.
    RK_IntegerXor,  ///< Bitwise or logical XOR of numbers.
    RK_IntegerMinMax, ///< Min/max implemented in terms of select(cmp()).
    RK_FloatAdd,    ///< Sum of floats.
    RK_FloatMult,   ///< Product of floats.
    RK_FloatMinMax  ///< Min/max implemented in terms of select(cmp()).
  };

  /// This enum represents the kinds of inductions that we support.
  enum InductionKind {
    IK_NoInduction,         ///< Not an induction variable.
    IK_IntInduction,        ///< Integer induction variable. Step = 1.
    IK_ReverseIntInduction, ///< Reverse int induction variable. Step = -1.
    IK_PtrInduction,        ///< Pointer induction var. Step = sizeof(elem).
    IK_ReversePtrInduction  ///< Reverse ptr indvar. Step = - sizeof(elem).
  };

  // This enum represents the kind of minmax reduction.
  enum MinMaxReductionKind {
    MRK_Invalid,
    MRK_UIntMin,
    MRK_UIntMax,
    MRK_SIntMin,
    MRK_SIntMax,
    MRK_FloatMin,
    MRK_FloatMax
  };

  /// This struct holds information about reduction variables.
  struct ReductionDescriptor {
    ReductionDescriptor() : StartValue(nullptr), LoopExitInstr(nullptr),
      Kind(RK_NoReduction), MinMaxKind(MRK_Invalid) {}

    ReductionDescriptor(Value *Start, Instruction *Exit, ReductionKind K,
                        MinMaxReductionKind MK)
        : StartValue(Start), LoopExitInstr(Exit), Kind(K), MinMaxKind(MK) {}

    // The starting value of the reduction.
    // It does not have to be zero!
    TrackingVH<Value> StartValue;
    // The instruction who's value is used outside the loop.
    Instruction *LoopExitInstr;
    // The kind of the reduction.
    ReductionKind Kind;
    // If this a min/max reduction the kind of reduction.
    MinMaxReductionKind MinMaxKind;
  };

  /// This POD struct holds information about a potential reduction operation.
  struct ReductionInstDesc {
    ReductionInstDesc(bool IsRedux, Instruction *I) :
      IsReduction(IsRedux), PatternLastInst(I), MinMaxKind(MRK_Invalid) {}

    ReductionInstDesc(Instruction *I, MinMaxReductionKind K) :
      IsReduction(true), PatternLastInst(I), MinMaxKind(K) {}

    // Is this instruction a reduction candidate.
    bool IsReduction;
    // The last instruction in a min/max pattern (select of the select(icmp())
    // pattern), or the current reduction instruction otherwise.
    Instruction *PatternLastInst;
    // If this is a min/max pattern the comparison predicate.
    MinMaxReductionKind MinMaxKind;
  };

  /// This struct holds information about the memory runtime legality
  /// check that a group of pointers do not overlap.
  struct RuntimePointerCheck {
    //RuntimePointerCheck() : Need(false), SE(NULL) {}
	RuntimePointerCheck(ScalarEvolution *SE) : Need(false), SE(SE){ }

    /// Reset the state of the pointer runtime information.
    void reset() {
      Need = false;
      Pointers.clear();
      Starts.clear();
      Ends.clear();
      IsWritePtr.clear();
      DependencySetId.clear();
      AliasSetId.clear();
	  
	  RemOffset.clear();
	  RemStride.clear();	
      PointerAs.clear();
      StartAs.clear();
      EndAs.clear();
      PointerBs.clear();
      StartBs.clear();
      EndBs.clear();	

	  ArrayRangeCheckPtrPair.clear();
	  RTCheckTimes = 0;
    }

    /// Insert a pointer and calculate the start and end SCEVs.
    void insert(ScalarEvolution *SE, Loop *Lp, Value *Ptr, bool WritePtr,
                unsigned DepSetId, unsigned ASId, ValueToValueMap &Strides) {
	  // Get the stride replaced scev.
	  const SCEV *Sc = replaceSymbolicStrideSCEV(SE, Strides, Ptr);
	  const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(Sc);
	  const SCEV *Ex = SE->getBackedgeTakenCount(Lp);  
	  if(AR){
		  const SCEV *ScEnd = AR->evaluateAtIteration(Ex, *SE);
		  Starts.push_back(AR->getStart());
		  Ends.push_back(ScEnd);
	  }else{
		  assert(Lp->isLoopInvariant(Ptr) && "unhandled case when inserting runtime check!\n");
		  Starts.push_back(SE->getSCEV(Ptr));
		  Ends.push_back(SE->getSCEV(Ptr));
	  }
	  Pointers.push_back(Ptr);
	  IsWritePtr.push_back(WritePtr);
	  DependencySetId.push_back(DepSetId);
	  AliasSetId.push_back(ASId);
	  RTCheckTimes++;
	}

	void addOffsetRemStrideCheck(const SCEV* Offset, const SCEV* Stride){
		Need = true;
		unsigned sz = RemOffset.size();

		////for runtime efficiency, we only check if offset<stride ( in which offset%stride is sure != 0 )
		////therefore, for the same stride, we only theck the biggest offset
		if(const SCEVConstant *C = dyn_cast<SCEVConstant>(Offset)){			
			const APInt &c_val = C->getValue()->getValue();
			int c_v = c_val.getSExtValue();
			c_v = c_v * c_v;  ////in order to handle negative value 
			for(unsigned ii = 0 ; ii < sz; ii++){
				const SCEVConstant *C_t = dyn_cast<SCEVConstant>(RemOffset[ii]);
				if( C_t && ((SE->getMinusSCEV(Stride, RemStride[ii]))->isZero())){
					const APInt &c_t_val = C_t->getValue()->getValue();
					int c_t_v = c_t_val.getSExtValue();
					c_t_v = c_t_v * c_t_v;
					if(c_v > c_t_v) ////keep the larger offset to be compared with fixed&unknown stride later 
						RemOffset[ii] = Offset;
					return;					
				}
			}
		}
		
		RemOffset.push_back(Offset);
		RemStride.push_back(Stride);
		RTCheckTimes++;
	}

	void addArrayRangeCheck(ScalarEvolution *SE, const Loop *Lp, Value *PtrA, Value *PtrB) 
	{	
	  Need = true;

      if(ArrayRangeCheckPtrPair.count(std::make_pair(PtrA,PtrB)) || ArrayRangeCheckPtrPair.count(std::make_pair(PtrB,PtrA)))
	  	return;
	  
	  const SCEV *Ex = SE->getBackedgeTakenCount(Lp);	
	  const SCEVAddRecExpr *AR1 = dyn_cast<SCEVAddRecExpr>(SE->getSCEV(PtrA));
	  const SCEVAddRecExpr *AR2 = dyn_cast<SCEVAddRecExpr>(SE->getSCEV(PtrB));
	  PointerAs.push_back(PtrA);
	  PointerBs.push_back(PtrB);	  
	  if(AR1 && AR2){
	  	  const SCEV *ScEnd1 = AR1->evaluateAtIteration(Ex, *SE);
	  	  const SCEV *ScEnd2 = AR2->evaluateAtIteration(Ex, *SE);	  	
		  StartAs.push_back(AR1->getStart());
		  EndAs.push_back(ScEnd1);
		  StartBs.push_back(AR2->getStart());
		  EndBs.push_back(ScEnd2);
	  }
	  else if(AR1 || AR2){
	  	  if(AR1){
			  const SCEV *ScEnd1 = AR1->evaluateAtIteration(Ex, *SE);	
			  StartAs.push_back(AR1->getStart());
			  EndAs.push_back(ScEnd1);
			  StartBs.push_back(SE->getSCEV(PtrB));
			  EndBs.push_back(SE->getSCEV(PtrB));
		  }else{
			  const SCEV *ScEnd2 = AR2->evaluateAtIteration(Ex, *SE);		
			  StartAs.push_back(SE->getSCEV(PtrA));
			  EndAs.push_back(SE->getSCEV(PtrA));
			  StartBs.push_back(AR2->getStart());
			  EndBs.push_back(ScEnd2);
		  }
	  }
	  else
	  	assert(false && "unhandled case: array range check!\n");

	  ArrayRangeCheckPtrPair.insert(std::make_pair(PtrA,PtrB));
	  RTCheckTimes++;
	}

    /// This flag indicates if we need to add the runtime check.
    bool Need;
	ScalarEvolution *SE;
	
    /// Holds the pointers that we need to check.
    SmallVector<TrackingVH<Value>, 2> Pointers;
    /// Holds the pointer value at the beginning of the loop.
    SmallVector<const SCEV*, 2> Starts;
    /// Holds the pointer value at the end of the loop.
    SmallVector<const SCEV*, 2> Ends;
    /// Holds the information if this pointer is used for writing to memory.
    SmallVector<bool, 2> IsWritePtr;
    /// Holds the id of the set of pointers that could be dependent because of a
    /// shared underlying object.
    SmallVector<unsigned, 2> DependencySetId;
    /// Holds the id of the disjoint alias set to which this pointer belongs.
	SmallVector<unsigned, 2> AliasSetId;
	
	/// hold the offset and stride SCEVs for offset%stride test  
	SmallVector<const SCEV*, 2> RemOffset;
	SmallVector<const SCEV*, 2> RemStride;
    /// Holds the pointers and ranges that we need to check.
    SmallVector<TrackingVH<Value>, 2> PointerAs;
    SmallVector<const SCEV*, 2> StartAs;
    SmallVector<const SCEV*, 2> EndAs;	
    SmallVector<TrackingVH<Value>, 2> PointerBs;
    SmallVector<const SCEV*, 2> StartBs;
    SmallVector<const SCEV*, 2> EndBs;	
	std::set< std::pair<Value*, Value*> > ArrayRangeCheckPtrPair;
	int RTCheckTimes;

  };


  /// A struct for saving information about induction variables.
  struct InductionInfo {
    InductionInfo(Value *Start, InductionKind K, int stride) : StartValue(Start), IK(K), Stride(stride) {}
    InductionInfo() : StartValue(nullptr), IK(IK_NoInduction), Stride(0) {}
    /// Start value.
    TrackingVH<Value> StartValue;
    /// Induction kind.
    InductionKind IK;
	int Stride;
  };


  /// ReductionList contains the reduction descriptors for all
  /// of the reductions that were found in the loop.
  typedef DenseMap<PHINode*, ReductionDescriptor> ReductionList;
  /// InductionList saves induction variables and maps them to the
  /// induction descriptor.
  typedef MapVector<PHINode*, InductionInfo> InductionList;

  typedef SmallVector<Value*, 4> ValuePack;
  typedef SmallVector<ValuePack, 4> SLPVarPackList;
  
  typedef DenseMap< std::pair<Value*, Value*>, int> DepDistanceMap;
  
  typedef DenseMap< Value*, Value* > MemAccMap;
  typedef SmallVector<ValuePack, 4> MemAccessChainList;
  
  /// Returns true if it is legal to vectorize this loop.
  /// This does not mean that it is profitable to vectorize this
  /// loop, only that it is legal to do so.
  bool canVectorize();
  /// Returns the Induction variable.
  PHINode *getInduction() { return Induction; }
  /// Returns the reduction variables found in the loop.
  ReductionList *getReductionVars() { return &Reductions; }
  /// Returns the induction variables found in the loop.
  InductionList *getInductionVars() { return &Inductions; }
  /// Returns the widest induction type.
  Type *getWidestInductionType() { return WidestIndTy; }
  /// Returns True if V is an induction variable in this loop.
  bool isInductionVariable(const Value *V);
  /// Return true if the block BB needs to be predicated in order for the loop
  /// to be vectorized.
  bool blockNeedsPredication(BasicBlock *BB);
  /// Check if this  pointer is consecutive when vectorizing. This happens
  /// when the last index of the GEP is the induction variable, or that the
  /// pointer itself is an induction variable.
  /// This check allows us to vectorize A[idx] into a wide load/store.
  /// Returns:
  /// 0 - Stride is unknown or non-consecutive.
  /// 1 - Address is consecutive.
  /// -1 - Address is consecutive, and decreasing.
  int isConsecutivePtr(Value *Ptr);
  /// Returns true if the value V is uniform within the loop.
  bool isUniform(Value *V);
  /// Returns true if this instruction will remain scalar after vectorization.
  bool isUniformAfterVectorization(Instruction* I) { return Uniforms.count(I); }
  /// Returns the information that we collected about runtime memory check.
  RuntimePointerCheck *getRuntimePointerCheck() { return &PtrRtCheck; }
  /// This function returns the identity element (or neutral element) for the operation K.
  static Constant *getReductionIdentity(ReductionKind K, Type *Tp);

  unsigned getMaxSafeDepDistBytes() { return MaxSafeDepDistBytes; }

  bool hasStride(Value *V) { return StrideSet.count(V); }
 
  bool mustCheckStrides() { return !StrideSet.empty(); }

  SmallPtrSet<Value *, 8>::iterator strides_begin() { return StrideSet.begin(); }

  SmallPtrSet<Value *, 8>::iterator strides_end() { return StrideSet.end(); }

/////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////begin: newly added functions/////////////////////////////////////////
private:
  ///get the consecutiveness information about all the memory access, 
  /// all the info is kept in the consecutiveMemAcc map
  bool addConsecutiveMemAccessinfo(ValuePack &mem_list);
  bool addConsecutiveMemAccessinfo(Value* mem_list); 
  
  ///check if the given two instrucitons are consecutive load/store instructions
  int isConsecutiveMemAcc(Instruction* mem_a, Instruction* mem_b);
  
  const SCEV * ignoreCastOfSCEV(const SCEV* Dist);

public:
  ////for a certain data type, get the number of elements inside the vector register 
  unsigned getNumofElementInVector(Type * type);
  ////for a certain data type, get the number of elements that form a full/partial vector 
  unsigned getBestVectorWidth(Type * ty, unsigned sz); 
  ///according to the consecutiveMemAcc map, get the consecutive memory accesses list
  void findConsecutiveMemAccess();
  ///return the consecutive memory accesses list
  MemAccessChainList& getConsecutiveStoreMemAccChainList(){
	return consecutiveMemAccChainList;
  }

  
  ///check if the given bundle is an existing slp bundle
  bool isExistingSlpBundle(ArrayRef<Value *> bundle){
	if(!isSLPVar(bundle[0]))
		return false;
	unsigned bundle_id = getSLPBundleID(bundle[0]);
	ValuePack bundle_l = slpVarPackList[bundle_id];
	unsigned bundle_sz = bundle.size();
	if(bundle_sz != bundle_l.size())
		return false;
	for(unsigned ii = 0; ii < bundle_sz; ii++)
		if(bundle_l[ii] != bundle[ii])
			return false;
	return true;
  }

  ///check if the given bundle is an existing slp bundle
  bool isReversedExistingSlpBundle(ArrayRef<Value *> bundle){
	if(!isSLPVar(bundle[0]))
		return false;
	unsigned bundle_id = getSLPBundleID(bundle[0]);
	ValuePack bundle_l = slpVarPackList[bundle_id];
	unsigned bundle_sz = bundle.size();
	if(bundle_sz != bundle_l.size())
		return false;
	
	for(unsigned ii = 0, jj = bundle_sz-1; ii < bundle_sz; ii++, jj--)
		if(bundle_l[ii] != bundle[jj])
			return false;
	return true;
  }
  
  ///check if the given var is a slp var 
  bool isSLPVar(Value * v){
	return SLPVars.count(v);
  }

  ///get the id of the corresponding slp bundle for a given var
  unsigned getSLPBundleID(Value * v){
  	assert(isSLPVar(v) && "this var is not a SLP var!");
  	return SLPVarToVarPack[v];
  }
  ///get the insert loc for a specific slp bundle id while flattening/unrolling the slp vars
  Instruction * getSLPBundleInsertLoc(unsigned bundle_id){
  	//assert(bundle_id >= 0 && "invalid slp bundle id!");
  	return SLPVarPackInstLoc[bundle_id];	
  }
  ///get the slp bundle for a specific slp bundle id
  ValuePack getSLPBundle(unsigned bundle_id){
    //assert(bundle_id >= 0 && "invalid slp bundle id!");
    return slpVarPackList[bundle_id];	  
  }
  
  ///return the instruction index of the instrction firstly occurs in the basic block
  int getFirstIndex(ArrayRef<Value *> VL) {
		BasicBlock *BB = cast<Instruction>(VL[0])->getParent();
		assert(BB == getSameBlock(VL) && "Invalid block");
		BlockNumbering &BN = getBlockNumbering(BB);
	  
		int MinIdx = BN.getIndex(BB->getTerminator());
		for (unsigned i = 0, e = VL.size(); i < e; ++i)
		  MinIdx = std::min(MinIdx, BN.getIndex(cast<Instruction>(VL[i])));
		return MinIdx;
  }
  ///return the instruction index of the instrction lastly occurs in the basic block
  int getLastIndex(ArrayRef<Value *> VL) {
		BasicBlock *BB = cast<Instruction>(VL[0])->getParent();
		assert(BB == getSameBlock(VL) && "Invalid block");
		BlockNumbering &BN = getBlockNumbering(BB);
	  
		int MaxIdx = BN.getIndex(BB->getFirstNonPHI());
		for (unsigned i = 0, e = VL.size(); i < e; ++i)
		  MaxIdx = std::max(MaxIdx, BN.getIndex(cast<Instruction>(VL[i])));
		return MaxIdx;
  }
  ///return the instruction firstly occurs in the basic block
  Instruction *getFirstInstruction(ArrayRef<Value *> VL) {
	  	BasicBlock *BB = cast<Instruction>(VL[0])->getParent();
	  	assert(BB == getSameBlock(VL) && "Invalid block");
	  	BlockNumbering &BN = getBlockNumbering(BB);

	  	int MinIdx = BN.getIndex(cast<Instruction>(VL[0]));
	  	for (unsigned i = 1, e = VL.size(); i < e; ++i)
	    	MinIdx = std::min(MinIdx, BN.getIndex(cast<Instruction>(VL[i])));
	  	Instruction *I = BN.getInstruction(MinIdx);
	  	assert(I && "bad location");
	  	return I;
  }
  ///return the instruction lastly occurs in the basic block
  Instruction *getLastInstruction(ArrayRef<Value *> VL) {
	  	BasicBlock *BB = cast<Instruction>(VL[0])->getParent();
	  	assert(BB == getSameBlock(VL) && "Invalid block");
	  	BlockNumbering &BN = getBlockNumbering(BB);

	  	int MaxIdx = BN.getIndex(cast<Instruction>(VL[0]));
	  	for (unsigned i = 1, e = VL.size(); i < e; ++i)
	    	MaxIdx = std::max(MaxIdx, BN.getIndex(cast<Instruction>(VL[i])));
	  	Instruction *I = BN.getInstruction(MaxIdx);
	  	assert(I && "bad location");
	  	return I;
  }
  
  ///redo BlockNumbering
  void refreshBlockNumbering(Instruction* I) {
	  	BasicBlock *BB = I->getParent();
	  	BlockNumbering &BN = getBlockNumbering(BB);
	  	BN.numberInstructions();
  }

  ///return the instruction following I in the basic block
  Instruction *getNextInstruction(Instruction* I) {
	  	BasicBlock *BB = I->getParent();
	  	BlockNumbering &BN = getBlockNumbering(BB);
		
	  	int Idx = BN.getIndex(I);
	  	Instruction *I_next = BN.getInstruction(Idx+1);
	  	assert(I_next && "bad location");
	  	return I_next;
  }
  
  ///get instruction index info of a given basic block
  BlockNumbering &getBlockNumbering(BasicBlock *BB) {
  		auto I = BlocksNumbers.insert(std::make_pair(BB, BlockNumbering(BB)));
  		return I.first->second;
  }

  ///check if it is legal to move Dst to the location closely after Src
  Value * getHoistBarrier(Instruction *Dst, Instruction *Src) {
	  assert(Src->getParent() == Dst->getParent() && "Not the same BB");
	  BasicBlock::iterator I = Src, E = Dst;
	  Value* APtr;
	  Value* BPtr = cast<LoadInst>(Dst)->getPointerOperand();
	  /// Scan all of the instruction from SRC to DST and check if
	  /// the source may alias.
	  for (++I; I != E; ++I) {
		if (!I->mayWriteToMemory())
			continue;
  //	AliasAnalysis::Location A = getLocation(&*I);
  //	AliasAnalysis::Location B = getLocation(Src);  
  //	if (!A.Ptr || !B.Ptr || AA->alias(A, B))
  //	  return I;
		if(!dyn_cast<Instruction>(I))
		  return I;
		else if(dyn_cast<CallInst>(I)) ////we have checked function calls at the legalitychecking stage
		  continue;
		else if(dyn_cast<StoreInst>(I))
		  APtr = cast<StoreInst>(I)->getPointerOperand();
		else
		  return I;
		  
		std::pair <Value*, Value*> mem_acc_p1 (APtr, BPtr);
		std::pair <Value*, Value*> mem_acc_p2 (BPtr, APtr);
		int dis = -1;
		if(DepDistance.count(mem_acc_p1))
		  dis = DepDistance[mem_acc_p1];
		if(dis != 0 && DepDistance.count(mem_acc_p2))
		  dis = DepDistance[mem_acc_p2];
  
		if(dis == 0)
		  return I;
	  }
	  return nullptr;
  }

  ///check if it is legal to move Src to the location closely before Dst
  Value * getSinkBarrier(Instruction *Src, Instruction *Dst) {
	assert(Src->getParent() == Dst->getParent() && "Not the same BB");
	BasicBlock::iterator I = Src, E = Dst;
	Value* APtr = (dyn_cast<StoreInst>(Src))? cast<StoreInst>(Src)->getPointerOperand(): cast<LoadInst>(Src)->getPointerOperand();
    Value* BPtr;
	/// Scan all of the instruction from SRC to DST and check if
	/// the source may alias.
	for (++I; I != E; ++I) {
	  if (Src->mayWriteToMemory()) /* Write */ {
		if (!I->mayReadOrWriteMemory())
		  continue;
	  } else /* Read */ {
		if (!I->mayWriteToMemory())
		  continue;
	  }
//	  AliasAnalysis::Location A = getLocation(&*I);
//	  AliasAnalysis::Location B = getLocation(Src);  
//	  if (!A.Ptr || !B.Ptr || AA->alias(A, B))
//		return I;
      if(!dyn_cast<Instruction>(I))
	  	return I;
	  else if(dyn_cast<CallInst>(I))
	  	continue;
	  else if(dyn_cast<StoreInst>(I))
	  	BPtr = cast<StoreInst>(I)->getPointerOperand();
	  else if(dyn_cast<LoadInst>(I))
	  	BPtr = cast<LoadInst>(I)->getPointerOperand();
	  else
	  	return I;
	  	
	  std::pair <Value*, Value*> mem_acc_p1 (APtr, BPtr);
	  std::pair <Value*, Value*> mem_acc_p2 (BPtr, APtr);
	  int dis = -1;
	  if(DepDistance.count(mem_acc_p1))
	  	dis = DepDistance[mem_acc_p1];
	  if(dis != 0 && DepDistance.count(mem_acc_p2))
	  	dis = DepDistance[mem_acc_p2];

	  if(dis == 0)
	  	return I;
	}
	return nullptr;
  }


  /// start from consecutive memory access, get all the slp vars
  void findSLPVars();
  /// bottom-up search to get slp vars
  bool findSLPVars(ValuePack value_pack);
  /// check if loads are slp vars
  bool addSLPLoadVars(ValuePack value_pack);

  /// determine if further switching between different SIMD parallelism is still beneficial or not
  bool noMoreSwitch(ValuePack root);    	
  /// print out all the confirmed slp vars
  void dumpSLPVars();
  ////get the distance of the memory access locations between two consecutive iterations
  int getMemLocationDistanceAccrossInteration(Value* v);
  ////check if all the users of the value v are slp vars
  bool onlyHasSlpVarUsers(Value* v);
  ////check if there is slp var for this loop
  bool HasSlpVars(){
  	return !SLPVars.empty();

  }  
////////////////////////////end: newly added functions/////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

private:
  /// Check if a single basic block loop is vectorizable.
  /// At this point we know that this is a loop with a constant trip count
  /// and we only need to check individual instructions.
  bool canVectorizeInstrs();
  /// When we vectorize loops we may change the order in which
  /// we read and write from memory. This method checks if it is
  /// legal to vectorize the code, considering only memory constrains.
  /// Returns true if the loop is vectorizable
  bool canVectorizeMemory();	
  /// Return true if we can vectorize this loop using the IF-conversion
  /// transformation.
  bool canVectorizeWithIfConvert();
  /// Collect the variables that need to stay uniform after vectorization.
  void collectLoopUniforms();
  /// Return true if all of the instructions in the block can be speculatively
  /// executed. \p SafePtrs is a list of addresses that are known to be legal
  /// and we know that we can read from them without segfault.
  bool blockCanBePredicated(BasicBlock *BB, SmallPtrSet<Value *, 8>& SafePtrs);
  /// Returns True, if 'Phi' is the kind of reduction variable for type
  /// 'Kind'. If this is a reduction variable, it adds it to ReductionList.
  bool AddReductionVar(PHINode *Phi, ReductionKind Kind);
  /// Returns a struct describing if the instruction 'I' can be a reduction
  /// variable of type 'Kind'. If the reduction is a min/max pattern of
  /// select(icmp()) this function advances the instruction pointer 'I' from the
  /// compare instruction to the select instruction and stores this pointer in
  /// 'PatternLastInst' member of the returned struct.
  ReductionInstDesc isReductionInstr(Instruction *I, ReductionKind Kind,
                                     ReductionInstDesc &Desc);
  /// Returns true if the instruction is a Select(ICmp(X, Y), X, Y) instruction
  /// pattern corresponding to a min(X, Y) or max(X, Y).
  static ReductionInstDesc isMinMaxSelectCmpPattern(Instruction *I,
                                                    ReductionInstDesc &Prev);
  /// Returns the induction kind of Phi. This function may return NoInduction
  /// if the PHI is not an induction variable.
  InductionKind isInductionVariable(PHINode *Phi, int & stride);
  /// \brief Collect memory access with loop invariant strides.
  /// Looks for accesses like "a[i * StrideA]" where "StrideA" is loop
  /// invariant.
  void collectStridedAcccess(Value *LoadOrStoreInst);
  /// Report an analysis message to assist the user in diagnosing loops that are
  /// not vectorized.
  void emitAnalysis(Report &Message) {
    DebugLoc DL = TheLoop->getStartLoc();
    if (Instruction *I = Message.getInstr())
      DL = I->getDebugLoc();
    emitOptimizationRemarkAnalysis(TheFunction->getContext(), DEBUG_TYPE,
                                   *TheFunction, DL, Message.str());
  }

  /// The loop that we evaluate.
  Loop *TheLoop;
  /// Scev analysis.
  ScalarEvolution *SE;
  /// DataLayout analysis.
  const DataLayout *DL;
  /// Dominators.
  DominatorTree *DT;
  /// Target Library Info.
  TargetLibraryInfo *TLI;
  /// Alias analysis.
  AliasAnalysis *AA;
  /// Parent function
  Function *TheFunction;

  //  ---  vectorization state --- //
  /// Holds the integer induction variable. This is the counter of the
  /// loop.
  PHINode *Induction;
  /// Holds the reduction variables.
  ReductionList Reductions;
  /// Holds all of the induction variables that we found in the loop.
  /// Notice that inductions don't need to start at zero and that induction
  /// variables can be pointers.
  InductionList Inductions;
  /// Holds the widest induction type encountered.
  Type *WidestIndTy;
  /// Allowed outside users. This holds the reduction
  /// vars which can be accessed from outside the loop.
  SmallPtrSet<Value*, 4> AllowedExit;
  /// This set holds the variables which are known to be uniform after
  /// vectorization.
  SmallPtrSet<Instruction*, 4> Uniforms;
  /// We need to check that all of the pointers in this list are disjoint
  /// at runtime.
  RuntimePointerCheck PtrRtCheck;
  /// Can we assume the absence of NaNs.
  bool HasFunNoNaNAttr;

  unsigned MaxSafeDepDistBytes;

  ValueToValueMap Strides;

  SmallPtrSet<Value *, 8> StrideSet;

  ///consecutive mem_acc pair/map
  MemAccMap consecutiveMemAcc; 
  MemAccMap consecutiveMemAccReverse;  
  ///list of consecutive mem_acc chains
  MemAccessChainList consecutiveMemAccChainList;  
  ///dependent mem_acc pair and its dependence distance
  DepDistanceMap DepDistance;

  ///set of values that should be vectorized in SLP manner
  SmallPtrSet<Value *, 8> SLPVars;   
  ///map between SLP var and its belonging bundle id
  std::map<Value *, unsigned> SLPVarToVarPack; 
  ///map between SLPVar bundle id and its unroll/flatten location
  std::map<unsigned, Instruction *> SLPVarPackInstLoc; 
  ///list of SLPVar bundles
  SLPVarPackList slpVarPackList;    

  /// Numbers instructions in different blocks.
  DenseMap<BasicBlock *, BlockNumbering> BlocksNumbers;

  std::set<Value*> SwitchVars;
  int SwitchTimes;  
  int SLPMemAccTimes;
  
};



/// LoopVectorizationCostModel - estimates the expected speedups due to
/// vectorization.
/// In many cases vectorization is not profitable. This can happen because of
/// a number of reasons. In this class we mainly attempt to predict the
/// expected speedup/slowdowns due to the supported instruction set. We use the
/// TargetTransformInfo to query the different backends for the cost of
/// different operations.
class LoopVectorizationCostModel {
typedef SmallVector<Value*, 4> ValuePack;
typedef SmallVector<ValuePack, 4> MemAccessChainList;

public:
  LoopVectorizationCostModel(Loop *L, ScalarEvolution *SE, LoopInfo *LI,
                             LoopVectorizationLegality *Legal,
                             const TargetTransformInfo &TTI,
                             const DataLayout *DL, const TargetLibraryInfo *TLI)
      : TheLoop(L), SE(SE), LI(LI), Legal(Legal), TTI(TTI), DL(DL), TLI(TLI) {}

  /// Information about vectorization costs
  struct VectorizationFactor {
    unsigned Width; // Vector width with best cost
    unsigned Cost; // Cost of the loop with that width
  };
  /// \return The most profitable vectorization factor and the cost of that VF.
  /// This method checks every power of two up to VF. If UserVF is not ZERO
  /// then this vectorization factor will be selected if vectorization is
  /// possible.
  VectorizationFactor selectVectorizationFactor(bool OptForSize,
                                                unsigned UserVF,
                                                bool ForceVectorization);

  /// \return The size (in bits) of the widest type in the code that
  /// needs to be vectorized. We ignore values that remain scalar such as
  /// 64 bit loop indices.
  unsigned getWidestType();

  /// \return The most profitable unroll factor.
  /// If UserUF is non-zero then this method finds the best unroll-factor
  /// based on register pressure and other parameters.
  /// VF and LoopCost are the selected vectorization factor and the cost of the
  /// selected VF.
  unsigned selectUnrollFactor(bool OptForSize, unsigned UserUF, unsigned VF,
                              unsigned LoopCost);

  /// \brief A struct that represents some properties of the register usage
  /// of a loop.
  struct RegisterUsage {
    /// Holds the number of loop invariant values that are used in the loop.
    unsigned LoopInvariantRegs;
    /// Holds the maximum number of concurrent live intervals in the loop.
    unsigned MaxLocalUsers;
    /// Holds the number of instructions in the loop.
    unsigned NumInstructions;
  };

  /// \return  information about the register usage of the loop.
  RegisterUsage calculateRegisterUsage();

private:
  /// Returns the expected execution cost. The unit of the cost does
  /// not matter because we use the 'cost' units to compare different
  /// vector widths. The cost that is returned is *not* normalized by
  /// the factor width.
  unsigned expectedCost(unsigned VF);

  /// Returns the execution time cost of an instruction for a given vector
  /// width. Vector width of one means scalar.
  unsigned getInstructionCost(Instruction *I, unsigned VF);

  ///return the slp vars' cost in BB
  unsigned getAllSLPInstructionCost(BasicBlock *BB, unsigned VF);
  ///return the slp var v's cost
  unsigned getSLPInstructionCost(ArrayRef<Value *> bundle, unsigned SLPVF, unsigned LoopVF);  

  /// A helper function for converting Scalar types to vector types.
  /// If the incoming type is void, we return void. If the VF is 1, we return
  /// the scalar type.
  static Type* ToVectorTy(Type *Scalar, unsigned VF);

  /// Returns whether the instruction is a load or store and will be a emitted
  /// as a vector operation.
  bool isConsecutiveLoadOrStore(Instruction *I);

  /// The loop that we evaluate.
  Loop *TheLoop;
  /// Scev analysis.
  ScalarEvolution *SE;
  /// Loop Info analysis.
  LoopInfo *LI;
  /// Vectorization legality.
  LoopVectorizationLegality *Legal;
  /// Vector target information.
  const TargetTransformInfo &TTI;
  /// Target data layout information.
  const DataLayout *DL;
  /// Target Library Info.
  const TargetLibraryInfo *TLI;
};



/// The LoopVectorize Pass.
struct LoopVectorize : public FunctionPass {
  /// Pass identification, replacement for typeid
  static char ID;

  explicit LoopVectorize(bool NoUnrolling = false, bool AlwaysVectorize = true)
    : FunctionPass(ID),
      DisableUnrolling(NoUnrolling),
      AlwaysVectorize(AlwaysVectorize) {
    initializeLoopVectorizePass(*PassRegistry::getPassRegistry());
  }

  ScalarEvolution *SE;
  const DataLayout *DL;
  LoopInfo *LI;
  TargetTransformInfo *TTI;
  DominatorTree *DT;
  BlockFrequencyInfo *BFI;
  TargetLibraryInfo *TLI;
  AliasAnalysis *AA;
  bool DisableUnrolling;
  bool AlwaysVectorize;
  BlockFrequency ColdEntryFreq;


  bool runOnFunction(Function &F) override {
	
    SE = &getAnalysis<ScalarEvolution>();
    DataLayoutPass *DLP = getAnalysisIfAvailable<DataLayoutPass>();
    DL = DLP ? &DLP->getDataLayout() : nullptr;
    LI = &getAnalysis<LoopInfo>();
    TTI = &getAnalysis<TargetTransformInfo>();
    DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
    BFI = &getAnalysis<BlockFrequencyInfo>();
    TLI = getAnalysisIfAvailable<TargetLibraryInfo>();
    AA = &getAnalysis<AliasAnalysis>();

    // Compute some weights outside of the loop over the loops. Compute this
    // using a BranchProbability to re-use its scaling math.
    const BranchProbability ColdProb(1, 5); // 20%
    ColdEntryFreq = BlockFrequency(BFI->getEntryFreq()) * ColdProb;

    // If the target claims to have no vector registers don't attempt
    // vectorization.
    if (!TTI->getNumberOfRegisters(true))
      return false;

    if (!DL) 
      return false;

    // Build up a worklist of inner-loops to vectorize. This is necessary as
    // the act of vectorizing or partially unrolling a loop creates new loops
    // and can invalidate iterators across the loops.
    SmallVector<Loop *, 8> Worklist;

    for (Loop *L : *LI)
      addInnerLoop(*L, Worklist);

    LoopsAnalyzed += Worklist.size();

    // Now walk the identified inner loops.
    bool Changed = false;
    while (!Worklist.empty()){
      Changed |= processLoop(Worklist.pop_back_val());
	}
    return Changed;
  }


  bool processLoop(Loop *L) {
  	
    assert(L->empty() && "Only process inner loops.");
#ifndef NDEBUG
    const std::string DebugLocStr = getDebugLocString(L);
#endif 
    LoopVectorizeHints Hints(L, DisableUnrolling);
    DEBUG(dbgs() << "LV: Loop hints:"
                 << " force="
                 << (Hints.getForce() == LoopVectorizeHints::FK_Disabled
                         ? "disabled"
                         : (Hints.getForce() == LoopVectorizeHints::FK_Enabled
                                ? "enabled"
                                : "?")) << " width=" << Hints.getWidth()
                 << " unroll=" << Hints.getUnroll() << "\n");

    // Function containing loop
    Function *F = L->getHeader()->getParent();
    // Looking at the diagnostic output is the only way to determine if a loop
    // was vectorized (other than looking at the IR or machine code), so it
    // is important to generate an optimization remark for each loop. Most of
    // these messages are generated by emitOptimizationRemarkAnalysis. Remarks
    // generated by emitOptimizationRemark and emitOptimizationRemarkMissed are
    // less verbose reporting vectorized loops and unvectorized loops that may
    // benefit from vectorization, respectively.
    if (Hints.getForce() == LoopVectorizeHints::FK_Disabled) {
      DEBUG(dbgs() << "LV: Not vectorizing: #pragma vectorize disable.\n");
      emitOptimizationRemarkAnalysis(F->getContext(), DEBUG_TYPE, *F,
                                     L->getStartLoc(), Hints.emitRemark());
      return false;
    }
    if (!AlwaysVectorize && Hints.getForce() != LoopVectorizeHints::FK_Enabled) {
      DEBUG(dbgs() << "LV: Not vectorizing: No #pragma vectorize enable.\n");
      emitOptimizationRemarkAnalysis(F->getContext(), DEBUG_TYPE, *F,
                                     L->getStartLoc(), Hints.emitRemark());
      return false;
    }
    if (Hints.getWidth() == 1 && Hints.getUnroll() == 1) {
      DEBUG(dbgs() << "LV: Not vectorizing: Disabled/already vectorized.\n");
      emitOptimizationRemarkAnalysis(
          F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
          "loop not vectorized: vector width and interleave count are "
          "explicitly set to 1");
      return false;
    }

    // Check the loop for a trip count threshold:
    // do not vectorize loops with a tiny trip count.
    BasicBlock *Latch = L->getLoopLatch();
    const unsigned TC = SE->getSmallConstantTripCount(L, Latch);
    if (TC > 0u && TC < TinyTripCountVectorThreshold) {
      errs() << "LV: Found a loop with a very small trip count. "
                   << "This loop is not worth vectorizing.\n";
      if (Hints.getForce() == LoopVectorizeHints::FK_Enabled)
        errs() << " But vectorizing was explicitly forced.\n";
      else {
        DEBUG(dbgs() << "\n");
        emitOptimizationRemarkAnalysis(
            F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
            "vectorization is not beneficial and is not explicitly forced");
        return false;
      }
    }

    // Check if it is legal to vectorize the loop.
    LoopVectorizationLegality LVL(L, SE, DL, DT, TLI, AA, F);
    if (!LVL.canVectorize()) {
	  errs()<<"no parallelism across iterations !!!\n";
      emitMissedWarning(F, L, Hints);
      return false;
    }

    // Check the function attributes to see if implicit floats are allowed.a
    // FIXME: This check doesn't seem possibly correct -- what if the loop is
    // an integer loop and the vector instructions selected are purely integer
    // vector instructions?
    if (F->hasFnAttribute(Attribute::NoImplicitFloat)) {
      errs() << "LV: Can't vectorize when the NoImplicitFloat"
            "attribute is used.\n";
      emitOptimizationRemarkAnalysis(
          F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
          "loop not vectorized due to NoImplicitFloat attribute");
      emitMissedWarning(F, L, Hints);
      return false;
    }
	
    //// *********************************************************************************
	//// here, we do bottom-up or top-down search to put variables in the loop into two category;
	//// i.e. find the variables which should be vectorized in SLP/loop manner (default: in loop manner);
	//// starting point is the consecutive memory accesses.
	////////1. identify all the consecutive loads/stores;
	////////2. trace def-use or use-def chain to explore further;
	////////3. solve conflict if it exists.	
	LVL.findConsecutiveMemAccess();
	LVL.findSLPVars();		
	//// *********************************************************************************
		
    // Use the cost model.
    LoopVectorizationCostModel CM(L, SE, LI, &LVL, *TTI, DL, TLI);
    // Check the function attributes to find out if this function should be
    // optimized for size.
    bool OptForSize = Hints.getForce() != LoopVectorizeHints::FK_Enabled &&
                      F->hasFnAttribute(Attribute::OptimizeForSize);
    // Compute the weighted frequency of this loop being executed and see if it
    // is less than 20% of the function entry baseline frequency. Note that we
    // always have a canonical loop here because we think we *can* vectoriez.
    // FIXME: This is hidden behind a flag due to pervasive problems with
    // exactly what block frequency models.
    if (LoopVectorizeWithBlockFrequency) {
      BlockFrequency LoopEntryFreq = BFI->getBlockFreq(L->getLoopPreheader());
      if (Hints.getForce() != LoopVectorizeHints::FK_Enabled &&
          LoopEntryFreq < ColdEntryFreq)
        OptForSize = true;
    }

    // Select the optimal vectorization factor.
    LoopVectorizationCostModel::VectorizationFactor VF =
        CM.selectVectorizationFactor(OptForSize, Hints.getWidth(),
                                     Hints.getForce() ==
                                         LoopVectorizeHints::FK_Enabled);
    // Select the unroll factor.
    unsigned UF =
        CM.selectUnrollFactor(OptForSize, Hints.getUnroll(), VF.Width, VF.Cost);



	// do actual code generation
    if (VF.Width == 1) {
      if (UF == 1) {
         emitOptimizationRemarkAnalysis(F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
            "not beneficial to vectorize and user disabled interleaving");
		 return false;
      }

      // Report the unrolling decision.
      emitOptimizationRemark(F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
                             Twine("unrolled with interleaving factor " +
                             Twine(UF) +  " (vectorization not beneficial)"));
      // We decided not to vectorize, but we may want to unroll.
      InnerLoopUnroller Unroller(L, SE, LI, DT, DL, TLI, UF);
      Unroller.vectorize(&LVL);
    } 
	else 
	{ 	
      InnerLoopVectorizer LB(L, SE, LI, DT, DL, *TTI, TLI, VF.Width, UF);
      LB.vectorize(&LVL);
      
      // Report the vectorization decision.
      emitOptimizationRemark(
          F->getContext(), DEBUG_TYPE, *F, L->getStartLoc(),
          Twine("vectorized loop (vectorization factor: ") + Twine(VF.Width) +
              ", unrolling interleave factor: " + Twine(UF) + ")");	  
    }

    // Mark the loop as already vectorized to avoid vectorizing again.
    Hints.setAlreadyVectorized(L);
    DEBUG(verifyFunction(*L->getHeader()->getParent()));
    return true;
	
  }

  void getAnalysisUsage(AnalysisUsage &AU) const override {
    AU.addRequiredID(LoopSimplifyID);
    AU.addRequiredID(LCSSAID);
    AU.addRequired<BlockFrequencyInfo>();
    AU.addRequired<DominatorTreeWrapperPass>();
    AU.addRequired<LoopInfo>();
    AU.addRequired<ScalarEvolution>();
    AU.addRequired<TargetTransformInfo>();
    AU.addRequired<AliasAnalysis>();
    AU.addPreserved<LoopInfo>();
    AU.addPreserved<DominatorTreeWrapperPass>();
    AU.addPreserved<AliasAnalysis>();
  }

};

} // end anonymous namespace



namespace {
struct CSEDenseMapInfo {
  static bool canHandle(Instruction *I) {
    return isa<InsertElementInst>(I) || isa<ExtractElementInst>(I) ||
           isa<ShuffleVectorInst>(I) || isa<GetElementPtrInst>(I);
  }
  static inline Instruction *getEmptyKey() {
    return DenseMapInfo<Instruction *>::getEmptyKey();
  }
  static inline Instruction *getTombstoneKey() {
    return DenseMapInfo<Instruction *>::getTombstoneKey();
  }
  static unsigned getHashValue(Instruction *I) {
    assert(canHandle(I) && "Unknown instruction!");
    return hash_combine(I->getOpcode(), hash_combine_range(I->value_op_begin(),
                                                           I->value_op_end()));
  }
  static bool isEqual(Instruction *LHS, Instruction *RHS) {
    if (LHS == getEmptyKey() || RHS == getEmptyKey() ||
        LHS == getTombstoneKey() || RHS == getTombstoneKey())
      return LHS == RHS;
    return LHS->isIdenticalTo(RHS);
  }
};
}



namespace {
/// \brief Analyses memory accesses in a loop.
///
/// Checks whether run time pointer checks are needed and builds sets for data
/// dependence checking.
class AccessAnalysis {
public:
  /// \brief Read or write access location.
  typedef PointerIntPair<Value *, 1, bool> MemAccessInfo;
  typedef SmallPtrSet<MemAccessInfo, 8> MemAccessInfoSet;

  /// \brief Set of potential dependent memory accesses.
  typedef EquivalenceClasses<MemAccessInfo> DepCandidates;

  AccessAnalysis(const DataLayout *Dl, AliasAnalysis *AA, DepCandidates &DA) :
    DL(Dl), AST(*AA), DepCands(DA), IsRTCheckNeeded(false) {}

  /// \brief Register a load  and whether it is only read from.
  void addLoad(AliasAnalysis::Location &Loc, bool IsReadOnly) {
    Value *Ptr = const_cast<Value*>(Loc.Ptr);
    AST.add(Ptr, AliasAnalysis::UnknownSize, Loc.TBAATag);
    Accesses.insert(MemAccessInfo(Ptr, false));
    if (IsReadOnly)
      ReadOnlyPtr.insert(Ptr);
  }

  /// \brief Register a store.
  void addStore(AliasAnalysis::Location &Loc) {
    Value *Ptr = const_cast<Value*>(Loc.Ptr);
    AST.add(Ptr, AliasAnalysis::UnknownSize, Loc.TBAATag);
    Accesses.insert(MemAccessInfo(Ptr, true));
  }

  /// \brief Check whether we can check the pointers at runtime for
  /// non-intersection.

  bool canTryCheckPtrAtRT(
	  LoopVectorizationLegality::RuntimePointerCheck &RtCheck,
	  unsigned &NumComparisons, ScalarEvolution *SE, Loop *TheLoop,
	  ValueToValueMap &StridesMap);

  /// \brief Goes over all memory accesses, checks whether a RT check is needed
  /// and builds sets of dependent accesses.
  void buildDependenceSets() {
    processMemAccesses();
  }

  bool isRTCheckNeeded() { return IsRTCheckNeeded; }

  bool isDependencyCheckNeeded() { return !CheckDeps.empty(); }
  void resetDepChecks() { CheckDeps.clear(); }

  MemAccessInfoSet &getDependenciesToCheck() { return CheckDeps; }

private:
  typedef SetVector<MemAccessInfo> PtrAccessSet;

  /// \brief Go over all memory access and check whether runtime pointer checks
  /// are needed /// and build sets of dependency check candidates.
  void processMemAccesses();

  /// Set of all accesses.
  PtrAccessSet Accesses;

  /// Set of accesses that need a further dependence check.
  MemAccessInfoSet CheckDeps;

  /// Set of pointers that are read only.
  SmallPtrSet<Value*, 16> ReadOnlyPtr;

  const DataLayout *DL;

  /// An alias set tracker to partition the access set by underlying object and
  //intrinsic property (such as TBAA metadata).
  AliasSetTracker AST;

  /// Sets of potentially dependent accesses - members of one set share an
  /// underlying pointer. The set "CheckDeps" identfies which sets really need a
  /// dependence check.
  DepCandidates &DepCands;

  bool IsRTCheckNeeded;
};

} // end anonymous namespace

void AccessAnalysis::processMemAccesses() {
  // We process the set twice: first we process read-write pointers, last we
  // process read-only pointers. This allows us to skip dependence tests for
  // read-only pointers.

  // The AliasSetTracker has nicely partitioned our pointers by metadata (tbaa)
  // compatibility and potential for underlying-object overlap. As a result, we
  // only need to check for potential pointer dependencies within each alias
  // set.
  for (auto &AS : AST) {
    // Note that both the alias-set tracker and the alias sets themselves used
    // linked lists internally and so the iteration order here is deterministic
    // (matching the original instruction order within each set).
    bool SetHasWrite = false;
    // Map of pointers to last access encountered.
    typedef DenseMap<Value*, MemAccessInfo> UnderlyingObjToAccessMap;
    UnderlyingObjToAccessMap ObjToLastAccess;

    // Set of access to check after all writes have been processed.
    PtrAccessSet DeferredAccesses;

    // Iterate over each alias set twice, once to process read/write pointers,
    // and then to process read-only pointers.
    for (int SetIteration = 0; SetIteration < 2; ++SetIteration) {
      bool UseDeferred = SetIteration > 0;
      PtrAccessSet &S = UseDeferred ? DeferredAccesses : Accesses;

      for (auto A : AS) {
        Value *Ptr = A.getValue();
        bool IsWrite = S.count(MemAccessInfo(Ptr, true));   ////because of this line, only the store access on the pointer can be recorded in DepCands and CheckDeps

        // If we're using the deferred access set, then it contains only reads.
        bool IsReadOnlyPtr = ReadOnlyPtr.count(Ptr) && !IsWrite;
        if (UseDeferred && !IsReadOnlyPtr)
          continue;
        // Otherwise, the pointer must be in the PtrAccessSet, either as a read
        // or a write.
        assert(((IsReadOnlyPtr && UseDeferred) || IsWrite ||
                 S.count(MemAccessInfo(Ptr, false))) &&
               "Alias-set pointer not in the access set?");

        MemAccessInfo Access(Ptr, IsWrite);
        DepCands.insert(Access);

        // Memorize read-only pointers for later processing and skip them in the
        // first round (they need to be checked after we have seen all write
        // pointers). Note: we also mark pointer that are not consecutive as
        // "read-only" pointers (so that we check "a[b[i]] +="). Hence, we need
        // the second check for "!IsWrite".
        if (!UseDeferred && IsReadOnlyPtr) {
          DeferredAccesses.insert(Access);
          continue;
        }

        // If this is a write - check other reads and writes for conflicts.  If
        // this is a read only check other writes for conflicts (but only if
        // there is no other write to the ptr - this is an optimization to
        // catch "a[i] = a[i] + " without having to do a dependence check).
        if ((IsWrite || IsReadOnlyPtr) && SetHasWrite) {
          CheckDeps.insert(Access);
          IsRTCheckNeeded = true;
        }

        if (IsWrite)
          SetHasWrite = true;

	// Create sets of pointers connected by a shared alias set and
	// underlying object.

	////all the access on the same object are now in the same set in DepCands.
        typedef SmallVector<Value*, 16> ValueVector;
        ValueVector TempObjects;
        GetUnderlyingObjects(Ptr, TempObjects, DL);
        for (Value *UnderlyingObj : TempObjects) {
          UnderlyingObjToAccessMap::iterator Prev =
            ObjToLastAccess.find(UnderlyingObj);
          if (Prev != ObjToLastAccess.end())
            DepCands.unionSets(Access, Prev->second); /////DepCands only contains access info (not precisely related to objs)
          ObjToLastAccess[UnderlyingObj] = Access;
        }

    ////***********************
    ////here is the code to fix the original bug of llvm 3.5.0, we record the information of the load access of a write-read pointer
	  	bool isRWMemAccess = IsWrite && S.count(MemAccessInfo(Ptr, false));
	  	if(isRWMemAccess){
			MemAccessInfo Access_ld(Ptr, false);
			DepCands.insert(Access_ld);

			TempObjects.clear();
			GetUnderlyingObjects(Ptr, TempObjects, DL);
			for (Value *UnderlyingObj : TempObjects) {
			  UnderlyingObjToAccessMap::iterator Prev =
				ObjToLastAccess.find(UnderlyingObj);
			  if (Prev != ObjToLastAccess.end())
				DepCands.unionSets(Access_ld, Prev->second);
			  else{
			  	assert(false && "unhandled case!");
			  }
			}		
	  	}
   ////***********************

      }
    }
  }
}


bool AccessAnalysis::canTryCheckPtrAtRT(
    LoopVectorizationLegality::RuntimePointerCheck &RtCheck,
    unsigned &NumComparisons, ScalarEvolution *SE, Loop *TheLoop,
    ValueToValueMap &StridesMap) {
  // Find pointers with computable bounds. We are going to use this information
  // to place a runtime bound check.
  bool CanDoRT = true;

  bool IsDepCheckNeeded = isDependencyCheckNeeded();
  NumComparisons = 0;

  // We assign a consecutive id to access from different alias sets.
  // Accesses between different groups doesn't need to be checked.
  unsigned ASId = 1;
  for (auto &AS : AST) {
    unsigned NumReadPtrChecks = 0;
    unsigned NumWritePtrChecks = 0;

    // We assign consecutive id to access from different dependence sets.
    // Accesses within the same set don't need a runtime check.
    unsigned RunningDepId = 1;
    DenseMap<Value *, unsigned> DepSetId;

    for (auto A : AS) {
      Value *Ptr = A.getValue();
      bool IsWrite = Accesses.count(MemAccessInfo(Ptr, true));
      MemAccessInfo Access(Ptr, IsWrite);

      if (IsWrite)
        ++NumWritePtrChecks;
      else
        ++NumReadPtrChecks;

      if ((hasComputableBounds(SE, StridesMap, Ptr)) || (TheLoop->isLoopInvariant(Ptr))) {
        // The id of the dependence set.
        unsigned DepId;
        /////if we apply runtime array overlap check, then we only need to test if the current objs being pointed are overlapped
        /////otherwise, we have to check any of the objs potentially could being pointed are overlapped or not
        /////here, we conservatively assume the accesses in the same DepCands subset targets on the same objs
        if (IsDepCheckNeeded) {
          Value *Leader = DepCands.getLeaderValue(Access).getPointer();
          unsigned &LeaderId = DepSetId[Leader];
          if (!LeaderId)
            LeaderId = RunningDepId++;
          DepId = LeaderId;
        } else
          // Each access has its own dependence set.
          DepId = RunningDepId++;

        RtCheck.insert(SE, TheLoop, Ptr, IsWrite, DepId, ASId, StridesMap);
      }
	  else{		
        CanDoRT = false;
      }
    }

    if (IsDepCheckNeeded && CanDoRT && RunningDepId == 2)
      NumComparisons += 0; // Only one dependence set.
    else {
      NumComparisons += (NumWritePtrChecks * (NumReadPtrChecks +
                                              NumWritePtrChecks - 1));
    }

    ++ASId;
  }

  // If the pointers that we would use for the bounds comparison have different
  // address spaces, assume the values aren't directly comparable, so we can't
  // use them for the runtime check. We also have to assume they could
  // overlap. In the future there should be metadata for whether address spaces
  // are disjoint.
  unsigned NumPointers = RtCheck.Pointers.size();
  for (unsigned i = 0; i < NumPointers; ++i) {
    for (unsigned j = i + 1; j < NumPointers; ++j) {
      // Only need to check pointers between two different dependency sets, which points to different objs.

      Value *PtrI = RtCheck.Pointers[i];
      Value *PtrJ = RtCheck.Pointers[j];

      if (RtCheck.DependencySetId[i] == RtCheck.DependencySetId[j]){
        continue;
	  }
      // Only need to check pointers in the same alias set.
      if (RtCheck.AliasSetId[i] != RtCheck.AliasSetId[j]){
        continue;
      }
	  
      unsigned ASi = PtrI->getType()->getPointerAddressSpace();
      unsigned ASj = PtrJ->getType()->getPointerAddressSpace();
      if (ASi != ASj) {
        return false;
      }
    }
  }

  return CanDoRT;
}



namespace {
/// \brief Checks memory dependences among accesses to the same underlying
/// object to determine whether there vectorization is legal or not (and at
/// which vectorization factor).
///
/// This class works under the assumption that we already checked that memory
/// locations with different underlying pointers are "must-not alias".
/// We use the ScalarEvolution framework to symbolically evalutate access
/// functions pairs. Since we currently don't restructure the loop we can rely
/// on the program order of memory accesses to determine their safety.
/// At the moment we will only deem accesses as safe for:
///  * A negative constant distance assuming program order.
///
///      Safe: tmp = a[i + 1];     OR     a[i + 1] = x;
///            a[i] = tmp;                y = a[i];
///
///   The latter case is safe because later checks guarantuee that there can't
///   be a cycle through a phi node (that is, we check that "x" and "y" is not
///   the same variable: a header phi can only be an induction or a reduction, a
///   reduction can't have a memory sink, an induction can't have a memory
///   source). This is important and must not be violated (or we have to
///   resort to checking for cycles through memory).
///
///  * A positive constant distance assuming program order that is bigger
///    than the biggest memory access.
///
///     tmp = a[i]        OR              b[i] = x
///     a[i+2] = tmp                      y = b[i+2];
///
///     Safe distance: 2 x sizeof(a[0]), and 2 x sizeof(b[0]), respectively.
///
///  * Zero distances and all accesses have the same size.
///
class MemoryDepChecker {
public:
  typedef PointerIntPair<Value *, 1, bool> MemAccessInfo;
  typedef SmallPtrSet<MemAccessInfo, 8> MemAccessInfoSet;
  typedef DenseMap< std::pair<Value*, Value*>, int> DepDistanceMap;

  MemoryDepChecker(ScalarEvolution *Se, const DataLayout *Dl, const Loop *L)
      : SE(Se), DL(Dl), InnermostLoop(L), AccessIdx(0),
        ShouldRetryWithRuntimeCheck(false) {}

  /// \brief Register the location (instructions are given increasing numbers)
  /// of a write access.
  void addAccess(StoreInst *SI) {
    Value *Ptr = SI->getPointerOperand();
    Accesses[MemAccessInfo(Ptr, true)].push_back(AccessIdx);
    InstMap.push_back(SI);
    ++AccessIdx;
  }

  /// \brief Register the location (instructions are given increasing numbers)
  /// of a write access.
  void addAccess(LoadInst *LI) {
    Value *Ptr = LI->getPointerOperand();
    Accesses[MemAccessInfo(Ptr, false)].push_back(AccessIdx);
    InstMap.push_back(LI);
    ++AccessIdx;
  }

  /// \brief Check whether the dependencies between the accesses are safe.
  ///
  /// Only checks sets with elements in \p CheckDeps.
  bool areDepsSafe(LoopVectorizationLegality::RuntimePointerCheck &RtCheck, AccessAnalysis::DepCandidates &AccessSets,
                   MemAccessInfoSet &CheckDeps, ValueToValueMap &Strides, DepDistanceMap &DepDistance);

  /// \brief The maximum number of bytes of a vector register we can vectorize
  /// the accesses safely with.
  unsigned getMaxSafeDepDistBytes() { return MaxSafeDepDistBytes; }

  /// \brief In same cases when the dependency check fails we can still
  /// vectorize the loop with a dynamic array access check.
  bool shouldRetryWithRuntimeCheck() { return ShouldRetryWithRuntimeCheck; }

private:
  ScalarEvolution *SE;
  const DataLayout *DL;
  const Loop *InnermostLoop;

  /// \brief Maps access locations (ptr, read/write) to program order.
  DenseMap<MemAccessInfo, std::vector<unsigned> > Accesses;

  /// \brief Memory access instructions in program order.
  SmallVector<Instruction *, 16> InstMap;

  /// \brief The program order index to be used for the next instruction.
  unsigned AccessIdx;

  // We can access this many bytes in parallel safely.
  unsigned MaxSafeDepDistBytes;

  /// \brief If we see a non-constant dependence distance we can still try to
  /// vectorize this loop with runtime checks.
  bool ShouldRetryWithRuntimeCheck;

  /// \brief Check whether there is a plausible dependence between the two
  /// accesses.
  ///
  /// Access \p A must happen before \p B in program order. The two indices
  /// identify the index into the program order map.
  ///
  /// This function checks  whether there is a plausible dependence (or the
  /// absence of such can't be proved) between the two accesses. If there is a
  /// plausible dependence but the dependence distance is bigger than one
  /// element access it records this distance in \p MaxSafeDepDistBytes (if this
  /// distance is smaller than any other distance encountered so far).
  /// Otherwise, this function returns true signaling a possible dependence.
  bool isDependent(LoopVectorizationLegality::RuntimePointerCheck &RtCheck,
                   const MemAccessInfo &A, unsigned AIdx,
                   const MemAccessInfo &B, unsigned BIdx,
                   ValueToValueMap &Strides, DepDistanceMap &DepDistance);

  /// \brief Check whether the data dependence could prevent store-load
  /// forwarding.
  bool couldPreventStoreLoadForward(unsigned Distance, unsigned TypeByteSize);
};

} // end anonymous namespace

bool MemoryDepChecker::couldPreventStoreLoadForward(unsigned Distance,
                                                    unsigned TypeByteSize) {
  // If loads occur at a distance that is not a multiple of a feasible vector
  // factor store-load forwarding does not take place.
  // Positive dependences might cause troubles because vectorizing them might
  // prevent store-load forwarding making vectorized code run a lot slower.
  //   a[i] = a[i-3] ^ a[i-8];
  //   The stores to a[i:i+1] don't align with the stores to a[i-3:i-2] and
  //   hence on your typical architecture store-load forwarding does not take
  //   place. Vectorizing in such cases does not make sense.
  // Store-load forwarding distance.
  const unsigned NumCyclesForStoreLoadThroughMemory = 8*TypeByteSize;
  // Maximum vector factor.
  unsigned MaxVFWithoutSLForwardIssues = MaxVectorWidth*TypeByteSize;
  if(MaxSafeDepDistBytes < MaxVFWithoutSLForwardIssues)
    MaxVFWithoutSLForwardIssues = MaxSafeDepDistBytes;

  for (unsigned vf = 2*TypeByteSize; vf <= MaxVFWithoutSLForwardIssues;
       vf *= 2) {
    if (Distance % vf && Distance / vf < NumCyclesForStoreLoadThroughMemory) {
      MaxVFWithoutSLForwardIssues = (vf >>=1);
      break;
    }
  }

  if (MaxVFWithoutSLForwardIssues< 2*TypeByteSize) {
    DEBUG(dbgs() << "LV: Distance " << Distance <<
          " that could cause a store-load forwarding conflict\n");
    return true;
  }

  if (MaxVFWithoutSLForwardIssues < MaxSafeDepDistBytes &&
      MaxVFWithoutSLForwardIssues != MaxVectorWidth*TypeByteSize)
    MaxSafeDepDistBytes = MaxVFWithoutSLForwardIssues;
  return false;
}

bool MemoryDepChecker::isDependent(LoopVectorizationLegality::RuntimePointerCheck &RtCheck,
	                               const MemAccessInfo &A, unsigned AIdx,
                                   const MemAccessInfo &B, unsigned BIdx,
                                   ValueToValueMap &Strides, DepDistanceMap &DepDistance) {
  assert (AIdx < BIdx && "Must pass arguments in program order");

  Value *APtr = A.getPointer();
  Value *BPtr = B.getPointer();
  bool AIsWrite = A.getInt();
  bool BIsWrite = B.getInt();

  // Two reads are independent.
  if (!AIsWrite && !BIsWrite){
    return false;
  }

  // We cannot check pointers in different address spaces.
  if (APtr->getType()->getPointerAddressSpace() !=
      BPtr->getType()->getPointerAddressSpace()){
    return true;
  }

  ////replace the symbolic stride in SCEV with 1
  const SCEV *AScev = replaceSymbolicStrideSCEV(SE, Strides, APtr);
  const SCEV *BScev = replaceSymbolicStrideSCEV(SE, Strides, BPtr);
  ////get the constant stride if possible, otherwise return 0
  int StrideAPtr = isStridedPtr(SE, DL, APtr, InnermostLoop, Strides);
  int StrideBPtr = isStridedPtr(SE, DL, BPtr, InnermostLoop, Strides);
  const SCEV *Src = AScev;
  const SCEV *Sink = BScev;

  // If the induction step is negative we have to invert source and sink of the dependence.
  if (StrideAPtr < 0) {
    std::swap(APtr, BPtr);
    std::swap(Src, Sink);
    std::swap(AIsWrite, BIsWrite);
    std::swap(AIdx, BIdx);
    std::swap(StrideAPtr, StrideBPtr);
  }

  // Bail out early if passed-in parameters make vectorization not feasible.
  unsigned ForcedFactor = VectorizationFactor ? VectorizationFactor : 1;
  unsigned ForcedUnroll = VectorizationUnroll ? VectorizationUnroll : 1;
  
  Type *ATy = APtr->getType()->getPointerElementType();
  Type *BTy = BPtr->getType()->getPointerElementType();
  unsigned TypeByteSize = DL->getTypeAllocSize(ATy);  

  const SCEV *Dist = SE->getMinusSCEV(Sink, Src);
  const SCEVConstant *C = dyn_cast<SCEVConstant>(Dist);  

  if(C){
	  const APInt &Val = C->getValue()->getValue();
	  int Distance = Val.getSExtValue();

	  std::pair <Value*, Value*> mem_acc_p (APtr, BPtr);
	  if(DepDistance.count(mem_acc_p))
		  if(DepDistance[mem_acc_p] != Distance)
			assert(false && "different dep distance for the same pair !!!");
	  DepDistance[mem_acc_p] = Distance;
  }

  bool LoopInvarAPtr = InnermostLoop->isLoopInvariant(APtr);
  bool LoopInvarBPtr = InnermostLoop->isLoopInvariant(BPtr);

  if(LoopInvarAPtr || LoopInvarBPtr){
  	if(LoopInvarAPtr && LoopInvarBPtr){
		if(Dist->isZero()){	
			return true;
		}			
	}else{
		RtCheck.addArrayRangeCheck(SE, InnermostLoop, APtr, BPtr);	
	}
	return false;	
  }
  ////if these are stride accesses, need to analysis more 
  else if ((StrideAPtr != StrideBPtr) || (StrideAPtr != 1 && StrideAPtr != -1)){
	////could insert runtime check (if necessary) to pass legality test for stride access
	////as long as the stride and offset is fixed (i.e. loop invariant)
	const SCEV *StrideA = getStride(SE, SE->getSCEV(APtr), InnermostLoop);
	const SCEV *StrideB = getStride(SE, SE->getSCEV(BPtr), InnermostLoop);	
	
    if(!SE->isLoopInvariant(StrideA, InnermostLoop) || !SE->isLoopInvariant(StrideB, InnermostLoop)){
		return true;
	}

    const SCEV *Diff_stride = SE->getMinusSCEV(StrideA, StrideB);	
	////if the two accesses has the same stride:
	if(Diff_stride->isZero()){
		if(!SE->isLoopInvariant(Dist, InnermostLoop)){
			return true;
		}
		////stride and offset are known constants, and stride != 0, we can simple do static check		
		if(StrideAPtr && C){
			const APInt &c_val = C->getValue()->getValue();
            int offset_t = c_val.getSExtValue()/TypeByteSize;//(unsigned) 

			if(!offset_t){
				return false;
			}
			else if(offset_t%abs(StrideAPtr)){  //// in this case, we dont care the value of VF and UF
				return false;
			}
			else if((abs(offset_t/StrideAPtr) >= ForcedUnroll*ForcedFactor) && (abs(offset_t/StrideAPtr) >= maxVecRegWidthInbit/(TypeByteSize*8))){
				return false;
			}
			else{
				return true;
			}
		}
		////either stride or offset is a unknown constant, we need to do runtime check
		else{
			if(!StrideAPtr){ ////check if stride equals zero
				if(dyn_cast<SCEVConstant>(StrideA)){ ////try static test first
                    assert(false && "unhandled case!!!");
					//return true;
				}
			}			
		    RtCheck.addOffsetRemStrideCheck(Dist, StrideA);
			////(optional) runtime check on if offset_t > StrideAPtr*VL
		}
	}
	////if the two accesses has the different stride: we do runtime range check
	else{
		if(!dyn_cast<SCEVAddRecExpr>(Dist)){
			return true;
		}		
		RtCheck.addArrayRangeCheck(SE, InnermostLoop, APtr, BPtr);	
	}	
    return false;
  }
  ////if these are stride accesses, but the offset is not constant
  else if(!C){
	if(!SE->isLoopInvariant(Dist, InnermostLoop)){
		return true;
	}	
	RtCheck.addArrayRangeCheck(SE, InnermostLoop, APtr, BPtr);
	return false;
  }  
  ////if these are not stride accesses, and the offset is constant  
  else{
	  const APInt &Val = C->getValue()->getValue();
	  unsigned Distance = (unsigned) Val.getZExtValue();
	  
	  // Negative distances are not plausible dependencies.  
	  if (Val.isNegative()) {
		bool IsTrueDataDependence = (AIsWrite && !BIsWrite);
		if (IsTrueDataDependence &&
			(couldPreventStoreLoadForward(Val.abs().getZExtValue(), TypeByteSize) ||
			 ATy != BTy)){
		  return true;
		}
		return false;
	  }
  
	  // Write to the same location with the same size.
	  // Could be improved to assert type sizes are the same (i32 == float, etc).
	  if (Val == 0) {
		if (ATy == BTy){
		  return false;
		}
		return true;
	  }
	  	  
	  assert(Val.isStrictlyPositive() && "Expect a positive value");
	  // Positive distance bigger than max vectorization factor.
	  if (ATy != BTy) {
		return true;
	  }
	  // The distance must be bigger than the size needed for a vectorized version
	  // of the operation and the size of the vectorized operation must not be
	  // bigger than the currrent maximum size.
	  if (Distance < 2*TypeByteSize ||
		  2*TypeByteSize > MaxSafeDepDistBytes ||
		  Distance < TypeByteSize * ForcedUnroll * ForcedFactor) {
		return true;
	  }
	  MaxSafeDepDistBytes = Distance < MaxSafeDepDistBytes ?
		Distance : MaxSafeDepDistBytes;
	  bool IsTrueDataDependence = (!AIsWrite && BIsWrite);
	  if (IsTrueDataDependence &&
		  couldPreventStoreLoadForward(Distance, TypeByteSize)){
		return true;
	  }
	  return false;

  } 
}

bool MemoryDepChecker::areDepsSafe(LoopVectorizationLegality::RuntimePointerCheck &RtCheck, 
	                               AccessAnalysis::DepCandidates &AccessSets,
                                   MemAccessInfoSet &CheckDeps,
                                   ValueToValueMap &Strides, DepDistanceMap &DepDistance) {

  MaxSafeDepDistBytes = -1U;
  while (!CheckDeps.empty()) {
    MemAccessInfo CurAccess = *CheckDeps.begin();

    // Get the relevant memory access set.
    EquivalenceClasses<MemAccessInfo>::iterator I =
      AccessSets.findValue(AccessSets.getLeaderValue(CurAccess));

    // Check accesses within this set.
    EquivalenceClasses<MemAccessInfo>::member_iterator AI, AE;
    AI = AccessSets.member_begin(I), AE = AccessSets.member_end();

    // Check every access pair.
    while (AI != AE) {
      CheckDeps.erase(*AI);
      EquivalenceClasses<MemAccessInfo>::member_iterator OI = std::next(AI);	  
      while (OI != AE) {
        // Check every accessing instruction pair in program order.
        ////unlike the runtime check, here we need to check every access instead of the access sets
        for (std::vector<unsigned>::iterator I1 = Accesses[*AI].begin(),
             I1E = Accesses[*AI].end(); I1 != I1E; ++I1){
          for (std::vector<unsigned>::iterator I2 = Accesses[*OI].begin(),
               I2E = Accesses[*OI].end(); I2 != I2E; ++I2) {
            if (*I1 < *I2 && isDependent(RtCheck, *AI, *I1, *OI, *I2, Strides, DepDistance))
              return false;
            if (*I2 < *I1 && isDependent(RtCheck, *OI, *I2, *AI, *I1, Strides, DepDistance))
              return false;
          }
		}
        ++OI;
      }
      AI++;
    }
  }
  return true;
}

static Type* getSameType(ArrayRef<Value *> VL) {
  Type *Ty = VL[0]->getType();
  for (unsigned i = 1, e = VL.size(); i < e; i++)
    if (VL[i]->getType() != Ty)
      return nullptr;
  return Ty;
}

static unsigned getSameOpcode(ArrayRef<Value *> VL) {
  Instruction *I0 = dyn_cast<Instruction>(VL[0]);
  if (!I0)
    return 0;
  unsigned Opcode = I0->getOpcode();
  for (unsigned i = 1, e = VL.size(); i < e; i++) {
    Instruction *I = dyn_cast<Instruction>(VL[i]);
    if (!I || Opcode != I->getOpcode()) {
      if (canCombineAsAltInst(Opcode) && i == 1)
        return isAltInst(VL);
      return 0;
    }
  }
  return Opcode;
}

///\returns Opcode that can be clubbed with \p Op to create an alternate
/// sequence which can later be merged as a ShuffleVector instruction.
static unsigned getAltOpcode(unsigned Op) {
  switch (Op) {
  case Instruction::FAdd:
    return Instruction::FSub;
  case Instruction::FSub:
    return Instruction::FAdd;
  case Instruction::Add:
    return Instruction::Sub;
  case Instruction::Sub:
    return Instruction::Add;
  default:
    return 0;
  }
}

///\returns bool representing if Opcode \p Op can be part
/// of an alternate sequence which can later be merged as
/// a ShuffleVector instruction.
static bool canCombineAsAltInst(unsigned Op) {
  if (Op == Instruction::Sub|| Op == Instruction::FSub
  	|| Op == Instruction::FAdd  || Op == Instruction::Add)
    return true;
  return false;
}

/// \returns ShuffleVector instruction if intructions in \p VL have
///  alternate fadd,fsub / fsub,fadd/add,sub/sub,add sequence.
/// (i.e. e.g. opcodes of fadd,fsub,fadd,fsub...)
static unsigned isAltInst(ArrayRef<Value *> VL) {
  Instruction *I0 = dyn_cast<Instruction>(VL[0]);
  unsigned Opcode = I0->getOpcode();
  unsigned AltOpcode = getAltOpcode(Opcode);
  for (int i = 1, e = VL.size(); i < e; i++) {
    Instruction *I = dyn_cast<Instruction>(VL[i]);
    if (!I || I->getOpcode() != ((i & 1) ? AltOpcode : Opcode))
      return 0;
  }
  return Instruction::ShuffleVector;
}

static BasicBlock *getSameBlock(ArrayRef<Value *> VL) {
	Instruction *I0 = dyn_cast<Instruction>(VL[0]);
	if (!I0)
	  return nullptr;
	BasicBlock *BB = I0->getParent();
	for (int i = 1, e = VL.size(); i < e; i++) {
	  Instruction *I = dyn_cast<Instruction>(VL[i]);
	  if (!I)
		return nullptr;
	  if (BB != I->getParent())
		return nullptr;
	}
	return BB;
}

static bool allConstant(ArrayRef<Value *> VL) {
  for (unsigned i = 0, e = VL.size(); i < e; ++i)
    if (!isa<Constant>(VL[i]))
      return false;
  return true;
}


static bool isSplat(ArrayRef<Value *> VL) {
  for (unsigned i = 1, e = VL.size(); i < e; ++i)
    if (VL[i] != VL[0])
      return false;
  return true;
}

static bool isPartialSplat(ArrayRef<Value *> VL){

  std::set<Value*> valueset;
  for (unsigned i = 0, e = VL.size(); i < e; ++i){
	  if(valueset.count(VL[i]))
		  return true;
	  else
	  	  valueset.insert(VL[i]);
  }
  return false;	
}

static void reorderInputsAccordingToOpcode(ArrayRef<Value *> VL,
                                           SmallVectorImpl<Value *> &Left,
                                           SmallVectorImpl<Value *> &Right) {

  SmallVector<Value *, 16> OrigLeft, OrigRight;

  bool AllSameOpcodeLeft = true;
  bool AllSameOpcodeRight = true;
  for (unsigned i = 0, e = VL.size(); i != e; ++i) {
    Instruction *I = cast<Instruction>(VL[i]);
    Value *V0 = I->getOperand(0);
    Value *V1 = I->getOperand(1);
    OrigLeft.push_back(V0);
    OrigRight.push_back(V1);

    Instruction *I0 = dyn_cast<Instruction>(V0);
    Instruction *I1 = dyn_cast<Instruction>(V1);
    // Check whether all operands on one side have the same opcode. In this case
    // we want to preserve the original order and not make things worse by
    // reordering.
    AllSameOpcodeLeft = I0;
    AllSameOpcodeRight = I1;
    if (i && AllSameOpcodeLeft) {
      if(Instruction *P0 = dyn_cast<Instruction>(OrigLeft[i-1])) {
        if(P0->getOpcode() != I0->getOpcode())
          AllSameOpcodeLeft = false;
        if(!AllSameOpcodeLeft && I1 && (P0->getOpcode() == I1->getOpcode())){
          /////switch operand
          Instruction * i_tmp = I0;
          I0 = I1;
          I1 = i_tmp;
          AllSameOpcodeRight = I1;
          Value * v_tmp =  OrigLeft[i];
          OrigLeft[i] = OrigRight[i];
          OrigRight[i] = v_tmp;
          AllSameOpcodeLeft = true;
        }
      }
      else
        AllSameOpcodeLeft = false;
    }
    if (i && AllSameOpcodeRight) {
      if(Instruction *P1 = dyn_cast<Instruction>(OrigRight[i-1])) {
        if(P1->getOpcode() != I1->getOpcode())
          AllSameOpcodeRight = false;
      } else
        AllSameOpcodeRight = false;
    }

    // Sort two opcodes. In the code below we try to preserve the ability to use
    // broadcast of values instead of individual inserts.
    // vl1 = load
    // vl2 = phi
    // vr1 = load
    // vr2 = vr2
    //    = vl1 x vr1
    //    = vl2 x vr2
    // If we just sorted according to opcode we would leave the first line in
    // tact but we would swap vl2 with vr2 because opcode(phi) > opcode(load).
    //    = vl1 x vr1
    //    = vr2 x vl2
    // Because vr2 and vr1 are from the same load we loose the opportunity of a
    // broadcast for the packed right side in the backend: we have [vr1, vl2]
    // instead of [vr1, vr2=vr1].
    if (I0 && I1) {
       if(!i && I0->getOpcode() > I1->getOpcode()) {
         Left.push_back(I1);
         Right.push_back(I0);
       } else if (i && I0->getOpcode() > I1->getOpcode() && Right[i-1] != I1) {
         // Try not to destroy a broad cast for no apparent benefit.
         Left.push_back(I1);
         Right.push_back(I0);
       } else if (i && I0->getOpcode() == I1->getOpcode() && Right[i-1] ==  I0) {
         // Try preserve broadcasts.
         Left.push_back(I1);
         Right.push_back(I0);
       } else if (i && I0->getOpcode() == I1->getOpcode() && Left[i-1] == I1) {
         // Try preserve broadcasts.
         Left.push_back(I1);
         Right.push_back(I0);
       } else {
         Left.push_back(I0);
         Right.push_back(I1);
       }
       continue;
    }
    // One opcode, put the instruction on the right.
    if (I0) {
      Left.push_back(V1);
      Right.push_back(I0);
      continue;
    }
    Left.push_back(V0);
    Right.push_back(V1);
  }


  bool LeftBroadcast  = true;//isSplat(Left);
  bool RightBroadcast = true;//isSplat(Right);
  for (unsigned i = 1, e = Left.size(); i != e; ++i) {
	  if(Left[i] != Left[0]){
		  LeftBroadcast  = false;
		  break;
	  }
  }
  for (unsigned i = 1, e = Right.size(); i != e; ++i) {
	  if(Right[i] != Right[0]){
		  RightBroadcast  = false;
		  break;
	  }
  }
  // Don't reorder if the operands where good to begin with.
  if (!(LeftBroadcast || RightBroadcast) &&
      (AllSameOpcodeRight || AllSameOpcodeLeft)) {
    Left = OrigLeft;
    Right = OrigRight;
  }

  bool changed = false;
  for (unsigned i = 0, e = VL.size(); i != e; ++i){
  	if(Left[i] != OrigLeft[i]){
		changed = true;
		break;
	}
  }
  if(changed){
	  for (unsigned i = 0, e = VL.size(); i != e; ++i){
		  Instruction *I = cast<Instruction>(VL[i]);
		  I->setOperand(0, Left[i]);
		  I->setOperand(1, Right[i]);
	  }
  }  

  return;
}



/// \brief Find the operand of the GEP that should be checked for consecutive
/// stores. This ignores trailing indices that have no effect on the final
/// pointer.
static unsigned getGEPInductionOperand(const DataLayout *DL,
                                       const GetElementPtrInst *Gep) {
  unsigned LastOperand = Gep->getNumOperands() - 1;
  unsigned GEPAllocSize = DL->getTypeAllocSize(
      cast<PointerType>(Gep->getType()->getScalarType())->getElementType());

  // Walk backwards and try to peel off zeros.
  while (LastOperand > 1 && match(Gep->getOperand(LastOperand), m_Zero())) {
    // Find the type we're currently indexing into.
    gep_type_iterator GEPTI = gep_type_begin(Gep);
    std::advance(GEPTI, LastOperand - 1);

    // If it's a type with the same allocation size as the result of the GEP we
    // can peel off the zero index.
    if (DL->getTypeAllocSize(*GEPTI) != GEPAllocSize)
      break;
    --LastOperand;
  }

  return LastOperand;
}


static Instruction *getFirstInst(Instruction *FirstInst, Value *V,
                                 Instruction *Loc) {
  if (FirstInst)
    return FirstInst;
  if (Instruction *I = dyn_cast<Instruction>(V))
    return I->getParent() == Loc->getParent() ? I : nullptr;
  return nullptr;
}


/// This function translates the reduction kind to an LLVM binary operator.
static unsigned
getReductionBinOp(LoopVectorizationLegality::ReductionKind Kind) {
  switch (Kind) {
    case LoopVectorizationLegality::RK_IntegerAdd:
      return Instruction::Add;
    case LoopVectorizationLegality::RK_IntegerMult:
      return Instruction::Mul;
    case LoopVectorizationLegality::RK_IntegerOr:
      return Instruction::Or;
    case LoopVectorizationLegality::RK_IntegerAnd:
      return Instruction::And;
    case LoopVectorizationLegality::RK_IntegerXor:
      return Instruction::Xor;
    case LoopVectorizationLegality::RK_FloatMult:
      return Instruction::FMul;
    case LoopVectorizationLegality::RK_FloatAdd:
      return Instruction::FAdd;
    case LoopVectorizationLegality::RK_IntegerMinMax:
      return Instruction::ICmp;
    case LoopVectorizationLegality::RK_FloatMinMax:
      return Instruction::FCmp;
    default:
      llvm_unreachable("Unknown reduction operation");
  }
}

Value *createMinMaxOp(IRBuilder<> &Builder,
                      LoopVectorizationLegality::MinMaxReductionKind RK,
                      Value *Left,
                      Value *Right) {
  CmpInst::Predicate P = CmpInst::ICMP_NE;
  switch (RK) {
  default:
    llvm_unreachable("Unknown min/max reduction kind");
  case LoopVectorizationLegality::MRK_UIntMin:
    P = CmpInst::ICMP_ULT;
    break;
  case LoopVectorizationLegality::MRK_UIntMax:
    P = CmpInst::ICMP_UGT;
    break;
  case LoopVectorizationLegality::MRK_SIntMin:
    P = CmpInst::ICMP_SLT;
    break;
  case LoopVectorizationLegality::MRK_SIntMax:
    P = CmpInst::ICMP_SGT;
    break;
  case LoopVectorizationLegality::MRK_FloatMin:
    P = CmpInst::FCMP_OLT;
    break;
  case LoopVectorizationLegality::MRK_FloatMax:
    P = CmpInst::FCMP_OGT;
    break;
  }

  Value *Cmp;
  if (RK == LoopVectorizationLegality::MRK_FloatMin ||
      RK == LoopVectorizationLegality::MRK_FloatMax)
    Cmp = Builder.CreateFCmp(P, Left, Right, "rdx.minmax.cmp");
  else
    Cmp = Builder.CreateICmp(P, Left, Right, "rdx.minmax.cmp");

  Value *Select = Builder.CreateSelect(Cmp, Left, Right, "rdx.minmax.select");
  return Select;
}


/// \brief Check whether this block is a predicated block.
/// Due to if predication of stores we might create a sequence of "if(pred) a[i]
/// = ...;  " blocks. We start with one vectorized basic block. For every
/// conditional block we split this vectorized block. Therefore, every second
/// block will be a predicated one.
static bool isPredicatedBlock(unsigned BlockNum) {
  return BlockNum % 2;
}

///\brief Perform cse of induction variable instructions.
static void cse(SmallVector<BasicBlock *, 4> &BBs) {
  // Perform simple cse.
  SmallDenseMap<Instruction *, Instruction *, 4, CSEDenseMapInfo> CSEMap;
  for (unsigned i = 0, e = BBs.size(); i != e; ++i) {
    BasicBlock *BB = BBs[i];
    for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E;) {
      Instruction *In = I++;

      if (!CSEDenseMapInfo::canHandle(In))
        continue;

      // Check if we can replace this instruction with any of the
      // visited instructions.
      if (Instruction *V = CSEMap.lookup(In)) {
        In->replaceAllUsesWith(V);
        In->eraseFromParent();
        continue;
      }
      // Ignore instructions in conditional blocks. We create "if (pred) a[i] =
      // ...;" blocks for predicated stores. Every second block is a predicated
      // block.
      if (isPredicatedBlock(i))
        continue;

      CSEMap[In] = In;
    }
  }
}

/// \brief Adds a 'fast' flag to floating point operations.
static Value *addFastMathFlag(Value *V) {
  if (isa<FPMathOperator>(V)){
    FastMathFlags Flags;
    Flags.setUnsafeAlgebra();
    cast<Instruction>(V)->setFastMathFlags(Flags);
  }
  return V;
}

/// \brief Check whether it is safe to if-convert this phi node.
///
/// Phi nodes with constant expressions that can trap are not safe to if
/// convert.
static bool canIfConvertPHINodes(BasicBlock *BB) {
  for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I) {
    PHINode *Phi = dyn_cast<PHINode>(I);
    if (!Phi)
      return true;
    for (unsigned p = 0, e = Phi->getNumIncomingValues(); p != e; ++p)
      if (Constant *C = dyn_cast<Constant>(Phi->getIncomingValue(p)))
        if (C->canTrap())
          return false;
  }
  return true;
}

static Type *convertPointerToIntegerType(const DataLayout &DL, Type *Ty) {
  if (Ty->isPointerTy())
    return DL.getIntPtrType(Ty);

  // It is possible that char's or short's overflow when we ask for the loop's
  // trip count, work around this by changing the type size.
  if (Ty->getScalarSizeInBits() < 32)
    return Type::getInt32Ty(Ty->getContext());

  return Ty;
}

static Type* getWiderType(const DataLayout &DL, Type *Ty0, Type *Ty1) {
  Ty0 = convertPointerToIntegerType(DL, Ty0);
  Ty1 = convertPointerToIntegerType(DL, Ty1);
  if (Ty0->getScalarSizeInBits() > Ty1->getScalarSizeInBits())
    return Ty0;
  return Ty1;
}

/// \brief Check that the instruction has outside loop users and is not an
/// identified reduction variable.
static bool hasOutsideLoopUser(const Loop *TheLoop, Instruction *Inst,
                               SmallPtrSet<Value *, 4> &Reductions) {
  // Reduction instructions are allowed to have exit users. All other
  // instructions must not have external users.
  if (!Reductions.count(Inst))
    //Check that all of the users of the loop are inside the BB.
    for (User *U : Inst->users()) {
      Instruction *UI = cast<Instruction>(U);
      // This user may be a reduction exit value.
      if (!TheLoop->contains(UI)) {
        DEBUG(dbgs() << "LV: Found an outside user for : " << *UI << '\n');
        return true;
      }
    }
  return false;
}

///\brief Remove GEPs whose indices but the last one are loop invariant and
/// return the induction operand of the gep pointer.
static Value *stripGetElementPtr(Value *Ptr, ScalarEvolution *SE,
                                 const DataLayout *DL, Loop *Lp) {
  GetElementPtrInst *GEP = dyn_cast<GetElementPtrInst>(Ptr);
  if (!GEP)
    return Ptr;

  unsigned InductionOperand = getGEPInductionOperand(DL, GEP);

  // Check that all of the gep indices are uniform except for our induction
  // operand.
  for (unsigned i = 0, e = GEP->getNumOperands(); i != e; ++i)
    if (i != InductionOperand &&
        !SE->isLoopInvariant(SE->getSCEV(GEP->getOperand(i)), Lp))
      return Ptr;
  return GEP->getOperand(InductionOperand);
}

///\brief Look for a cast use of the passed value.
static Value *getUniqueCastUse(Value *Ptr, Loop *Lp, Type *Ty) {
  Value *UniqueCast = nullptr;
  for (User *U : Ptr->users()) {
    CastInst *CI = dyn_cast<CastInst>(U);
    if (CI && CI->getType() == Ty) {
      if (!UniqueCast)
        UniqueCast = CI;
      else
        return nullptr;
    }
  }
  return UniqueCast;
}

///\brief Get the stride of a pointer access in a loop.
/// Looks for symbolic strides "a[i*stride]". Returns the symbolic stride as a
/// pointer to the Value, or null otherwise.
static Value *getStrideFromPointer(Value *Ptr, ScalarEvolution *SE,
                                   const DataLayout *DL, Loop *Lp) {
  const PointerType *PtrTy = dyn_cast<PointerType>(Ptr->getType());
  if (!PtrTy || PtrTy->isAggregateType()){
//  	DEBUG_WITH_TYPE("LV_LEGAL", llvm::dbgs() << "unsupported ptr type of strided access: " << *Ptr << " with type: " << Ptr->getType() << "\n"); 
    return nullptr;
  }
  // Try to remove a gep instruction to make the pointer (actually index at this
  // point) easier analyzable. If OrigPtr is equal to Ptr we are analzying the
  // pointer, otherwise, we are analyzing the index.
  Value *OrigPtr = Ptr;

  // The size of the pointer access.
  int64_t PtrAccessSize = 1;

  Ptr = stripGetElementPtr(Ptr, SE, DL, Lp);
  const SCEV *V = SE->getSCEV(Ptr);

  if (Ptr != OrigPtr)
    // Strip off casts.
    while (const SCEVCastExpr *C = dyn_cast<SCEVCastExpr>(V))
      V = C->getOperand();

  const SCEVAddRecExpr *S = dyn_cast<SCEVAddRecExpr>(V);
  if (!S){ 	
    return nullptr;
  }
  
  V = S->getStepRecurrence(*SE);
  if (!V){  	
    return nullptr;
  }
  
  // Strip off the size of access multiplication if we are still analyzing the
  // pointer.
  if (OrigPtr == Ptr) {
    DL->getTypeAllocSize(PtrTy->getElementType());
    if (const SCEVMulExpr *M = dyn_cast<SCEVMulExpr>(V)) {
      if (M->getOperand(0)->getSCEVType() != scConstant){	 	
        return nullptr;
	  }

      const APInt &APStepVal =
          cast<SCEVConstant>(M->getOperand(0))->getValue()->getValue();

      // Huge step value - give up.
      if (APStepVal.getBitWidth() > 64){	  
        return nullptr;
	  }

      int64_t StepVal = APStepVal.getSExtValue();
      if (PtrAccessSize != StepVal){	  
        return nullptr;  ///for an SCEVMulExpr, the factor to scale the step should be 1
      }
      V = M->getOperand(1);
    }
  }

  // Strip off casts.
  Type *StripedOffRecurrenceCast = nullptr;
  if (const SCEVCastExpr *C = dyn_cast<SCEVCastExpr>(V)) {
    StripedOffRecurrenceCast = C->getType();
    V = C->getOperand();
  }

  // Look for the loop invariant symbolic value.
  const SCEVUnknown *U = dyn_cast<SCEVUnknown>(V);
  if (!U){	          
    return nullptr;
  }

  Value *Stride = U->getValue();
  if (!Lp->isLoopInvariant(Stride)){          	
    return nullptr;
  }

  // If we have stripped off the recurrence cast we have to make sure that we
  // return the value that is used in this loop so that we can replace it later.
  if (StripedOffRecurrenceCast)
    Stride = getUniqueCastUse(Stride, Lp, StripedOffRecurrenceCast);

  return Stride;
}


/// \brief Check whether a pointer can participate in a runtime bounds check.
static bool hasComputableBounds(ScalarEvolution *SE, ValueToValueMap &Strides,
                                Value *Ptr) {
  const SCEV *PtrScev = replaceSymbolicStrideSCEV(SE, Strides, Ptr);  
  const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(PtrScev);
  if (!AR){
    return false;
  }

  if(AR->isAffine()){
	return true;
  }else{
	return false;
  }  
}


static bool isInBoundsGep(Value *Ptr) {
  if (GetElementPtrInst *GEP = dyn_cast<GetElementPtrInst>(Ptr))
    return GEP->isInBounds();
  return false;
}


/// \brief Check whether the access through \p Ptr has a constant stride.
/// \brief Check the stride of the pointer and ensure that it does not wrap in
/// the address space.
//static int isStridedPtr(ScalarEvolution *SE, const DataLayout *DL, Value *Ptr,
//                        const Loop *Lp, ValueToValueMap &StridesMap);
static int isStridedPtr(ScalarEvolution *SE, const DataLayout *DL, Value *Ptr,
                        const Loop *Lp, ValueToValueMap &StridesMap) {
  const Type *Ty = Ptr->getType();
  assert(Ty->isPointerTy() && "Unexpected non-ptr");

  // Make sure that the pointer does not point to aggregate types.
  const PointerType *PtrTy = cast<PointerType>(Ty);

  if (PtrTy->getElementType()->isAggregateType()) {
    return 0;
  }

  const SCEV *PtrScev = replaceSymbolicStrideSCEV(SE, StridesMap, Ptr);

  const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(PtrScev);
  if (!AR) {
    return 0;
  }

  // The accesss function must stride over the innermost loop.
  if (Lp != AR->getLoop()) {
	return 0;
  }

  // The address calculation must not wrap. Otherwise, a dependence could be
  // inverted.
  // An inbounds getelementptr that is a AddRec with a unit stride
  // cannot wrap per definition. The unit stride requirement is checked later.
  // An getelementptr without an inbounds attribute and unit stride would have
  // to access the pointer value "0" which is undefined behavior in address
  // space 0, therefore we can also vectorize this case.
  bool IsInBoundsGEP = isInBoundsGep(Ptr);
  bool IsNoWrapAddRec = AR->getNoWrapFlags(SCEV::NoWrapMask);
  bool IsInAddressSpaceZero = PtrTy->getAddressSpace() == 0;
  if (!IsInBoundsGEP && !IsInAddressSpaceZero && !IsNoWrapAddRec) {
    return 0;
  }

  // Check the step is constant.
  const SCEV *Step = AR->getStepRecurrence(*SE);

  // Calculate the pointer stride and check if it is consecutive.
  const SCEVConstant *C = dyn_cast<SCEVConstant>(Step);
  if (!C) {
    return 0;
  }

  int64_t Size = DL->getTypeAllocSize(PtrTy->getElementType());
  const APInt &APStepVal = C->getValue()->getValue();

  // Huge step value - give up.
  if (APStepVal.getBitWidth() > 64){
    return 0;
  }

  int64_t StepVal = APStepVal.getSExtValue();

  // Strided access.
  int64_t Stride = StepVal / Size;
  int64_t Rem = StepVal % Size;
  if (Rem){
    return 0;
  }
  
  return Stride;
}

const SCEV * getStride(ScalarEvolution *SE, const SCEV *PtrScev, const Loop *Lp) {

  const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(PtrScev);
  if (!AR) {
    return PtrScev;
  }

  // The accesss function must stride over the innermost loop.
  if (Lp != AR->getLoop()) {
	return PtrScev;
  }

  return AR->getStepRecurrence(*SE);
}

static bool hasMultipleUsesOf(Instruction *I,
                              SmallPtrSet<Instruction *, 8> &Insts) {
  unsigned NumUses = 0;
  for(User::op_iterator Use = I->op_begin(), E = I->op_end(); Use != E; ++Use) {
    if (Insts.count(dyn_cast<Instruction>(*Use)))
      ++NumUses;
    if (NumUses > 1)
      return true;
  }

  return false;
}

static bool areAllUsesIn(Instruction *I, SmallPtrSet<Instruction *, 8> &Set) {
  for(User::op_iterator Use = I->op_begin(), E = I->op_end(); Use != E; ++Use)
    if (!Set.count(dyn_cast<Instruction>(*Use)))
      return false;
  return true;
}


/// \brief Check whether the address computation for a non-consecutive memory
/// access looks like an unlikely candidate for being merged into the indexing
/// mode.
///
/// We look for a GEP which has one index that is an induction variable and all
/// other indices are loop invariant. If the stride of this access is also
/// within a small bound we decide that this address computation can likely be
/// merged into the addressing mode.
/// In all other cases, we identify the address computation as complex.
static bool isLikelyComplexAddressComputation(Value *Ptr,
                                              LoopVectorizationLegality *Legal,
                                              ScalarEvolution *SE,
                                              const Loop *TheLoop) {
  GetElementPtrInst *Gep = dyn_cast<GetElementPtrInst>(Ptr);
  if (!Gep)
    return true;

  // We are looking for a gep with all loop invariant indices except for one
  // which should be an induction variable.
  unsigned NumOperands = Gep->getNumOperands();
  for (unsigned i = 1; i < NumOperands; ++i) {
    Value *Opd = Gep->getOperand(i);
    if (!SE->isLoopInvariant(SE->getSCEV(Opd), TheLoop) &&
        !Legal->isInductionVariable(Opd))
      return true;
  }

  // Now we know we have a GEP ptr, %inv, %ind, %inv. Make sure that the step
  // can likely be merged into the address computation.
  unsigned MaxMergeDistance = 64;

  const SCEVAddRecExpr *AddRec = dyn_cast<SCEVAddRecExpr>(SE->getSCEV(Ptr));
  if (!AddRec)
    return true;

  // Check the step is constant.
  const SCEV *Step = AddRec->getStepRecurrence(*SE);
  // Calculate the pointer stride and check if it is consecutive.
  const SCEVConstant *C = dyn_cast<SCEVConstant>(Step);
  if (!C)
    return true;

  const APInt &APStepVal = C->getValue()->getValue();

  // Huge step value - give up.
  if (APStepVal.getBitWidth() > 64)
    return true;

  int64_t StepVal = APStepVal.getSExtValue();

  return StepVal > MaxMergeDistance;
}

static bool isStrideMul(Instruction *I, LoopVectorizationLegality *Legal) {
  if (Legal->hasStride(I->getOperand(0)) || Legal->hasStride(I->getOperand(1)))
    return true;
  return false;
}


//===----------------------------------------------------------------------===//
// Implementation of LoopVectorizationLegality, InnerLoopVectorizer and
// LoopVectorizationCostModel.
//===----------------------------------------------------------------------===//


Value *InnerLoopVectorizer::getBroadcastInstrs(Value *V) {
  // We need to place the broadcast of invariant variables outside the loop.
  Instruction *Instr = dyn_cast<Instruction>(V);
  bool NewInstr =
      (Instr && std::find(LoopVectorBody.begin(), LoopVectorBody.end(),
                          Instr->getParent()) != LoopVectorBody.end());
  bool Invariant = OrigLoop->isLoopInvariant(V) && !NewInstr;

  // Place the code for broadcasting invariant variables in the new preheader.
  IRBuilder<>::InsertPointGuard Guard(Builder);
 
  if (Invariant)
    Builder.SetInsertPoint(LoopVectorPreHeader->getTerminator());

  // Broadcast the scalar into all locations in the vector.
  Value *Shuf = Builder.CreateVectorSplat(VF, V, "broadcast");

  return Shuf;
}


Value *InnerLoopVectorizer::getConsecutiveVector(Value* Val, int StartIdx,
                                                 bool Negate, int Stride) {
  assert(Val->getType()->isVectorTy() && "Must be a vector");
  assert(Val->getType()->getScalarType()->isIntegerTy() &&
         "Elem must be an integer");
  // Create the types.
  Type *ITy = Val->getType()->getScalarType();
  VectorType *Ty = cast<VectorType>(Val->getType());
  int VLen = Ty->getNumElements();
  SmallVector<Constant*, 8> Indices;

  // Create a vector of consecutive numbers from zero to VF.
  for (int i = 0; i < VLen; ++i) {
    int64_t Idx = Negate ? (-i)*Stride : i*Stride;
    Indices.push_back(ConstantInt::get(ITy, StartIdx + Idx, Negate));
  }

  // Add the consecutive indices to the vector value.
  Constant *Cv = ConstantVector::get(Indices);
  assert(Cv->getType() == Val->getType() && "Invalid consecutive vec");
  return Builder.CreateAdd(Val, Cv, "induction");
}


InnerLoopVectorizer::VectorParts&
InnerLoopVectorizer::getVectorValue(Value *V) {
  assert(V != Induction && "The new induction variable should not be used.");
  assert(!V->getType()->isVectorTy() && "Can't widen a vector");

  // If we have this scalar in the map, return it.
  if (WidenMap.has(V))
    return WidenMap.get(V);

  //////solve slp-->loop conflicts here
  if(Legal->isSLPVar(V)){
    if(!SLPScaMap.has(V)){
       errs() << *V << "\n";
	}
  	assert(SLPScaMap.has(V) && "should have info for this value in slp bundle!");
	switchSLPToLoopLevel(V); 
  	return WidenMap.get(V);
  }
  
  // If this scalar is unknown, assume that it is a constant or that it is
  // loop invariant. Broadcast V and save the value for future uses.
  Value *B = getBroadcastInstrs(V);
  return WidenMap.splat(V, B);
}

Value *InnerLoopVectorizer::reverseVector(Value *Vec) {
  assert(Vec->getType()->isVectorTy() && "Invalid type");
  SmallVector<Constant*, 8> ShuffleMask;
  for (unsigned i = 0; i < VF; ++i)
    ShuffleMask.push_back(Builder.getInt32(VF - i - 1));

  return Builder.CreateShuffleVector(Vec, UndefValue::get(Vec->getType()),
                                     ConstantVector::get(ShuffleMask),
                                     "reverse");
}

void InnerLoopVectorizer::vectorizeMemoryInstruction(Instruction *Instr) {
  // Attempt to issue a wide load.
  LoadInst *LI = dyn_cast<LoadInst>(Instr);
  StoreInst *SI = dyn_cast<StoreInst>(Instr);

  assert((LI || SI) && "Invalid Load/Store instruction");

  Type *ScalarDataTy = LI ? LI->getType() : SI->getValueOperand()->getType();
  Type *DataTy = VectorType::get(ScalarDataTy, VF);
  Value *Ptr = LI ? LI->getPointerOperand() : SI->getPointerOperand();
  unsigned Alignment = LI ? LI->getAlignment() : SI->getAlignment();
  // An alignment of 0 means target abi alignment. We need to use the scalar's
  // target abi alignment in such a case.
  if (!Alignment)
    Alignment = DL->getABITypeAlignment(ScalarDataTy);
  unsigned AddressSpace = Ptr->getType()->getPointerAddressSpace();
  unsigned ScalarAllocatedSize = DL->getTypeAllocSize(ScalarDataTy);
  unsigned VectorElementSize = DL->getTypeStoreSize(DataTy)/VF;

  if (SI && Legal->blockNeedsPredication(SI->getParent()))
    return scalarizeInstruction(Instr, true); /////here could use masked vector store operation

  if (ScalarAllocatedSize != VectorElementSize)
    return scalarizeInstruction(Instr);

  // If the pointer is loop invariant or if it is non-consecutive,
  // scalarize the load.
  int ConsecutiveStride = Legal->isConsecutivePtr(Ptr);
  bool Reverse = ConsecutiveStride < 0;
  bool UniformLoad = LI && Legal->isUniform(Ptr);
  if (!ConsecutiveStride || UniformLoad)
    return scalarizeInstruction(Instr);

  Constant *Zero = Builder.getInt32(0);
  VectorParts &Entry = WidenMap.get(Instr);

  // Handle consecutive loads/stores.
  GetElementPtrInst *Gep = dyn_cast<GetElementPtrInst>(Ptr);
  if (Gep && Legal->isInductionVariable(Gep->getPointerOperand())) {
    setDebugLocFromInst(Builder, Gep);
    Value *PtrOperand = Gep->getPointerOperand();
    Value *FirstBasePtr = getVectorValue(PtrOperand)[0];
    FirstBasePtr = Builder.CreateExtractElement(FirstBasePtr, Zero);

    // Create the new GEP with the new induction variable.
    GetElementPtrInst *Gep2 = cast<GetElementPtrInst>(Gep->clone());
    Gep2->setOperand(0, FirstBasePtr);
    Gep2->setName("gep.indvar.base");
    Ptr = Builder.Insert(Gep2);
  } 
  else if (Gep) {
    setDebugLocFromInst(Builder, Gep);
    assert(SE->isLoopInvariant(SE->getSCEV(Gep->getPointerOperand()),
                               OrigLoop) && "Base ptr must be invariant");

    // The last index does not have to be the induction. It can be
    // consecutive and be a function of the index. For example A[I+1];
    unsigned NumOperands = Gep->getNumOperands();
    unsigned InductionOperand = getGEPInductionOperand(DL, Gep);
    // Create the new GEP with the new induction variable.
    GetElementPtrInst *Gep2 = cast<GetElementPtrInst>(Gep->clone());

    for (unsigned i = 0; i < NumOperands; ++i) {
      Value *GepOperand = Gep->getOperand(i);
      Instruction *GepOperandInst = dyn_cast<Instruction>(GepOperand);

      // Update last index or loop invariant instruction anchored in loop.
      if (i == InductionOperand ||
          (GepOperandInst && OrigLoop->contains(GepOperandInst))) {
        assert((i == InductionOperand ||
               SE->isLoopInvariant(SE->getSCEV(GepOperandInst), OrigLoop)) &&
               "Must be last index or loop invariant");

        VectorParts &GEPParts = getVectorValue(GepOperand);
        Value *Index = GEPParts[0];
        Index = Builder.CreateExtractElement(Index, Zero);
        Gep2->setOperand(i, Index);
        Gep2->setName("gep.indvar.idx");
      }
    }
    Ptr = Builder.Insert(Gep2);
  } 
  else {
    // Use the induction element ptr.
    assert(isa<PHINode>(Ptr) && "Invalid induction ptr");
    setDebugLocFromInst(Builder, Ptr);
    VectorParts &PtrVal = getVectorValue(Ptr);
    Ptr = Builder.CreateExtractElement(PtrVal[0], Zero);
  }

  // Handle Stores:
  if (SI) {
    assert(!Legal->isUniform(SI->getPointerOperand()) &&
           "We do not allow storing to uniform addresses");
    setDebugLocFromInst(Builder, SI);
    // We don't want to update the value in the map as it might be used in
    // another expression. So don't use a reference type for "StoredVal".
    VectorParts StoredVal = getVectorValue(SI->getValueOperand());

    for (unsigned Part = 0; Part < UF; ++Part) {
      // Calculate the pointer for the specific unroll-part.
      Value *PartPtr = Builder.CreateGEP(Ptr, Builder.getInt32(Part * VF));

      if (Reverse) {
        // If we store to reverse consecutive memory locations then we need
        // to reverse the order of elements in the stored value.
        StoredVal[Part] = reverseVector(StoredVal[Part]);
        // If the address is consecutive but reversed, then the
        // wide store needs to start at the last vector element.
        PartPtr = Builder.CreateGEP(Ptr, Builder.getInt32(-Part * VF));
        PartPtr = Builder.CreateGEP(PartPtr, Builder.getInt32(1 - VF));
      }

      Value *VecPtr = Builder.CreateBitCast(PartPtr,
                                            DataTy->getPointerTo(AddressSpace));
      StoreInst *NewSI =
        Builder.CreateAlignedStore(StoredVal[Part], VecPtr, Alignment);
      propagateMetadata(NewSI, SI);
    }
    return;
  }

  // Handle loads.
  assert(LI && "Must have a load instruction");
  setDebugLocFromInst(Builder, LI);
  for (unsigned Part = 0; Part < UF; ++Part) {
    // Calculate the pointer for the specific unroll-part.
    Value *PartPtr = Builder.CreateGEP(Ptr, Builder.getInt32(Part * VF));

    if (Reverse) {
      // If the address is consecutive but reversed, then the
      // wide store needs to start at the last vector element.
      PartPtr = Builder.CreateGEP(Ptr, Builder.getInt32(-Part * VF));
      PartPtr = Builder.CreateGEP(PartPtr, Builder.getInt32(1 - VF));
    }

    Value *VecPtr = Builder.CreateBitCast(PartPtr,
                                          DataTy->getPointerTo(AddressSpace));
    LoadInst *NewLI = Builder.CreateAlignedLoad(VecPtr, Alignment, "wide.load");
    propagateMetadata(NewLI, LI);
    Entry[Part] = Reverse ? reverseVector(NewLI) :  NewLI;
  }
}

void InnerLoopVectorizer::scalarizeInstruction(Instruction *Instr, bool IfPredicateStore) {
  assert(!Instr->getType()->isAggregateType() && "Can't handle vectors");
  // Holds vector parameters or scalars, in case of uniform vals.
  SmallVector<VectorParts, 4> Params;

  setDebugLocFromInst(Builder, Instr);

  // Find all of the vectorized parameters.
  for (unsigned op = 0, e = Instr->getNumOperands(); op != e; ++op) {
    Value *SrcOp = Instr->getOperand(op);

    // If we are accessing the old induction variable, use the new one.
    if (SrcOp == OldInduction) {
      Params.push_back(getVectorValue(SrcOp));
      continue;
    }

    // Try using previously calculated values.
    Instruction *SrcInst = dyn_cast<Instruction>(SrcOp);

    // If the src is an instruction that appeared earlier in the basic block
    // then it should already be vectorized.
    if (SrcInst && OrigLoop->contains(SrcInst)) {

	  if(Legal->isSLPVar(SrcInst)){
			assert(SLPScaMap.has(SrcInst) && "should have info for this value in slp bundle!");
			switchSLPToLoopLevel(SrcInst); 
	  }

	  assert(WidenMap.has(SrcInst) && "Source operand is unavailable");
	  // The parameter is a vector value from earlier.
	  Params.push_back(WidenMap.get(SrcInst));
	 
    } else {
      // The parameter is a scalar from outside the loop. Maybe even a constant.
      VectorParts Scalars;
      Scalars.append(UF, SrcOp);
      Params.push_back(Scalars);
    }
  }

  assert(Params.size() == Instr->getNumOperands() &&
         "Invalid number of operands");

  // Does this instruction return a value ?
  bool IsVoidRetTy = Instr->getType()->isVoidTy();

  Value *UndefVec = IsVoidRetTy ? nullptr :
    UndefValue::get(VectorType::get(Instr->getType(), VF));
  // Create a new entry in the WidenMap and initialize it to Undef or Null.
  VectorParts &VecResults = WidenMap.splat(Instr, UndefVec);

  Instruction *InsertPt = Builder.GetInsertPoint();
  BasicBlock *IfBlock = Builder.GetInsertBlock();
  BasicBlock *CondBlock = nullptr;

  VectorParts Cond;
  Loop *VectorLp = nullptr;
  if (IfPredicateStore) {
    assert(Instr->getParent()->getSinglePredecessor() &&
           "Only support single predecessor blocks");
    Cond = createEdgeMask(Instr->getParent()->getSinglePredecessor(),
                          Instr->getParent());
    VectorLp = LI->getLoopFor(IfBlock);
    assert(VectorLp && "Must have a loop for this block");
  }

  // For each vector unroll 'part':
  for (unsigned Part = 0; Part < UF; ++Part) {
    // For each scalar that we create:
    for (unsigned Width = 0; Width < VF; ++Width) {

      // Start if-block.
      Value *Cmp = nullptr;
      if (IfPredicateStore) {
        Cmp = Builder.CreateExtractElement(Cond[Part], Builder.getInt32(Width));
        Cmp = Builder.CreateICmp(ICmpInst::ICMP_EQ, Cmp, ConstantInt::get(Cmp->getType(), 1));
        CondBlock = IfBlock->splitBasicBlock(InsertPt, "cond.store");
        LoopVectorBody.push_back(CondBlock);
        VectorLp->addBasicBlockToLoop(CondBlock, LI->getBase());
        // Update Builder with newly created basic block.
        Builder.SetInsertPoint(InsertPt);
      }

      Instruction *Cloned = Instr->clone();
      if (!IsVoidRetTy)
        Cloned->setName(Instr->getName() + ".cloned");
      // Replace the operands of the cloned instructions with extracted scalars.
      for (unsigned op = 0, e = Instr->getNumOperands(); op != e; ++op) {
        Value *Op = Params[op][Part];
        // Param is a vector. Need to extract the right lane.
        if (Op->getType()->isVectorTy())
          Op = Builder.CreateExtractElement(Op, Builder.getInt32(Width));
        Cloned->setOperand(op, Op);
      }

      // Place the cloned scalar in the new loop.
      Builder.Insert(Cloned);

      // If the original scalar returns a value we need to place it in a vector
      // so that future users will be able to use it.
      if (!IsVoidRetTy)
        VecResults[Part] = Builder.CreateInsertElement(VecResults[Part], Cloned,
                                                       Builder.getInt32(Width));
      // End if-block.
      if (IfPredicateStore) {
         BasicBlock *NewIfBlock = CondBlock->splitBasicBlock(InsertPt, "else");
         LoopVectorBody.push_back(NewIfBlock);
         VectorLp->addBasicBlockToLoop(NewIfBlock, LI->getBase());
         Builder.SetInsertPoint(InsertPt);
         Instruction *OldBr = IfBlock->getTerminator();
         BranchInst::Create(CondBlock, NewIfBlock, Cmp, OldBr);
         OldBr->eraseFromParent();
         IfBlock = NewIfBlock;
      }
    }
  }
}



std::pair<Instruction *, Instruction *>
InnerLoopVectorizer::addOffsetRemStrideCheck(Instruction *Loc) {
  LoopVectorizationLegality::RuntimePointerCheck *PtrRtCheck = Legal->getRuntimePointerCheck();
  Instruction *tnullptr = nullptr;
  unsigned check_num = PtrRtCheck->RemOffset.size();
  assert(check_num == PtrRtCheck->RemStride.size() && "different num of offset and stride !!!");
  if (PtrRtCheck->RemOffset.empty())
    return std::pair<Instruction *, Instruction *>(tnullptr, tnullptr);

  IRBuilder<> ChkBuilder(Loc);
  LLVMContext &Ctx = Loc->getContext();
  SCEVExpander Exp(*SE, "induction");

  // Emit checks.
  Value *Check = nullptr;
  Instruction *FirstInst = nullptr;
  for (unsigned ii = 0; ii < check_num; ii++) {	
	
	unsigned ty_a_width = SE->getTypeSizeInBits(PtrRtCheck->RemOffset[ii]->getType());
    Type *ArithTyOffset = Type::getIntNTy(Ctx, ty_a_width);	
	Value * offset_v = Exp.expandCodeFor(PtrRtCheck->RemOffset[ii], ArithTyOffset, Loc);
	
	unsigned ty_b_width = SE->getTypeSizeInBits(PtrRtCheck->RemStride[ii]->getType());
    Type *ArithTyStride = Type::getIntNTy(Ctx, ty_b_width);			
	Value * stride_v = Exp.expandCodeFor(PtrRtCheck->RemStride[ii], ArithTyStride, Loc);
	if(ty_a_width > ty_b_width)
		stride_v = ChkBuilder.CreateBitCast(stride_v, ArithTyOffset);
	if(ty_b_width > ty_a_width)
		offset_v = ChkBuilder.CreateBitCast(offset_v, ArithTyStride);

	offset_v = ChkBuilder.CreateMul(offset_v, offset_v);
	stride_v = ChkBuilder.CreateMul(stride_v, stride_v);	
	Value *C = ChkBuilder.CreateICmpUGE(offset_v, stride_v, "remstride.chk");
	
    // Store the first instruction we create.
    FirstInst = getFirstInst(FirstInst, C, Loc);
    if (Check)
      Check = ChkBuilder.CreateOr(Check, C);
    else
      Check = C;
  }

  // We have to do this trickery because the IRBuilder might fold the check to a
  // constant expression in which case there is no Instruction anchored in a
  // the block.
  Instruction *TheCheck =
      BinaryOperator::CreateAnd(Check, ConstantInt::getTrue(Ctx));
  ChkBuilder.Insert(TheCheck, "stride.not.one");
  FirstInst = getFirstInst(FirstInst, TheCheck, Loc);

  return std::make_pair(FirstInst, TheCheck);
}


std::pair<Instruction *, Instruction *>
InnerLoopVectorizer::addArrayRangeCheck(Instruction *Loc) {
  LoopVectorizationLegality::RuntimePointerCheck *PtrRtCheck = Legal->getRuntimePointerCheck();

  Instruction *tnullptr = nullptr;
  unsigned NumPointers = PtrRtCheck->PointerAs.size();  
  if (!NumPointers)
    return std::pair<Instruction *, Instruction *>(tnullptr, tnullptr);

  SmallVector<TrackingVH<Value> , 2> StartAs;
  SmallVector<TrackingVH<Value> , 2> EndAs;
  SmallVector<TrackingVH<Value> , 2> StartBs;
  SmallVector<TrackingVH<Value> , 2> EndBs;  
  LLVMContext &Ctx = Loc->getContext();
  SCEVExpander Exp(*SE, "induction");
  Instruction *FirstInst = nullptr;

  for (unsigned i = 0; i < NumPointers; ++i) {
    Value *Ptr = PtrRtCheck->PointerAs[i];
    const SCEV *Sc = SE->getSCEV(Ptr);
    if (SE->isLoopInvariant(Sc, OrigLoop)) {
      DEBUG(dbgs() << "LV: Adding array range check for a loop invariant ptr:" << *Ptr <<"\n");
      StartAs.push_back(Ptr);
      EndAs.push_back(Ptr);
    } else {
      DEBUG(dbgs() << "LV: Adding array range check for range:" << *Ptr << '\n');
      unsigned AS = Ptr->getType()->getPointerAddressSpace();
      // Use this type for pointer arithmetic.
      Type *PtrArithTy = Type::getInt8PtrTy(Ctx, AS);
      Value *Start = Exp.expandCodeFor(PtrRtCheck->StartAs[i], PtrArithTy, Loc);
      Value *End = Exp.expandCodeFor(PtrRtCheck->EndAs[i], PtrArithTy, Loc);
      StartAs.push_back(Start);
      EndAs.push_back(End);
    }
  }
  for (unsigned i = 0; i < NumPointers; ++i) {
    Value *Ptr = PtrRtCheck->PointerBs[i];
    const SCEV *Sc = SE->getSCEV(Ptr);

    if (SE->isLoopInvariant(Sc, OrigLoop)) {
      DEBUG(dbgs() << "LV: Adding array range check for a loop invariant ptr:" << *Ptr <<"\n");
      StartBs.push_back(Ptr);
      EndBs.push_back(Ptr);
    } else {
      DEBUG(dbgs() << "LV: Adding array range check for range:" << *Ptr << '\n');
      unsigned AS = Ptr->getType()->getPointerAddressSpace();
      // Use this type for pointer arithmetic.
      Type *PtrArithTy = Type::getInt8PtrTy(Ctx, AS);  
      Value *Start = Exp.expandCodeFor(PtrRtCheck->StartBs[i], PtrArithTy, Loc);
      Value *End = Exp.expandCodeFor(PtrRtCheck->EndBs[i], PtrArithTy, Loc);
      StartBs.push_back(Start);
      EndBs.push_back(End);
    }
  }
 
  IRBuilder<> ChkBuilder(Loc);
  Value *MemoryRuntimeCheck = nullptr;
  for (unsigned i = 0; i < NumPointers; ++i) {
      unsigned AS0 = StartAs[i]->getType()->getPointerAddressSpace();
      unsigned AS1 = StartBs[i]->getType()->getPointerAddressSpace();

      assert((AS0 == EndBs[i]->getType()->getPointerAddressSpace()) &&
             (AS1 == EndAs[i]->getType()->getPointerAddressSpace()) &&
             "Trying to bounds check pointers with different address spaces");

      Type *PtrArithTy0 = Type::getInt8PtrTy(Ctx, AS0);
      Type *PtrArithTy1 = Type::getInt8PtrTy(Ctx, AS1);

      Value *Start0 = ChkBuilder.CreateBitCast(StartAs[i], PtrArithTy0, "bc");
      Value *Start1 = ChkBuilder.CreateBitCast(StartBs[i], PtrArithTy1, "bc");
      Value *End0 =   ChkBuilder.CreateBitCast(EndAs[i],   PtrArithTy1, "bc");
      Value *End1 =   ChkBuilder.CreateBitCast(EndBs[i],   PtrArithTy0, "bc");

      Value *Cmp0 = ChkBuilder.CreateICmpULE(Start0, End1, "bound0");
      FirstInst = getFirstInst(FirstInst, Cmp0, Loc);
      Value *Cmp1 = ChkBuilder.CreateICmpULE(Start1, End0, "bound1");
      FirstInst = getFirstInst(FirstInst, Cmp1, Loc);
      Value *IsConflict = ChkBuilder.CreateAnd(Cmp0, Cmp1, "found.conflict");
      FirstInst = getFirstInst(FirstInst, IsConflict, Loc);
      if (MemoryRuntimeCheck) {
        IsConflict = ChkBuilder.CreateOr(MemoryRuntimeCheck, IsConflict,
                                         "conflict.rdx");
        FirstInst = getFirstInst(FirstInst, IsConflict, Loc);
      }
      MemoryRuntimeCheck = IsConflict;
  }

  // We have to do this trickery because the IRBuilder might fold the check to a
  // constant expression in which case there is no Instruction anchored in a
  // the block.
  Instruction *Check = BinaryOperator::CreateAnd(MemoryRuntimeCheck,
                                                 ConstantInt::getTrue(Ctx));
  ChkBuilder.Insert(Check, "memcheck.conflict");
  FirstInst = getFirstInst(FirstInst, Check, Loc);
  return std::make_pair(FirstInst, Check);
}


void InnerLoopVectorizer::createEmptyLoop() {
  /*
   In this function we generate a new loop. The new loop will contain
   the vectorized instructions while the old loop will continue to run the
   scalar remainder.

       [ ] <-- Back-edge taken count overflow check.
    /   |
   /    v
  |    [ ] <-- vector loop bypass (may consist of multiple blocks).
  |  /  |
  | /   v
  ||   [ ]     <-- vector pre header.
  ||    |
  ||    v
  ||   [  ] \
  ||   [  ]_|   <-- vector loop.
  ||    |
  | \   v
  |   >[ ]   <--- middle-block.
  |  /  |
  | /   v
  -|- >[ ]     <--- new preheader.
   |    |
   |    v
   |   [ ] \
   |   [ ]_|   <-- old scalar loop to handle remainder.
    \   |
     \  v
      >[ ]     <-- exit block.
   ...
   */

  BasicBlock *OldBasicBlock = OrigLoop->getHeader();
  BasicBlock *BypassBlock = OrigLoop->getLoopPreheader();
  BasicBlock *ExitBlock = OrigLoop->getExitBlock();
  assert(BypassBlock && "Invalid loop structure");
  assert(ExitBlock && "Must have an exit block");

  // Some loops have a single integer induction variable, while other loops
  // don't. One example is c++ iterators that often have multiple pointer
  // induction variables. In the code below we also support a case where we
  // don't have a single induction variable.
  OldInduction = Legal->getInduction();
  Type *IdxTy = Legal->getWidestInductionType();

  // Find the loop boundaries.
  const SCEV *ExitCount = SE->getBackedgeTakenCount(OrigLoop);
  assert(ExitCount != SE->getCouldNotCompute() && "Invalid loop count");

  // The exit count might have the type of i64 while the phi is i32. This can
  // happen if we have an induction variable that is sign extended before the
  // compare. The only way that we get a backedge taken count is that the
  // induction variable was signed and as such will not overflow. In such a case
  // truncation is legal.
  if (ExitCount->getType()->getPrimitiveSizeInBits() >
      IdxTy->getPrimitiveSizeInBits())
    ExitCount = SE->getTruncateOrNoop(ExitCount, IdxTy);

  const SCEV *BackedgeTakeCount = SE->getNoopOrZeroExtend(ExitCount, IdxTy);
  // Get the total trip count from the count by adding 1.
  ExitCount = SE->getAddExpr(BackedgeTakeCount,
                             SE->getConstant(BackedgeTakeCount->getType(), 1));

  // Expand the trip count and place the new instructions in the preheader.
  // Notice that the pre-header does not change, only the loop body.
  SCEVExpander Exp(*SE, "induction");

  // We need to test whether the backedge-taken count is uint##_max. Adding one
  // to it will cause overflow and an incorrect loop trip count in the vector
  // body. In case of overflow we want to directly jump to the scalar remainder
  // loop.
  Value *BackedgeCount =
      Exp.expandCodeFor(BackedgeTakeCount, BackedgeTakeCount->getType(),
                        BypassBlock->getTerminator());
  if (BackedgeCount->getType()->isPointerTy())
    BackedgeCount = CastInst::CreatePointerCast(BackedgeCount, IdxTy,
                                                "backedge.ptrcnt.to.int",
                                                BypassBlock->getTerminator());
  Instruction *CheckBCOverflow =
      CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_EQ, BackedgeCount,
                      Constant::getAllOnesValue(BackedgeCount->getType()),
                      "backedge.overflow", BypassBlock->getTerminator());

  // The loop index does not have to start at Zero. Find the original start
  // value from the induction PHI node. If we don't have an induction variable
  // then we know that it starts at zero.
  Builder.SetInsertPoint(BypassBlock->getTerminator());
  Value *StartIdx = ExtendedIdx = OldInduction ?
    Builder.CreateZExt(OldInduction->getIncomingValueForBlock(BypassBlock),
                       IdxTy):
    ConstantInt::get(IdxTy, 0);

  // We need an instruction to anchor the overflow check on. StartIdx needs to
  // be defined before the overflow check branch. Because the scalar preheader
  // is going to merge the start index and so the overflow branch block needs to
  // contain a definition of the start index.
  Instruction *OverflowCheckAnchor = BinaryOperator::CreateAdd(
      StartIdx, ConstantInt::get(IdxTy, 0), "overflow.check.anchor",
      BypassBlock->getTerminator());

  // Count holds the overall loop count (N).
  Value *Count = Exp.expandCodeFor(ExitCount, ExitCount->getType(),
                                   BypassBlock->getTerminator());

  LoopBypassBlocks.push_back(BypassBlock);

  // Split the single block loop into the two loop structure described above.
  BasicBlock *VectorPH =
  BypassBlock->splitBasicBlock(BypassBlock->getTerminator(), "vector.ph");
  BasicBlock *VecBody =
  VectorPH->splitBasicBlock(VectorPH->getTerminator(), "vector.body");
  BasicBlock *MiddleBlock =
  VecBody->splitBasicBlock(VecBody->getTerminator(), "middle.block");
  BasicBlock *ScalarPH =
  MiddleBlock->splitBasicBlock(MiddleBlock->getTerminator(), "scalar.ph");

  // Create and register the new vector loop.
  Loop* Lp = new Loop();
  Loop *ParentLoop = OrigLoop->getParentLoop();

  // Insert the new loop into the loop nest and register the new basic blocks
  // before calling any utilities such as SCEV that require valid LoopInfo.
  if (ParentLoop) {
    ParentLoop->addChildLoop(Lp);
    ParentLoop->addBasicBlockToLoop(ScalarPH, LI->getBase());
    ParentLoop->addBasicBlockToLoop(VectorPH, LI->getBase());
    ParentLoop->addBasicBlockToLoop(MiddleBlock, LI->getBase());
  } else {
    LI->addTopLevelLoop(Lp);
  }
  Lp->addBasicBlockToLoop(VecBody, LI->getBase());

  // Use this IR builder to create the loop instructions (Phi, Br, Cmp)
  // inside the loop.
  Builder.SetInsertPoint(VecBody->getFirstNonPHI());

  // Generate the induction variable.
  setDebugLocFromInst(Builder, getDebugLocFromInstOrOperands(OldInduction));
  Induction = Builder.CreatePHI(IdxTy, 2, "index");
  // The loop step is equal to the vectorization factor (num of SIMD elements)
  // times the unroll factor (num of SIMD instructions).
  Constant *Step = ConstantInt::get(IdxTy, VF * UF);

  // This is the IR builder that we use to add all of the logic for bypassing
  // the new vector loop.
  IRBuilder<> BypassBuilder(BypassBlock->getTerminator());
  setDebugLocFromInst(BypassBuilder,
                      getDebugLocFromInstOrOperands(OldInduction));

  // We may need to extend the index in case there is a type mismatch.
  // We know that the count starts at zero and does not overflow.
  if (Count->getType() != IdxTy) {
    // The exit count can be of pointer type. Convert it to the correct
    // integer type.
    if (ExitCount->getType()->isPointerTy())
      Count = BypassBuilder.CreatePointerCast(Count, IdxTy, "ptrcnt.to.int");
    else
      Count = BypassBuilder.CreateZExtOrTrunc(Count, IdxTy, "cnt.cast");
  }

  // Add the start index to the loop count to get the new end index.
  Value *IdxEnd = BypassBuilder.CreateAdd(Count, StartIdx, "end.idx");

  // Now we need to generate the expression for N - (N % VF), which is
  // the part that the vectorized body will execute.
  Value *R = BypassBuilder.CreateURem(Count, Step, "n.mod.vf");
  Value *CountRoundDown = BypassBuilder.CreateSub(Count, R, "n.vec");
  Value *IdxEndRoundDown = BypassBuilder.CreateAdd(CountRoundDown, StartIdx,
                                                     "end.idx.rnd.down");

  // Now, compare the new count to zero. If it is zero skip the vector loop and
  // jump to the scalar loop.
  Value *Cmp =
      BypassBuilder.CreateICmpEQ(IdxEndRoundDown, StartIdx, "cmp.zero");

  BasicBlock *LastBypassBlock = BypassBlock;

  // Generate code to check that the loops trip count that we computed by adding
  // one to the backedge-taken count will not overflow.
  {
    auto PastOverflowCheck =
        std::next(BasicBlock::iterator(OverflowCheckAnchor));
    BasicBlock *CheckBlock =
      LastBypassBlock->splitBasicBlock(PastOverflowCheck, "overflow.checked");
    if (ParentLoop)
      ParentLoop->addBasicBlockToLoop(CheckBlock, LI->getBase());
    LoopBypassBlocks.push_back(CheckBlock);
    Instruction *OldTerm = LastBypassBlock->getTerminator();
    BranchInst::Create(ScalarPH, CheckBlock, CheckBCOverflow, OldTerm);
    OldTerm->eraseFromParent();
    LastBypassBlock = CheckBlock;
  }

  Instruction *FirstCheckInst;
//inserting RTcheck on offset % stride!!!;
  Instruction *RemCheck;
  std::tie(FirstCheckInst, RemCheck) =
      addOffsetRemStrideCheck(LastBypassBlock->getTerminator());
  if (RemCheck) {
    // Create a new block containing the memory check.
    BasicBlock *CheckBlock =
        LastBypassBlock->splitBasicBlock(RemCheck, "vector.OffsetRemStrideCheck");
    if (ParentLoop)
      ParentLoop->addBasicBlockToLoop(CheckBlock, LI->getBase());
    LoopBypassBlocks.push_back(CheckBlock);

    // Replace the branch into the memory check block with a conditional branch
    // for the "few elements case".
    Instruction *OldTerm = LastBypassBlock->getTerminator();
    BranchInst::Create(MiddleBlock, CheckBlock, Cmp, OldTerm);
    OldTerm->eraseFromParent();

    Cmp = RemCheck;
    LastBypassBlock = CheckBlock;
  }

//inserting RTcheck on array range!!!;	
  Instruction *MemRuntimeCheck;
  std::tie(FirstCheckInst, MemRuntimeCheck) =
      addArrayRangeCheck(LastBypassBlock->getTerminator());
  if (MemRuntimeCheck) {
    // Create a new block containing the memory check.
    BasicBlock *CheckBlock =
        LastBypassBlock->splitBasicBlock(MemRuntimeCheck, "vector.ArrayRangeCheck");
    if (ParentLoop)
      ParentLoop->addBasicBlockToLoop(CheckBlock, LI->getBase());
    LoopBypassBlocks.push_back(CheckBlock);

    // Replace the branch into the memory check block with a conditional branch
    // for the "few elements case".
    Instruction *OldTerm = LastBypassBlock->getTerminator();
    BranchInst::Create(MiddleBlock, CheckBlock, Cmp, OldTerm);
    OldTerm->eraseFromParent();

    Cmp = MemRuntimeCheck;
    LastBypassBlock = CheckBlock;
  }
	
  LastBypassBlock->getTerminator()->eraseFromParent();
  BranchInst::Create(MiddleBlock, VectorPH, Cmp,
                     LastBypassBlock);

  // We are going to resume the execution of the scalar loop.
  // Go over all of the induction variables that we found and fix the
  // PHIs that are left in the scalar version of the loop.
  // The starting values of PHI nodes depend on the counter of the last
  // iteration in the vectorized loop.
  // If we come from a bypass edge then we need to start from the original
  // start value.

  // This variable saves the new starting index for the scalar loop.
  PHINode *ResumeIndex = nullptr;
  LoopVectorizationLegality::InductionList::iterator I, E;
  LoopVectorizationLegality::InductionList *List = Legal->getInductionVars();
  // Set builder to point to last bypass block.
  BypassBuilder.SetInsertPoint(LoopBypassBlocks.back()->getTerminator());
  for (I = List->begin(), E = List->end(); I != E; ++I) {
    PHINode *OrigPhi = I->first;
    LoopVectorizationLegality::InductionInfo II = I->second;

    Type *ResumeValTy = (OrigPhi == OldInduction) ? IdxTy : OrigPhi->getType();
    PHINode *ResumeVal = PHINode::Create(ResumeValTy, 2, "resume.val",
                                         MiddleBlock->getTerminator());
    // We might have extended the type of the induction variable but we need a
    // truncated version for the scalar loop.
    PHINode *TruncResumeVal = (OrigPhi == OldInduction) ?
      PHINode::Create(OrigPhi->getType(), 2, "trunc.resume.val",
                      MiddleBlock->getTerminator()) : nullptr;

    // Create phi nodes to merge from the  backedge-taken check block.
    // only backedge-taken check block branch to scalar ph
    // other runtime check branch to middle block
    PHINode *BCResumeVal = PHINode::Create(ResumeValTy, 3, "bc.resume.val",
                                           ScalarPH->getTerminator());
    BCResumeVal->addIncoming(ResumeVal, MiddleBlock);
    PHINode *BCTruncResumeVal = nullptr;
    if (OrigPhi == OldInduction) {
      BCTruncResumeVal =
          PHINode::Create(OrigPhi->getType(), 2, "bc.trunc.resume.val",
                          ScalarPH->getTerminator());
      BCTruncResumeVal->addIncoming(TruncResumeVal, MiddleBlock);
    }


    Value *EndValue = nullptr;
    switch (II.IK) {
    case LoopVectorizationLegality::IK_NoInduction:
      llvm_unreachable("Unknown induction");
    case LoopVectorizationLegality::IK_IntInduction: {
      // Handle the integer induction counter.
      assert(OrigPhi->getType()->isIntegerTy() && "Invalid type");

      // We have the canonical induction variable.
      if (OrigPhi == OldInduction) {
        // Create a truncated version of the resume value for the scalar loop,
        // we might have promoted the type to a larger width.
        EndValue =
          BypassBuilder.CreateTrunc(IdxEndRoundDown, OrigPhi->getType());
        // The new PHI merges the original incoming value, in case of a bypass,
        // or the value at the end of the vectorized loop.
        for (unsigned I = 1, E = LoopBypassBlocks.size(); I != E; ++I)
          TruncResumeVal->addIncoming(II.StartValue, LoopBypassBlocks[I]);
        TruncResumeVal->addIncoming(EndValue, VecBody);

        BCTruncResumeVal->addIncoming(II.StartValue, LoopBypassBlocks[0]);

        // We know what the end value is.
        EndValue = IdxEndRoundDown;
        // We also know which PHI node holds it.
        ResumeIndex = ResumeVal;
        break;
      }

      // Not the canonical induction variable - add the vector loop count to the start value.
      //// need to comsider step info here !
      Value *CRD;
      if(II.Stride == 1)
     		CRD = BypassBuilder.CreateSExtOrTrunc(CountRoundDown,
                                                   II.StartValue->getType(),
                                                   "cast.crd");
	  else{
		assert((II.Stride > 1) && "unexpected stride!");
		CRD = BypassBuilder.CreateSExtOrTrunc(CountRoundDown, II.StartValue->getType(), "cast.crd");
		CRD = BypassBuilder.CreateMul(CRD, llvm::ConstantInt::get(II.StartValue->getType(), II.Stride, true));
		//CRD = BypassBuilder.CreateSExtOrTrunc(CRD, II.StartValue->getType(), "cast.crd");
	  }
      EndValue = BypassBuilder.CreateAdd(CRD, II.StartValue, "ind.end");
      break;
    }
    case LoopVectorizationLegality::IK_ReverseIntInduction: {
/*		
      Value *CRD = BypassBuilder.CreateSExtOrTrunc(CountRoundDown,
                                                   II.StartValue->getType(),
                                                   "cast.crd");
      EndValue = BypassBuilder.CreateSub(II.StartValue, CRD, "rev.ind.end");
*/
	  assert((II.Stride < 0) && "unexpected stride!!!");
	  Value * CRD = BypassBuilder.CreateSExtOrTrunc(CountRoundDown, II.StartValue->getType(), "cast.crd");
	  CRD = BypassBuilder.CreateMul(CRD, llvm::ConstantInt::get(II.StartValue->getType(), II.Stride, true));
	  EndValue = BypassBuilder.CreateAdd(CRD, II.StartValue, "ind.end");
      break;
    }
    case LoopVectorizationLegality::IK_PtrInduction: {
      // For pointer induction variables, calculate the offset using
      // the end index.
      EndValue = BypassBuilder.CreateGEP(II.StartValue, CountRoundDown,
                                         "ptr.ind.end");
      break;
    }
    case LoopVectorizationLegality::IK_ReversePtrInduction: {
      // The value at the end of the loop for the reverse pointer is calculated
      // by creating a GEP with a negative index starting from the start value.
      Value *Zero = ConstantInt::get(CountRoundDown->getType(), 0);
      Value *NegIdx = BypassBuilder.CreateSub(Zero, CountRoundDown,
                                              "rev.ind.end");
      EndValue = BypassBuilder.CreateGEP(II.StartValue, NegIdx,
                                         "rev.ptr.ind.end");
      break;
    }
    }// end of case

    // The new PHI merges the original incoming value, in case of a bypass,
    // or the value at the end of the vectorized loop.
    for (unsigned I = 1, E = LoopBypassBlocks.size(); I != E; ++I) {
      if (OrigPhi == OldInduction)
        ResumeVal->addIncoming(StartIdx, LoopBypassBlocks[I]);
      else
        ResumeVal->addIncoming(II.StartValue, LoopBypassBlocks[I]);
    }
    ResumeVal->addIncoming(EndValue, VecBody);

    // Fix the scalar body counter (PHI node).
    unsigned BlockIdx = OrigPhi->getBasicBlockIndex(ScalarPH);

    // The old induction's phi node in the scalar body needs the truncated
    // value.
    if (OrigPhi == OldInduction) {
      BCResumeVal->addIncoming(StartIdx, LoopBypassBlocks[0]);
      OrigPhi->setIncomingValue(BlockIdx, BCTruncResumeVal);
    } else {
      BCResumeVal->addIncoming(II.StartValue, LoopBypassBlocks[0]);
      OrigPhi->setIncomingValue(BlockIdx, BCResumeVal);
    }
  }

  // If we are generating a new induction variable then we also need to
  // generate the code that calculates the exit value. This value is not
  // simply the end of the counter because we may skip the vectorized body
  // in case of a runtime check.
  if (!OldInduction){
    assert(!ResumeIndex && "Unexpected resume value found");
    ResumeIndex = PHINode::Create(IdxTy, 2, "new.indc.resume.val",
                                  MiddleBlock->getTerminator());
    for (unsigned I = 1, E = LoopBypassBlocks.size(); I != E; ++I)
      ResumeIndex->addIncoming(StartIdx, LoopBypassBlocks[I]);
    ResumeIndex->addIncoming(IdxEndRoundDown, VecBody);
  }

  // Make sure that we found the index where scalar loop needs to continue.
  assert(ResumeIndex && ResumeIndex->getType()->isIntegerTy() &&
         "Invalid resume Index");

  // Add a check in the middle block to see if we have completed
  // all of the iterations in the first vector loop.
  // If (N - N%VF) == N, then we *don't* need to run the remainder.
  Value *CmpN = CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_EQ, IdxEnd,
                                ResumeIndex, "cmp.n",
                                MiddleBlock->getTerminator());
  BranchInst::Create(ExitBlock, ScalarPH, CmpN, MiddleBlock->getTerminator());
  // Remove the old terminator.
  MiddleBlock->getTerminator()->eraseFromParent();

  ////create the phi in VecBody
  // Create i+1 and fill the PHINode.
  Value *NextIdx = Builder.CreateAdd(Induction, Step, "index.next");
  Induction->addIncoming(StartIdx, VectorPH);
  Induction->addIncoming(NextIdx, VecBody);
  // Create the compare.
  Value *ICmp = Builder.CreateICmpEQ(NextIdx, IdxEndRoundDown);
  Builder.CreateCondBr(ICmp, MiddleBlock, VecBody);

  // Now we have two terminators. Remove the old one from the block.
  VecBody->getTerminator()->eraseFromParent();

  // Get ready to start creating new instructions into the vectorized body.
  Builder.SetInsertPoint(VecBody->getFirstInsertionPt());

  // Save the state.
  LoopVectorPreHeader = VectorPH;
  LoopScalarPreHeader = ScalarPH;
  LoopMiddleBlock = MiddleBlock;
  LoopExitBlock = ExitBlock;
  LoopVectorBody.push_back(VecBody);
  LoopScalarBody = OldBasicBlock;

  LoopVectorizeHints Hints(Lp, true);
  Hints.setAlreadyVectorized(Lp);
}


void InnerLoopVectorizer::vectorizeLoop() {
  //===------------------------------------------------===//
  //
  // Notice: any optimization or new instruction that go
  // into the code below should be also be implemented in
  // the cost-model.
  //
  //===------------------------------------------------===//
  Constant *Zero = Builder.getInt32(0);

  // In order to support reduction variables we need to be able to vectorize
  // Phi nodes. Phi nodes have cycles, so we need to vectorize them in two
  // stages. First, we create a new vector PHI node with no incoming edges.
  // We use this value when we vectorize all of the instructions that use the
  // PHI. Next, after all of the instructions in the block are complete we
  // add the new incoming edges to the PHI. At this point all of the
  // instructions in the basic block are vectorized, so we can use them to
  // construct the PHI.
  PhiVector RdxPHIsToFix;

  // Scan the loop in a topological order to ensure that defs are vectorized
  // before users.
  LoopBlocksDFS DFS(OrigLoop);
  DFS.perform(LI);

  // Vectorize all of the blocks in the original loop.
  for (LoopBlocksDFS::RPOIterator bb = DFS.beginRPO(),
       be = DFS.endRPO(); bb != be; ++bb){
    vectorizeBlockInLoop(*bb, &RdxPHIsToFix);
  }
  ////after the previous phase, slp vars are gethered and located at the same location but unvectorized;
  ////now vectorize SLP vars here 
  vectorizeSLPBundles(); 
  
  // At this point every instruction in the original loop is widened to
  // a vector form. We are almost done. Now, we need to fix the PHI nodes
  // that we vectorized. The PHI nodes are currently empty because we did
  // not want to introduce cycles. Notice that the remaining PHI nodes
  // that we need to fix are reduction variables.

  // Create the 'reduced' values for each of the induction vars.
  // The reduced values are the vector values that we scalarize and combine
  // after the loop is finished.
  for (PhiVector::iterator it = RdxPHIsToFix.begin(), e = RdxPHIsToFix.end();
       it != e; ++it) {
    PHINode *RdxPhi = *it;
    assert(RdxPhi && "Unable to recover vectorized PHI");

    // Find the reduction variable descriptor.
    assert(Legal->getReductionVars()->count(RdxPhi) &&
           "Unable to find the reduction variable");
    LoopVectorizationLegality::ReductionDescriptor RdxDesc =
    (*Legal->getReductionVars())[RdxPhi];

    setDebugLocFromInst(Builder, RdxDesc.StartValue);

    // We need to generate a reduction vector from the incoming scalar.
    // To do so, we need to generate the 'identity' vector and override
    // one of the elements with the incoming scalar reduction. We need
    // to do it in the vector-loop preheader.
    Builder.SetInsertPoint(LoopBypassBlocks[1]->getTerminator());

    // This is the vector-clone of the value that leaves the loop.
    VectorParts &VectorExit = getVectorValue(RdxDesc.LoopExitInstr);
    Type *VecTy = VectorExit[0]->getType();

    // Find the reduction identity variable. Zero for addition, or, xor,
    // one for multiplication, -1 for And.
    Value *Identity;
    Value *VectorStart;
    if (RdxDesc.Kind == LoopVectorizationLegality::RK_IntegerMinMax ||
        RdxDesc.Kind == LoopVectorizationLegality::RK_FloatMinMax) {
      // MinMax reduction have the start value as their identify.
      if (VF == 1) {
        VectorStart = Identity = RdxDesc.StartValue;
      } else {
        VectorStart = Identity = Builder.CreateVectorSplat(VF,
                                                           RdxDesc.StartValue,
                                                           "minmax.ident");
      }
    } else {
      // Handle other reduction kinds:
      Constant *Iden =
      LoopVectorizationLegality::getReductionIdentity(RdxDesc.Kind,
                                                      VecTy->getScalarType());
      if (VF == 1) {
        Identity = Iden;
        // This vector is the Identity vector where the first element is the
        // incoming scalar reduction.
        VectorStart = RdxDesc.StartValue;
      } else {
        Identity = ConstantVector::getSplat(VF, Iden);
        // This vector is the Identity vector where the first element is the
        // incoming scalar reduction.
        VectorStart = Builder.CreateInsertElement(Identity,
                                                  RdxDesc.StartValue, Zero);
      }
    }

    // Fix the vector-loop phi.
    // We created the induction variable so we know that the
    // preheader is the first entry.
    BasicBlock *VecPreheader = Induction->getIncomingBlock(0);

    // Reductions do not have to start at zero. They can start with
    // any loop invariant values.
    VectorParts &VecRdxPhi = WidenMap.get(RdxPhi);
    BasicBlock *Latch = OrigLoop->getLoopLatch();
    Value *LoopVal = RdxPhi->getIncomingValueForBlock(Latch);
    VectorParts &Val = getVectorValue(LoopVal);
    for (unsigned part = 0; part < UF; ++part) {
      // Make sure to add the reduction stat value only to the
      // first unroll part.
      Value *StartVal = (part == 0) ? VectorStart : Identity;
      cast<PHINode>(VecRdxPhi[part])->addIncoming(StartVal, VecPreheader);
      cast<PHINode>(VecRdxPhi[part])->addIncoming(Val[part],
                                                  LoopVectorBody.back());
    }

    // Before each round, move the insertion point right between
    // the PHIs and the values we are going to write.
    // This allows us to write both PHINodes and the extractelement
    // instructions.
    Builder.SetInsertPoint(LoopMiddleBlock->getFirstInsertionPt());

    VectorParts RdxParts;
    setDebugLocFromInst(Builder, RdxDesc.LoopExitInstr);
    for (unsigned part = 0; part < UF; ++part) {
      // This PHINode contains the vectorized reduction variable, or
      // the initial value vector, if we bypass the vector loop.
      VectorParts &RdxExitVal = getVectorValue(RdxDesc.LoopExitInstr);
      PHINode *NewPhi = Builder.CreatePHI(VecTy, 2, "rdx.vec.exit.phi");
      Value *StartVal = (part == 0) ? VectorStart : Identity;
      for (unsigned I = 1, E = LoopBypassBlocks.size(); I != E; ++I)
        NewPhi->addIncoming(StartVal, LoopBypassBlocks[I]);
      NewPhi->addIncoming(RdxExitVal[part],
                          LoopVectorBody.back());
      RdxParts.push_back(NewPhi);
    }

    // Reduce all of the unrolled parts into a single vector.
    Value *ReducedPartRdx = RdxParts[0];
    unsigned Op = getReductionBinOp(RdxDesc.Kind);
    setDebugLocFromInst(Builder, ReducedPartRdx);
    for (unsigned part = 1; part < UF; ++part) {
      if (Op != Instruction::ICmp && Op != Instruction::FCmp)
        // Floating point operations had to be 'fast' to enable the reduction.
        ReducedPartRdx = addFastMathFlag(
            Builder.CreateBinOp((Instruction::BinaryOps)Op, RdxParts[part],
                                ReducedPartRdx, "bin.rdx"));
      else
        ReducedPartRdx = createMinMaxOp(Builder, RdxDesc.MinMaxKind,
                                        ReducedPartRdx, RdxParts[part]);
    }

    if (VF > 1) {
      // VF is a power of 2 so we can emit the reduction using log2(VF) shuffles
      // and vector ops, reducing the set of values being computed by half each
      // round.
      assert(isPowerOf2_32(VF) &&
             "Reduction emission only supported for pow2 vectors!");
      Value *TmpVec = ReducedPartRdx;
      SmallVector<Constant*, 32> ShuffleMask(VF, nullptr);
      for (unsigned i = VF; i != 1; i >>= 1) {
        // Move the upper half of the vector to the lower half.
        for (unsigned j = 0; j != i/2; ++j)
          ShuffleMask[j] = Builder.getInt32(i/2 + j);

        // Fill the rest of the mask with undef.
        std::fill(&ShuffleMask[i/2], ShuffleMask.end(),
                  UndefValue::get(Builder.getInt32Ty()));

        Value *Shuf =
        Builder.CreateShuffleVector(TmpVec,
                                    UndefValue::get(TmpVec->getType()),
                                    ConstantVector::get(ShuffleMask),
                                    "rdx.shuf");

        if (Op != Instruction::ICmp && Op != Instruction::FCmp)
          // Floating point operations had to be 'fast' to enable the reduction.
          TmpVec = addFastMathFlag(Builder.CreateBinOp(
              (Instruction::BinaryOps)Op, TmpVec, Shuf, "bin.rdx"));
        else
          TmpVec = createMinMaxOp(Builder, RdxDesc.MinMaxKind, TmpVec, Shuf);
      }

      // The result is in the first element of the vector.
      ReducedPartRdx = Builder.CreateExtractElement(TmpVec,
                                                    Builder.getInt32(0));
    }

    // Create a phi node that merges control-flow from the backedge-taken check
    // block and the middle block.
    PHINode *BCBlockPhi = PHINode::Create(RdxPhi->getType(), 2, "bc.merge.rdx",
                                          LoopScalarPreHeader->getTerminator());
    BCBlockPhi->addIncoming(RdxDesc.StartValue, LoopBypassBlocks[0]);
    BCBlockPhi->addIncoming(ReducedPartRdx, LoopMiddleBlock);

    // Now, we need to fix the users of the reduction variable
    // inside and outside of the scalar remainder loop.
    // We know that the loop is in LCSSA form. We need to update the
    // PHI nodes in the exit blocks.
    for (BasicBlock::iterator LEI = LoopExitBlock->begin(),
         LEE = LoopExitBlock->end(); LEI != LEE; ++LEI) {
      PHINode *LCSSAPhi = dyn_cast<PHINode>(LEI);
      if (!LCSSAPhi) break;

      // All PHINodes need to have a single entry edge, or two if
      // we already fixed them.
      assert(LCSSAPhi->getNumIncomingValues() < 3 && "Invalid LCSSA PHI");

      // We found our reduction value exit-PHI. Update it with the
      // incoming bypass edge.
      if (LCSSAPhi->getIncomingValue(0) == RdxDesc.LoopExitInstr) {
        // Add an edge coming from the bypass.
        LCSSAPhi->addIncoming(ReducedPartRdx, LoopMiddleBlock);
        break;
      }
    }// end of the LCSSA phi scan.

    // Fix the scalar loop reduction variable with the incoming reduction sum
    // from the vector body and from the backedge value.
    int IncomingEdgeBlockIdx =
    (RdxPhi)->getBasicBlockIndex(OrigLoop->getLoopLatch());
    assert(IncomingEdgeBlockIdx >= 0 && "Invalid block index");
    // Pick the other block.
    int SelfEdgeBlockIdx = (IncomingEdgeBlockIdx ? 0 : 1);
    (RdxPhi)->setIncomingValue(SelfEdgeBlockIdx, BCBlockPhi);
    (RdxPhi)->setIncomingValue(IncomingEdgeBlockIdx, RdxDesc.LoopExitInstr);
  }// end of for each redux variable.

  fixLCSSAPHIs();
   

  // Remove redundant induction instructions.
  cse(LoopVectorBody);
}

void InnerLoopVectorizer::fixLCSSAPHIs() {
  for (BasicBlock::iterator LEI = LoopExitBlock->begin(),
       LEE = LoopExitBlock->end(); LEI != LEE; ++LEI) {
    PHINode *LCSSAPhi = dyn_cast<PHINode>(LEI);
    if (!LCSSAPhi) break;
    if (LCSSAPhi->getNumIncomingValues() == 1)
      LCSSAPhi->addIncoming(UndefValue::get(LCSSAPhi->getType()),
                            LoopMiddleBlock);
  }
} 

InnerLoopVectorizer::VectorParts
InnerLoopVectorizer::createEdgeMask(BasicBlock *Src, BasicBlock *Dst) {
  assert(std::find(pred_begin(Dst), pred_end(Dst), Src) != pred_end(Dst) &&
         "Invalid edge");

  // Look for cached value.
  std::pair<BasicBlock*, BasicBlock*> Edge(Src, Dst);
  EdgeMaskCache::iterator ECEntryIt = MaskCache.find(Edge);
  if (ECEntryIt != MaskCache.end())
    return ECEntryIt->second;

  VectorParts SrcMask = createBlockInMask(Src);

  // The terminator has to be a branch inst!
  BranchInst *BI = dyn_cast<BranchInst>(Src->getTerminator());
  assert(BI && "Unexpected terminator found");

  if (BI->isConditional()) {
    VectorParts EdgeMask = getVectorValue(BI->getCondition());

    if (BI->getSuccessor(0) != Dst)
      for (unsigned part = 0; part < UF; ++part)
        EdgeMask[part] = Builder.CreateNot(EdgeMask[part]);

    for (unsigned part = 0; part < UF; ++part)
      EdgeMask[part] = Builder.CreateAnd(EdgeMask[part], SrcMask[part]);

    MaskCache[Edge] = EdgeMask;
    return EdgeMask;
  }

  MaskCache[Edge] = SrcMask;
  return SrcMask;
}

InnerLoopVectorizer::VectorParts
InnerLoopVectorizer::createBlockInMask(BasicBlock *BB) {
  assert(OrigLoop->contains(BB) && "Block is not a part of a loop");

  // Loop incoming mask is all-one.
  if (OrigLoop->getHeader() == BB) {
    Value *C = ConstantInt::get(IntegerType::getInt1Ty(BB->getContext()), 1);
    return getVectorValue(C);
  }

  // This is the block mask. We OR all incoming edges, and with zero.
  Value *Zero = ConstantInt::get(IntegerType::getInt1Ty(BB->getContext()), 0);
  VectorParts BlockMask = getVectorValue(Zero);

  // For each pred:
  for (pred_iterator it = pred_begin(BB), e = pred_end(BB); it != e; ++it) {
    VectorParts EM = createEdgeMask(*it, BB);
    for (unsigned part = 0; part < UF; ++part)
      BlockMask[part] = Builder.CreateOr(BlockMask[part], EM[part]);
  }

  return BlockMask;
}

void InnerLoopVectorizer::widenPHIInstruction(Instruction *PN,
                                              InnerLoopVectorizer::VectorParts &Entry,
                                              unsigned UF, unsigned VF, PhiVector *PV) {
  PHINode* P = cast<PHINode>(PN);
  // Handle reduction variables:
  if (Legal->getReductionVars()->count(P)) {
    for (unsigned part = 0; part < UF; ++part) {
      // This is phase one of vectorizing PHIs.
      Type *VecTy = (VF == 1) ? PN->getType() :
      VectorType::get(PN->getType(), VF);
      Entry[part] = PHINode::Create(VecTy, 2, "vec.phi",
                                    LoopVectorBody.back()-> getFirstInsertionPt());
    }
    PV->push_back(P);
    return;
  }

  setDebugLocFromInst(Builder, P);
  // Check for PHI nodes that are lowered to vector selects.
  if (P->getParent() != OrigLoop->getHeader()) {
    // We know that all PHIs in non-header blocks are converted into
    // selects, so we don't have to worry about the insertion order and we
    // can just use the builder.
    // At this point we generate the predication tree. There may be
    // duplications since this is a simple recursive scan, but future
    // optimizations will clean it up.

    unsigned NumIncoming = P->getNumIncomingValues();

    // Generate a sequence of selects of the form:
    // SELECT(Mask3, In3,
    //      SELECT(Mask2, In2,
    //                   ( ...)))
    for (unsigned In = 0; In < NumIncoming; In++) {
      VectorParts Cond = createEdgeMask(P->getIncomingBlock(In),
                                        P->getParent());
      VectorParts &In0 = getVectorValue(P->getIncomingValue(In));

      for (unsigned part = 0; part < UF; ++part) {
        // We might have single edge PHIs (blocks) - use an identity
        // 'select' for the first PHI operand.
        if (In == 0)
          Entry[part] = Builder.CreateSelect(Cond[part], In0[part],
                                             In0[part]);
        else
          // Select between the current value and the previous incoming edge
          // based on the incoming mask.
          Entry[part] = Builder.CreateSelect(Cond[part], In0[part],
                                             Entry[part], "predphi");
      }
    }
    return;
  }

  // This PHINode must be an induction variable.
  // Make sure that we know about it.
  assert(Legal->getInductionVars()->count(P) &&
         "Not an induction variable");

  LoopVectorizationLegality::InductionInfo II =
  Legal->getInductionVars()->lookup(P);

  switch (II.IK) {
    case LoopVectorizationLegality::IK_NoInduction:
      llvm_unreachable("Unknown induction");
    case LoopVectorizationLegality::IK_IntInduction: {
      assert(P->getType() == II.StartValue->getType() && "Types must match");
      Type *PhiTy = P->getType();
      Value *Broadcasted;
      if (P == OldInduction) {
        // Handle the canonical induction variable. We might have had to
        // extend the type.
        Broadcasted = Builder.CreateTrunc(Induction, PhiTy);
        Broadcasted = getBroadcastInstrs(Broadcasted);	
		assert((II.Stride == 1) && "unexpected stride of indvar!\n");
        for (unsigned part = 0; part < UF; ++part)
          Entry[part] = getConsecutiveVector(Broadcasted, VF * part, false);		
      } else {
        // Handle other induction variables that are now based on the
        // canonical one.
        Value *NormalizedIdx = Builder.CreateSub(Induction, ExtendedIdx,
                                                 "normalized.idx");
        NormalizedIdx = Builder.CreateSExtOrTrunc(NormalizedIdx, PhiTy);
		if(II.Stride > 1)
			NormalizedIdx = Builder.CreateMul(NormalizedIdx, llvm::ConstantInt::get(PhiTy, II.Stride, true));		
        Broadcasted = Builder.CreateAdd(II.StartValue, NormalizedIdx, "offset.idx");
        Broadcasted = getBroadcastInstrs(Broadcasted);	
        for (unsigned part = 0; part < UF; ++part)
          Entry[part] = getConsecutiveVector(Broadcasted, VF * part, false, II.Stride);		
      }
      return;
    }
    case LoopVectorizationLegality::IK_ReverseIntInduction:
    case LoopVectorizationLegality::IK_PtrInduction:
    case LoopVectorizationLegality::IK_ReversePtrInduction:
      // Handle reverse integer and pointer inductions.
      Value *StartIdx = ExtendedIdx;
      // This is the normalized GEP that starts counting at zero.
      Value *NormalizedIdx = Builder.CreateSub(Induction, StartIdx,
                                               "normalized.idx");

      // Handle the reverse integer induction variable case.
      if (LoopVectorizationLegality::IK_ReverseIntInduction == II.IK) {
        IntegerType *DstTy = cast<IntegerType>(II.StartValue->getType());
        Value *CNI = Builder.CreateSExtOrTrunc(NormalizedIdx, DstTy,
                                               "resize.norm.idx");

		Value *ReverseInd = Builder.CreateMul(CNI, llvm::ConstantInt::get(II.StartValue->getType(), II.Stride, true));		
	    Value *Broadcasted = Builder.CreateAdd(II.StartValue, ReverseInd,
									"offset.idx");
	    Broadcasted = getBroadcastInstrs(Broadcasted);	
	    for (unsigned part = 0; part < UF; ++part)
	       Entry[part] = getConsecutiveVector(Broadcasted, -(int)VF * part, false, II.Stride);

        return;
      }

      // Handle the pointer induction variable case.
      assert(P->getType()->isPointerTy() && "Unexpected type.");

      // Is this a reverse induction ptr or a consecutive induction ptr.
      bool Reverse = (LoopVectorizationLegality::IK_ReversePtrInduction ==
                      II.IK);

      // This is the vector of results. Notice that we don't generate
      // vector geps because scalar geps result in better code.
      for (unsigned part = 0; part < UF; ++part) {
        if (VF == 1) {
          int EltIndex = (part) * (Reverse ? -1 : 1);
          Constant *Idx = ConstantInt::get(Induction->getType(), EltIndex);
          Value *GlobalIdx;
          if (Reverse)
            GlobalIdx = Builder.CreateSub(Idx, NormalizedIdx, "gep.ridx");
          else
            GlobalIdx = Builder.CreateAdd(NormalizedIdx, Idx, "gep.idx");

          Value *SclrGep = Builder.CreateGEP(II.StartValue, GlobalIdx,
                                             "next.gep");
          Entry[part] = SclrGep;
          continue;
        }

        Value *VecVal = UndefValue::get(VectorType::get(P->getType(), VF));
        for (unsigned int i = 0; i < VF; ++i) {
          int EltIndex = (i + part * VF) * (Reverse ? -1 : 1);
          Constant *Idx = ConstantInt::get(Induction->getType(), EltIndex);
          Value *GlobalIdx;
          if (!Reverse)
            GlobalIdx = Builder.CreateAdd(NormalizedIdx, Idx, "gep.idx");
          else
            GlobalIdx = Builder.CreateSub(Idx, NormalizedIdx, "gep.ridx");

          Value *SclrGep = Builder.CreateGEP(II.StartValue, GlobalIdx,
                                             "next.gep");
          VecVal = Builder.CreateInsertElement(VecVal, SclrGep,
                                               Builder.getInt32(i),
                                               "insert.gep");
        }
        Entry[part] = VecVal;
      }
      return;
  }
}

void InnerLoopVectorizer::vectorizeBlockInLoop(BasicBlock *BB, PhiVector *PV) {
  
  // For each instruction in the old loop.
  for (BasicBlock::iterator it = BB->begin(), e = BB->end(); it != e; ++it) {
    ////check if the value is in a slp bundle, if so, check if the code need to be inserted right now
    if(Legal->isSLPVar(it)){
		unsigned bundle_id = Legal->getSLPBundleID(it);
		Instruction * loc = Legal->getSLPBundleInsertLoc(bundle_id);
		if(loc == cast<Instruction>(it)){
			vectorizeSLPValuePack(it);		
		}	
		continue;
	}

	/////vector the var across loop iterations
	/////here, we need to take care of the operands, since they may be vectorized in SLP manner
    VectorParts &Entry = WidenMap.get(it);

    switch (it->getOpcode()) {
    case Instruction::Br:
      // Nothing to do for PHIs and BR, since we already took care of the
      // loop control flow instructions.
      continue;
    case Instruction::PHI:{
      // Vectorize PHINodes.
      widenPHIInstruction(it, Entry, UF, VF, PV);
      continue;
    }// End of PHI.

    case Instruction::Add:
    case Instruction::FAdd:
    case Instruction::Sub:
    case Instruction::FSub:
    case Instruction::Mul:
    case Instruction::FMul:
    case Instruction::UDiv:
    case Instruction::SDiv:
    case Instruction::FDiv:
    case Instruction::URem:
    case Instruction::SRem:
    case Instruction::FRem:
    case Instruction::Shl:
    case Instruction::LShr:
    case Instruction::AShr:
    case Instruction::And:
    case Instruction::Or:
    case Instruction::Xor: {
      // Just widen binops.	  
      BinaryOperator *BinOp = dyn_cast<BinaryOperator>(it);
      setDebugLocFromInst(Builder, BinOp);
      VectorParts &A = getVectorValue(it->getOperand(0));
      VectorParts &B = getVectorValue(it->getOperand(1));

      // Use this vector value for all users of the original instruction.
      for (unsigned Part = 0; Part < UF; ++Part) {
        Value *V = Builder.CreateBinOp(BinOp->getOpcode(), A[Part], B[Part]);

        // Update the NSW, NUW and Exact flags. Notice: V can be an Undef.
        BinaryOperator *VecOp = dyn_cast<BinaryOperator>(V);
        if (VecOp && isa<OverflowingBinaryOperator>(BinOp)) {
          VecOp->setHasNoSignedWrap(BinOp->hasNoSignedWrap());
          VecOp->setHasNoUnsignedWrap(BinOp->hasNoUnsignedWrap());
        }
        if (VecOp && isa<PossiblyExactOperator>(VecOp))
          VecOp->setIsExact(BinOp->isExact());

        // Copy the fast-math flags.
        if (VecOp && isa<FPMathOperator>(V))
          VecOp->setFastMathFlags(it->getFastMathFlags());

        Entry[Part] = V;
      }

      propagateMetadata(Entry, it);
      break;
    }
    case Instruction::Select: {
      // Widen selects.
      // If the selector is loop invariant we can create a select
      // instruction with a scalar condition. Otherwise, use vector-select.
      bool InvariantCond = SE->isLoopInvariant(SE->getSCEV(it->getOperand(0)),
                                               OrigLoop);
      setDebugLocFromInst(Builder, it);

      // The condition can be loop invariant  but still defined inside the
      // loop. This means that we can't just use the original 'cond' value.
      // We have to take the 'vectorized' value and pick the first lane.
      // Instcombine will make this a no-op.
      VectorParts &Cond = getVectorValue(it->getOperand(0));
      VectorParts &Op0  = getVectorValue(it->getOperand(1));
      VectorParts &Op1  = getVectorValue(it->getOperand(2));

      Value *ScalarCond = (VF == 1) ? Cond[0] :
        Builder.CreateExtractElement(Cond[0], Builder.getInt32(0));

      for (unsigned Part = 0; Part < UF; ++Part) {
        Entry[Part] = Builder.CreateSelect(
          InvariantCond ? ScalarCond : Cond[Part],
          Op0[Part],
          Op1[Part]);
      }

      propagateMetadata(Entry, it);
      break;
    }

    case Instruction::ICmp:
    case Instruction::FCmp: {
      // Widen compares. Generate vector compares.
      bool FCmp = (it->getOpcode() == Instruction::FCmp);
      CmpInst *Cmp = dyn_cast<CmpInst>(it);
      setDebugLocFromInst(Builder, it);
      VectorParts &A = getVectorValue(it->getOperand(0));
      VectorParts &B = getVectorValue(it->getOperand(1));
      for (unsigned Part = 0; Part < UF; ++Part) {
        Value *C = nullptr;
        if (FCmp)
          C = Builder.CreateFCmp(Cmp->getPredicate(), A[Part], B[Part]);
        else
          C = Builder.CreateICmp(Cmp->getPredicate(), A[Part], B[Part]);
        Entry[Part] = C;
      }

      propagateMetadata(Entry, it);
      break;
    }

    case Instruction::Store:
    case Instruction::Load:
      vectorizeMemoryInstruction(it);
        break;
    case Instruction::ZExt:
    case Instruction::SExt:
    case Instruction::FPToUI:
    case Instruction::FPToSI:
    case Instruction::FPExt:
    case Instruction::PtrToInt:
    case Instruction::IntToPtr:
    case Instruction::SIToFP:
    case Instruction::UIToFP:
    case Instruction::Trunc:
    case Instruction::FPTrunc:
    case Instruction::BitCast: {
      CastInst *CI = dyn_cast<CastInst>(it);
      setDebugLocFromInst(Builder, it);
      /// Optimize the special case where the source is the induction
      /// variable. Notice that we can only optimize the 'trunc' case
      /// because: a. FP conversions lose precision, b. sext/zext may wrap,
      /// c. other casts depend on pointer size.
      if (CI->getOperand(0) == OldInduction &&
          it->getOpcode() == Instruction::Trunc) {
        Value *ScalarCast = Builder.CreateCast(CI->getOpcode(), Induction,
                                               CI->getType());
        Value *Broadcasted = getBroadcastInstrs(ScalarCast);
        for (unsigned Part = 0; Part < UF; ++Part)
          Entry[Part] = getConsecutiveVector(Broadcasted, VF * Part, false);
        propagateMetadata(Entry, it);
        break;
      }
      /// Vectorize casts.
      Type *DestTy = (VF == 1) ? CI->getType() :
                                 VectorType::get(CI->getType(), VF);

      VectorParts &A = getVectorValue(it->getOperand(0));
      for (unsigned Part = 0; Part < UF; ++Part)
        Entry[Part] = Builder.CreateCast(CI->getOpcode(), A[Part], DestTy);
      propagateMetadata(Entry, it);
      break;
    }

    case Instruction::Call: {
      // Ignore dbg intrinsics.
      if (isa<DbgInfoIntrinsic>(it))
        break;
      setDebugLocFromInst(Builder, it);

      Module *M = BB->getParent()->getParent();
      CallInst *CI = cast<CallInst>(it);
      Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
      assert(ID && "Not an intrinsic call!");
      switch (ID) {
      case Intrinsic::lifetime_end:
      case Intrinsic::lifetime_start:
        scalarizeInstruction(it);
        break;
      default:
        bool HasScalarOpd = hasVectorInstrinsicScalarOpd(ID, 1);
        for (unsigned Part = 0; Part < UF; ++Part) {
          SmallVector<Value *, 4> Args;
          for (unsigned i = 0, ie = CI->getNumArgOperands(); i != ie; ++i) {
            if (HasScalarOpd && i == 1) {
              Args.push_back(CI->getArgOperand(i));
              continue;
            }
            VectorParts &Arg = getVectorValue(CI->getArgOperand(i));
            Args.push_back(Arg[Part]);
          }
          Type *Tys[] = {CI->getType()};
          if (VF > 1)
            Tys[0] = VectorType::get(CI->getType()->getScalarType(), VF);

          Function *F = Intrinsic::getDeclaration(M, ID, Tys);
          Entry[Part] = Builder.CreateCall(F, Args);
        }

        propagateMetadata(Entry, it);
        break;
      }
      break;
    }

    default:
      // All other instructions are unsupported. Scalarize them.
      scalarizeInstruction(it);
      break;
    }// end of switch.	
  }// end of for_each instr.
}



void InnerLoopVectorizer::vectorizeSLPValuePack(Value * sca) {

	unsigned bundle_id = Legal->getSLPBundleID(sca);
    ValuePack slp_bundle = Legal->getSLPBundle(bundle_id);
	unsigned bundle_size = slp_bundle.size();   

    switch (getSameOpcode(slp_bundle))
    {
	
    case Instruction::Add:
    case Instruction::FAdd:
    case Instruction::Sub:
    case Instruction::FSub:
    case Instruction::Mul:
    case Instruction::FMul:
    case Instruction::UDiv:
    case Instruction::SDiv:
    case Instruction::FDiv:
    case Instruction::URem:
    case Instruction::SRem:
    case Instruction::FRem:
    case Instruction::Shl:
    case Instruction::LShr:
    case Instruction::AShr:
    case Instruction::And:
    case Instruction::Or:
    case Instruction::Xor: {
	  ////need to check the source operands, if any of them is not a slp bundle, then resolve the conflicts here		
      BinaryOperator *BinOp = dyn_cast<BinaryOperator>(sca);
      setDebugLocFromInst(Builder, BinOp);
	  
      unsigned opr_num = cast<Instruction>(sca)->getNumOperands();	  
	  for(unsigned ii = 0; ii < opr_num; ii++){
	  	ValuePack opr_bundle;
		for(unsigned jj = 0; jj < bundle_size; jj++){
			Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
			opr_bundle.push_back(op_t);
		}
		bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		if(!findit)
			switchLoopLevelToSLP(opr_bundle);		
		////handle the case if the slp vars are from the loop
		////as we only scan the loop body, these vars are not processed
		else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			for(unsigned jj = 0; jj < bundle_size; jj++){
				Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
				SLPScaMap.splat(op_t, op_t);
			}
		}		
	  }
  	  
	  for(unsigned ii = 0; ii < bundle_size; ii++){
		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(0))){
			assert(false && "missing operand value 0 in SLPScaMap?!\n");
		}
		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(1))){
			assert(false && "missing operand value 1 in SLPScaMap?!\n");
		}	
		if(SLPScaMap.has(slp_bundle[ii]))
			assert(false && "the slp value already been processed?!\n");

		VectorParts & slpsca = SLPScaMap.get(slp_bundle[ii]);			
        VectorParts &A = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(0));
        VectorParts &B = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(1));	

		for(unsigned jj = 0; jj < UF*VF; jj++){
			Value *V = Builder.CreateBinOp(BinOp->getOpcode(), A[jj], B[jj]);

			// Update the NSW, NUW and Exact flags. Notice: V can be an Undef.
			BinaryOperator *ScaOp = dyn_cast<BinaryOperator>(V);
			if (ScaOp && isa<OverflowingBinaryOperator>(BinOp)) {
			  ScaOp->setHasNoSignedWrap(BinOp->hasNoSignedWrap());
			  ScaOp->setHasNoUnsignedWrap(BinOp->hasNoUnsignedWrap());
			}
			if (ScaOp && isa<PossiblyExactOperator>(ScaOp))
			  ScaOp->setIsExact(BinOp->isExact());			
			// Copy the fast-math flags.
			if (ScaOp && isa<FPMathOperator>(V))
			  ScaOp->setFastMathFlags(cast<Instruction>(sca)->getFastMathFlags());
			slpsca[jj] = V;
		}
	  }	  
      break;
    }

    case Instruction::ShuffleVector: {
	  ////need to check the source operands, if any of them is not a slp bundle, then resolve the conflicts here			  
      unsigned opr_num = cast<Instruction>(sca)->getNumOperands();	  
	  for(unsigned ii = 0; ii < opr_num; ii++){
	  	ValuePack opr_bundle;
		for(unsigned jj = 0; jj < bundle_size; jj++){
			Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
			opr_bundle.push_back(op_t);
		}
		bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		if(!findit)
			switchLoopLevelToSLP(opr_bundle);		
		////handle the case if the slp vars are from the outside of the loop
		////as we only scan the loop body, these vars are not processed
		else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			for(unsigned jj = 0; jj < bundle_size; jj++){
				Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
				SLPScaMap.splat(op_t, op_t);
			}
		}		
	  }
  	  
	  for(unsigned ii = 0; ii < bundle_size; ii++){
		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(0))){
			assert(false && "missing operand value 0 in SLPScaMap?!\n");
		}
		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(1))){
			assert(false && "missing operand value 1 in SLPScaMap?!\n");
		}	
		if(SLPScaMap.has(slp_bundle[ii]))
			assert(false && "the slp value already been processed?!\n");

		VectorParts & slpsca = SLPScaMap.get(slp_bundle[ii]);			
        VectorParts &A = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(0));
        VectorParts &B = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(1));	

        BinaryOperator *BinOp = dyn_cast<BinaryOperator>(slp_bundle[ii]);
		for(unsigned jj = 0; jj < UF*VF; jj++){
			Value *V = Builder.CreateBinOp(BinOp->getOpcode(), A[jj], B[jj]);

			// Update the NSW, NUW and Exact flags. Notice: V can be an Undef.
			BinaryOperator *ScaOp = dyn_cast<BinaryOperator>(V);
			if (ScaOp && isa<OverflowingBinaryOperator>(BinOp)) {
			  ScaOp->setHasNoSignedWrap(BinOp->hasNoSignedWrap());
			  ScaOp->setHasNoUnsignedWrap(BinOp->hasNoUnsignedWrap());
			}
			if (ScaOp && isa<PossiblyExactOperator>(ScaOp))
			  ScaOp->setIsExact(BinOp->isExact());			
			// Copy the fast-math flags.
			if (ScaOp && isa<FPMathOperator>(V))
			  ScaOp->setFastMathFlags(cast<Instruction>(sca)->getFastMathFlags());
			slpsca[jj] = V;
		}
	  }	  
      break;
    }
	
    case Instruction::ZExt:
    case Instruction::SExt:
    case Instruction::FPToUI:
    case Instruction::FPToSI:
    case Instruction::FPExt:
    case Instruction::PtrToInt:
    case Instruction::IntToPtr:
    case Instruction::SIToFP:
    case Instruction::UIToFP:
    case Instruction::Trunc:
    case Instruction::FPTrunc:
    case Instruction::BitCast: {

      CastInst *CI = dyn_cast<CastInst>(sca);
      setDebugLocFromInst(Builder, sca);
      Type *DestTy = CI->getType();  		

      unsigned opr_num = cast<Instruction>(sca)->getNumOperands();	  
	  for(unsigned ii = 0; ii < opr_num; ii++){
	  	ValuePack opr_bundle;
		for(unsigned jj = 0; jj < bundle_size; jj++){
			opr_bundle.push_back(cast<Instruction>(slp_bundle[jj])->getOperand(ii));
		}
		bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		if(!findit)
			switchLoopLevelToSLP(opr_bundle);
		else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			for(unsigned jj = 0; jj < bundle_size; jj++){
				Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
				SLPScaMap.splat(op_t, op_t);
			}
		}		
	  }

	  for(unsigned ii = 0; ii < bundle_size; ii++){

		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(0)))
			assert(false && "missing operand value in SLPScaMap?!\n");
		if(SLPScaMap.has(slp_bundle[ii]))
			assert(false && "the slp value already been processed?!\n");

		VectorParts & slpsca = SLPScaMap.get(slp_bundle[ii]);			
        VectorParts & A = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(0));
		for(unsigned jj = 0; jj < UF*VF; jj++){
			Value *V = Builder.CreateCast(CI->getOpcode(), A[jj], DestTy);
			slpsca[jj] = V;
		}
	  }
      break;
    }

    case Instruction::ICmp:
    case Instruction::FCmp: {

      bool FCmp = (cast<Instruction>(sca)->getOpcode() == Instruction::FCmp);
      CmpInst *Cmp = dyn_cast<CmpInst>(sca);
      setDebugLocFromInst(Builder, sca);
	  
      unsigned opr_num = cast<Instruction>(sca)->getNumOperands();	  
	  for(unsigned ii = 0; ii < opr_num; ii++){
	  	ValuePack opr_bundle;
		for(unsigned jj = 0; jj < bundle_size; jj++){
			opr_bundle.push_back(cast<Instruction>(slp_bundle[jj])->getOperand(ii));
		}
		bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		if(!findit)
			switchLoopLevelToSLP(opr_bundle);
		else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			for(unsigned jj = 0; jj < bundle_size; jj++){
				Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
				SLPScaMap.splat(op_t, op_t);
			}
		}			
	  }
	  
	  for(unsigned ii = 0; ii < bundle_size; ii++){

		if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(0)) || !SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(1)))
			assert(false && "missing operand value in SLPScaMap?!\n");
		if(SLPScaMap.has(slp_bundle[ii]))
			assert(false && "the slp value already been processed?!\n");

		VectorParts &slpsca = SLPScaMap.get(slp_bundle[ii]);			
        VectorParts &A = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(0));
        VectorParts &B = SLPScaMap.get(cast<Instruction>(slp_bundle[ii])->getOperand(1));	

		for(unsigned jj = 0; jj < UF*VF; jj++){
			Value *C = nullptr;
			if (FCmp)
			  C = Builder.CreateFCmp(Cmp->getPredicate(), A[jj], B[jj]);
			else
			  C = Builder.CreateICmp(Cmp->getPredicate(), A[jj], B[jj]);
			slpsca[jj] = C;
		}
	  }
      break;
    }
	
    case Instruction::Call: {
      // Ignore dbg intrinsics.
      if (isa<DbgInfoIntrinsic>(sca))
        break;
      setDebugLocFromInst(Builder, sca);

      CallInst *CI = cast<CallInst>(sca);
	  Module *M = CI->getParent()->getParent()->getParent();
      Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
      assert(ID && "Not an intrinsic call!");
      switch (ID) {
      case Intrinsic::lifetime_end:
      case Intrinsic::lifetime_start:{
	  	for(unsigned ii = 0; ii < bundle_size; ii++)
        	scalarizeInstruction(cast<Instruction>(slp_bundle[ii]));
        break;
	  }
      default:{
		unsigned opr_num = CI->getNumArgOperands();	
		for(unsigned ii = 0; ii < opr_num; ii++){
		  ValuePack opr_bundle;
		  for(unsigned jj = 0; jj < bundle_size; jj++){
			  opr_bundle.push_back(cast<Instruction>(slp_bundle[jj])->getOperand(ii));
		  }
		  bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		  if(!findit)
			  switchLoopLevelToSLP(opr_bundle);		  
		  else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			  for(unsigned jj = 0; jj < bundle_size; jj++){
				  Value * op_t = cast<Instruction>(slp_bundle[jj])->getOperand(ii);
				  SLPScaMap.splat(op_t, op_t);
			  }
		  }
		}
		
		for(unsigned ii = 0; ii < bundle_size; ii++){
		  if(SLPScaMap.has(slp_bundle[ii]))
			  assert(false && "the slp value already been processed?!\n");
		  for(unsigned kk = 0; kk < opr_num; kk++){
			  if(!SLPScaMap.has(cast<Instruction>(slp_bundle[ii])->getOperand(kk))){
			  	errs() << *(cast<Instruction>(slp_bundle[ii])->getOperand(kk)) << "\n";			  
				assert(false && "missing operand value in SLPScaMap?!\n");
		      }
		  }
		
		  VectorParts & slpsca = SLPScaMap.get(slp_bundle[ii]); 		  
		
		  for(unsigned jj = 0; jj < UF*VF; jj++){
			  SmallVector<Value *, 4> Args;
			  for (unsigned i = 0, ie = CI->getNumArgOperands(); i != ie; ++i) {
				  VectorParts &A = SLPScaMap.get(CI->getArgOperand(i));
				  Args.push_back(A[jj]);
				  continue;
			  }
              Function *F = Intrinsic::getDeclaration(M, ID, CI->getType()->getScalarType());			
			  Value *C = Builder.CreateCall(F, Args);
			  slpsca[jj] = C;
		  }
		}
        break;
      }
      }
      break;
    }

    case Instruction::Load:{
	    for(unsigned ii = 0; ii < bundle_size; ii++){
		    if(SLPScaMap.has(slp_bundle[ii]))
			   assert(false && "the slp value already been processed?!\n");
	    }			
      	vectorizeSLPMemAccPack(slp_bundle);
        break;
    }
	
    case Instruction::Store:{
		ValuePack opr_bundle;
		for(unsigned jj = 0; jj < bundle_size; jj++){
			opr_bundle.push_back(cast<StoreInst>(slp_bundle[jj])->getValueOperand());
		}
		bool findit = Legal->isExistingSlpBundle(opr_bundle) || Legal->isReversedExistingSlpBundle(opr_bundle);
		if(!findit)
			switchLoopLevelToSLP(opr_bundle);
		else if(!OrigLoop->contains(cast<Instruction>(opr_bundle[0]))){
			for(unsigned jj = 0; jj < bundle_size; jj++){
				Value * op_t = cast<StoreInst>(slp_bundle[jj])->getValueOperand();
				SLPScaMap.splat(op_t, op_t);
			}
		}			
	  
	    for(unsigned ii = 0; ii < bundle_size; ii++){
		    if(!SLPScaMap.has(cast<StoreInst>(slp_bundle[ii])->getValueOperand()))
			   assert(false && "missing operand value in SLPScaMap?!\n");
		    if(SLPScaMap.has(slp_bundle[ii]))
			   assert(false && "the slp value already been processed?!\n");
	    }		  
        vectorizeSLPMemAccPack(slp_bundle);
		break;
    }
	
    default:
        assert(false && "could not be this type of inst!"); 
		break;

    }// end of switch.
}// end of for_each instr.



void InnerLoopVectorizer::vectorizeSLPMemAccPack(ValuePack bundle){

    Instruction * Instr = cast<Instruction>(bundle[0]);
	LoadInst *LdI = dyn_cast<LoadInst>(Instr);
	StoreInst *SI = dyn_cast<StoreInst>(Instr);
	assert((LdI || SI) && "Invalid Load/Store instruction");

	Value *Ptr = LdI ? LdI->getPointerOperand() : SI->getPointerOperand();
	Type *ScalarDataTy = LdI ? LdI->getType() : SI->getValueOperand()->getType();
    unsigned Alignment = LdI ? LdI->getAlignment() : SI->getAlignment();
    if (!Alignment)
       Alignment = DL->getABITypeAlignment(ScalarDataTy);	

	bool isPredicateStore = false;
	if (SI && Legal->blockNeedsPredication(SI->getParent()))
	  isPredicateStore = true;

////now get the first load/store in the bundle, get the ptr of the instruction across iterations
////generate corresponding load/store, record the location of the load/store	

	////get the ptr operand 
    setDebugLocFromInst(Builder, Instr);
	Instruction *SrcInst = dyn_cast<Instruction>(Ptr);
	if (SrcInst && OrigLoop->contains(SrcInst)) {
		assert(WidenMap.has(SrcInst) && "Source operand is unavailable!");
	} 
	
	VectorParts & PtrPara= getVectorValue(Ptr);;
	if ((Ptr != OldInduction) && (!SrcInst || !OrigLoop->contains(SrcInst))){
		PtrPara = WidenMap.splat(Ptr, Ptr);
	}

	Instruction *InsertPt = Builder.GetInsertPoint();
	BasicBlock  *IfBlock = Builder.GetInsertBlock();
	BasicBlock  *CondBlock = nullptr;
	VectorParts Cond;
	Loop *VectorLp = nullptr;
	if (isPredicateStore) {
	  assert(Instr->getParent()->getSinglePredecessor() &&
			 "Only support single predecessor blocks");
	  Cond = createEdgeMask(Instr->getParent()->getSinglePredecessor(),
							Instr->getParent());
	  VectorLp = LI->getLoopFor(IfBlock);
	  assert(VectorLp && "Must have a loop for this block");
	}

	ValuePack insertLoc;
	VectorParts & slpsca = SLPScaMap.get(bundle[0]);		
	for (unsigned Part = 0; Part < UF; ++Part) {
	 for (unsigned Width = 0; Width < VF; ++Width) {
	  // For each scalar that we create:	
	  // Start an "if (pred) a[i] = ..." block.
	  Value *Cmp = nullptr;
	  if (isPredicateStore) {
        Cmp = Builder.CreateExtractElement(Cond[Part], Builder.getInt32(Width));
		Cmp = Builder.CreateICmp(ICmpInst::ICMP_EQ, Cmp, ConstantInt::get(Cmp->getType(), 1));
		CondBlock = IfBlock->splitBasicBlock(InsertPt, "cond.store");
		LoopVectorBody.push_back(CondBlock);
		VectorLp->addBasicBlockToLoop(CondBlock, LI->getBase());
		Builder.SetInsertPoint(InsertPt);
	  }

	  Instruction *Cloned = Instr->clone();
	  Value *ptr_opr = PtrPara[Part];
	  if (ptr_opr->getType()->isVectorTy())
          ptr_opr = Builder.CreateExtractElement(ptr_opr, Builder.getInt32(Width));
	  if(SI){
	        VectorParts &val_opr = SLPScaMap.get(SI->getValueOperand());
			Cloned->setOperand(0, val_opr[Part*VF+Width]);			
			Cloned->setOperand(1, ptr_opr);
	  }
	  if(LdI){
			Cloned->setOperand(0, ptr_opr);
	  }
      Builder.Insert(Cloned);
	  slpsca[Part*VF+Width] = Cloned;	  
	  insertLoc.push_back(Cloned);
	  		
	  // End if-block.
	  if (isPredicateStore) {
		BasicBlock *NewIfBlock = CondBlock->splitBasicBlock(InsertPt, "else");
		LoopVectorBody.push_back(NewIfBlock);
		VectorLp->addBasicBlockToLoop(NewIfBlock, LI->getBase());
		Builder.SetInsertPoint(InsertPt);
		Instruction *OldBr = IfBlock->getTerminator();
		BranchInst::Create(CondBlock, NewIfBlock, Cmp, OldBr);
		OldBr->eraseFromParent();
		IfBlock = NewIfBlock;
	  }	  
	 }
	}
	
////based on the ptr of the first load/store in the bundle, 
////add offset to it and get the ptr for the following consecutive load/store
////generate load/store for the following load/store
	unsigned bundle_sz = bundle.size();
	ValuePack base_ptr;
	for (unsigned Part = 0; Part < UF; ++Part) {
	 for (unsigned Width = 0; Width < VF; ++Width) {
	 	Value* mem_tmp = slpsca[Part*VF+Width];
		Value* ptr_tmp = LdI ? cast<LoadInst>(mem_tmp)->getPointerOperand() : cast<StoreInst>(mem_tmp)->getPointerOperand();
		base_ptr.push_back(ptr_tmp);
	 }
	}	

	for(unsigned ii = 1; ii < bundle_sz; ii++){
		
	 VectorParts & slpsca_c = SLPScaMap.get(bundle[ii]);
	 
	 for(unsigned jj = 0; jj < VF*UF;  jj++){
		
		Builder.SetInsertPoint(cast<Instruction>(insertLoc[jj]));
		////get the corresponding by add offset jj to the base ptr		
		Value * mem_ptr = Builder.CreateConstGEP1_32(base_ptr[jj], ii);
		if(dyn_cast<GetElementPtrInst>(mem_ptr)){
			Instruction * gep_ptr = cast<GetElementPtrInst>(mem_ptr);
			gep_ptr->removeFromParent();
			gep_ptr->insertAfter(cast<Instruction>(insertLoc[jj]));
		}
		////generate the ld/st instructions, insert it at the right place			
		if(SI){
			VectorParts &val_opr = SLPScaMap.get(cast<StoreInst>(bundle[ii])->getValueOperand());
			StoreInst *NewSI = Builder.CreateAlignedStore(val_opr[jj], mem_ptr, Alignment);
			if(dyn_cast<GetElementPtrInst>(mem_ptr)){
				Instruction * gep_ptr = cast<GetElementPtrInst>(mem_ptr);
				NewSI->removeFromParent();
				NewSI->insertAfter(gep_ptr);
			}else if(dyn_cast<Constant>(mem_ptr)){
				NewSI->removeFromParent();
				NewSI->insertAfter(cast<Instruction>(insertLoc[jj]));                
			}			
			insertLoc[jj] = NewSI;
			slpsca_c[jj] = NewSI;
		}
		if(LdI){
			LoadInst *NewLI = Builder.CreateAlignedLoad(mem_ptr, Alignment);
			if(dyn_cast<GetElementPtrInst>(mem_ptr)){
				Instruction * gep_ptr = cast<GetElementPtrInst>(mem_ptr);
				NewLI->removeFromParent();
				NewLI->insertAfter(gep_ptr);
			}else if(dyn_cast<Constant>(mem_ptr)){
				NewLI->removeFromParent();
				NewLI->insertAfter(cast<Instruction>(insertLoc[jj]));                
			}					
			insertLoc[jj] = NewLI;
			slpsca_c[jj] = NewLI;
		}			
	 }	 
	}
	///adjust the insert loc, in order to make sure all the users would be inserted after the defination 
	Builder.SetInsertPoint(InsertPt);	
	return;

}



void InnerLoopVectorizer::switchLoopLevelToSLP(ValuePack bundle){

	unsigned bundle_sz = bundle.size();
	for(unsigned ii = 0; ii < bundle_sz; ii++){

		if(SLPScaMap.has(bundle[ii]))
			continue;

		if(WidenMap.has(bundle[ii])){
			VectorParts & loopvec = WidenMap.get(bundle[ii]);
            VectorParts & loop2slpsca = SLPScaMap.get(bundle[ii]);		
		    
			// Use this vector value for all users of the original instruction.
			for (unsigned Part = 0; Part < UF; ++Part) {
			  Value *V = loopvec[Part];
              if(VF > 1){
					for (unsigned jj = 0; jj < VF; ++jj) {
						 Value * eleV = Builder.CreateExtractElement(V, Builder.getInt32(jj));
						 loop2slpsca[Part*VF+jj] = eleV;
					}
			  }
			  else{
				  loop2slpsca[Part] = V;
			  }
			}
			assert((loop2slpsca.size()==UF*VF) && "SLPScaMap has wrong size of SlpVar! \n");
			continue;
		}

		if(!dyn_cast<Instruction>(bundle[ii]) || !OrigLoop->contains(cast<Instruction>(bundle[ii]))){
			SLPScaMap.splat(bundle[ii],bundle[ii]);
			continue;
		}

		assert(false && "unhandled case ! \n");

	}

    return;

}



void InnerLoopVectorizer::switchSLPToLoopLevel(Value* sca){

     VectorParts & slpsca2loopvec = WidenMap.get(sca);
     VectorParts & slpscas = SLPScaMap.get(sca);

	 assert(!(sca->getType()->isVoidTy()) && "void type value being used !");
	 Value *UndefVec = UndefValue::get(VectorType::get(sca->getType(), VF));
			
	 // Use this vector value for all users of the original instruction.
	 for (unsigned Part = 0; Part < UF; ++Part) {
	 	if(VF > 1){
			Value * V = UndefVec;
			for (unsigned jj = 0; jj < VF; ++jj) {
				V = Builder.CreateInsertElement(V, slpscas[Part*VF+jj], Builder.getInt32(jj));
			}
			slpsca2loopvec[Part] = V;			
		}
		else{
			slpsca2loopvec[Part] = slpscas[Part];
		}
	 }
	
     return;

}


Value* InnerLoopVectorizer::optimizeShuffles(Value * shuf){
//////this function specially optimize shuffles generated from wider vector loads that cover loads from consecutive iterations 	
	ShuffleVectorInst * shuf_inst = dyn_cast<ShuffleVectorInst>(shuf);
	if(!shuf_inst)
		return shuf;

    Value * opr_a, * opr_b;
	opr_a = shuf_inst->getOperand(0);
	opr_b = shuf_inst->getOperand(1);	
	ShuffleVectorInst * shuf_opr_a = dyn_cast<ShuffleVectorInst>(opr_a);
	ShuffleVectorInst * shuf_opr_b = dyn_cast<ShuffleVectorInst>(opr_b);
    if(!shuf_opr_a || !shuf_opr_b)
		return shuf;

	Type * shuf_Ty = shuf->getType();
	unsigned mask_sz = cast<VectorType>(shuf_Ty)->getNumElements();
	for(unsigned ii = 0; ii < mask_sz; ++ii){
		if(shuf_inst->getMaskValue(ii) != ii)
			return shuf;
	}
	
	Value * opr_a_a, * opr_b_b;
	opr_a_a = shuf_opr_a->getOperand(0);
	if(opr_a_a != shuf_opr_a->getOperand(1))
		return shuf;
	opr_b_b = shuf_opr_b->getOperand(0);
	if(opr_b_b != shuf_opr_b->getOperand(1))
		return shuf;
	if(opr_a_a->getType() != opr_b_b->getType())
		return shuf;

    Instruction * ori_insert_pos = Builder.GetInsertPoint();
	Builder.SetInsertPoint(shuf_inst);
	Value * shuf_new;

		Type * vec_pre_Ty = opr_a_a->getType(); ////we know these two loads have the same type
		unsigned vec_pre_sz = cast<VectorType>(vec_pre_Ty)->getNumElements();
		SmallVector<Constant*, 8> ShuffleMask;
		for (unsigned ii = 0; ii < mask_sz/2; ++ii)
			 ShuffleMask.push_back(Builder.getInt32((shuf_opr_a->getMaskValue(ii))%vec_pre_sz));	
		for (unsigned ii = 0; ii < mask_sz/2; ++ii)
			 ShuffleMask.push_back(Builder.getInt32((shuf_opr_b->getMaskValue(ii))%vec_pre_sz+vec_pre_sz));			
	    shuf_new = Builder.CreateShuffleVector(opr_a_a, opr_b_b, ConstantVector::get(ShuffleMask));
	
	Builder.SetInsertPoint(ori_insert_pos);
    return shuf_new;
}


void InnerLoopVectorizer::generateShuffleForVecLoads (unsigned subgroup_sz, ValuePack & vecLoads){

    assert(llvm::APInt(32, UF).isPowerOf2() && "unexpected unroll factor!!!\n");

    Value * vector_t = vecLoads[0];
	Type * vec_ty = vector_t->getType();
	assert(dyn_cast<VectorType>(vec_ty) && "vector load is not of vector type !!!\n");
	Type * sca_ty = cast<VectorType>(vec_ty)->getElementType();
	unsigned vec_ele_num = cast<VectorType>(vec_ty)->getNumElements();
	assert((vec_ele_num == subgroup_sz) && "we have enforced this constraint! \n");
	
	////max_vec_width is the widest vector length we could use to shuffle elements
    unsigned max_vec_ele_num = Legal->getNumofElementInVector(sca_ty);
    max_vec_ele_num = (max_vec_ele_num <= VF*UF) ?  max_vec_ele_num : VF*UF;
	unsigned max_vec_width = max_vec_ele_num * DL->getTypeSizeInBits(sca_ty);	

	////here we could combine the vector loads if possible (i.e. VF*UF > subgroup_sz && max_vec_ele_num > subgroup_sz)
	////therefore to apply wider shuffle
	if(subgroup_sz < max_vec_ele_num){ 
		ValuePack vecLoads_t;
		Value * vec_load_t; 	
		unsigned subgroup_sz_t = subgroup_sz;
		while(subgroup_sz_t != max_vec_ele_num){ 
			
		  SmallVector<Constant*, 8> ShuffleMask;
		  for (unsigned ii = 0; ii < subgroup_sz_t*2; ++ii)
			  ShuffleMask.push_back(Builder.getInt32(ii));
		  
		  vecLoads_t.clear();
		  unsigned vec_shuffle_num = vecLoads.size()/2; 
		  unsigned ii = 0, jj = 0;
          while(vec_shuffle_num){
		  	  if(ii == subgroup_sz){
			  	jj += subgroup_sz;
                ii = 0;
			  }
			  vec_load_t = Builder.CreateShuffleVector(vecLoads[jj], vecLoads[jj+subgroup_sz], ConstantVector::get(ShuffleMask));
			  vecLoads_t.push_back(vec_load_t);
              ii++;
			  jj++;
			  vec_shuffle_num--;
		  }
		  subgroup_sz_t = subgroup_sz_t << 1;

		  vecLoads.clear();					
		  for(unsigned ii = 0; ii < vecLoads_t.size(); ii++){
			  vecLoads.push_back(vecLoads_t[ii]);
		  } 					  
		}
	    //////optimize the shuffled value here
		///////////////////////////////////////
		for(unsigned ii = 0; ii < vecLoads.size(); ii++){
		   Value * shuffle_opt = optimizeShuffles(vecLoads[ii]);
		   vecLoads[ii] = shuffle_opt;
		} 		
	}

	unsigned vecLoads_sz = vecLoads.size();
	if(vecLoads_sz < 2){
		assert(false && "vector combination leads to only one vector !\n");
	}

    ////////////apply shuffle	
	unsigned shuf_interval = (subgroup_sz < vecLoads_sz)? (subgroup_sz >> 1):(vecLoads_sz >> 1);
	unsigned shufmask_sz = cast<VectorType>(vecLoads[0]->getType())->getNumElements();
	unsigned shuf_dis = shuf_interval;

	while(shuf_interval){
		
		SmallVector<unsigned, 8> index;
		bool skip = false;
        for(unsigned ii = 0, pp = 0, jj = shuf_interval; ii < shufmask_sz; ii++, pp++, jj--){
			unsigned idx;
			if(jj == 0){
					skip = !skip;
					jj = shuf_interval;
			}
			idx = skip ? (index[ii-shuf_interval]+shufmask_sz) : ii;
            index.push_back(idx);
		}
		
		SmallVector<Constant*, 8> ShuffleMask1, ShuffleMask2;
        for(unsigned ii = 0; ii < shufmask_sz; ii++){
            unsigned jj = index[ii];	
			ShuffleMask1.push_back(Builder.getInt32(jj));
			ShuffleMask2.push_back(Builder.getInt32(jj+shuf_interval));
		}

	    ValuePack vecShuf;
		Value * vec_shuf;
		unsigned shuf_num = shuf_dis, ttt = 0;
		while(ttt*2*shuf_num < vecLoads.size()){
			unsigned pp = ttt*2*shuf_num, qq = ttt*2*shuf_num+shuf_dis; 
			while(pp!=qq){
				vec_shuf = Builder.CreateShuffleVector(vecLoads[pp], vecLoads[pp+shuf_dis], ConstantVector::get(ShuffleMask1));
				vecShuf.push_back(vec_shuf);
				vec_shuf = Builder.CreateShuffleVector(vecLoads[pp], vecLoads[pp+shuf_dis], ConstantVector::get(ShuffleMask2));
				vecShuf.push_back(vec_shuf);
				pp++;
			}	
			ttt++;
		}
	
		shuf_interval = shuf_interval >> 1;

		vecLoads.clear();
		unsigned vec_shuf_num = vecShuf.size(); 					
		for(unsigned ii = 0; ii < vec_shuf_num; ii++){
			vecLoads.push_back(vecShuf[ii]);
		}
	}

    ////////////extract the shuffled vector if its length is greater than VF
    max_vec_ele_num = cast<VectorType>(vecLoads[0]->getType())->getNumElements();
	if(max_vec_ele_num > VF){
		ValuePack vecExts;
		Value * vec_ext;  
		unsigned ext_times = max_vec_ele_num/VF; 						
		for(unsigned jj = 0 ; jj < ext_times; jj++){					  
		  SmallVector<Constant*, 8> ShuffleMask;
		  for (unsigned ii = jj*VF; ii < (jj+1)*VF; ++ii)
			  ShuffleMask.push_back(Builder.getInt32(ii));

		  unsigned vec_loads_num = vecLoads.size(); 								  
		  for(unsigned ii = 0; ii < vec_loads_num; ii++){
			  vec_ext = Builder.CreateShuffleVector(vecLoads[ii], vecLoads[ii], ConstantVector::get(ShuffleMask));
			  vecExts.push_back(vec_ext);
		  } 	  
		}
		
		vecLoads.clear();
		unsigned vec_ext_num = vecExts.size();						
		for(unsigned ii = 0; ii < vec_ext_num; ii++){
			vecLoads.push_back(vecExts[ii]);
		}
	}

	if(vecLoads.size() != UF*subgroup_sz){  		
  	   assert(false && "the optimized shuffle is incorrect!\n");
	}
	
}


void InnerLoopVectorizer::generateShuffleForVecStores (ValuePack vecStores){

    assert(llvm::APInt(32, UF).isPowerOf2() && "unexpected unroll factor!!!\n");
    Value * store_v = cast<StoreInst>(vecStores[0])->getValueOperand();
	VectorParts & store_v_vec = getVectorValue(store_v);
	Type * vec_ty = store_v_vec[0]->getType();
	assert(dyn_cast<VectorType>(vec_ty) && "store_v_vec is not of vector type !!!\n");
	Type * sca_ty = cast<VectorType>(vec_ty)->getElementType();
	unsigned vec_ele_num = cast<VectorType>(vec_ty)->getNumElements();
	assert((vec_ele_num == VF) && "we have enforced this constraint!!! \n");
	////max_vec_width is the widest vector length we could use to shuffle elements
    unsigned bundle_sz = vecStores.size();
	unsigned bundle_sz_ori = bundle_sz;

	while(bundle_sz){
		unsigned max_vec_ele_num = Legal->getNumofElementInVector(sca_ty);
		max_vec_ele_num = (max_vec_ele_num <= VF*UF) ?	max_vec_ele_num : VF*UF;
		unsigned max_vec_width = max_vec_ele_num * DL->getTypeSizeInBits(sca_ty);	
			
		unsigned sub_bundle_sz = max_vec_ele_num;
		while(sub_bundle_sz > bundle_sz)
			sub_bundle_sz = sub_bundle_sz >> 1;

		ValuePack store_v_vecs;
		for(unsigned ii = 0; ii < UF; ii++){
			for(unsigned jj = 0; jj < sub_bundle_sz; jj++){
				store_v = cast<StoreInst>(vecStores[bundle_sz_ori-bundle_sz+jj])->getValueOperand();
				VectorParts & store_v_vec = getVectorValue(store_v);
				store_v_vecs.push_back(store_v_vec[ii]);
			}
		}
		unsigned store_v_vecs_sz = store_v_vecs.size();


		if(getSameBlock(SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz]))){
			ValuePack store_scas0;
			////before we generate stores, all the st_value has been generated (no matter generated vectors or extracted scalars)
			for(unsigned jj = 0; jj < sub_bundle_sz; jj++){
				VectorParts & sca_sts = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz+jj]);
				for(unsigned kk = 0; kk < sca_sts.size(); kk++){
					Value * sca_st_v = sca_sts[kk];
					store_scas0.push_back(sca_st_v);
				}
			}	
		    Legal->refreshBlockNumbering(cast<Instruction>(store_scas0[0]));			
			Instruction * insertpos_new = Legal->getLastInstruction(store_scas0);
			insertpos_new = Legal->getNextInstruction(insertpos_new);
			Builder.SetInsertPoint(insertpos_new);
		}else{
		    /////these stores are predicated stores
		    /////we insert the shuflle at the end of the basic block, in which the masks are generated
			VectorParts & sca_sts = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz]);
			BasicBlock* bb_sca_st0 = cast<Instruction>(sca_sts[0])->getParent();
			BasicBlock* bb_insert_into = bb_sca_st0->getSinglePredecessor();
			assert(bb_insert_into && "can not get the basic block contains the store masks ! \n");
			Instruction * insertpos_new = bb_insert_into->getTerminator();
			assert(insertpos_new && "can not get the Terminator of the basic block contains the store masks ! \n");
			Builder.SetInsertPoint(insertpos_new);
		}
			

		if(sub_bundle_sz > 1){			
            //the vector length for loop-level and slp-level may be different
			//here we could combine the vectors if possible, therefore to apply wider shuffle
			////after combining different vectors, each vector still only contains instantced of the same variable
			if(VF < max_vec_ele_num){ 
				ValuePack vec_stores_v_t;
				Value * vec_store_v_t; 	
				unsigned VF_store_v = VF;

				while(VF_store_v < max_vec_ele_num){

					SmallVector<Constant*, 8> ShuffleMask;
					for (unsigned ii = 0; ii < VF_store_v*2; ++ii)
						ShuffleMask.push_back(Builder.getInt32(ii));
					
					vec_stores_v_t.clear();
					for(unsigned iii = 0, jjj = 0; iii < store_v_vecs.size(); iii++){
						vec_store_v_t = Builder.CreateShuffleVector(store_v_vecs[iii], store_v_vecs[iii+sub_bundle_sz], ConstantVector::get(ShuffleMask));
						vec_stores_v_t.push_back(vec_store_v_t);	
						jjj++;
						if(jjj == sub_bundle_sz){
							jjj = 0;
	                        iii = iii + sub_bundle_sz;
						}
					}

					store_v_vecs.clear();				  
					for(unsigned ii = 0; ii < vec_stores_v_t.size(); ii++){
						store_v_vecs.push_back(vec_stores_v_t[ii]);
					}	  

	                VF_store_v = VF_store_v << 1;
				}		
			}
			store_v_vecs_sz = store_v_vecs.size();
	        assert((store_v_vecs_sz%sub_bundle_sz == 0) && "incorrect size of store_v_vecs ! \n");
			assert((DL->getTypeSizeInBits(store_v_vecs[0]->getType()) == max_vec_width) && "wrong vec width after combining ! \n");

			////////////apply shuffle
			unsigned shuf_interval = sub_bundle_sz >> 1;
			unsigned shufmask_sz = max_vec_ele_num;	
			while(shuf_interval){
				
				SmallVector<unsigned, 8> index;
				bool skip = false;
				for(unsigned ii = 0, pp = 0, jj = shuf_interval; ii < shufmask_sz; ii++, pp++, jj--){
					unsigned idx;
			
					if(jj == 0){
							skip = !skip;
							jj = shuf_interval;
					}
					idx = skip ? (index[ii-shuf_interval]+shufmask_sz) : ii;
					
					index.push_back(idx);
				}

				SmallVector<Constant*, 8> ShuffleMask1, ShuffleMask2;
				for(unsigned ii = 0; ii < shufmask_sz; ii++){
					unsigned jj = index[ii];
					ShuffleMask1.push_back(Builder.getInt32(jj));
					ShuffleMask2.push_back(Builder.getInt32(jj+shuf_interval));
				}
						
				ValuePack vecShuf;
				Value * vec_shuf;	
	            store_v_vecs_sz = store_v_vecs.size();
				unsigned ttt = 0; 			
				while(ttt*sub_bundle_sz < store_v_vecs_sz){
					unsigned shuf_num = sub_bundle_sz >> 1;	
					unsigned shuf_dis = shuf_num;
					unsigned pp = ttt*sub_bundle_sz, qq = ttt*sub_bundle_sz+shuf_dis; 
					while(shuf_num){
						vec_shuf = Builder.CreateShuffleVector(store_v_vecs[pp], store_v_vecs[pp+shuf_dis], ConstantVector::get(ShuffleMask1));
						vecShuf.push_back(vec_shuf);
						vec_shuf = Builder.CreateShuffleVector(store_v_vecs[pp], store_v_vecs[pp+shuf_dis], ConstantVector::get(ShuffleMask2));
						vecShuf.push_back(vec_shuf);
						pp++;
						shuf_num--;
					}	
					ttt++;
				}
							
				shuf_interval = shuf_interval >> 1;
			
				store_v_vecs.clear();
				unsigned vec_shuf_num = vecShuf.size(); 					
				for(unsigned ii = 0; ii < vec_shuf_num; ii++){
					store_v_vecs.push_back(vecShuf[ii]);
				}
			}

			////////////extract the shuffled vector if its length is greater than sub_bundle_sz
			if(max_vec_ele_num > sub_bundle_sz){
				ValuePack vecExts;
				Value * vec_ext;  
		        store_v_vecs_sz = store_v_vecs.size();
				unsigned ttt = 0; 
				unsigned ext_times = max_vec_ele_num/sub_bundle_sz; 

				while(ttt*sub_bundle_sz < store_v_vecs_sz){
					for(unsigned jj = 0 ; jj < ext_times; jj++){					  
						  SmallVector<Constant*, 8> ShuffleMask;
						  for (unsigned ii = jj*sub_bundle_sz; ii < (jj+1)*sub_bundle_sz; ++ii)
							  ShuffleMask.push_back(Builder.getInt32(ii));														  
						  for(unsigned ii = ttt*sub_bundle_sz; ii < (ttt+1)*sub_bundle_sz; ii++){
							  vec_ext = Builder.CreateShuffleVector(store_v_vecs[ii], store_v_vecs[ii], ConstantVector::get(ShuffleMask));
							  vecExts.push_back(vec_ext);
						  } 	  
					}
	                ttt++;
				}		
			
				store_v_vecs.clear();
				unsigned vec_ext_num = vecExts.size();						
				for(unsigned ii = 0; ii < vec_ext_num; ii++){
					store_v_vecs.push_back(vecExts[ii]);
				}
			}		
			assert((store_v_vecs.size() == UF*VF) && "the optimized shuffle is not correct!\n");


		    //// check if we can can use wider vector store
		    //// generate the vector stores!
		    VectorParts & vp_a = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz]);
		    VectorParts & vp_b = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz+sub_bundle_sz-1]);
			bool acrossIteration = consecutiveMemAccs.count(vp_b[0]) && (consecutiveMemAccs[vp_b[0]] == vp_a[1]);
			bool acorssBB = (cast<Instruction>(vp_a[0])->getParent() != cast<Instruction>(vp_a[1])->getParent());
		    bool needWiderStore = acrossIteration && !acorssBB; 		
			if(needWiderStore){
				max_vec_ele_num = Legal->getNumofElementInVector(sca_ty);
				ValuePack vec_stores_v_t;
				Value * vec_store_v_t;	
				unsigned VF_store_v = sub_bundle_sz;
				while(VF_store_v < max_vec_ele_num){
				
					SmallVector<Constant*, 8> ShuffleMask;
					for (unsigned ii = 0; ii < VF_store_v*2; ++ii)
						ShuffleMask.push_back(Builder.getInt32(ii));
					
					vec_stores_v_t.clear();
					for(unsigned iii = 0; iii < store_v_vecs.size(); iii = iii+2){
						vec_store_v_t = Builder.CreateShuffleVector(store_v_vecs[iii], store_v_vecs[iii+1], ConstantVector::get(ShuffleMask));
						vec_stores_v_t.push_back(vec_store_v_t);
					}
				
					store_v_vecs.clear();				  
					for(unsigned ii = 0; ii < vec_stores_v_t.size(); ii++){
						store_v_vecs.push_back(vec_stores_v_t[ii]);
					}	  
				
					VF_store_v = VF_store_v << 1;
				}	

                ////optimized shuffled value here
				for(unsigned ii = 0; ii < store_v_vecs.size(); ii++){
				   Value * shuffle_opt = optimizeShuffles(store_v_vecs[ii]);
				   store_v_vecs[ii] = shuffle_opt;
				} 	                
				
				assert((store_v_vecs.size() == UF*VF*sub_bundle_sz/max_vec_ele_num) && "the optimized shuffle is not correct! ! !\n");
			}
			else{
				max_vec_ele_num = sub_bundle_sz;
			}

			ValuePack scaStores;
			for(unsigned ii = 0; ii < VF*UF ; ii++){
				for(unsigned jj = 0; jj < sub_bundle_sz; jj++){
					VectorParts & vp = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz+jj]);
					scaStores.push_back(vp[ii]);
				}
			}

            for(unsigned ii = 0, jj = scaStores.size(); ii < jj; ii = ii+max_vec_ele_num){
				ValuePack sub_scaStores;
				for(unsigned iii = 0; iii < max_vec_ele_num; iii++)
					sub_scaStores.push_back(scaStores[ii+iii]);

				unsigned bundle_id = SLPVecValues.size();
				assert((SLPScaValues.size() == bundle_id) && "SLPScaValues and SLPVecValues have different element total number !!!");					
				for (unsigned iii = 0; iii !=  max_vec_ele_num; ++iii) {
					assert(!SLPVarScatoVec.count(sub_scaStores[iii]) && "slp sca already vectorized ?!!");
					SLPVarScatoVec[sub_scaStores[iii]] = bundle_id;
				}
				SLPScaValues.push_back(sub_scaStores);
				SLPVecValues.push_back(UndefValue::get(sub_scaStores[0]->getType()));
				
				Value * sub_stores_vec_v = store_v_vecs[ii/max_vec_ele_num];
				vectorizeStoreSLPBundle(sub_scaStores, sub_stores_vec_v);
			}

		}
		else{
            VectorParts & scaStores = SLPScaMap.get(vecStores[bundle_sz_ori-bundle_sz]);			
            for(unsigned ii = 0; ii < UF; ii++){
				ValuePack sub_scaStores;
				for(unsigned iii = 0; iii < VF; iii++)
					sub_scaStores.push_back(scaStores[ii*VF+iii]);

				unsigned bundle_id = SLPVecValues.size();
				assert((SLPScaValues.size() == bundle_id) && "SLPScaValues and SLPVecValues have different element total number !!!");					
				for (unsigned iii = 0; iii != VF; ++iii) {
					assert(!SLPVarScatoVec.count(sub_scaStores[iii]) && "slp sca already vectorized ?!!");
					SLPVarScatoVec[sub_scaStores[iii]] = bundle_id;
				}
				SLPScaValues.push_back(sub_scaStores);
				SLPVecValues.push_back(UndefValue::get(sub_scaStores[0]->getType()));
				
				Value * sub_stores_vec_v = store_v_vecs[ii];
				vectorizeStoreSLPBundle(sub_scaStores, sub_stores_vec_v);
			}
		}

		bundle_sz = bundle_sz - sub_bundle_sz;
	}
	
}



void InnerLoopVectorizer::getConsecutivenessSLPMemAccPack(){

	MemAccessChainList& memChains = Legal->getConsecutiveStoreMemAccChainList();
	MemAccessChainList::iterator I, IE;	
 
	////collect the consecutiveness of the flattened stores/loads (for all the scalar value after vectorization)
	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I)

	{		
		ValuePack root = *I;		
		unsigned bundle_sz = root.size(); 

		assert(llvm::APInt(32, bundle_sz).isPowerOf2() && "unexpected bundle size !!! \n");
   		assert((Legal->isExistingSlpBundle(root) || Legal->isReversedExistingSlpBundle(root)) && "should be a slp var here!!!");
        assert(SLPScaMap.has(root[0]) && "should have the flattened sca values for this slp var!!!");

        ////check if the insts are in the same basic block
		bool accrossBasicBlock = false;
		VectorParts & vp_0 = SLPScaMap.get(root[0]);
		for(unsigned ii = 1; ii < vp_0.size(); ii++){
			if(cast<Instruction>(vp_0[ii-1])->getParent() != cast<Instruction>(vp_0[ii])->getParent()){			
				accrossBasicBlock = true;
				break;
			}
		}

        ////get the consecutiveness of the mem access
		int dis = Legal->getMemLocationDistanceAccrossInteration(root[0]);
		if((dis == bundle_sz) && (!accrossBasicBlock)){
			////could use purely consecutive vec stores, but need to get the correct ptr for vector stores
			ValuePack memlist;
			for(unsigned ii = 0; ii < VF*UF; ii++){
				for(unsigned jj = 0; jj < bundle_sz; jj++){
					VectorParts & vp = SLPScaMap.get(root[jj]);
					memlist.push_back(vp[ii]);
				}
			}
			assert((memlist.size() == bundle_sz*UF*VF) && "incorrect memlist size!!!");
			for(unsigned ii = 1, jj = memlist.size(); ii < jj; ii++){				
				assert(!consecutiveMemAccs.count(memlist[ii-1]) && "an existing value in the consecutive mem access map!");
				consecutiveMemAccs[memlist[ii-1]] = memlist[ii];
			}
		}
		else{
			////need to use extracts and shorter vector stores
			for(unsigned ii = 0; ii < VF*UF; ii++){
				ValuePack memlist;
				for(unsigned jj = 0; jj < bundle_sz; jj++){
					VectorParts & vp = SLPScaMap.get(root[jj]);
					memlist.push_back(vp[ii]);
				}
				assert((memlist.size() == bundle_sz) && "incorrect loadlist size!!!");
				for(unsigned jj = 1; jj < memlist.size(); jj++){
					assert(!consecutiveMemAccs.count(memlist[jj-1]) && "an existing value in the consecutive mem access map!!!");
					consecutiveMemAccs[memlist[jj-1]] = memlist[jj];
				}				
			}
		}		
	}

}



void InnerLoopVectorizer::vectorizeSLPBundles(){

	if(!Legal->HasSlpVars())
		return;
	
	getConsecutivenessSLPMemAccPack();

    Instruction *oriInsertPt = Builder.GetInsertPoint();	
	MemAccessChainList& memChains = Legal->getConsecutiveStoreMemAccChainList();
	MemAccessChainList::iterator I, IE;	
	////start from store bundles, do bottom-up vectorization	
	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I){
		
		ValuePack root = *I;	
		/////skip load bundle		
		if(dyn_cast<LoadInst>(root[0]))
			continue;
		
		unsigned bundle_sz = root.size();
		
        ValuePack store_value_bud;
		for(unsigned j = 0; j < bundle_sz; j++)
			store_value_bud.push_back(cast<StoreInst>(root[j])->getValueOperand());		
	    bool isAllSLPVar = true;

	    if(!Legal->isExistingSlpBundle(store_value_bud) && !Legal->isReversedExistingSlpBundle(store_value_bud)){
		   isAllSLPVar = false;
	    }

        if(!isAllSLPVar){
			////apply shuffle to the stored value for LoopVec
			generateShuffleForVecStores(root);			
		}
		else{
  		    Value * store_value =  store_value_bud[0];			
			Type * sca_ty = store_value->getType();
			unsigned max_vl = Legal->getNumofElementInVector(sca_ty);
			max_vl = (bundle_sz*VF*UF < max_vl) ? bundle_sz*VF*UF : max_vl;  ////because the VF might not be the largest VL available
			
			/////here, strip-mining the scalar stores
			/////to maximize consecutive store access and simplify shuffle operations 			
			SLPVarPackList	scaStoresList;
		    ValuePack scaStores;

            bool crossIteration = false;
			if((VF > 1) || (UF > 1)){
				VectorParts & vp_p_1 = SLPScaMap.get(root[bundle_sz-1]);
				Value * store_1 = vp_p_1[0];
				VectorParts & vp_p_2 = SLPScaMap.get(root[0]);
				Value * store_2 = vp_p_2[1];
				if(consecutiveMemAccs.count(store_1) && (consecutiveMemAccs[store_1] == store_2))
					crossIteration = true;	
			}				
		    ////if stores are consecutive across iterations, then store wont require extract
			////if the vector used is 128 bit, then load and shuffle can match better, if store are combined in this way
			if(crossIteration || (max_vl*DL->getTypeSizeInBits(sca_ty) != 256) || (VF*UF < max_vl)){
				for(unsigned ii = 0; ii < VF*UF ; ii++){
					for(unsigned jj = 0; jj < bundle_sz; jj++){
						VectorParts & vp = SLPScaMap.get(root[jj]);
						scaStores.push_back(vp[ii]);
						if(scaStores.size() == max_vl){
							scaStoresList.push_back(scaStores);
							scaStores.clear();
						}
					}
				}
			}
			////if the vector used is 256 bit, then load and shuffle can match better, if store are combined in this way
			else{
				for(unsigned pp = 0; pp < VF*UF ; pp = pp+max_vl){
					for(unsigned kk = 0; kk < bundle_sz; kk++){
						for(unsigned ii = 0; ii < max_vl; ii = ii+bundle_sz){
							for(unsigned jj = 0; jj < bundle_sz; jj++){									
								VectorParts & vp = SLPScaMap.get(root[jj]);
								scaStores.push_back(vp[ii+kk+pp]);									
								if(scaStores.size() == max_vl){
									scaStoresList.push_back(scaStores);
									scaStores.clear();
								}
							}
						}
					}
				}
			}
			
			////as sub_bundle_sz_p*VF*UF must can be divided by max_vl
			assert(scaStores.empty() && "store bundle is not proporly combined!\n");	

			//////vectorized the stores
			for(unsigned ii = 0, jj = scaStoresList.size(); ii < jj; ii++){
				
				ValuePack stores = scaStoresList[ii];
				unsigned g_sz = stores.size();
				
				unsigned bundle_id = SLPVecValues.size();
				assert((SLPScaValues.size() == bundle_id) && "SLPScaValues and SLPVecValues have different element total number !!!");					
				for (unsigned i = 0; i != g_sz; ++i) {
					assert(!SLPVarScatoVec.count(stores[i]) && "slp sca already vectorized ?!!");
					SLPVarScatoVec[stores[i]] = bundle_id;
				}
				SLPScaValues.push_back(stores);
				SLPVecValues.push_back(UndefValue::get(root[0]->getType()));
				
				ValuePack combined_store_v; 
				for(unsigned kk = 0; kk < g_sz; kk++){
					Value * store_v = cast<StoreInst>(stores[kk])->getValueOperand();
					combined_store_v.push_back(store_v);
				}	
				
				Type * sca_ty = combined_store_v[0]->getType();
				unsigned vec_sz = Legal->getBestVectorWidth(sca_ty, g_sz);
				Legal->refreshBlockNumbering(cast<StoreInst>(stores[0]));
				if(getSameBlock(combined_store_v)){
					Instruction * insertpos = Legal->getLastInstruction(combined_store_v);
					insertpos = Legal->getNextInstruction(insertpos);
					Builder.SetInsertPoint(insertpos);
				}else{
					Instruction * insertpos = Legal->getLastInstruction(stores);
					Builder.SetInsertPoint(insertpos);
				}				
				Value* vec_st_v = vectorizeSLPBundles(combined_store_v, vec_sz);
                  
				Legal->refreshBlockNumbering(cast<StoreInst>(stores[0]));
				Instruction * insertpos = Legal->getLastInstruction(stores);
				insertpos = Legal->getNextInstruction(insertpos);
				Builder.SetInsertPoint(insertpos);
				vectorizeStoreSLPBundle(stores, vec_st_v);							
				////handle external user
				HandleExternalUsers();
			}			
		}		
	}	

		////handle other loads
		////we know the loaded value will be reorgnized for some loop-level vectorized users, here we handle it specially
	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I){
				
				ValuePack root = *I;		
				/////skip store bundle		
				if(dyn_cast<StoreInst>(root[0]))
					continue;
		
				unsigned bundle_sz = root.size();						
				Type* sca_ty = cast<LoadInst>(root[0])->getType();
				ValuePack scaLoads, vecLoads;	 
				
				////some loads probably have already been vectorized with vec-load, here we can reuse them.
				////note that if the loads has been vectorized, then they must be conseucutive in the slp bundle, i.e. not random insertion

				///check if the loads have been vectorized by bottom-up vectorization
				bool done_vec = true;
				unsigned v_t_idx;
				for(unsigned kk = 0; kk < bundle_sz; kk++){
					VectorParts & vp_ld = SLPScaMap.get(root[kk]);
					if(SLPVarScatoVec.count(vp_ld[0])){
						if(!kk)
							 v_t_idx = SLPVarScatoVec[vp_ld[0]];
						else{
							if(v_t_idx != SLPVarScatoVec[vp_ld[0]]){
								done_vec = false;
								break;
							}
						}
					}else{
						done_vec = false;
						break;
					}
				}

		        ////the loads have been vectorized by bottom-up vectorization
				if(done_vec){
					//if the bundle is vectorized reversely, we need to reverse the bundle now
					ValuePack v_t_scas = SLPScaValues[v_t_idx];
					VectorParts & vp_ld = SLPScaMap.get(root[0]);
					if(consecutiveMemAccs.count(v_t_scas[1]) && (consecutiveMemAccs[v_t_scas[1]] = v_t_scas[0]) && (v_t_scas[bundle_sz-1] == vp_ld[0])){
						ValuePack root_tt;
						for(unsigned ii = 0; ii < bundle_sz; ii++)	
							root_tt.push_back(root[bundle_sz-1-ii]);
						root = root_tt;
					}
					
					scaLoads = root;	
					Value * v_t_vec = SLPVecValues[v_t_idx];
					VectorType * v_t_vec_Ty = cast<VectorType>(v_t_vec->getType());
					unsigned ele_num_in_vec = v_t_vec_Ty->getNumElements();
					vp_ld = SLPScaMap.get(root[0]);
				
					if(ele_num_in_vec > bundle_sz){
						assert((dyn_cast<ShuffleVectorInst>(v_t_vec)||dyn_cast<LoadInst>(v_t_vec)) && "should be a vector shuffle or wider load ! \n");
						assert((ele_num_in_vec%bundle_sz == 0) && "vector load width error !!!\n");

						unsigned ext_times = ele_num_in_vec/bundle_sz;		
						////figure out how the loads from different iterations are combined in the vector
						bool isInterleaveCombined = false;					
						if((DL->getTypeSizeInBits(v_t_vec_Ty) == 256) && (bundle_sz < UF*VF) && (v_t_scas[bundle_sz] == vp_ld[bundle_sz])){
							for(unsigned ii = 0; ii < ext_times; ii++){
								assert((v_t_scas[bundle_sz*ii] == vp_ld[bundle_sz*ii]) && "unexpected case of combined vec loads !!! should not happen! (A) \n");
							}
							isInterleaveCombined = true;
						}
						else{
							for(unsigned ii = 0; ii < ext_times; ii++){
								assert((v_t_scas[bundle_sz*ii] == vp_ld[ii]) && "unexpected case of combined vec loads !!! should not happen! (B) \n");
							}
						}
		
						////extract the vector loads out
						Instruction * insertpos_ori = Builder.GetInsertPoint(); 			
						if(isInterleaveCombined){
							for(unsigned ii = 0; ii < UF*VF/(ext_times*bundle_sz); ii++){

		                        ////check if the vector loads generated is in the right form
								ValuePack combined_vld;
								for(unsigned kk = 0; kk < bundle_sz; kk++){
									unsigned ld_idx = kk + ii*(ext_times*bundle_sz);
									unsigned v_t_idx = SLPVarScatoVec[vp_ld[ld_idx]];  
									Value * v_t = SLPVecValues[v_t_idx];
									unsigned v_t_idx_p = SLPVarScatoVec[vp_ld[ld_idx + bundle_sz*(ext_times-1)]]; 
									Value * v_t_p = SLPVecValues[v_t_idx_p];
									assert((v_t_p == v_t) && "vector load error!!!\n"); 			
									combined_vld.push_back(v_t);
								}
								
								Legal->refreshBlockNumbering(cast<Instruction>(combined_vld[0]));
								Instruction * insertpos_new = Legal->getLastInstruction(combined_vld);
								insertpos_new = Legal->getNextInstruction(insertpos_new);
								Builder.SetInsertPoint(insertpos_new);
		
								for(unsigned jj = 0; jj < ext_times; jj++){
									SmallVector<Constant*, 8> ShuffleMask;
									for (unsigned iii = jj*bundle_sz; iii < (jj+1)*bundle_sz; ++iii)
										ShuffleMask.push_back(Builder.getInt32(iii));
		
									for(unsigned kk = 0; kk < bundle_sz; kk++){
										Value * v_t = combined_vld[kk];
										Value * vec_ext = Builder.CreateShuffleVector(v_t, v_t, ConstantVector::get(ShuffleMask));
										vecLoads.push_back(vec_ext);
									}
								}
							}
						}
						else{
							for(unsigned ii = 0; ii < UF*VF/ext_times; ii++){
								unsigned v_t_idx = SLPVarScatoVec[vp_ld[ii*ext_times]];  
								Value * v_t = SLPVecValues[v_t_idx];
								unsigned v_t_idx_p = SLPVarScatoVec[vp_ld[ii*ext_times + ext_times - 1]]; 
								Value * v_t_p = SLPVecValues[v_t_idx_p];
								assert((v_t_p == v_t) && "vector load error!\n");
									
								/////if the loads originally from two consecutive iterations now are cimbined with one vector load
								/////in order to handle it simply afterward, we generate extraxt here
								Legal->refreshBlockNumbering(cast<Instruction>(v_t));
								Instruction * insertpos_new = Legal->getNextInstruction(cast<Instruction>(v_t));
								Builder.SetInsertPoint(insertpos_new);
											
								for(unsigned jj = 0 ; jj < ext_times; jj++){					  
									SmallVector<Constant*, 8> ShuffleMask;
									for (unsigned iii = jj*bundle_sz; iii < (jj+1)*bundle_sz; ++iii)
										ShuffleMask.push_back(Builder.getInt32(iii));
									Value * vec_ext = Builder.CreateShuffleVector(v_t, v_t, ConstantVector::get(ShuffleMask));
									vecLoads.push_back(vec_ext);
								}		
							}
						}
						Builder.SetInsertPoint(insertpos_ori);
		
					}
					else{
						if(!(ele_num_in_vec == bundle_sz) && dyn_cast<LoadInst>(v_t_vec)){
							assert(false && "unhandled case! \n");
						}
						for(unsigned ii = 0; ii < UF*VF; ii++){
							unsigned v_t_idx_t = SLPVarScatoVec[vp_ld[ii]];  
							Value * v_t_t = SLPVecValues[v_t_idx_t];
							vecLoads.push_back(v_t_t);
						}
					}		
				}
				////if the loads have not been vectorized, do it here
				else{			
					for(unsigned ii = 0; ii < bundle_sz; ii++){
						scaLoads.push_back(root[ii]);
					}	
		            ////determine the load vector width
					VectorParts & vp_a = SLPScaMap.get(root[0]);
					VectorParts & vp_b = SLPScaMap.get(root[bundle_sz-1]);
					bool needWiderLoad = consecutiveMemAccs.count(vp_b[0]) && (consecutiveMemAccs[vp_b[0]] == vp_a[1]);
					unsigned load_vec_ele_num = bundle_sz;
					if(needWiderLoad){
						load_vec_ele_num = Legal->getNumofElementInVector(sca_ty);
						if(load_vec_ele_num > VF*UF*bundle_sz)
							load_vec_ele_num = VF*UF*bundle_sz;
					}

					////get all the flattened scalat loads
					ValuePack scaLoads_p;
					for(unsigned ii = 0; ii < VF*UF ; ii++){
						for(unsigned jj = 0; jj < bundle_sz; jj++){
							VectorParts & vp = SLPScaMap.get(root[jj]);
							scaLoads_p.push_back(vp[ii]);
						}
					}
					////strip-mining and generate vector loads
					ValuePack vecLoads_p;
					Instruction * insertpos_ori_p = Builder.GetInsertPoint();			
					for(unsigned ii = 0, jj = scaLoads_p.size(); ii < jj; ii = ii + load_vec_ele_num){
						ValuePack sub_scaLoads;
						for(unsigned iii = 0; (iii < load_vec_ele_num) && (ii+iii < jj); iii++)
							sub_scaLoads.push_back(scaLoads_p[ii+iii]);
						assert((sub_scaLoads.size() == load_vec_ele_num) && "shuffle for loads may base on two vecter with different lengths ! \n");
		
						Legal->refreshBlockNumbering(cast<Instruction>(sub_scaLoads[0]));							
						Instruction * insertpos_new_p = Legal->getLastInstruction(sub_scaLoads);
						insertpos_new_p = Legal->getNextInstruction(insertpos_new_p);
						Builder.SetInsertPoint(insertpos_new_p);
						
						Value * v_ld_p = vectorizeLoadSLPBundle(sub_scaLoads, load_vec_ele_num);
						vecLoads_p.push_back(v_ld_p);							
					}
					Builder.SetInsertPoint(insertpos_ori_p);			
					
                    ////if we use wider vector loads, and the vector width is larger than bundle size
					if(needWiderLoad){
						Instruction * insertpos_ori_p = Builder.GetInsertPoint();
						Legal->refreshBlockNumbering(cast<Instruction>(vecLoads_p[0])); 							
						Instruction * insertpos_new_p = Legal->getLastInstruction(vecLoads_p);
						insertpos_new_p = Legal->getNextInstruction(insertpos_new_p);
						Builder.SetInsertPoint(insertpos_new_p);
						
						unsigned ext_times = load_vec_ele_num/bundle_sz;
						for(unsigned ii = 0 ; ii < vecLoads_p.size(); ii++){
							Value * v_t = vecLoads_p[ii];
							for(unsigned jj = 0 ; jj < ext_times; jj++){					  
							  SmallVector<Constant*, 8> ShuffleMask;
							  for (unsigned iii = jj*bundle_sz; iii < (jj+1)*bundle_sz; ++iii)
								  ShuffleMask.push_back(Builder.getInt32(iii));
							  Value * vec_ext = Builder.CreateShuffleVector(v_t, v_t, ConstantVector::get(ShuffleMask));
							  vecLoads.push_back(vec_ext);
							}
						}
						
						Builder.SetInsertPoint(insertpos_ori_p);
					}
					////otherwise, the vector load's width is equal with the bundle size
					else{
						for(unsigned ii = 0 ; ii < vecLoads_p.size(); ii++){
							Value * v_t = vecLoads_p[ii];
							vecLoads.push_back(v_t);
						}
					}
				}
				
				assert((vecLoads.size()==VF*UF) && "vecLoads has incorrect size after combining !!!\n");
				Legal->refreshBlockNumbering(cast<Instruction>(vecLoads[0]));
				Instruction * insert_pos = Legal->getLastInstruction(vecLoads); 	
				Builder.SetInsertPoint(Legal->getNextInstruction(insert_pos));		

                ////switch from slp to loop level for these loads
				generateShuffleForVecLoads(bundle_sz, vecLoads);
				assert((vecLoads.size() == bundle_sz*UF) && "shuffled value not correct!\n");

		        ////update all users with the generated shuffle insts
				for(unsigned ii = 0; ii < bundle_sz; ii++){
					if(WidenMap.has(scaLoads[ii])){
						VectorParts & ld_inserted = WidenMap.get(scaLoads[ii]);
						assert((ld_inserted.size() == UF) && "wrong size of the widened value!\n");
						for(unsigned jj = 0; jj < UF; jj++){
							cast<Instruction>(ld_inserted[jj])->replaceAllUsesWith(vecLoads[bundle_sz*jj+ii]);
						}
					}
		
				}
		
	}

	////remove scalar stores 
	ValuePack all_sca_stores;
	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I){
		ValuePack root = *I;		
		/////skip load bundle		
		if(dyn_cast<LoadInst>(root[0]))
			continue;
		unsigned bundle_sz = root.size();
		for(unsigned jj = 0; jj < bundle_sz; jj++){
			VectorParts & vp = SLPScaMap.get(root[jj]);
			for(unsigned ii = 0; ii < VF*UF ; ii++){
				assert(SLPVarScatoVec.count(vp[ii]) && "this store is not vectorized !!!");
				all_sca_stores.push_back(vp[ii]);
			}
		}    
	}	
	for(unsigned ii = 0, jj = all_sca_stores.size(); ii < jj; ii++){
		Value * sca_st = all_sca_stores[ii];
		cast<Instruction>(sca_st)->eraseFromParent();
	}

	Builder.SetInsertPoint(oriInsertPt);
	return;		

}


void InnerLoopVectorizer::HandleExternalUsers(){
	
	ExternalUserMap::iterator ext_u_i, ext_u_e;
	for(ext_u_i = externalUserMap.begin(), ext_u_e = externalUserMap.end(); ext_u_i != ext_u_e; ext_u_i++){
		Value * ext_u_def = ext_u_i->first;
		assert(SLPVarScatoVec.count(ext_u_def) && " def of the external user is not in any slp bundle !!!");
	
		unsigned bundle_id = SLPVarScatoVec[ext_u_def];
		ValuePack  bundle_scas = SLPScaValues[bundle_id];
		unsigned iii = 0, jjj = bundle_scas.size();
		for(; iii < jjj; iii++){
			if(bundle_scas[iii] == ext_u_def)
				break;
		}
		assert((iii < jjj) && "can not find the value in this slp bundle !!!");
		Value * bundle_vec = SLPVecValues[bundle_id];

		Builder.SetInsertPoint(Legal->getLastInstruction(bundle_scas));
		Value * def_ext = Builder.CreateExtractElement(bundle_vec, Builder.getInt32(iii));
		
		ValuePack ext_u_users = ext_u_i->second;						
		unsigned user_num = ext_u_users.size();
		for(unsigned ll = 0; ll < user_num; ll++){
			Value * user_t = ext_u_users[ll];			
			if (std::find(ext_u_def->user_begin(), ext_u_def->user_end(), user_t) == ext_u_def->user_end()){
				continue;
			}
	
			if (PHINode *PH = dyn_cast<PHINode>(user_t)) {
				for (int i = 0, e = PH->getNumIncomingValues(); i != e; ++i) {
				  if (PH->getIncomingValue(i) == ext_u_def) {
					PH->setOperand(i, def_ext);
				  }
				}
			} else {
				cast<User>(user_t)->replaceUsesOfWith(ext_u_def, def_ext);
			}
		}
	}

	externalUserMap.clear();
	return;
}


bool InnerLoopVectorizer::isSubBundle(ValuePack bd_a, ValuePack bd_b){

	std::set<Value*> bd_a_v_set;
	for(unsigned j = 0, m = bd_a.size(); j < m; j++)
		bd_a_v_set.insert(bd_a[j]);

	bool foundit = true;
	for(unsigned j = 0, m = bd_b.size(); j < m; j++){
		if(!bd_a_v_set.count(bd_b[j])){
			foundit = false;
			break;
		}
	}

	return foundit;
}

int InnerLoopVectorizer::isExistingPackedValue(ValuePack bundle){

    int bd_idx = -1;	
	for(unsigned i = 0, e = PackedValueVec.size(); i < e; i++){
		ValuePack bd_t = PackedValueSca[i];
		if(isSubBundle(bd_t, bundle)){
			bd_idx = i;
			break;
		}
	}

	return bd_idx;
}

bool InnerLoopVectorizer::isExactExistingPackedValue(ValuePack bundle, int bd_idx){

	ValuePack bd = PackedValueSca[bd_idx];
    unsigned bundle_sz = bundle.size();
	if(bd.size() != bundle_sz)
		return false;

    bool isExact = true;	
	for(unsigned i = 0; i < bundle_sz; i++){
		if(bundle[i] != bd[i]){
			isExact = false;
			break;
		}
	}

	return isExact;
}

Value* InnerLoopVectorizer::getShuffleMaskForBundles(ValuePack from, ValuePack to){

	SmallVector<unsigned, 8> ShuffleIdx;
    for(unsigned i = 0, e = to.size(); i < e; i++){
		for(unsigned j = 0, m = from.size(); j < m; j++){
			if(to[i] == from[j]){
				ShuffleIdx.push_back(j);
				break;
			}
		}
	}
	assert((ShuffleIdx.size() == to.size())  && "incorrect shuffle index for shuffleing packed value !!!\n");

	SmallVector<Constant*, 8> ShuffleMask;		
	for(unsigned i = 0, e = to.size(); i < e; ++i)
		ShuffleMask.push_back(Builder.getInt32(ShuffleIdx[i]));
	
    return ConstantVector::get(ShuffleMask);
}


Value * InnerLoopVectorizer::reusePackedValue(ValuePack root, Instruction * ori_insert_pos){

	////check if we could reuse the already packed value
	int exist_bd_idx = isExistingPackedValue(root);
	if(exist_bd_idx >= 0){
		Value * V = PackedValueVec[exist_bd_idx];
		Instruction * vec_t = dyn_cast<Instruction>(V);

		bool couldopt = true;
		if(vec_t){
		////check if need to relocate the vector inst
			  Instruction * new_insert_pos = Builder.GetInsertPoint();
			  if(new_insert_pos == ori_insert_pos){
				  ////insert position is not at the loop preheader
				  Legal->refreshBlockNumbering(vec_t);
				  ValuePack inst_tmp;
				  inst_tmp.push_back(ori_insert_pos);
				  inst_tmp.push_back(V);			  
				  if(!getSameBlock(inst_tmp) || (Legal->getLastInstruction(inst_tmp) == vec_t)){
					  couldopt = false;
				  }
			  }
		}
		else
			  couldopt = false;

		if(couldopt){
			 if(!isExactExistingPackedValue(root, exist_bd_idx)){				
				  Builder.SetInsertPoint(vec_t);
				  V = Builder.CreateShuffleVector(V, V, getShuffleMaskForBundles(PackedValueSca[exist_bd_idx], root));			
				  vec_t->moveBefore(cast<Instruction>(V));
				  Legal->refreshBlockNumbering(vec_t);
			 }

			 Builder.SetInsertPoint(ori_insert_pos);
			 return V;			 
		}	
		else
			 return NULL;
	}
	
    return NULL;
}



Value * InnerLoopVectorizer::dataPacking(ValuePack root, unsigned v_sz){

	unsigned bundle_sz = root.size(); 
	Value *V = UndefValue::get(VectorType::get(root[0]->getType(), v_sz));
  	Instruction * ori_insert_pos = Builder.GetInsertPoint();

	////determine the location to insert the vectorized code
	bool Invariant = true;
	for (unsigned Part = 0; Part < bundle_sz; ++Part){
		Instruction *Instr = dyn_cast<Instruction>(root[Part]);
		bool NewInstr =(Instr && std::find(LoopVectorBody.begin(), LoopVectorBody.end(),
								Instr->getParent()) != LoopVectorBody.end());
		Invariant = OrigLoop->isLoopInvariant(root[Part]) && !NewInstr;
		if(!Invariant)
			break;
	}
	if(Invariant){
		if(std::find(LoopVectorBody.begin(), LoopVectorBody.end(),
					ori_insert_pos->getParent()) != LoopVectorBody.end())
			Builder.SetInsertPoint(LoopVectorPreHeader->getTerminator());
	}

    
    ////check if we could reuse the already packed value
    V = reusePackedValue(root, ori_insert_pos);
	if(V)
		return V;

	V = UndefValue::get(VectorType::get(root[0]->getType(), v_sz));
	for (unsigned Part = 0; Part < bundle_sz; ++Part)
	    V = Builder.CreateInsertElement(V, root[Part], Builder.getInt32(Part));

	PackedValueSca.push_back(root);
	PackedValueVec.push_back(V);		

	Builder.SetInsertPoint(ori_insert_pos);
	return V;
}



Value * InnerLoopVectorizer::dataSplating(ValuePack root, unsigned v_sz){

	unsigned bundle_sz = root.size(); 
  	Instruction * ori_insert_pos = Builder.GetInsertPoint();
	
	bool Invariant = true;
	for (unsigned Part = 0; Part < bundle_sz; ++Part){
		Instruction *Instr = dyn_cast<Instruction>(root[Part]);
		bool NewInstr =(Instr && std::find(LoopVectorBody.begin(), LoopVectorBody.end(),
								Instr->getParent()) != LoopVectorBody.end());
		Invariant = OrigLoop->isLoopInvariant(root[Part]) && !NewInstr;
		if(!Invariant)
			break;
	}
	if(Invariant){
		if(std::find(LoopVectorBody.begin(), LoopVectorBody.end(),
					ori_insert_pos->getParent()) != LoopVectorBody.end())
			Builder.SetInsertPoint(LoopVectorPreHeader->getTerminator());
	}

    ////check if we could reuse the already packed value
    Value * V = reusePackedValue(root, ori_insert_pos);
	if(V)
		return V;	

	////if is pure identical splat, then its easy
	if(isSplat(root)){
		V = Builder.CreateVectorSplat(v_sz, root[0]);
		Builder.SetInsertPoint(ori_insert_pos); 
		PackedValueSca.push_back(root);
		PackedValueVec.push_back(V);		
		return V;	
	}

	///collect the times each value repeats
	ValuePack valuelist;
	std::set<Value*> valueset;
	for (unsigned Part = 0; Part < bundle_sz; ++Part){
		if(!valueset.count(root[Part])){
			valuelist.push_back(root[Part]);
			valueset.insert(root[Part]);
		}
	}

    ///if no value is repeated, then apply pure data packing
    if(valueset.size() == bundle_sz)
		return dataPacking(root, v_sz);	

    ///use simple shuffle for the following special case
    if(dyn_cast<ExtractElementInst>(valuelist[0]) && getSameOpcode(valuelist)){
		bool could_opt = true;
		Value * rsc = cast<ExtractElementInst>(valuelist[0])->getVectorOperand();
		for(unsigned Part = 1; Part < valuelist.size(); ++Part){
			Value * rsc_t = cast<ExtractElementInst>(valuelist[Part])->getVectorOperand();
			if(rsc_t != rsc){
				could_opt = false;
				break;
			}
		}
		if(could_opt){
            SmallVector<unsigned, 8> ShuffleIdx;
			for(unsigned Part = 0; Part < bundle_sz; ++Part){
				Value * idx = cast<ExtractElementInst>(root[Part])->getIndexOperand();
				unsigned idx_t = cast<ConstantInt>(idx)->getZExtValue();
				ShuffleIdx.push_back(idx_t);
			}
			
			SmallVector<Constant*, 8> ShuffleMask;		
			for(unsigned Part = 0; Part < bundle_sz; ++Part){
				ShuffleMask.push_back(Builder.getInt32(ShuffleIdx[Part]));
			}
		    V = Builder.CreateShuffleVector(rsc, rsc, ConstantVector::get(ShuffleMask));
			return V;
		}
	}

    ///or generate shuffle for the general case
	V = UndefValue::get(VectorType::get(root[0]->getType(), v_sz));
	for(unsigned Part = 0; Part < valuelist.size(); ++Part){
		Value * V_t = Builder.CreateVectorSplat(v_sz, valuelist[Part]);
		SmallVector<Constant*, 8> ShuffleMask;
		for (unsigned i = 0; i < v_sz; ++i){
			if((i < bundle_sz)&&(valuelist[Part] == root[i])){
				ShuffleMask.push_back(Builder.getInt32(i));
			}else{
				ShuffleMask.push_back(Builder.getInt32(i+v_sz));
			}
		}
		V = Builder.CreateShuffleVector(V_t, V, ConstantVector::get(ShuffleMask));
	}

	PackedValueSca.push_back(root);
	PackedValueVec.push_back(V);	
	Builder.SetInsertPoint(ori_insert_pos);	
	return V;	

}



bool InnerLoopVectorizer::isSlpBundleVectorized(ValuePack root){
	if(!SLPVarScatoVec.count(root[0]))
		return false;

	unsigned vec_id = SLPVarScatoVec[root[0]];
	ValuePack scalars = SLPScaValues[vec_id];

	unsigned bundle_sz = root.size();
	if(scalars.size() != bundle_sz)
		return false;
	for(unsigned i = 0; i < bundle_sz; i++){
		if(scalars[i] != root[i])
			return false;
	}

	return true;
}



Value * InnerLoopVectorizer::vectorizeSLPBundles(ValuePack root, unsigned v_sz){
	
		unsigned bundle_sz = root.size();
		assert((bundle_sz <= v_sz) && "bundle_sz is larger than the vector length!!!\n");
		
		////here, check for valid SLP variables based on the root, by doing bottom-up search
		bool SameTy = getSameType(root); 
		(void)SameTy;
		assert(SameTy && "Invalid types!");
	
		if (root[0]->getType()->isVectorTy())
			assert(false && "vectorize value with vector types!");		  	
	
		if (allConstant(root))
			return dataPacking(root, v_sz);
		if (isPartialSplat(root))
			return dataSplating(root,v_sz);//dataPacking(root, v_sz)
	    unsigned Opcode = getSameOpcode(root);	  
	    if (!getSameBlock(root) || !Opcode)
			return dataPacking(root, v_sz);
		if(dyn_cast<ExtractElementInst>(root[0])){
			if(isExtractedVectorReusable(root))
				return cast<ExtractElementInst>(root[0])->getVectorOperand();
			else
				return dataPacking(root, v_sz);
	    }

		if(isSlpBundleVectorized(root)){
		    unsigned vec_id = SLPVarScatoVec[root[0]];
			return SLPVecValues[vec_id];
		}

        ////generate vector based on existing vector
        ////in the following case, the values in root are completely different with each other
		if(SLPVarScatoVec.count(root[0])){
			unsigned vec_id = SLPVarScatoVec[root[0]];
			ValuePack root_s = SLPScaValues[vec_id];

			if(root_s.size() == bundle_sz){
				std::set<Value*> v_set; 
				for (unsigned i = 0; i < bundle_sz; ++i)
					v_set.insert(root_s[i]);

				bool couldOpt = true;
				for (unsigned i = 0; i < bundle_sz; ++i){
					if(!(v_set.count(root[i])))	{
						couldOpt = false;
						break;
					}
				}

				if(couldOpt){
					SmallVector<unsigned, 8> ShuffleIdx;
					for(unsigned i = 0; i < bundle_sz; ++i){
						for(unsigned j = 0; j < bundle_sz; ++j){
							if(root[i] == root_s[j]){
								ShuffleIdx.push_back(j);
								break;
							}
						}
					}
					assert((ShuffleIdx.size() == bundle_sz)  && "incorrect shuffle index for reverse vector!\n");

					SmallVector<Constant*, 8> ShuffleMask;		
					for(unsigned i = 0; i < bundle_sz; ++i)
						ShuffleMask.push_back(Builder.getInt32(ShuffleIdx[i]));

					Value* root_v = SLPVecValues[vec_id];
					Instruction * ori_insert_pos = Builder.GetInsertPoint();
					Instruction * root_v_i = cast<Instruction>(root_v);
					Builder.SetInsertPoint(root_v_i);
					Value * V = Builder.CreateShuffleVector(root_v, root_v, ConstantVector::get(ShuffleMask));
					root_v_i->moveBefore(cast<Instruction>(V));
					Builder.SetInsertPoint(ori_insert_pos);
					Legal->refreshBlockNumbering(root_v_i);
					return V;
				}
			}
		}

		for (unsigned i = 0; i != bundle_sz; ++i) {
			if(SLPVarScatoVec.count(root[i])){
				assert(false && "slp sca already vectorized ?!!");
			}
		}

		//if it is load bundle, then do datapacking if loads in bundle are not consecutive (or reversely consecutive)	
		bool isConsecutiveLoad = false;
		bool isReversedConsecutiveLoad = false;
		unsigned chain_len = 1;	
        if(Opcode == Instruction::Load){					
			if(consecutiveMemAccs.count(root[0]) && (consecutiveMemAccs[root[0]] == root[1])){							   
			   for (unsigned e = bundle_sz; chain_len < e; chain_len++) {
				   if (!(consecutiveMemAccs.count(root[chain_len-1]) && (consecutiveMemAccs[root[chain_len-1]] == root[chain_len])))
				   	break;		   
			   }
			   if(chain_len > 1){
			       assert(llvm::APInt(32, chain_len).isPowerOf2() && "unexpected consecutive load chain length !!!\n");
				   isConsecutiveLoad = true;
				   for (unsigned i = 0, e = bundle_sz; i < e; i = i+chain_len) {
					  for (unsigned j = 0; j < chain_len-1; j++){
					  	if (!(consecutiveMemAccs.count(root[i+j]) && (consecutiveMemAccs[root[i+j]] == root[i+j+1]))){
							isConsecutiveLoad = false;
							break;
						}
					  }
				      if(!isConsecutiveLoad)
						break;
				   }					   
			   }
			}	
			
			chain_len = 1;	
			if(consecutiveMemAccs.count(root[1]) && (consecutiveMemAccs[root[1]] == root[0])){							   
			   for (unsigned e = bundle_sz; chain_len < e; chain_len++) {
				   if (!(consecutiveMemAccs.count(root[chain_len]) && (consecutiveMemAccs[root[chain_len]] == root[chain_len-1])))
				   	break;		   
			   }
			   if(chain_len > 1){
			       assert(llvm::APInt(32, chain_len).isPowerOf2() && "unexpected consecutive load chain length !!!\n");
				   isReversedConsecutiveLoad = true;
				   for (unsigned i = 0, e = bundle_sz; i < e; i = i+chain_len) {
					  for (unsigned j = 1; j < chain_len; j++){
					  	if (!(consecutiveMemAccs.count(root[i+j]) && (consecutiveMemAccs[root[i+j]] == root[i+j-1]))){
							isReversedConsecutiveLoad = false;
							break;
						}
					  }
				      if(!isReversedConsecutiveLoad)
						break;
				   }					   
			   }
			}

			if(!isReversedConsecutiveLoad && !isConsecutiveLoad)
				return dataPacking(root, v_sz);
		}
		

		unsigned bundle_id = SLPVecValues.size();
		assert((SLPScaValues.size() == bundle_id) && "SLPScaValues and SLPVecValues have different element total number !!!");		
		for (unsigned i = 0; i != bundle_sz; ++i)
			SLPVarScatoVec[root[i]] = bundle_id;		
		SLPScaValues.push_back(root);
		SLPVecValues.push_back(UndefValue::get(root[0]->getType()));

		////collect external user info		
		Instruction *VL0 = cast<Instruction>(root[0]);
		Legal->refreshBlockNumbering(VL0);		
		BasicBlock *BB = cast<Instruction>(VL0)->getParent();
		int MyLastIndex = Legal->getLastIndex(root);

		for (unsigned i = 0; i != bundle_sz; ++i) {
			Instruction *Scalar = cast<Instruction>(root[i]);
			ValuePack externalUsers;
			for (User *U : Scalar->users()) {
			  Instruction *UI = dyn_cast<Instruction>(U);	
			  ////if user is not an instruction, then stop further searching. (when could this happen?)
			  if (!UI) {
				assert(false && "this user is not an instruction!!!");
			  }
		
			  // We don't care if the user is in a different basic block.
			  BasicBlock *UserBlock = UI->getParent();
			  if (UserBlock != BB) {
				externalUsers.push_back(UI);
				////need to resolve conflicts later for this case			
				continue;
			  }
		
			  // If this is a PHINode within this basic block then we can place the extract wherever we want.
			  if (isa<PHINode>(*UI)) {
				////need to resolve conflicts later for this case
				externalUsers.push_back(UI);
				continue;
			  }
		
			  // Check if this is a safe in-tree user.
			  if (SLPVarScatoVec.count(UI)) {
				unsigned id = SLPVarScatoVec[UI];
				ValuePack scalars_t = SLPScaValues[id];
				assert((scalars_t.size() == bundle_sz) && "slp user bundle size inconsistant !!!");
				unsigned lane;
				for(lane = 0; lane < bundle_sz; lane++)
					if(UI == scalars_t[lane])
						break;
				if(lane != i){
					externalUsers.push_back(UI);					
				}
				continue;
			  }

			  BlockNumbering &BN = Legal->getBlockNumbering(BB);
			  int UserIndex = BN.getIndex(UI);
			  assert((UserIndex > MyLastIndex) && "unschedualble external user!!!");
			  externalUsers.push_back(UI);
			}

			assert(!externalUserMap.count(root[i]) && " external user record already exsit!!!");
			externalUserMap[root[i]] = externalUsers;
			
		}	  


		Instruction * insert_loc = Legal->getLastInstruction(root);
		Builder.SetInsertPoint(insert_loc);
		Value * V = NULL;
		
		switch (Opcode) {
			case Instruction::ZExt:
			case Instruction::SExt:
			case Instruction::FPToUI:
			case Instruction::FPToSI:
			case Instruction::FPExt:
			case Instruction::PtrToInt:
			case Instruction::IntToPtr:
			case Instruction::SIToFP:
			case Instruction::UIToFP:
			case Instruction::Trunc:
			case Instruction::FPTrunc:
			case Instruction::BitCast: {			
			  Type *SrcTy = VL0->getOperand(0)->getType();
			  for (unsigned i = 0; i < root.size(); ++i) {
				Type *Ty = cast<Instruction>(root[i])->getOperand(0)->getType();
				if (Ty != SrcTy || Ty->isAggregateType() || Ty->isVectorTy())
				  assert(false && "can not do vectorization on this slp bundle!!! (Cast) ");
			  }
	  
		  	  ValuePack Operands;
			  for (unsigned j = 0; j < bundle_sz; ++j)
				  Operands.push_back(cast<Instruction>(root[j])->getOperand(0));
			  Builder.SetInsertPoint(insert_loc);
			  Value * V_t = vectorizeSLPBundles(Operands, v_sz); 
			  			  
			  Type *DestTy = VectorType::get(VL0->getType(), v_sz);
			  Builder.SetInsertPoint(insert_loc);
			  V = Builder.CreateCast(dyn_cast<CastInst>(VL0)->getOpcode(), V_t, DestTy);			  
			  break;			
			}
							
			case Instruction::Add:
			case Instruction::FAdd:
			case Instruction::Sub:
			case Instruction::FSub:
			case Instruction::Mul:
			case Instruction::FMul:
			case Instruction::UDiv:
			case Instruction::SDiv:
			case Instruction::FDiv:
			case Instruction::URem:
			case Instruction::SRem:
			case Instruction::FRem:
			case Instruction::Shl:
			case Instruction::LShr:
			case Instruction::AShr:
			case Instruction::And:
			case Instruction::Or:
			case Instruction::Xor: {
			  // Sort operands of the instructions so that each side is more likely to
			  // have the same opcode.
			  BinaryOperator *BinOp = dyn_cast<BinaryOperator>(VL0);
			  if (isa<BinaryOperator>(VL0) && VL0->isCommutative()) {
				ValuePack Left, Right;
				reorderInputsAccordingToOpcode(root, Left, Right);
				BasicBlock *LeftBB = getSameBlock(Left);
				BasicBlock *RightBB = getSameBlock(Right);
				// If we have common uses on separate paths in the tree make sure we
				// process the one with greater common depth first.
				// We can use block numbering to determine the subtree traversal as
				// earler user has to come in between the common use and the later user.
				Value *V1, *V2;
				if (LeftBB && RightBB && LeftBB == RightBB &&
					Legal->getLastIndex(Right) > Legal->getLastIndex(Left)) {
				  Builder.SetInsertPoint(insert_loc);
				  V2 = vectorizeSLPBundles(Right, v_sz);
				  Builder.SetInsertPoint(insert_loc);
				  V1 = vectorizeSLPBundles(Left, v_sz);
				} else {
				  Builder.SetInsertPoint(insert_loc);
				  V1 = vectorizeSLPBundles(Left, v_sz);	
				  Builder.SetInsertPoint(insert_loc);
				  V2 = vectorizeSLPBundles(Right, v_sz);
				}
				Builder.SetInsertPoint(insert_loc);
				V = Builder.CreateBinOp(BinOp->getOpcode(), V1, V2);
			  } 
			  else{
				ValuePack Left, Right;
				for (unsigned j = 0; j < bundle_sz; ++j){
					Left.push_back(cast<Instruction>(root[j])->getOperand(0));
					Right.push_back(cast<Instruction>(root[j])->getOperand(1));
				}
				Value *V1, *V2;
				Builder.SetInsertPoint(insert_loc);
				V1 = vectorizeSLPBundles(Left, v_sz);
				Builder.SetInsertPoint(insert_loc);
				V2 = vectorizeSLPBundles(Right, v_sz);
				Builder.SetInsertPoint(insert_loc);
				V = Builder.CreateBinOp(BinOp->getOpcode(), V1, V2);
			  }

			  // Update the NSW, NUW and Exact flags. Notice: V can be an Undef.
			  BinaryOperator *VecOp = dyn_cast<BinaryOperator>(V);
			  if (VecOp && isa<OverflowingBinaryOperator>(BinOp)) {
				VecOp->setHasNoSignedWrap(BinOp->hasNoSignedWrap());
				VecOp->setHasNoUnsignedWrap(BinOp->hasNoUnsignedWrap());
			  }
			  if (VecOp && isa<PossiblyExactOperator>(VecOp))
				VecOp->setIsExact(BinOp->isExact());			  
			  // Copy the fast-math flags.
			  if (VecOp && isa<FPMathOperator>(V))
				VecOp->setFastMathFlags(VL0->getFastMathFlags());

			  break;
			}

		    case Instruction::ShuffleVector: {

			  ValuePack Left, Right;
			  for (unsigned j = 0; j < bundle_sz; ++j){
				  Left.push_back(cast<Instruction>(root[j])->getOperand(0));
				  Right.push_back(cast<Instruction>(root[j])->getOperand(1));
			  }
			  Value *V1, *V2;
			  Builder.SetInsertPoint(insert_loc);
			  V1 = vectorizeSLPBundles(Left, v_sz);
			  Builder.SetInsertPoint(insert_loc);
			  V2 = vectorizeSLPBundles(Right, v_sz);
			  Builder.SetInsertPoint(insert_loc);	 
		      // Create a vector of LHS op1 RHS
		      BinaryOperator *BinOp0 = cast<BinaryOperator>(VL0);
		      Value *V_a = Builder.CreateBinOp(BinOp0->getOpcode(), V1, V2);
		      // Create a vector of LHS op2 RHS
		      Instruction *VL1 = cast<Instruction>(root[1]);
		      BinaryOperator *BinOp1 = cast<BinaryOperator>(VL1);
		      Value *V_b = Builder.CreateBinOp(BinOp1->getOpcode(), V1, V2);

		      std::vector<Constant *> Mask(bundle_sz);
		      for (unsigned i = 0; i < bundle_sz; ++i) {
		        if (i & 1)
		          Mask[i] = Builder.getInt32(bundle_sz + i);
		        else
		          Mask[i] = Builder.getInt32(i);
		      }
		      Value *ShuffleMask = ConstantVector::get(Mask);
		      V = Builder.CreateShuffleVector(V_a, V_b, ShuffleMask);
			  break;
		    }			

			case Instruction::ICmp:
			case Instruction::FCmp: {
			  // Check that all of the compares have the same predicate.
			  CmpInst::Predicate P0 = dyn_cast<CmpInst>(VL0)->getPredicate();
			  Type *ComparedTy = VL0->getOperand(0)->getType();
			  for (unsigned i = 1, e = root.size(); i < e; ++i) {
				CmpInst *Cmp = cast<CmpInst>(root[i]);
				if (Cmp->getPredicate() != P0 || Cmp->getOperand(0)->getType() != ComparedTy)
					assert(false && "can not do vectorization on this slp bundle!!! (Cmp) ");
			  }

		  	  ValuePack Operand1, Operand2;
			  for (unsigned j = 0; j < bundle_sz; ++j){
				  Operand1.push_back(cast<Instruction>(root[j])->getOperand(0));
				  Operand2.push_back(cast<Instruction>(root[j])->getOperand(1));

			  }
			  Builder.SetInsertPoint(insert_loc);
			  Value * V1 = vectorizeSLPBundles(Operand1, v_sz);
			  Builder.SetInsertPoint(insert_loc);
			  Value * V2 = vectorizeSLPBundles(Operand2, v_sz);

			  bool FCmp = (Opcode == Instruction::FCmp);
			  Builder.SetInsertPoint(insert_loc);
			  if (FCmp)
				  V = Builder.CreateFCmp(P0, V1, V2);
			  else
				  V = Builder.CreateICmp(P0, V1, V2);
			  break;
			}

			case Instruction::Call: {
			  // Check if the calls are all to the same vectorizable intrinsic.
			  CallInst *CI = cast<CallInst>(root[0]);
			  // Check if this is an Intrinsic call or something that can be
			  // represented by an intrinsic call
			  Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
			  if (!isTriviallyVectorizable(ID))
				assert(false && "can not do vectorization on this slp bundle!!! (Call) ");
			  		  
			  Function *Int = CI->getCalledFunction();
			  Value *A1I;
			  bool HasScalarOpd = hasVectorInstrinsicScalarOpd(ID, 1);
			  if (HasScalarOpd)
				A1I = CI->getArgOperand(1);
			  for (unsigned i = 1; i != bundle_sz; ++i) {
				CallInst *CI2 = dyn_cast<CallInst>(root[i]);
				if (!CI2 || CI2->getCalledFunction() != Int ||
					getIntrinsicIDForCall(CI2, TLI) != ID)
					assert(false && "can not do vectorization on this slp bundle!!! (Call) ");
				if (hasVectorInstrinsicScalarOpd(ID, 1)) {
				  Value *A1J = CI2->getArgOperand(1);
				  if (A1I != A1J)
				  	assert(false && "can not do vectorization on this slp bundle!!! (Call) ");
				}
			  }
	
			  Module *M = BB->getParent()->getParent();
			  Type *Tys[] = {VectorType::get(CI->getType()->getScalarType(), v_sz)};
			  Function *F = Intrinsic::getDeclaration(M, ID, Tys);

			  SmallVector<Value *, 4> Args;
			  for (unsigned i = 0, ie = CI->getNumArgOperands(); i != ie; ++i) {
				if (HasScalarOpd && i == 1) {
				  Args.push_back(CI->getArgOperand(i));
				  continue;
				}

				ValuePack Operand;
			    for (unsigned j = 0; j < bundle_sz; ++j)
				  Operand.push_back(cast<Instruction>(root[j])->getOperand(i));
				Builder.SetInsertPoint(insert_loc);
				Value * vec_opr = vectorizeSLPBundles(Operand, v_sz);
				Args.push_back(vec_opr);
			  }
			  Builder.SetInsertPoint(insert_loc);
			  V = Builder.CreateCall(F, Args);
			  break;
			}
			case Instruction::Load: {
			  // Check if the loads are consecutive or of we need to swizzle them.
			  Builder.SetInsertPoint(insert_loc);

			  ValuePack root_t;
			  if(isReversedConsecutiveLoad){		   
				   for (unsigned i = 0, e = bundle_sz; i < e; i = i+chain_len) {
					  for (unsigned j = 1; j <= chain_len; j++){
					  	  root_t.push_back(root[i+chain_len-j]);
					  }
				   }
				   assert((root.size() == root_t.size()) && "incorrect reversed bundle!\n");
			  }
			  else
				  root_t = root;
			  
			  V = vectorizeLoadSLPBundle(root_t, v_sz);

			  if(isReversedConsecutiveLoad){
				  SmallVector<unsigned, 8> ShuffleIdx;
				  for (unsigned i = 0, e = bundle_sz; i < e; i = i+chain_len) {
					 for (unsigned j = 1; j <= chain_len; j++){
						 ShuffleIdx.push_back(i+chain_len-j);
					 }
				  }
				  assert((ShuffleIdx.size() == bundle_sz)  && "incorrect shuffle index for reverse vector load !!!\n");
				  
				  SmallVector<Constant*, 8> ShuffleMask;	  
				  for(unsigned i = 0; i < bundle_sz; ++i)
					  ShuffleMask.push_back(Builder.getInt32(ShuffleIdx[i]));
					  
				  V = Builder.CreateShuffleVector(V, V, ConstantVector::get(ShuffleMask));
			  }
			  
			  break;
			}	
			default:
			  assert(false && "can not vectorize this slp bundle!!!");
			  
		}
		
        SLPVecValues[bundle_id] = V;
		return V;

}

void InnerLoopVectorizer::vectorizeStoreSLPBundle(ValuePack root, Value * v){

	unsigned bundle_sz = root.size();
	if(bundle_sz == 1){		
		Instruction *Cloned = cast<Instruction>(root[0])->clone();
		Cloned->setOperand(0, v);		
		Builder.Insert(Cloned);
		return;
	}

	Type * sca_ty = cast<StoreInst>(root[0])->getValueOperand()->getType();
	unsigned vec_sz = Legal->getBestVectorWidth(sca_ty, bundle_sz);
	assert(((vec_sz >= bundle_sz) && (bundle_sz >= vec_sz/2)) && "invalid bundle size!!!\n");
	assert((vec_sz == cast<VectorType>(v->getType())->getNumElements()) && "store value vector is incorrect!\n");

	bool needunpack = false;
	if(vec_sz != bundle_sz)
		needunpack = true;
	if(!needunpack){
		for (unsigned i = 1; i < bundle_sz; ++i) {
		  StoreInst *S = cast<StoreInst>(root[i]);
		  if (!S->isSimple() || !consecutiveMemAccs.count(root[i-1]) || root[i] != consecutiveMemAccs[root[i-1]]){
			needunpack = true;
			break;
		  }
		}
	}

	if(needunpack){
		ValuePack root_p1, root_p2;
		for(unsigned i = 0; i < bundle_sz; ++i){
			if(i < vec_sz/2)
				root_p1.push_back(root[i]);
			else
				root_p2.push_back(root[i]);
		}

		if((root_p1.size() == 1) && (root_p2.size() == 1)){////pure extract
			Value *V_t = Builder.CreateExtractElement(v, Builder.getInt32(0));
			vectorizeStoreSLPBundle(root_p1, V_t);
			V_t = Builder.CreateExtractElement(v, Builder.getInt32(1));		
			vectorizeStoreSLPBundle(root_p2, V_t);
			return;
		}
		
		////shuffle
		SmallVector<Constant*, 8> ShuffleMask;
		for (unsigned i = 0; i < vec_sz/2; ++i)
		  ShuffleMask.push_back(Builder.getInt32(i));
		Value * V_shuf = Builder.CreateShuffleVector(v, v, ConstantVector::get(ShuffleMask));
		vectorizeStoreSLPBundle(root_p1, V_shuf);

		if(root_p2.size() == 1){
			Value *V_t = Builder.CreateExtractElement(v, Builder.getInt32(root_p1.size()));
			vectorizeStoreSLPBundle(root_p2, V_t);
		}
		else if(root_p2.size() > 0){
			ShuffleMask.clear();
			for (unsigned i = vec_sz/2; i < vec_sz; ++i){
				if(i < bundle_sz)
			  		ShuffleMask.push_back(Builder.getInt32(i));
			}
			V_shuf = Builder.CreateShuffleVector(v, v, ConstantVector::get(ShuffleMask));
			vectorizeStoreSLPBundle(root_p2, V_shuf);		
		}		
	}
	else{		
		////get pointer operand for vec store
		StoreInst * SI = cast<StoreInst>(root[0]);
		Value * ptr_v = SI->getPointerOperand();		
		////insert the store inst						
		Type * vec_ty = VectorType::get(sca_ty, vec_sz);
		unsigned Alignment = SI->getAlignment();
		if (!Alignment)
		  Alignment = DL->getABITypeAlignment(sca_ty);
		unsigned AddressSpace = ptr_v->getType()->getPointerAddressSpace();
		
		Value *VecPtr = Builder.CreateBitCast(ptr_v, vec_ty->getPointerTo(AddressSpace));
		Builder.CreateAlignedStore(v, VecPtr, Alignment);
		return;
	}

}



Value * InnerLoopVectorizer::vectorizeLoadSLPBundle(ValuePack root, unsigned size){

	unsigned bundle_sz = root.size();
	if(bundle_sz == 0)
		return NULL;
	if(bundle_sz == 1){
		Instruction *Cloned = cast<Instruction>(root[0])->clone();
		Builder.Insert(Cloned);
		root[0]->replaceAllUsesWith(Cloned);
		return Cloned;
	}

	Type * sca_ty = root[0]->getType();
	unsigned vec_sz = size;
	assert(((vec_sz >= bundle_sz) && (bundle_sz >= vec_sz/2)) && "invalid bundle size!!!\n");

	bool needpack = false;
	if(vec_sz != bundle_sz)
		needpack = true;
	if(!needpack){
		for (unsigned i = 1; i < bundle_sz; ++i) {
		  LoadInst *L = cast<LoadInst>(root[i]);
		  if (!L->isSimple() || !consecutiveMemAccs.count(root[i-1]) || root[i] != consecutiveMemAccs[root[i-1]]){
			needpack = true;
			break;
		  }
		}
	}

	if(needpack){
		ValuePack root_p1, root_p2;
		for(unsigned i = 0; i < bundle_sz; ++i){
			if(i < vec_sz/2)
				root_p1.push_back(root[i]);
			else
				root_p2.push_back(root[i]);
		}

		Value * V1 = vectorizeLoadSLPBundle(root_p1, vec_sz/2);
		Value * V2 = vectorizeLoadSLPBundle(root_p2, vec_sz/2);

		if((root_p1.size() == 1) && (root_p2.size() == 1)){////pure insert
			Value *V = UndefValue::get(VectorType::get(sca_ty, 2));	   
  		    V = Builder.CreateInsertElement(V, V1, Builder.getInt32(0));
  		    V = Builder.CreateInsertElement(V, V2, Builder.getInt32(1));			
			return V;
		}

		if(root_p2.size() == 0)
			V2 = UndefValue::get(VectorType::get(sca_ty, vec_sz/2));		
		if(root_p2.size() == 1)
			V2 = Builder.CreateVectorSplat(vec_sz/2, V2);

		////shuffle
		SmallVector<Constant*, 8> ShuffleMask;	
		for (unsigned i = 0; i < vec_sz; ++i){
			if(i < bundle_sz)
		  		ShuffleMask.push_back(Builder.getInt32(i));
			else
				ShuffleMask.push_back(Builder.getInt32(bundle_sz-1));
		}		
		return Builder.CreateShuffleVector(V1, V2, ConstantVector::get(ShuffleMask));		
	}
	else{
		////get pointer operand for vec store
		LoadInst * LDI = cast<LoadInst>(root[0]);
		Value * ptr_v = LDI->getPointerOperand();
		Type * vec_ty = VectorType::get(sca_ty, bundle_sz);
		unsigned Alignment = LDI->getAlignment();
		if (!Alignment)
		  Alignment = DL->getABITypeAlignment(sca_ty);
		unsigned AddressSpace = ptr_v->getType()->getPointerAddressSpace();
		
		Value *VecPtr = Builder.CreateBitCast(ptr_v, vec_ty->getPointerTo(AddressSpace));
		return Builder.CreateAlignedLoad(VecPtr, Alignment);
	}

}



void InnerLoopVectorizer::updateAnalysis() {
  // Forget the original basic block.
  SE->forgetLoop(OrigLoop);

  // Update the dominator tree information.
  assert(DT->properlyDominates(LoopBypassBlocks.front(), LoopExitBlock) &&
         "Entry does not dominate exit.");

  for (unsigned I = 1, E = LoopBypassBlocks.size(); I != E; ++I)
    DT->addNewBlock(LoopBypassBlocks[I], LoopBypassBlocks[I-1]);
  DT->addNewBlock(LoopVectorPreHeader, LoopBypassBlocks.back());

  // Due to if predication of stores we might create a sequence of "if(pred)
  // a[i] = ...;  " blocks.
  for (unsigned i = 0, e = LoopVectorBody.size(); i != e; ++i) {
    if (i == 0)
      DT->addNewBlock(LoopVectorBody[0], LoopVectorPreHeader);
    else if (isPredicatedBlock(i)) {
      DT->addNewBlock(LoopVectorBody[i], LoopVectorBody[i-1]);
    } else {
      DT->addNewBlock(LoopVectorBody[i], LoopVectorBody[i-2]);
    }
  }

  DT->addNewBlock(LoopMiddleBlock, LoopBypassBlocks[1]);
  DT->addNewBlock(LoopScalarPreHeader, LoopBypassBlocks[0]);
  DT->changeImmediateDominator(LoopScalarBody, LoopScalarPreHeader);
  DT->changeImmediateDominator(LoopExitBlock, LoopMiddleBlock);

  DEBUG(DT->verifyDomTree());
}


bool LoopVectorizationLegality::isUniform(Value *V) {
  return (SE->isLoopInvariant(SE->getSCEV(V), TheLoop));
}

/// This function returns the identity element (or neutral element) for
/// the operation K.
Constant*
LoopVectorizationLegality::getReductionIdentity(ReductionKind K, Type *Tp) {
  switch (K) {
  case RK_IntegerXor:
  case RK_IntegerAdd:
  case RK_IntegerOr:
    // Adding, Xoring, Oring zero to a number does not change it.
    return ConstantInt::get(Tp, 0);
  case RK_IntegerMult:
    // Multiplying a number by 1 does not change it.
    return ConstantInt::get(Tp, 1);
  case RK_IntegerAnd:
    // AND-ing a number with an all-1 value does not change it.
    return ConstantInt::get(Tp, -1, true);
  case  RK_FloatMult:
    // Multiplying a number by 1 does not change it.
    return ConstantFP::get(Tp, 1.0L);
  case  RK_FloatAdd:
    // Adding zero to a number does not change it.
    return ConstantFP::get(Tp, 0.0L);
  default:
    llvm_unreachable("Unknown reduction kind");
  }
}


int LoopVectorizationLegality::isConsecutivePtr(Value *Ptr) {
  assert(Ptr->getType()->isPointerTy() && "Unexpected non-ptr");
  // Make sure that the pointer does not point to structs.
  if (Ptr->getType()->getPointerElementType()->isAggregateType())
    return 0;

  // If this value is a pointer induction variable we know it is consecutive.
  PHINode *Phi = dyn_cast_or_null<PHINode>(Ptr);
  if (Phi && Inductions.count(Phi)) {
    InductionInfo II = Inductions[Phi];
    if (IK_PtrInduction == II.IK)
      return 1;
    else if (IK_ReversePtrInduction == II.IK)
      return -1;
  }

  GetElementPtrInst *Gep = dyn_cast_or_null<GetElementPtrInst>(Ptr);
  if (!Gep)
    return 0;

  unsigned NumOperands = Gep->getNumOperands();
  Value *GpPtr = Gep->getPointerOperand();
  // If this GEP value is a consecutive pointer induction variable and all of
  // the indices are constant then we know it is consecutive. We can
  Phi = dyn_cast<PHINode>(GpPtr);
  if (Phi && Inductions.count(Phi)) {

    // Make sure that the pointer does not point to structs.
    PointerType *GepPtrType = cast<PointerType>(GpPtr->getType());
    if (GepPtrType->getElementType()->isAggregateType())
      return 0;

    // Make sure that all of the index operands are loop invariant.
    for (unsigned i = 1; i < NumOperands; ++i)
      if (!SE->isLoopInvariant(SE->getSCEV(Gep->getOperand(i)), TheLoop))
        return 0;

    InductionInfo II = Inductions[Phi];
    if (IK_PtrInduction == II.IK)
      return 1;
    else if (IK_ReversePtrInduction == II.IK)
      return -1;
  }

  unsigned InductionOperand = getGEPInductionOperand(DL, Gep);

  // Check that all of the gep indices are uniform except for our induction
  // operand.
  for (unsigned i = 0; i != NumOperands; ++i)
    if (i != InductionOperand &&
        !SE->isLoopInvariant(SE->getSCEV(Gep->getOperand(i)), TheLoop))
      return 0;

  // We can emit wide load/stores only if the last non-zero index is the
  // induction variable.
  const SCEV *Last = nullptr;
  if (!Strides.count(Gep))
    Last = SE->getSCEV(Gep->getOperand(InductionOperand));
  else {
/* 
    // Because of the multiplication by a stride we can have a s/zext cast.
    // We are going to replace this stride by 1 so the cast is safe to ignore.
    //
    //  %indvars.iv = phi i64 [ 0, %entry ], [ %indvars.iv.next, %for.body ]
    //  %0 = trunc i64 %indvars.iv to i32
    //  %mul = mul i32 %0, %Stride1
    //  %idxprom = zext i32 %mul to i64  << Safe cast.
    //  %arrayidx = getelementptr inbounds i32* %B, i64 %idxprom
    //
    Last = replaceSymbolicStrideSCEV(SE, Strides,
                                     Gep->getOperand(InductionOperand), Gep);
    if (const SCEVCastExpr *C = dyn_cast<SCEVCastExpr>(Last))
      Last =
          (C->getSCEVType() == scSignExtend || C->getSCEVType() == scZeroExtend)
              ? C->getOperand()
              : Last;
*/
	return 0;
  }
  if (const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(Last)) {
    const SCEV *Step = AR->getStepRecurrence(*SE);

    // The memory is consecutive because the last index is consecutive
    // and all other indices are loop invariant.
    if (Step->isOne())
      return 1;
    if (Step->isAllOnesValue())
      return -1;
  }

  return 0;
}


unsigned LoopVectorizationLegality::getNumofElementInVector(Type * type){

	unsigned vec_reg_sz = maxVecRegWidthInbit;
	unsigned ty_sz = (unsigned)DL->getTypeSizeInBits(type);
	return vec_reg_sz/ty_sz;
}


//////get the vector width for partial vectorization
unsigned LoopVectorizationLegality::getBestVectorWidth(Type * ty, unsigned sz){
	unsigned vec_sz = getNumofElementInVector(ty);
	unsigned sz_best = vec_sz;
	while(vec_sz > 1){
		vec_sz = vec_sz/2;
		if(vec_sz < sz)
			break;
		else
			sz_best = vec_sz;
	}
	return sz_best;
}




bool LoopVectorizationLegality::canVectorizeWithIfConvert() {
  if (!EnableIfConversion) {
    emitAnalysis(Report() << "if-conversion is disabled");
    return false;
  }

  assert(TheLoop->getNumBlocks() > 1 && "Single block loops are vectorizable");

  // A list of pointers that we can safely read and write to.
  SmallPtrSet<Value *, 8> SafePointes;

  // Collect safe addresses.
  for (Loop::block_iterator BI = TheLoop->block_begin(),
         BE = TheLoop->block_end(); BI != BE; ++BI) {
    BasicBlock *BB = *BI;

    if (blockNeedsPredication(BB))
      continue;

    for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I) {
      if (LoadInst *LI = dyn_cast<LoadInst>(I))
        SafePointes.insert(LI->getPointerOperand());
      else if (StoreInst *SI = dyn_cast<StoreInst>(I))
        SafePointes.insert(SI->getPointerOperand());
    }
  }

  // Collect the blocks that need predication.
  BasicBlock *Header = TheLoop->getHeader();
  for (Loop::block_iterator BI = TheLoop->block_begin(),
         BE = TheLoop->block_end(); BI != BE; ++BI) {
    BasicBlock *BB = *BI;

    // We don't support switch statements inside loops.
    if (!isa<BranchInst>(BB->getTerminator())) {
      emitAnalysis(Report(BB->getTerminator())
                   << "loop contains a switch statement");
      return false;
    }

    // We must be able to predicate all blocks that need to be predicated.
    if (blockNeedsPredication(BB)) {
      if (!blockCanBePredicated(BB, SafePointes)) {
        emitAnalysis(Report(BB->getTerminator())
                     << "control flow cannot be substituted for a select");
        return false;
      }
    } else if (BB != Header && !canIfConvertPHINodes(BB)) {
      emitAnalysis(Report(BB->getTerminator())
                   << "control flow cannot be substituted for a select");
      return false;
    }
  }

  // We can if-convert this loop.
  return true;
}

bool LoopVectorizationLegality::canVectorize() {
  // We must have a loop in canonical form. Loops with indirectbr in them cannot
  // be canonicalized.
  if (!TheLoop->getLoopPreheader()) {
    emitAnalysis(
        Report() << "loop control flow is not understood by vectorizer");
    return false;
  }

  // We can only vectorize innermost loops.
  if (TheLoop->getSubLoopsVector().size()) {
    emitAnalysis(Report() << "loop is not the innermost loop");
    return false;
  }

  // We must have a single backedge.
  if (TheLoop->getNumBackEdges() != 1) {
    emitAnalysis(
        Report() << "loop control flow is not understood by vectorizer");
    return false;
  }

  // We must have a single exiting block.
  if (!TheLoop->getExitingBlock()) {
    emitAnalysis(
        Report() << "loop control flow is not understood by vectorizer");
    return false;
  }

  // We need to have a loop header.
  DEBUG(dbgs() << "LV: Found a loop: " << TheLoop->getHeader()->getName() << '\n');

  // Check if we can if-convert non-single-bb loops.
  unsigned NumBlocks = TheLoop->getNumBlocks();
  if (NumBlocks != 1 && !canVectorizeWithIfConvert()) {
    return false;
  }

  // ScalarEvolution needs to be able to find the exit count.
  const SCEV *ExitCount = SE->getBackedgeTakenCount(TheLoop);
  if (ExitCount == SE->getCouldNotCompute()) {
    emitAnalysis(Report() << "could not determine number of loop iterations");
    return false;
  }

  // Check if we can vectorize the instructions and CFG in this loop.
  if (!canVectorizeInstrs()) {
    return false;
  }

  // Go over each instruction and look at memory deps.
  if (!canVectorizeMemory()) {
    return false;
  }

  // Collect all of the variables that remain uniform after vectorization.
  collectLoopUniforms();

  DEBUG(dbgs() << "LV: We can vectorize this loop" <<
        (PtrRtCheck.Need ? " (with a runtime bound check)" : "")
        <<"!\n");

  // Okay! We can vectorize. At this point we don't have any other mem analysis
  // which may limit our maximum vectorization factor, so just return true with
  // no restrictions.
  return true;
}



bool LoopVectorizationLegality::canVectorizeInstrs() {
  BasicBlock *PreHeader = TheLoop->getLoopPreheader();
  BasicBlock *Header = TheLoop->getHeader();

  // Look for the attribute signaling the absence of NaNs.
  Function &F = *Header->getParent();
  if (F.hasFnAttribute("no-nans-fp-math"))
    HasFunNoNaNAttr = F.getAttributes().getAttribute(
      AttributeSet::FunctionIndex,
      "no-nans-fp-math").getValueAsString() == "true";

  // For each block in the loop.
  for (Loop::block_iterator bb = TheLoop->block_begin(),
       be = TheLoop->block_end(); bb != be; ++bb) {

    // Scan the instructions in the block and look for hazards.
    for (BasicBlock::iterator it = (*bb)->begin(), e = (*bb)->end(); it != e;
         ++it) {

      if (PHINode *Phi = dyn_cast<PHINode>(it)) {
        Type *PhiTy = Phi->getType();
        // Check that this PHI type is allowed.
        if (!PhiTy->isIntegerTy() &&
            !PhiTy->isFloatingPointTy() &&
            !PhiTy->isPointerTy()) {
          emitAnalysis(Report(it)
                       << "loop control flow is not understood by vectorizer");
          return false;
        }

        // If this PHINode is not in the header block, then we know that we
        // can convert it to select during if-conversion. No need to check if
        // the PHIs in this block are induction or reduction variables.
        if (*bb != Header) {
          // Check that this instruction has no outside users or is an
          // identified reduction value with an outside user.
          if (!hasOutsideLoopUser(TheLoop, it, AllowedExit))
            continue;
		  
          emitAnalysis(Report(it) << "value that could not be identified as "
                                     "reduction is used outside the loop");
          return false;
        }

        // We only allow if-converted PHIs with more than two incoming values.
        if (Phi->getNumIncomingValues() != 2) {
          emitAnalysis(Report(it)
                       << "control flow not understood by vectorizer");
          return false;
        }

        // This is the value coming from the preheader.
        Value *StartValue = Phi->getIncomingValueForBlock(PreHeader);
		
        // Check if this is an induction variable.
		int stride_induction = 0;
        InductionKind IK = isInductionVariable(Phi, stride_induction);
        if (IK_NoInduction != IK) {
          // Get the widest type.
          if (!WidestIndTy)
            WidestIndTy = convertPointerToIntegerType(*DL, PhiTy);
          else
            WidestIndTy = getWiderType(*DL, PhiTy, WidestIndTy);

          // Int inductions are special because we only allow one IV.
          if (IK == IK_IntInduction) {
            // Use the phi node with the widest type as induction. Use the last
            // one if there are multiple (no good reason for doing this other
            // than it is expedient).
            if ((!Induction || PhiTy == WidestIndTy) && (stride_induction == 1))
              Induction = Phi;
          }

          Inductions[Phi] = InductionInfo(StartValue, IK, stride_induction);

          // Until we explicitly handle the case of an induction variable with
          // an outside loop user we have to give up vectorizing this loop.
          if (hasOutsideLoopUser(TheLoop, it, AllowedExit)) {
            emitAnalysis(Report(it) << "use of induction value outside of the "
                                       "loop is not handled by vectorizer");
            return false;
          }

          continue;
        }

        // Check if this is an reduction variable.
        if (AddReductionVar(Phi, RK_IntegerAdd)) {
          DEBUG(dbgs() << "LV: Found an ADD reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_IntegerMult)) {
          DEBUG(dbgs() << "LV: Found a MUL reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_IntegerOr)) {
          DEBUG(dbgs() << "LV: Found an OR reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_IntegerAnd)) {
          DEBUG(dbgs() << "LV: Found an AND reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_IntegerXor)) {
          DEBUG(dbgs() << "LV: Found a XOR reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_IntegerMinMax)) {
          DEBUG(dbgs() << "LV: Found a MINMAX reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_FloatMult)) {
          DEBUG(dbgs() << "LV: Found an FMult reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_FloatAdd)) {
          DEBUG(dbgs() << "LV: Found an FAdd reduction PHI."<< *Phi <<"\n");
          continue;
        }
        if (AddReductionVar(Phi, RK_FloatMinMax)) {
          DEBUG(dbgs() << "LV: Found an float MINMAX reduction PHI."<< *Phi <<
                "\n");
          continue;
        }

        emitAnalysis(Report(it) << "unvectorizable operation");
        return false;
      }// end of PHI handling

      // We still don't handle functions. However, we can ignore dbg intrinsic
      // calls and we do handle certain intrinsic and libm functions.
      CallInst *CI = dyn_cast<CallInst>(it);
      if (CI && !getIntrinsicIDForCall(CI, TLI) && !isa<DbgInfoIntrinsic>(CI)) {
        emitAnalysis(Report(it) << "call instruction cannot be vectorized");
        return false;
      }

      // Intrinsics such as powi,cttz and ctlz are legal to vectorize if the
      // second argument is the same (i.e. loop invariant)
      if (CI && hasVectorInstrinsicScalarOpd(getIntrinsicIDForCall(CI, TLI), 1)) {
        if (!SE->isLoopInvariant(SE->getSCEV(CI->getOperand(1)), TheLoop)) {
          emitAnalysis(Report(it) << "intrinsic instruction cannot be vectorized");
          return false;
        }
      }

      // Check that the instruction return type is vectorizable.
      // Also, we can't vectorize extractelement instructions.
      if ((!VectorType::isValidElementType(it->getType()) &&
           !it->getType()->isVoidTy()) || isa<ExtractElementInst>(it)) {
        emitAnalysis(Report(it) << "instruction return type cannot be vectorized");
        return false;
      }

      // Check that the stored type is vectorizable.
      if (StoreInst *ST = dyn_cast<StoreInst>(it)) {
        Type *T = ST->getValueOperand()->getType();
        if (!VectorType::isValidElementType(T)) {
          emitAnalysis(Report(ST) << "store instruction cannot be vectorized");
          return false;
        }
        if (EnableMemAccessVersioning)
          collectStridedAcccess(ST);
      }

      if (EnableMemAccessVersioning)
        if (LoadInst *LI = dyn_cast<LoadInst>(it))
          collectStridedAcccess(LI);

      // Reduction instructions are allowed to have exit users.
      // All other instructions must not have external users.
      if (hasOutsideLoopUser(TheLoop, it, AllowedExit)) {
        emitAnalysis(Report(it) << "value cannot be used outside the loop");
        return false;
      }

    } // next instr.

  }

  if (!Induction) {
    if (Inductions.empty()) {
      emitAnalysis(Report()
                   << "loop induction variable could not be identified");
      return false;
    }
  }

  return true;
}


void LoopVectorizationLegality::collectStridedAcccess(Value *MemAccess) {
  Value *Ptr = nullptr;
  if (LoadInst *LI = dyn_cast<LoadInst>(MemAccess))
    Ptr = LI->getPointerOperand();
  else if (StoreInst *SI = dyn_cast<StoreInst>(MemAccess))
    Ptr = SI->getPointerOperand();
  else
    return;

  Value *Stride = getStrideFromPointer(Ptr, SE, DL, TheLoop);
  if (!Stride){
    return;
  }
  Strides[Ptr] = Stride;
  StrideSet.insert(Stride);
}

void LoopVectorizationLegality::collectLoopUniforms() {
  // We now know that the loop is vectorizable!
  // Collect variables that will remain uniform after vectorization.
  std::vector<Value*> Worklist;
  BasicBlock *Latch = TheLoop->getLoopLatch();

  // Start with the conditional branch and walk up the block.
  Worklist.push_back(Latch->getTerminator()->getOperand(0));

  // Also add all consecutive pointer values; these values will be uniform
  // after vectorization (and subsequent cleanup) and, until revectorization is
  // supported, all dependencies must also be uniform.
  for (Loop::block_iterator B = TheLoop->block_begin(),
       BE = TheLoop->block_end(); B != BE; ++B)
    for (BasicBlock::iterator I = (*B)->begin(), IE = (*B)->end();
         I != IE; ++I)
      if (I->getType()->isPointerTy() && isConsecutivePtr(I))
        Worklist.insert(Worklist.end(), I->op_begin(), I->op_end());

  while (Worklist.size()) {
    Instruction *I = dyn_cast<Instruction>(Worklist.back());
    Worklist.pop_back();

    // Look at instructions inside this loop.
    // Stop when reaching PHI nodes.
    // TODO: we need to follow values all over the loop, not only in this block.
    if (!I || !TheLoop->contains(I) || isa<PHINode>(I))
      continue;

    // This is a known uniform.
    Uniforms.insert(I);

    // Insert all operands.
    Worklist.insert(Worklist.end(), I->op_begin(), I->op_end());
  }
}


bool LoopVectorizationLegality::addConsecutiveMemAccessinfo(ValuePack &memacc_list){

	ValuePack::iterator I, IE;
	for (I = memacc_list.begin(), IE = memacc_list.end(); I != IE; ++I){
		assert(dyn_cast<Instruction>(*I) && "memory access must be an instruction!!!");
		addConsecutiveMemAccessinfo(*I);
	}
	return true;
}



bool LoopVectorizationLegality::addConsecutiveMemAccessinfo(Value* mem_acc){

    Instruction* cur_inst = cast<Instruction>(mem_acc);
	BasicBlock* cur_bb = cur_inst->getParent();
	bool foundit = false; 
	
	MemAccMap::iterator I, IE;
	for(I = consecutiveMemAcc.begin(), IE = consecutiveMemAcc.end(); I != IE; ++I){
		Instruction* m_inst = cast<Instruction>(I->first);
		BasicBlock* m_bb = m_inst->getParent();
        if(m_bb != cur_bb)
			continue;
		if(cur_inst->getOpcode()!= m_inst->getOpcode())
			continue;
		////now we could check if two stores or two loads are consecutive
		if(int cons = isConsecutiveMemAcc(cur_inst, m_inst)){			
			if(cons > 0){
				if(I->second != NULL)
					continue;		
				consecutiveMemAccReverse[mem_acc] = I->first;				
				consecutiveMemAcc[I->first] = mem_acc;
				consecutiveMemAcc[mem_acc] = NULL;	
			}
			else if(cons < 0){
				if(consecutiveMemAccReverse.count(m_inst) && (consecutiveMemAccReverse[m_inst]!= NULL))
					continue;
				consecutiveMemAccReverse[I->first] = mem_acc;
				consecutiveMemAccReverse[mem_acc] = NULL;				
				consecutiveMemAcc[mem_acc] = I->first;
			}
			foundit = true; 
			break;			
		}
	}
	if(!foundit){
		consecutiveMemAcc[mem_acc] = NULL;
		consecutiveMemAccReverse[mem_acc] = NULL;
	}
	return true;

}



////hanlde some special case, e.g., for 301.apsi from spec 2000, there the induction var is of typr i32 not i64
const SCEV *  LoopVectorizationLegality::ignoreCastOfSCEV(const SCEV* Dist){

	if(!dyn_cast<SCEVAddExpr>(Dist))
		return Dist;

	const SCEVAddExpr* dist_t = cast<SCEVAddExpr>(Dist);
    if(dist_t->getNumOperands()!=2)
		return Dist;
	
	const SCEV* dist_t_opr1 = dist_t->getOperand(0);
	const SCEV* dist_t_opr2 = dist_t->getOperand(1);	
	if(!dyn_cast<SCEVMulExpr>(dist_t_opr1) || !dyn_cast<SCEVMulExpr>(dist_t_opr2))
		return Dist;
	
	const SCEVMulExpr* dist_tt1 = cast<SCEVMulExpr>(dist_t_opr1);
    if(dist_tt1->getNumOperands()!=2)
		return Dist;
	const SCEV* dist_tt1_opr1 = dist_tt1->getOperand(0);
	const SCEV* dist_tt1_opr2 = dist_tt1->getOperand(1);
	
	const SCEVMulExpr* dist_tt2 = cast<SCEVMulExpr>(dist_t_opr2);
    if(dist_tt2->getNumOperands()!=2)
		return Dist;	
	const SCEV* dist_tt2_opr1 = dist_tt2->getOperand(0);
	const SCEV* dist_tt2_opr2 = dist_tt2->getOperand(1);	

	if(!dyn_cast<SCEVConstant>(dist_tt1_opr1) || !dyn_cast<SCEVConstant>(dist_tt2_opr1))
		return Dist;		


	if(!(SE->getAddExpr(dist_tt1_opr1, dist_tt2_opr1))->isZero())
		return Dist;

	while(dyn_cast<SCEVCastExpr>(dist_tt1_opr2) || dyn_cast<SCEVCastExpr>(dist_tt2_opr2)){

		if(!dyn_cast<SCEVCastExpr>(dist_tt1_opr2) || !dyn_cast<SCEVCastExpr>(dist_tt2_opr2))
			return Dist;		

		const SCEVCastExpr* cast_opr1 = cast<SCEVCastExpr>(dist_tt1_opr2);
		const SCEVCastExpr* cast_opr2 = cast<SCEVCastExpr>(dist_tt2_opr2);

		if(cast_opr1->getType() != cast_opr2->getType())
			return Dist;
		
		dist_tt1_opr2 = cast_opr1->getOperand();
		dist_tt2_opr2 = cast_opr2->getOperand();
	}

	const SCEV* dist_ttt = SE->getMinusSCEV(dist_tt1_opr2, dist_tt2_opr2);
	if(dyn_cast<SCEVConstant>(dist_ttt)){

		const SCEVConstant *C = cast<SCEVConstant>(dist_ttt);
		const APInt &Val = C->getValue()->getValue();
		int Distance = Val.getSExtValue();

		C = cast<SCEVConstant>(dist_tt1_opr1);
		const APInt &Val1 = C->getValue()->getValue();
		Distance = Distance*(Val1.getSExtValue());		
		
		return SE->getConstant(ConstantInt::getSigned(C->getValue()->getType(), Distance));
	}

	return Dist;

}



int LoopVectorizationLegality::isConsecutiveMemAcc(Instruction* mem_a, Instruction* mem_b){
	
	Value *APtr;
	Value *BPtr;
	StoreInst* cur_inst = dyn_cast<StoreInst>(mem_a);
    if(cur_inst){
		APtr = cast<StoreInst>(mem_a)->getPointerOperand();
		BPtr = cast<StoreInst>(mem_b)->getPointerOperand();
	}else{
		APtr = cast<LoadInst>(mem_a)->getPointerOperand();
		BPtr = cast<LoadInst>(mem_b)->getPointerOperand();
	}

	// We cannot check pointers in different address spaces.
	if (APtr->getType()->getPointerAddressSpace() !=
		BPtr->getType()->getPointerAddressSpace())
	  return 0;
	
	const SCEV *AScev = SE->getSCEV(APtr);
	const SCEV *BScev = SE->getSCEV(BPtr);	

	const SCEV *Dist = SE->getMinusSCEV(AScev, BScev);
    Dist = ignoreCastOfSCEV(Dist);	
	const SCEVConstant *C = dyn_cast<SCEVConstant>(Dist);
	if (!C) {
	  return 0;
	}

	Type *ATy = APtr->getType()->getPointerElementType();
	Type *BTy = BPtr->getType()->getPointerElementType();
	if (ATy != BTy) {
	  return 0;
	}
	
	unsigned TypeByteSize = DL->getTypeAllocSize(ATy);
	const APInt &Val = C->getValue()->getValue();
	int Distance = Val.getSExtValue();

	if(Distance == TypeByteSize)
		return 1;
	else if(Distance == (-1)*TypeByteSize){
		return -1;
	}
	else
		return 0;

}



void LoopVectorizationLegality::findConsecutiveMemAccess(){

	SetVector<Value *> Heads, Tails;	
	for(MemAccMap::iterator I = consecutiveMemAcc.begin(), IE = consecutiveMemAcc.end(); I != IE; ++I){
		if(I->second){
			Heads.insert(I->first);
			Tails.insert(I->second);
		}
	}
	
	std::set<Value *> checked;
	for (SetVector<Value *>::iterator it = Heads.begin(), e = Heads.end(); it != e; ++it) {
	  if (Tails.count(*it))
		continue;

	  ValuePack mem_list;
	  Value *I = *it;
	  while (Tails.count(I) || Heads.count(I)) {
		if(checked.count(I))
			break;
		mem_list.push_back(I);
		I = consecutiveMemAcc[I];
	  }

	  if(mem_list.size()>1)
		  consecutiveMemAccChainList.push_back(mem_list);
 	  checked.insert(mem_list.begin(), mem_list.end());	  
	}

	return;
}



void LoopVectorizationLegality::findSLPVars(){

    MemAccessChainList consecutiveMemAccChainList_t; 
	MemAccessChainList::iterator I, IE;	
	for(MemAccessChainList::iterator I = consecutiveMemAccChainList.begin(), IE = consecutiveMemAccChainList.end(); I != IE; ++I)
		consecutiveMemAccChainList_t.push_back(*I);		
	consecutiveMemAccChainList.clear();

	////handle stores	first
	for(I = consecutiveMemAccChainList_t.begin(), IE = consecutiveMemAccChainList_t.end(); I != IE; ++I){

		ValuePack root= *I;	
		if(dyn_cast<LoadInst>(root[0]))
			continue;

		Type * ty_v = cast<StoreInst>(root[0])->getValueOperand()->getType();
		unsigned maxVF = getNumofElementInVector(ty_v);

		for(unsigned ii = 0, jj = root.size(); ii < jj; ii += maxVF){
			ValuePack root_sub;
			while((maxVF > 0) && (ii+maxVF > jj))
				maxVF = maxVF >> 1;
			if(maxVF < 2)
				break;
		
			for(unsigned kk = 0; kk < maxVF; kk++)
				root_sub.push_back(root[ii+kk]);
			
			SwitchTimes = 0;
			SLPMemAccTimes = 0;			
			if(!findSLPVars(root_sub)) ////trace from stores, until reach loads.					
				consecutiveMemAccChainList.push_back(root_sub);			
		}		
	}	
    
	////handle other loads that haven't be collected by the bottom-up searching
	for(I = consecutiveMemAccChainList_t.begin(), IE = consecutiveMemAccChainList_t.end(); I != IE; ++I){

		ValuePack root= *I;	
		if(dyn_cast<StoreInst>(root[0]))
			continue;
		      
        SLPVarPackList load_bundles;
		ValuePack loads;
		for(unsigned ii = 0, jj = root.size(); ii < jj; ii++){
			if(!isSLPVar(root[ii]))
				loads.push_back(root[ii]);
			
			if(isSLPVar(root[ii]) || (ii == jj-1)){
				if(loads.size() >= 2)
					load_bundles.push_back(loads);
				loads.clear();
			}
		}

		Type * ty_v = cast<LoadInst>(root[0])->getType();
		unsigned maxVF = getNumofElementInVector(ty_v);

        ////strip-mining obtained consecutive loads		
		for(unsigned ii = 0, jj = load_bundles.size(); ii < jj; ii++){
			ValuePack loads_t = load_bundles[ii];
			unsigned bundle_sz = loads_t.size();
			unsigned bundle_sz_ori = bundle_sz;

			while(bundle_sz){
				unsigned sub_bundle_sz = maxVF;
				while(sub_bundle_sz > bundle_sz)
					sub_bundle_sz = sub_bundle_sz >> 1;

				if(sub_bundle_sz > 1){
					ValuePack sub_loads;
					for(unsigned iii = 0; iii < sub_bundle_sz; iii++)
						sub_loads.push_back(loads_t[bundle_sz_ori-bundle_sz+iii]);					
					
					if(addSLPLoadVars(sub_loads)){
						consecutiveMemAccChainList.push_back(sub_loads);
					}
				}

			    bundle_sz = bundle_sz - sub_bundle_sz;
			}
		}
	}

	return;	
}


bool LoopVectorizationLegality::addSLPLoadVars(ValuePack root){

	bool SameTy = getSameType(root); 
	(void)SameTy;
	assert(SameTy && "Invalid types!");
	if (root[0]->getType()->isVectorTy())
		return false;
			
	for (unsigned i = 0, e = root.size(); i != e; ++i) {
		if (isSLPVar(root[i]))   
			return false;
		LoadInst *L = cast<LoadInst>(root[i]);
		if (!L->isSimple())
			return false;
	}

	BlockNumbering &BN = getBlockNumbering(cast<LoadInst>(root[0])->getParent());
	BN.numberInstructions();		
	Instruction *First = getFirstInstruction(root);

	int idx = 0; 
	for (unsigned i = 0, e = root.size(); i < e; ++i) {
	  	if (root[i] == First){
			idx = i;
			continue;
		}
		Value *Barrier = getHoistBarrier(cast<Instruction>(root[i]), First);
		if (Barrier)
			return false;
	}
	
    LoadInst * Load0 = cast<LoadInst>(root[0]);
	if(Load0 != cast<LoadInst>(First)){
		assert( idx > 0 && "unhandled case!" );
		Value * ptr_f = cast<LoadInst>(First)->getPointerOperand();
		Value * ptr_o = Load0->getPointerOperand();	
		IRBuilder<> Builder_t(SE->getContext());
		Builder_t.SetInsertPoint(First);

        SmallVector<Value *, 1> Idx;
		Type *IdxType = DL->getIntPtrType(ptr_f->getType());		
        Idx.push_back(ConstantInt::getSigned(IdxType, -idx));
        Value *ptr_n = Builder_t.CreateInBoundsGEP(ptr_f, Idx);		
		
		Load0->replaceUsesOfWith(ptr_o, ptr_n);	
		BN.numberInstructions();
	}
		
    unsigned new_vpack_id = slpVarPackList.size();
	slpVarPackList.push_back(root);
	SLPVarPackInstLoc[new_vpack_id] = getFirstInstruction(root);
	for (unsigned j = 0; j < root.size(); ++j){
	  SLPVarToVarPack[root[j]] = new_vpack_id;
	  SLPVars.insert(root[j]);
	}
	return true;
	
}


bool LoopVectorizationLegality::noMoreSwitch(ValuePack root){
	for (unsigned j = 0; j < root.size(); ++j)
		SwitchVars.insert(root[j]);
	SwitchTimes++;
	///we simplify the analysis here
	///if we need to switich very often, then exploit SLP to utilize consecutive mem access may not be a good diea 
	if(SwitchTimes > SLPMemAccTimes){ 
		return true;
	}else
		return false;
}


bool LoopVectorizationLegality::findSLPVars(ValuePack root){
////return false means tend to handle the vars in the slp manner
////return true means tend not to handle the vars in the slp manner, swtiching from loop to slp is  required

    ////here, check for valid SLP variables based on the root, by doing bottom-up search
		  
    //need to have same type, can not be vector type
	  bool SameTy = getSameType(root); 
	  (void)SameTy;
	  assert(SameTy && "Invalid types!");
	  
    //can not be vector type
	  if (root[0]->getType()->isVectorTy())
		assert(false && "can not vector types!");
	  if (StoreInst *SI = dyn_cast<StoreInst>(root[0]))
		if (SI->getValueOperand()->getType()->isVectorTy())
		  assert(false && "can not vector types!!!");
		
	//if they are all constant
	  if (allConstant(root))
		return false;
	  
	//if they are all loop invariant
      bool allLoopInvariant = true;
	  for (unsigned i = 0, e = root.size(); i != e; ++i) {
	      if(!TheLoop->isLoopInvariant(root[i])){
			 allLoopInvariant = false;
			 break;
		  }
	  }	
	  if(allLoopInvariant){
	  	return false;
	  }

	//if they are loop invariant and variant mixed
	  for (unsigned i = 0, e = root.size(); i != e; ++i) {
		if(TheLoop->isLoopInvariant(root[i])){
			return true;
		}
	  }	  

	//now we know they are loop variant values

	////if the SLPVarPack is identical with the existing one, then we could reuse it 
      if(isExistingSlpBundle(root)){	  	
	  	return false;
	  }

      if(isReversedExistingSlpBundle(root)){	
	  	return false;
	  }

	//if they are all identical
	  if (isSplat(root)){	  	
	  	if(isSLPVar(root[0]) || dyn_cast<LoadInst>(root[0]))
			return false;
		else
			return true;
	  }	  

	////otherwise, there is confilct, which need be resolved later with shuffling !!!!
	  for (unsigned i = 0, e = root.size(); i != e; ++i) {	  	
		if (isSLPVar(root[i])){		
		    return true;	
		}
	  } 	  

	//if they are all the same ops and inside the same block
	  unsigned Opcode = getSameOpcode(root);	  
	  if (!getSameBlock(root) || !Opcode){
		return true;
	  }
	  
    ////to get slp load vars, handle consecutive loads bundle here
      if(Opcode == Instruction::Load){

		bool isConsecutive = true, isConsecutiveReverse = true;        	
		for (unsigned i = 0, e = root.size() - 1; i < e; ++i) {
			LoadInst *L = cast<LoadInst>(root[i]);
			if (!L->isSimple() || !consecutiveMemAcc.count(root[i]) || root[i + 1]!=consecutiveMemAcc[root[i]]){
				isConsecutive = false;
				break;
			}			
		}	
		for (unsigned i = 0, e = root.size() - 1; i < e; ++i) {
			LoadInst *L = cast<LoadInst>(root[i]);
			if (!L->isSimple() || !consecutiveMemAccReverse.count(root[i]) || root[i + 1]!=consecutiveMemAccReverse[root[i]]){
				isConsecutiveReverse = false;
				break;
			}			
		}	

		ValuePack root_t;
		if(isConsecutive){
			root_t = root;
		}
		else if(isConsecutiveReverse){
			for (int i = root.size() - 1, j = 0; i >= 0; i--, j++){
				root_t.push_back(root[i]);
			}
		}else{
			return true;
		}
	
		if(!addSLPLoadVars(root_t)){			
			return true;
		}			
		consecutiveMemAccChainList.push_back(root_t);

        bool HasNonSLPUser = false;
		for (unsigned i = 0, e = root_t.size(); i < e; ++i){
			Instruction *load_t = cast<Instruction>(root_t[i]);
			for (User *U : load_t->users()){
				Instruction *UI = dyn_cast<Instruction>(U);
				if(!isSLPVar(UI)){
					 HasNonSLPUser = true;
					 break;
				}
			}
			if(HasNonSLPUser)
				break;
		}		
		if(!HasNonSLPUser){
			SLPMemAccTimes++;
		}

		return false;
	  }
 		  
	  // Check that all of the users of the scalars that we want to vectorize are schedulable.
	  Instruction *VL0 = cast<Instruction>(root[0]);
	  BasicBlock *BB = VL0->getParent();
	  int MyLastIndex = getLastIndex(root);
	
	  for (unsigned i = 0, e = root.size(); i != e; ++i) {
		Instruction *Scalar = cast<Instruction>(root[i]);
		for (User *U : Scalar->users()) {
		  Instruction *UI = dyn_cast<Instruction>(U);	  
		  ////if user is not an instruction, then stop further searching. 
		  if (!UI) {
			assert(false && "this user is not an instruction!!!");
		  }
		  // We don't care if the user is in a different basic block.
		  BasicBlock *UserBlock = UI->getParent();
		  if (UserBlock != BB) {		
			continue;
		  }	
		  // If this is a PHINode within this basic block then we can place the extract wherever we want.
		  if (isa<PHINode>(*UI)) {
			continue;
		  }

		  // Check if this is a safe in-tree user.
		  if (isSLPVar(UI)) {
			unsigned Idx = SLPVarToVarPack[UI];
			ValuePack vec_p = slpVarPackList[Idx];
			int VecLocation = getLastIndex(vec_p);
			if (VecLocation <= MyLastIndex){
			  return true;
			}
			continue;
		  }	
		  // Make sure that we can schedule this unknown user.
		  BlockNumbering &BN = getBlockNumbering(BB);
		  int UserIndex = BN.getIndex(UI);
		  if (UserIndex <= MyLastIndex){
			  return true;
		  }
		}
	  }
	
	  // Check that every instructions appears once in this bundle.
	  for (unsigned i = 0, e = root.size(); i < e; ++i)
		for (unsigned j = i+1; j < e; ++j)
		  if (root[i] == root[j]) {
			return true;
		  }
	
	  // Check that instructions in this bundle don't reference other instructions.
	  // The runtime of this check is O(N * N-1 * uses(N)) and a typical N is 4.
	  for (unsigned i = 0, e = root.size(); i < e; ++i) {
		for (User *U : root[i]->users()) {
		  for (unsigned j = 0; j < e; ++j) {
			if (i != j && U == root[j]){			
			  return true;
			}
		  }
		}
	  }
	
	  // Check if it is safe to sink the stores.
	  if (Opcode == Instruction::Store) {
		Instruction *Last = getLastInstruction(root);
		for (unsigned i = 0, e = root.size(); i < e; ++i) {
		  if (root[i] == Last)
			continue;
		  Value *Barrier = getSinkBarrier(cast<Instruction>(root[i]), Last);
		  if (Barrier){	  	
			return true;
		  }
		}
	  }

      ////backup some important data, in case the bottom-up trace stops at the next step
	  std::set<Value*> SwitchVars_ori = SwitchVars;
	  int SwitchTimes_ori = SwitchTimes;	  
	  SmallPtrSet<Value *, 8> SLPVars_ori = SLPVars; 	  
	  std::map<Value *, unsigned> SLPVarToVarPack_ori = SLPVarToVarPack; 
	  std::map<unsigned, Instruction *> SLPVarPackInstLoc_ori = SLPVarPackInstLoc; 
	  SLPVarPackList slpVarPackList_ori = slpVarPackList;
	  MemAccessChainList consecutiveMemAccChainList_ori = consecutiveMemAccChainList;  

      unsigned new_vpack_id = slpVarPackList.size();
	  Instruction *Last = getLastInstruction(root);
	  slpVarPackList.push_back(root);
	  SLPVarPackInstLoc[new_vpack_id] = Last;	
	  for (unsigned j = 0; j < root.size(); ++j){
	    SLPVarToVarPack[root[j]] = new_vpack_id;
	    SLPVars.insert(root[j]);
	  }

	  ////for addsub vector inst		
	  // Check that this shuffle vector refers to the alternate sequence of opcodes.	  
	  bool isAltShuffle = false;
	  if (Opcode == Instruction::ShuffleVector) {
		  Instruction *I0 = dyn_cast<Instruction>(root[0]);
		  unsigned Op = I0->getOpcode();
		  if (Op != Instruction::ShuffleVector)
			isAltShuffle = true;
	  }
	  
	  bool endpoint = false;
	  switch (Opcode) {
		case Instruction::PHI: 
		case Instruction::ExtractElement:
		case Instruction::Select: {
			endpoint = true;
			break;
		}
		case Instruction::ZExt:
		case Instruction::SExt:
		case Instruction::FPToUI:
		case Instruction::FPToSI:
		case Instruction::FPExt:
		case Instruction::PtrToInt:
		case Instruction::IntToPtr:
		case Instruction::SIToFP:
		case Instruction::UIToFP:
		case Instruction::Trunc:
		case Instruction::FPTrunc:
		case Instruction::BitCast: {
		  Type *SrcTy = VL0->getOperand(0)->getType();
		  for (unsigned i = 0; i < root.size(); ++i) {
			Type *Ty = cast<Instruction>(root[i])->getOperand(0)->getType();
			if (Ty != SrcTy || Ty->isAggregateType() || Ty->isVectorTy()){ 
			  endpoint = true;
			  break;
			}
		  }
		  if(endpoint)
		  	break;	      

		  endpoint = true;
		  for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
			ValuePack Operands;
			for (unsigned j = 0; j < root.size(); ++j)
			  Operands.push_back(cast<Instruction>(root[j])->getOperand(i));
			bool needSwitch = findSLPVars(Operands);
			if(needSwitch){
				if(noMoreSwitch(Operands)){
					endpoint = true;
					break;
				}
			}
			endpoint &= needSwitch;
		  }
	      break;		  	
		}
		case Instruction::ICmp:
		case Instruction::FCmp: {
		  // Check that all of the compares have the same predicate.
		  CmpInst::Predicate P0 = dyn_cast<CmpInst>(VL0)->getPredicate();
		  Type *ComparedTy = cast<Instruction>(root[0])->getOperand(0)->getType();
		  for (unsigned i = 1, e = root.size(); i < e; ++i) {
			CmpInst *Cmp = cast<CmpInst>(root[i]);
			if (Cmp->getPredicate() != P0 || Cmp->getOperand(0)->getType() != ComparedTy){
				endpoint = true;
				break;
			}
		  }
		  if(endpoint)
		  	break;	      

		  endpoint = true;	
		  for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
			ValuePack Operands;
			for (unsigned j = 0; j < root.size(); ++j)
			  Operands.push_back(cast<Instruction>(root[j])->getOperand(i));	
			bool needSwitch = findSLPVars(Operands);
			if(needSwitch){
				if(noMoreSwitch(Operands)){
					endpoint = true;
					break;
				}
			}
			endpoint &= needSwitch;
		  }
		  break;
		}

		case Instruction::Add:
		case Instruction::FAdd:
		case Instruction::Sub:
		case Instruction::FSub:
		case Instruction::Mul:
		case Instruction::FMul:
		case Instruction::UDiv:
		case Instruction::SDiv:
		case Instruction::FDiv:
		case Instruction::URem:
		case Instruction::SRem:
		case Instruction::FRem:
		case Instruction::Shl:
		case Instruction::LShr:
		case Instruction::AShr:
		case Instruction::And:
		case Instruction::Or:
		case Instruction::Xor: {
		  // Sort operands of the instructions so that each side is more likely to
		  // have the same opcode.
		  if (isa<BinaryOperator>(VL0) && VL0->isCommutative()) {
			ValuePack Left, Right;
            ////if the operand is the existing bundle, then use the exsiting one;
            ////to match the existing bundle, we may need to switch the original operands of the binary op;
            bool foundExistingBundle = false;
			for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
				Left.clear();
				Value * opr = VL0->getOperand(i);
				if(!isSLPVar(opr)){
					foundExistingBundle = false;
					continue;
				}
				else{				
					unsigned opr_bundle_id = getSLPBundleID(opr);
					ValuePack opr_bundle_l = slpVarPackList[opr_bundle_id];
					unsigned opr_bundle_sz = root.size();
					if(opr_bundle_sz != opr_bundle_l.size()){
						foundExistingBundle = false;
						continue;
					}
					
					std::set<Value*> v_set_tmp;	
					for(unsigned ii = 0; ii < opr_bundle_sz; ii++)
						v_set_tmp.insert(opr_bundle_l[ii]);

					foundExistingBundle = true;
					Left.push_back(opr);					
					for (unsigned j = 1; j < root.size(); ++j){
						for (unsigned i_t = 0; i_t < e; ++i_t){
							Value * opr_i = cast<Instruction>(root[j])->getOperand(i_t);
							if(v_set_tmp.count(opr_i)){
								Left.push_back(opr_i);
								break;
							}
							else
								continue;
						}
						if(Left.size() > j) ////we add a new value into Left
							continue;
						else{
							foundExistingBundle = false;
							break;
						}
					}

					if(foundExistingBundle){
						assert((Left.size()==opr_bundle_sz) && "operand bundle size is incorrect!\n");
						////check if is the exact exsiting bundle
						if(isExistingSlpBundle(Left)||isReversedExistingSlpBundle(Left)){
							////if so, change the operand order of the original bin ops
							for (unsigned i_t = 0; i_t != opr_bundle_sz; ++i_t){
								 Instruction *Inst = cast<Instruction>(root[i_t]);
								 if(Inst->getOperand(0) != Left[i_t]){
								    Value * tmp_opr = Inst->getOperand(0);
								    Inst->setOperand(0, Left[i_t]);
								 	Inst->setOperand(1, tmp_opr);
								 }		  
							}
							////get the operand bundles	
							Left.clear();
							Right.clear();
							for (unsigned i_t = 0; i_t != opr_bundle_sz; ++i_t){
								 Instruction *Inst = cast<Instruction>(root[i_t]);
								 Left.push_back(Inst->getOperand(0));
								 Right.push_back(Inst->getOperand(1));		  
							}
							break;					
						}
					}
					////check the next operand or terminate
					foundExistingBundle = false;
					continue;
				}
		    }			
			  
			if(!foundExistingBundle){
				Left.clear();
				Right.clear();
				reorderInputsAccordingToOpcode(root, Left, Right);
			}
			
			BasicBlock *LeftBB = getSameBlock(Left);
			BasicBlock *RightBB = getSameBlock(Right);
			// If we have common uses on separate paths in the tree make sure we
			// process the one with greater common depth first.
			// We can use block numbering to determine the subtree traversal as
			// earler user has to come in between the common use and the later user.
			endpoint = true;
			if (LeftBB && RightBB && LeftBB == RightBB &&
				getLastIndex(Right) > getLastIndex(Left)) {
			  bool needSwitch = findSLPVars(Right);
			  if(needSwitch){
				  if(noMoreSwitch(Right)){
					  endpoint = true;
					  break;
				  }
			  }
			  endpoint &= needSwitch;
			  needSwitch = findSLPVars(Left);
			  if(needSwitch){
				  if(noMoreSwitch(Left)){
					  endpoint = true;
					  break;
				  }
			  }
 			  endpoint &= needSwitch;			  
			} 
			else 
			{
			  bool needSwitch = findSLPVars(Left);
			  if(needSwitch){
				  if(noMoreSwitch(Left)){
					  endpoint = true;
					  break;
				  }
			  }
 			  endpoint &= needSwitch;			
			  needSwitch = findSLPVars(Right);
			  if(needSwitch){
				  if(noMoreSwitch(Right)){
					  endpoint = true;
					  break;
				  }
			  }
			  endpoint &= needSwitch;		  
			}
			break;		
		  }	

		  endpoint = true;
		  for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
			ValuePack Operands;
			for (unsigned j = 0; j < root.size(); ++j)
			  Operands.push_back(cast<Instruction>(root[j])->getOperand(i));
			bool needSwitch = findSLPVars(Operands);
			if(needSwitch){
				if(noMoreSwitch(Operands)){
					endpoint = true;
					break;
				}
			}
			endpoint &= needSwitch;			
		  }
		  break;
		}
		case Instruction::ShuffleVector:{
			endpoint = true;	
			if(isAltShuffle){
				ValuePack opr_bd;
				////if the operand is the existing bundle, then use the exsiting one
				////to match the existing bundle, we may need to switch the original operands of the binary op
				bool foundExistingBundle = false;
				for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
					opr_bd.clear();
					Value * opr = VL0->getOperand(i);
					if(!isSLPVar(opr)){
						foundExistingBundle = false;
						continue;
					}
					else{				
						unsigned opr_bundle_id = getSLPBundleID(opr);
						ValuePack opr_bundle_l = slpVarPackList[opr_bundle_id];
						unsigned opr_bundle_sz = root.size();
						if(opr_bundle_sz != opr_bundle_l.size()){
							foundExistingBundle = false;
							continue;
						}
						std::set<Value*> v_set_tmp; 		
						for(unsigned ii = 0; ii < opr_bundle_sz; ii++)
							v_set_tmp.insert(opr_bundle_l[ii]);
				
						foundExistingBundle = true;
						opr_bd.push_back(opr);
						
						for (unsigned j = 1; j < root.size(); ++j){
                            Instruction * inst_t = cast<Instruction>(root[j]);
							if(isa<BinaryOperator>(inst_t) && inst_t->isCommutative()){
								for (unsigned i_t = 0; i_t < e; ++i_t){
									Value * opr_i = inst_t->getOperand(i_t);
									if(v_set_tmp.count(opr_i)){
										opr_bd.push_back(opr_i);
										break;
									}
									else
										continue;
								}
							}
							else{
								Value * opr_i = inst_t->getOperand(i);
								if(v_set_tmp.count(opr_i)){
									opr_bd.push_back(opr_i);
								}
							}
							
							if(opr_bd.size() > j)
								continue;
							else{
								foundExistingBundle = false;
								break;
							}
						}
				
						if(foundExistingBundle){
							assert((opr_bd.size()==opr_bundle_sz) && "operand bundle size is incorrect!\n");
							////check if is the exact exsiting bundle
							if(isExistingSlpBundle(opr_bd)||isReversedExistingSlpBundle(opr_bd)){								
								////if so, change the operand order of the original bin ops
								for (unsigned i_t = 0; i_t != opr_bundle_sz; ++i_t){
									 Instruction *inst_t = cast<Instruction>(root[i_t]);
									 if(inst_t->getOperand(i) != opr_bd[i_t]){
									 	assert((isa<BinaryOperator>(inst_t) && inst_t->isCommutative()) && "operand can not be switched!!!\n");
										if(i == 0){
											Value * tmp_opr = inst_t->getOperand(0);
											inst_t->setOperand(0, opr_bd[i_t]);
											inst_t->setOperand(1, tmp_opr);
										}
										else if(i == 1){
											Value * tmp_opr = inst_t->getOperand(1);
											inst_t->setOperand(1, opr_bd[i_t]);
											inst_t->setOperand(0, tmp_opr);
										}
										assert((i < 2) && "incorrect operand index!\n");
									 }	  
								}
								break;					
							}
						}
						////check the next operand or terminate
						foundExistingBundle = false;
						continue;
					}
				}			
			
				for (unsigned i = 0, e = VL0->getNumOperands(); i < e; ++i) {
				  ValuePack Operands;
				  for (unsigned j = 0; j < root.size(); ++j)
					Operands.push_back(cast<Instruction>(root[j])->getOperand(i));
				  bool needSwitch = findSLPVars(Operands);
				  if(needSwitch){
					  if(noMoreSwitch(Operands)){
						  endpoint = true;
						  break;
					  }
				  }
				  endpoint &= needSwitch;					  
				}
			}	  				
			break;
		}		

		case Instruction::Store: {
		  // Check if the stores are consecutive or of we need to swizzle them.
		  for (unsigned i = 0, e = root.size() - 1; i < e; ++i)
			if (!consecutiveMemAcc.count(root[i]) || root[i + 1]!=consecutiveMemAcc[root[i]]){
			  endpoint = true;
			  break;
			}
		  if(endpoint)
		  	break;

		  SLPMemAccTimes = 1;
		  ValuePack Operands;
		  for (unsigned j = 0; j < root.size(); ++j)
			Operands.push_back(cast<Instruction>(root[j])->getOperand(0));
		  findSLPVars(Operands); 
		  break;
		}
		
		case Instruction::Call: {
		  // Check if the calls are all to the same vectorizable intrinsic.
		  CallInst *CI = cast<CallInst>(root[0]);
		  // Check if this is an Intrinsic call or something that can be
		  // represented by an intrinsic call
		  Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
		  if (!isTriviallyVectorizable(ID)){
		  	endpoint = true;
			break;
		  }
		  if(endpoint)
		  	break;
		  
		  Function *Int = CI->getCalledFunction();
		  Value *A1I = nullptr;
		  if (hasVectorInstrinsicScalarOpd(ID, 1))
			A1I = CI->getArgOperand(1);
		  for (unsigned i = 1, e = root.size(); i != e; ++i) {
			CallInst *CI2 = dyn_cast<CallInst>(root[i]);
			if (!CI2 || CI2->getCalledFunction() != Int ||
				getIntrinsicIDForCall(CI2, TLI) != ID) {
			  endpoint = true;
			  break;
			}
			// ctlz,cttz and powi are special intrinsics whose second argument
			// should be same in order for them to be vectorized.
			if (hasVectorInstrinsicScalarOpd(ID, 1)) {
			  Value *A1J = CI2->getArgOperand(1);
			  if (A1I != A1J){
				endpoint = true;
				break;
			  }
			}
		  }
		  if(endpoint)
		  	break;

		  endpoint = true;
		  for (unsigned i = 0, e = CI->getNumArgOperands(); i != e; ++i) {
			ValuePack Operands;
			for (unsigned j = 0; j < root.size(); ++j) {
			  CallInst *CI2 = dyn_cast<CallInst>(root[j]);
			  Operands.push_back(CI2->getArgOperand(i));
			}
			bool needSwitch = findSLPVars(Operands);
			if(needSwitch){
				if(noMoreSwitch(Operands)){
					endpoint = true;
					break;
				}
			}
			endpoint &= needSwitch;				
		  }
		  break;
		}
		default:
		  endpoint = true;			
		  break;
	  }

	if(endpoint){
	    SLPVars = SLPVars_ori;	
		SwitchVars = SwitchVars_ori;	
		SwitchTimes = SwitchTimes_ori;
		SLPVarToVarPack = SLPVarToVarPack_ori; 
		SLPVarPackInstLoc = SLPVarPackInstLoc_ori; 
		slpVarPackList = slpVarPackList_ori;
		consecutiveMemAccChainList = consecutiveMemAccChainList_ori;
		return true;
	}    
	return false;
	
}


void LoopVectorizationLegality::dumpSLPVars() {

    unsigned list_sz = slpVarPackList.size();
	for(unsigned i = 0; i < list_sz; i++){
		ValuePack slpVP = slpVarPackList[i];
		unsigned pack_sz = slpVP.size();
		if(!isSLPVar(slpVP[0]))
			continue;
		errs()<< "@@@@@@@found SLP Vars Pack: \n";
		for(unsigned j = 0; j < pack_sz; j++)
			errs() << *(slpVP[j]) << "\n"; 
	}
	return;
}


int LoopVectorizationLegality::getMemLocationDistanceAccrossInteration(Value* v){

	StoreInst* SI = dyn_cast<StoreInst>(v);
	LoadInst* LLI = dyn_cast<LoadInst>(v);
	assert((SI||LLI) && "must be load or store instruction here!!!");
		
	int ptrDis = 0;
	Value * ptr = (SI)?SI->getPointerOperand():LLI->getPointerOperand();	
	const SCEV *scev = SE->getSCEV(ptr);	
	
	Type * type = (SI)? SI->getValueOperand()->getType():LLI->getType();
	unsigned ty_sz = (unsigned)DL->getTypeStoreSize(type);

	if(const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(scev)){
		const SCEV *Step = AR->getStepRecurrence(*SE);
		const SCEVConstant *C = dyn_cast<SCEVConstant>(Step);
		if (C) {
			const APInt &Val = C->getValue()->getValue();
			ptrDis = Val.getSExtValue()/ty_sz;
		}
	}

	return ptrDis;

}



bool LoopVectorizationLegality::onlyHasSlpVarUsers(Value* v){

    Instruction * I = dyn_cast<Instruction>(v);
	if(!I)
		return true;
	
    for (User *U : I->users()) {
		if(!isSLPVar(U))
			return false;
    }
	return true;

}


bool LoopVectorizationLegality::canVectorizeMemory() {

  // Holds the Load and Store *instructions*.
  ValuePack Loads;
  ValuePack Stores;

  // Holds all the different accesses in the loop.
  unsigned NumReads = 0;
  unsigned NumReadWrites = 0;

  PtrRtCheck.Pointers.clear();
  PtrRtCheck.Need = false;

  const bool IsAnnotatedParallel = TheLoop->isAnnotatedParallel(); 
  MemoryDepChecker DepChecker(SE, DL, TheLoop);

  // For each block.
  for (Loop::block_iterator bb = TheLoop->block_begin(),
       be = TheLoop->block_end(); bb != be; ++bb) {

    // Scan the BB and collect legal loads and stores.
    for (BasicBlock::iterator it = (*bb)->begin(), e = (*bb)->end(); it != e;
         ++it) {

      // If this is a load, save it. If this instruction can read from memory
      // but is not a load, then we quit. Notice that we don't handle function
      // calls that read or write.
      if (it->mayReadFromMemory()) {
        // Many math library functions read the rounding mode. We will only
        // vectorize a loop if it contains known function calls that don't set
        // the flag. Therefore, it is safe to ignore this read from memory.
        CallInst *Call = dyn_cast<CallInst>(it);
        if (Call && getIntrinsicIDForCall(Call, TLI))
          continue;

        LoadInst *Ld = dyn_cast<LoadInst>(it);
        if (!Ld || (!Ld->isSimple() && !IsAnnotatedParallel)) {
          emitAnalysis(Report(Ld)
                       << "read with atomic ordering or volatile read");
          DEBUG(dbgs() << "LV: Found a non-simple load.\n");
          return false;
        }
        NumLoads++;
        Loads.push_back(Ld);
        DepChecker.addAccess(Ld);
        continue;
      }

      // Save 'store' instructions. Abort if other instructions write to memory.
      if (it->mayWriteToMemory()) {
        StoreInst *St = dyn_cast<StoreInst>(it);
        if (!St) {
          emitAnalysis(Report(it) << "instruction cannot be vectorized");
          return false;
        }
        if (!St->isSimple() && !IsAnnotatedParallel) {
          emitAnalysis(Report(St)
                       << "write with atomic ordering or volatile write");
          DEBUG(dbgs() << "LV: Found a non-simple store.\n");
          return false;
        }
        NumStores++;
        Stores.push_back(St);
        DepChecker.addAccess(St);
      }
    } // Next instr.
  } // Next block.

  //////******************************************************************
  //we have colloect all the load/store now, here we find consecutive LD/ST access
  //...requirements: in the same BB; consecutiveness; 
  addConsecutiveMemAccessinfo(Stores); 
  addConsecutiveMemAccessinfo(Loads); 
  //////****************************************************************** 

  // Check if we see any stores. If there are no stores, then we don't
  // care if the pointers are *restrict*.
  if (!Stores.size()) {
    return true;
  }

  AccessAnalysis::DepCandidates DependentAccesses;
  AccessAnalysis Accesses(DL, AA, DependentAccesses);

  // Holds the analyzed pointers. We don't want to call GetUnderlyingObjects
  // multiple times on the same object. If the ptr is accessed twice, once
  // for read and once for write, it will only appear once (on the write
  // list). This is okay, since we are going to check for conflicts between
  // writes and between reads and writes, but not between reads and reads.
  
  SmallPtrSet<Value*, 16> Seen;
  
  ValuePack::iterator I, IE;
  for (I = Stores.begin(), IE = Stores.end(); I != IE; ++I) {
    StoreInst *ST = cast<StoreInst>(*I);
    Value* Ptr = ST->getPointerOperand();

    if (isUniform(Ptr)) {
      emitAnalysis(
          Report(ST)
          << "write to a loop invariant address could not be vectorized");
      return false;
    }

    // If we did *not* see this pointer before, insert it to  the read-write
    // list. At this phase it is only a 'write' list.
    if (Seen.insert(Ptr)) {
      ++NumReadWrites;

      AliasAnalysis::Location Loc = AA->getLocation(ST);
      // The TBAA metadata could have a control dependency on the predication
      // condition, so we cannot rely on it when determining whether or not we
      // need runtime pointer checks.
      if (blockNeedsPredication(ST->getParent()))
        Loc.TBAATag = nullptr;
	  
      Accesses.addStore(Loc);
    }
  }

  if (IsAnnotatedParallel) {
    return true;
  }

  for (I = Loads.begin(), IE = Loads.end(); I != IE; ++I) {
    LoadInst *LD = cast<LoadInst>(*I);
    Value* Ptr = LD->getPointerOperand();
    // If we did *not* see this pointer before, insert it to the
    // read list. If we *did* see it before, then it is already in
    // the read-write list. This allows us to vectorize expressions
    // such as A[i] += x;  Because the address of A[i] is a read-write
    // pointer. This only works if the index of A[i] is consecutive.
    // If the address of i is unknown (for example A[B[i]]) then we may
    // read a few words, modify, and write a few words, and some of the
    // words may be written to the same address.
    bool IsReadOnlyPtr = false;
	int stride_t = isStridedPtr(SE, DL, Ptr, TheLoop, Strides);
	
    //////***********************************************************
    //////as long as the stride is fixed value, i.e. A[i+step] += x, 
    //////then the case is similar to A[i] += x, assuming there is no pointer wrap; 
    //////so we lose the restricts in the original source code
    //////***********************************************************    	
	
    //if (Seen.insert(Ptr) || stride_t*stride_t != 1) 
    if (Seen.insert(Ptr) || !stride_t) {
      ++NumReads;
      IsReadOnlyPtr = true;
    }  
    
    AliasAnalysis::Location Loc = AA->getLocation(LD);
    // The TBAA metadata could have a control dependency on the predication
    // condition, so we cannot rely on it when determining whether or not we
    // need runtime pointer checks.
    if (blockNeedsPredication(LD->getParent()))
      Loc.TBAATag = nullptr;

    Accesses.addLoad(Loc, IsReadOnlyPtr);
  }

  // If we write (or read-write) to a single destination and there are no
  // other reads in this loop then is it safe to vectorize.
  if (NumReadWrites == 1 && NumReads == 0) {
    return true;
  }

  // Build dependence sets and check whether we need a runtime pointer bounds
  // check.
  Accesses.buildDependenceSets();
  bool NeedRTCheck = Accesses.isRTCheckNeeded();

  // Find pointers with computable bounds. We are going to use this information
  // to place a runtime bound check.
  unsigned NumComparisons = 0;
  bool CanDoRT = false;
  if (NeedRTCheck)
    CanDoRT = Accesses.canTryCheckPtrAtRT(PtrRtCheck, NumComparisons, SE, TheLoop, Strides);
  // If we only have one set of dependences to check pointers among we don't
  // need a runtime check.
  if (NumComparisons == 0 && NeedRTCheck)
    NeedRTCheck = false;
  // Check that we did not collect too many pointers or found an unsizeable
  // pointer.
  if (!CanDoRT){
    PtrRtCheck.reset();
  }
 
  if (NeedRTCheck && !CanDoRT) {
    emitAnalysis(Report() << "cannot identify array bounds");
    return false;
  }

  PtrRtCheck.Need = NeedRTCheck;
  bool CanVecMem = true;
  if (Accesses.isDependencyCheckNeeded()) {
	PtrRtCheck.reset();
    CanVecMem = DepChecker.areDepsSafe(PtrRtCheck,
        DependentAccesses, Accesses.getDependenciesToCheck(), Strides, DepDistance);
    MaxSafeDepDistBytes = DepChecker.getMaxSafeDepDistBytes();
  
	if(PtrRtCheck.RTCheckTimes > RuntimeMemoryCheckThreshold){
		emitAnalysis(Report()
					 << PtrRtCheck.RTCheckTimes << " exceeds limit of "
					 << RuntimeMemoryCheckThreshold
					 << " dependent memory operations checked at runtime");
		PtrRtCheck.reset();
		CanVecMem = false;
	}
  }
  return CanVecMem;
}



bool LoopVectorizationLegality::AddReductionVar(PHINode *Phi,
                                                ReductionKind Kind) {
  if (Phi->getNumIncomingValues() != 2){
    return false;
  }
  // Reduction variables are only found in the loop header block.
  if (Phi->getParent() != TheLoop->getHeader())
    return false;

  // Obtain the reduction start value from the value that comes from the loop
  // preheader.
  Value *RdxStart = Phi->getIncomingValueForBlock(TheLoop->getLoopPreheader());

  // ExitInstruction is the single value which is used outside the loop.
  // We only allow for a single reduction value to be used outside the loop.
  // This includes users of the reduction, variables (which form a cycle
  // which ends in the phi node).
  Instruction *ExitInstruction = nullptr;
  // Indicates that we found a reduction operation in our scan.
  bool FoundReduxOp = false;

  // We start with the PHI node and scan for all of the users of this
  // instruction. All users must be instructions that can be used as reduction
  // variables (such as ADD). We must have a single out-of-block user. The cycle
  // must include the original PHI.
  bool FoundStartPHI = false;

  // To recognize min/max patterns formed by a icmp select sequence, we store
  // the number of instruction we saw from the recognized min/max pattern,
  //  to make sure we only see exactly the two instructions.
  unsigned NumCmpSelectPatternInst = 0;
  ReductionInstDesc ReduxDesc(false, nullptr);

  SmallPtrSet<Instruction *, 8> VisitedInsts;
  SmallVector<Instruction *, 8> Worklist;
  Worklist.push_back(Phi);
  VisitedInsts.insert(Phi);

  // A value in the reduction can be used:
  //  - By the reduction:
  //      - Reduction operation:
  //        - One use of reduction value (safe).
  //        - Multiple use of reduction value (not safe).
  //      - PHI:
  //        - All uses of the PHI must be the reduction (safe).
  //        - Otherwise, not safe.
  //  - By one instruction outside of the loop (safe).
  //  - By further instructions outside of the loop (not safe).
  //  - By an instruction that is not part of the reduction (not safe).
  //    This is either:
  //      * An instruction type other than PHI or the reduction operation.
  //      * A PHI in the header other than the initial PHI.
  while (!Worklist.empty()) {
    Instruction *Cur = Worklist.back();
    Worklist.pop_back();

    // No Users.
    // If the instruction has no users then this is a broken chain and can't be
    // a reduction variable.
    if (Cur->use_empty())
      return false;

    bool IsAPhi = isa<PHINode>(Cur);

    // A header PHI use other than the original PHI.
    if (Cur != Phi && IsAPhi && Cur->getParent() == Phi->getParent())
      return false;

    // Reductions of instructions such as Div, and Sub is only possible if the
    // LHS is the reduction variable.
    if (!Cur->isCommutative() && !IsAPhi && !isa<SelectInst>(Cur) &&
        !isa<ICmpInst>(Cur) && !isa<FCmpInst>(Cur) &&
        !VisitedInsts.count(dyn_cast<Instruction>(Cur->getOperand(0))))
      return false;

    // Any reduction instruction must be of one of the allowed kinds.
    ReduxDesc = isReductionInstr(Cur, Kind, ReduxDesc);
    if (!ReduxDesc.IsReduction)
      return false;

    // A reduction operation must only have one use of the reduction value.
    if (!IsAPhi && Kind != RK_IntegerMinMax && Kind != RK_FloatMinMax &&
        hasMultipleUsesOf(Cur, VisitedInsts))
      return false;

    // All inputs to a PHI node must be a reduction value.
    if(IsAPhi && Cur != Phi && !areAllUsesIn(Cur, VisitedInsts))
      return false;

    if (Kind == RK_IntegerMinMax && (isa<ICmpInst>(Cur) ||
                                     isa<SelectInst>(Cur)))
      ++NumCmpSelectPatternInst;
    if (Kind == RK_FloatMinMax && (isa<FCmpInst>(Cur) ||
                                   isa<SelectInst>(Cur)))
      ++NumCmpSelectPatternInst;

    // Check  whether we found a reduction operator.
    FoundReduxOp |= !IsAPhi;

    // Process users of current instruction. Push non-PHI nodes after PHI nodes
    // onto the stack. This way we are going to have seen all inputs to PHI
    // nodes once we get to them.
    SmallVector<Instruction *, 8> NonPHIs;
    SmallVector<Instruction *, 8> PHIs;
    for (User *U : Cur->users()) {
      Instruction *UI = cast<Instruction>(U);

      // Check if we found the exit user.
      BasicBlock *Parent = UI->getParent();
      if (!TheLoop->contains(Parent)) {
        // Exit if you find multiple outside users or if the header phi node is
        // being used. In this case the user uses the value of the previous
        // iteration, in which case we would loose "VF-1" iterations of the
        // reduction operation if we vectorize.
        if (ExitInstruction != nullptr || Cur == Phi)
          return false;

        // The instruction used by an outside user must be the last instruction
        // before we feed back to the reduction phi. Otherwise, we loose VF-1
        // operations on the value.
        if (std::find(Phi->op_begin(), Phi->op_end(), Cur) == Phi->op_end())
         return false;

        ExitInstruction = Cur;
        continue;
      }

      // Process instructions only once (termination). Each reduction cycle
      // value must only be used once, except by phi nodes and min/max
      // reductions which are represented as a cmp followed by a select.
      ReductionInstDesc IgnoredVal(false, nullptr);
      if (VisitedInsts.insert(UI)) {
        if (isa<PHINode>(UI))
          PHIs.push_back(UI);
        else
          NonPHIs.push_back(UI);
      } else if (!isa<PHINode>(UI) &&
                 ((!isa<FCmpInst>(UI) &&
                   !isa<ICmpInst>(UI) &&
                   !isa<SelectInst>(UI)) ||
                  !isMinMaxSelectCmpPattern(UI, IgnoredVal).IsReduction))
        return false;

      // Remember that we completed the cycle.
      if (UI == Phi)
        FoundStartPHI = true;
    }
    Worklist.append(PHIs.begin(), PHIs.end());
    Worklist.append(NonPHIs.begin(), NonPHIs.end());
  }

  // This means we have seen one but not the other instruction of the
  // pattern or more than just a select and cmp.
  if ((Kind == RK_IntegerMinMax || Kind == RK_FloatMinMax) &&
      NumCmpSelectPatternInst != 2)
    return false;

  if (!FoundStartPHI || !FoundReduxOp || !ExitInstruction)
    return false;

  // We found a reduction var if we have reached the original phi node and we
  // only have a single instruction with out-of-loop users.

  // This instruction is allowed to have out-of-loop users.
  AllowedExit.insert(ExitInstruction);

  // Save the description of this reduction variable.
  ReductionDescriptor RD(RdxStart, ExitInstruction, Kind,
                         ReduxDesc.MinMaxKind);
  Reductions[Phi] = RD;
  // We've ended the cycle. This is a reduction variable if we have an
  // outside user and it has a binary op.

  return true;
}

/// Returns true if the instruction is a Select(ICmp(X, Y), X, Y) instruction
/// pattern corresponding to a min(X, Y) or max(X, Y).
LoopVectorizationLegality::ReductionInstDesc
LoopVectorizationLegality::isMinMaxSelectCmpPattern(Instruction *I,
                                                    ReductionInstDesc &Prev) {

  assert((isa<ICmpInst>(I) || isa<FCmpInst>(I) || isa<SelectInst>(I)) &&
         "Expect a select instruction");
  Instruction *Cmp = nullptr;
  SelectInst *Select = nullptr;

  // We must handle the select(cmp()) as a single instruction. Advance to the
  // select.
  if ((Cmp = dyn_cast<ICmpInst>(I)) || (Cmp = dyn_cast<FCmpInst>(I))) {
    if (!Cmp->hasOneUse() || !(Select = dyn_cast<SelectInst>(*I->user_begin())))
      return ReductionInstDesc(false, I);
    return ReductionInstDesc(Select, Prev.MinMaxKind);
  }

  // Only handle single use cases for now.
  if (!(Select = dyn_cast<SelectInst>(I)))
    return ReductionInstDesc(false, I);
  if (!(Cmp = dyn_cast<ICmpInst>(I->getOperand(0))) &&
      !(Cmp = dyn_cast<FCmpInst>(I->getOperand(0))))
    return ReductionInstDesc(false, I);
  if (!Cmp->hasOneUse())
    return ReductionInstDesc(false, I);

  Value *CmpLeft;
  Value *CmpRight;

  // Look for a min/max pattern.
  if (m_UMin(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_UIntMin);
  else if (m_UMax(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_UIntMax);
  else if (m_SMax(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_SIntMax);
  else if (m_SMin(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_SIntMin);
  else if (m_OrdFMin(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_FloatMin);
  else if (m_OrdFMax(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_FloatMax);
  else if (m_UnordFMin(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_FloatMin);
  else if (m_UnordFMax(m_Value(CmpLeft), m_Value(CmpRight)).match(Select))
    return ReductionInstDesc(Select, MRK_FloatMax);

  return ReductionInstDesc(false, I);
}

LoopVectorizationLegality::ReductionInstDesc
LoopVectorizationLegality::isReductionInstr(Instruction *I,
                                            ReductionKind Kind,
                                            ReductionInstDesc &Prev) {
  bool FP = I->getType()->isFloatingPointTy();
  bool FastMath = (FP && I->isCommutative() && I->isAssociative());
  switch (I->getOpcode()) {
  default:
    return ReductionInstDesc(false, I);
  case Instruction::PHI:
      if (FP && (Kind != RK_FloatMult && Kind != RK_FloatAdd &&
                 Kind != RK_FloatMinMax))
        return ReductionInstDesc(false, I);
    return ReductionInstDesc(I, Prev.MinMaxKind);
  case Instruction::Sub:
  case Instruction::Add:
    return ReductionInstDesc(Kind == RK_IntegerAdd, I);
  case Instruction::Mul:
    return ReductionInstDesc(Kind == RK_IntegerMult, I);
  case Instruction::And:
    return ReductionInstDesc(Kind == RK_IntegerAnd, I);
  case Instruction::Or:
    return ReductionInstDesc(Kind == RK_IntegerOr, I);
  case Instruction::Xor:
    return ReductionInstDesc(Kind == RK_IntegerXor, I);
  case Instruction::FMul:
    return ReductionInstDesc(Kind == RK_FloatMult && FastMath, I);
  case Instruction::FAdd:
    return ReductionInstDesc(Kind == RK_FloatAdd && FastMath, I);
  case Instruction::FCmp:
  case Instruction::ICmp:
  case Instruction::Select:
    if (Kind != RK_IntegerMinMax &&
        (!HasFunNoNaNAttr || Kind != RK_FloatMinMax))
      return ReductionInstDesc(false, I);
    return isMinMaxSelectCmpPattern(I, Prev);
  }
}

LoopVectorizationLegality::InductionKind
LoopVectorizationLegality::isInductionVariable(PHINode *Phi, int & Stride) {
  Type *PhiTy = Phi->getType();
  // We only handle integer and pointer inductions variables.
  if (!PhiTy->isIntegerTy() && !PhiTy->isPointerTy())
    return IK_NoInduction;

  // Check that the PHI is consecutive.
  const SCEV *PhiScev = SE->getSCEV(Phi);
  const SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(PhiScev);
  if (!AR) {
    DEBUG(dbgs() << "LV: PHI is not a poly recurrence.\n");
    return IK_NoInduction;
  }
  const SCEV *Step = AR->getStepRecurrence(*SE);

  const SCEVConstant *C = dyn_cast<SCEVConstant>(Step);
  if (!C)
    return IK_NoInduction;
  int step_num = C->getValue()->getSExtValue(); 

  // Integer inductions need to have a stride of one.
  if (PhiTy->isIntegerTy()) {
    if (Step->isOne()){
	  Stride = 1;
      return IK_IntInduction;
	}
    if (Step->isAllOnesValue()){
	  Stride = -1;
      return IK_ReverseIntInduction;
	}
	if(step_num > 0){
		Stride = step_num;
		return IK_IntInduction;
	}
	if(step_num < 0){
		Stride = step_num;
		return IK_ReverseIntInduction;
	}	
    return IK_NoInduction;
  }

  // Calculate the pointer stride and check if it is consecutive.
  assert(PhiTy->isPointerTy() && "The PHI must be a pointer");
  uint64_t Size = DL->getTypeAllocSize(PhiTy->getPointerElementType());
  if (C->getValue()->equalsInt(Size))
    return IK_PtrInduction;
  else if (C->getValue()->equalsInt(0 - Size))
    return IK_ReversePtrInduction;

  return IK_NoInduction;
}

bool LoopVectorizationLegality::isInductionVariable(const Value *V) {
  Value *In0 = const_cast<Value*>(V);
  PHINode *PN = dyn_cast_or_null<PHINode>(In0);
  if (!PN)
    return false;

  return Inductions.count(PN);
}

bool LoopVectorizationLegality::blockNeedsPredication(BasicBlock *BB)  {
  assert(TheLoop->contains(BB) && "Unknown block used");

  // Blocks that do not dominate the latch need predication.
  BasicBlock* Latch = TheLoop->getLoopLatch();
  return !DT->dominates(BB, Latch);
}

bool LoopVectorizationLegality::blockCanBePredicated(BasicBlock *BB,
                                            SmallPtrSet<Value *, 8>& SafePtrs) {
  for (BasicBlock::iterator it = BB->begin(), e = BB->end(); it != e; ++it) {
    // We might be able to hoist the load.
    if (it->mayReadFromMemory()) {
      LoadInst *LI = dyn_cast<LoadInst>(it);
      if (!LI || !SafePtrs.count(LI->getPointerOperand()))
        return false;
    }

    // We don't predicate stores at the moment.
    if (it->mayWriteToMemory()) {
      StoreInst *SI = dyn_cast<StoreInst>(it);
      // We only support predication of stores in basic blocks with one
      // predecessor.
      if (!SI || ++NumPredStores > NumberOfStoresToPredicate ||
          !SafePtrs.count(SI->getPointerOperand()) ||
          !SI->getParent()->getSinglePredecessor())
        return false;
    }
    if (it->mayThrow())
      return false;

    // Check that we don't have a constant expression that can trap as operand.
    for (Instruction::op_iterator OI = it->op_begin(), OE = it->op_end();
         OI != OE; ++OI) {
      if (Constant *C = dyn_cast<Constant>(*OI))
        if (C->canTrap())
          return false;
    }

    // The instructions below can trap.
    switch (it->getOpcode()) {
    default: continue;
    case Instruction::UDiv:
    case Instruction::SDiv:
    case Instruction::URem:
    case Instruction::SRem:
             return false;
    }
  }

  return true;
}


void InnerLoopUnroller::scalarizeInstruction(Instruction *Instr,
                                             bool IfPredicateStore) {
  assert(!Instr->getType()->isAggregateType() && "Can't handle vectors");
  // Holds vector parameters or scalars, in case of uniform vals.
  SmallVector<VectorParts, 4> Params;

  setDebugLocFromInst(Builder, Instr);

  // Find all of the vectorized parameters.
  for (unsigned op = 0, e = Instr->getNumOperands(); op != e; ++op) {
    Value *SrcOp = Instr->getOperand(op);

    // If we are accessing the old induction variable, use the new one.
    if (SrcOp == OldInduction) {
      Params.push_back(getVectorValue(SrcOp));
      continue;
    }

    // Try using previously calculated values.
    Instruction *SrcInst = dyn_cast<Instruction>(SrcOp);

    // If the src is an instruction that appeared earlier in the basic block
    // then it should already be vectorized.
    if (SrcInst && OrigLoop->contains(SrcInst)) {
      assert(WidenMap.has(SrcInst) && "Source operand is unavailable");
      // The parameter is a vector value from earlier.
      Params.push_back(WidenMap.get(SrcInst));
    } else {
      // The parameter is a scalar from outside the loop. Maybe even a constant.
      VectorParts Scalars;
      Scalars.append(UF, SrcOp);
      Params.push_back(Scalars);
    }
  }

  assert(Params.size() == Instr->getNumOperands() &&
         "Invalid number of operands");

  // Does this instruction return a value ?
  bool IsVoidRetTy = Instr->getType()->isVoidTy();

  Value *UndefVec = IsVoidRetTy ? nullptr :
  UndefValue::get(Instr->getType());
  // Create a new entry in the WidenMap and initialize it to Undef or Null.
  VectorParts &VecResults = WidenMap.splat(Instr, UndefVec);

  Instruction *InsertPt = Builder.GetInsertPoint();
  BasicBlock *IfBlock = Builder.GetInsertBlock();
  BasicBlock *CondBlock = nullptr;

  VectorParts Cond;
  Loop *VectorLp = nullptr;
  if (IfPredicateStore) {
    assert(Instr->getParent()->getSinglePredecessor() &&
           "Only support single predecessor blocks");
    Cond = createEdgeMask(Instr->getParent()->getSinglePredecessor(),
                          Instr->getParent());
    VectorLp = LI->getLoopFor(IfBlock);
    assert(VectorLp && "Must have a loop for this block");
  }

  // For each vector unroll 'part':
  for (unsigned Part = 0; Part < UF; ++Part) {
    // For each scalar that we create:

    // Start an "if (pred) a[i] = ..." block.
    Value *Cmp = nullptr;
    if (IfPredicateStore) {
      if (Cond[Part]->getType()->isVectorTy())
        Cond[Part] =
            Builder.CreateExtractElement(Cond[Part], Builder.getInt32(0));
      Cmp = Builder.CreateICmp(ICmpInst::ICMP_EQ, Cond[Part],
                               ConstantInt::get(Cond[Part]->getType(), 1));
      CondBlock = IfBlock->splitBasicBlock(InsertPt, "cond.store");
      LoopVectorBody.push_back(CondBlock);
      VectorLp->addBasicBlockToLoop(CondBlock, LI->getBase());
      // Update Builder with newly created basic block.
      Builder.SetInsertPoint(InsertPt);
    }

    Instruction *Cloned = Instr->clone();
      if (!IsVoidRetTy)
        Cloned->setName(Instr->getName() + ".cloned");
      // Replace the operands of the cloned instructions with extracted scalars.
      for (unsigned op = 0, e = Instr->getNumOperands(); op != e; ++op) {
        Value *Op = Params[op][Part];
        Cloned->setOperand(op, Op);
      }

      // Place the cloned scalar in the new loop.
      Builder.Insert(Cloned);

      // If the original scalar returns a value we need to place it in a vector
      // so that future users will be able to use it.
      if (!IsVoidRetTy)
        VecResults[Part] = Cloned;

    // End if-block.
      if (IfPredicateStore) {
        BasicBlock *NewIfBlock = CondBlock->splitBasicBlock(InsertPt, "else");
        LoopVectorBody.push_back(NewIfBlock);
        VectorLp->addBasicBlockToLoop(NewIfBlock, LI->getBase());
        Builder.SetInsertPoint(InsertPt);
        Instruction *OldBr = IfBlock->getTerminator();
        BranchInst::Create(CondBlock, NewIfBlock, Cmp, OldBr);
        OldBr->eraseFromParent();
        IfBlock = NewIfBlock;
      }
  }
}

void InnerLoopUnroller::vectorizeMemoryInstruction(Instruction *Instr) {
  StoreInst *SI = dyn_cast<StoreInst>(Instr);
  bool IfPredicateStore = (SI && Legal->blockNeedsPredication(SI->getParent()));

  return scalarizeInstruction(Instr, IfPredicateStore);
}

Value *InnerLoopUnroller::reverseVector(Value *Vec) {
  return Vec;
}

Value *InnerLoopUnroller::getBroadcastInstrs(Value *V) {
  return V;
}

Value *InnerLoopUnroller::getConsecutiveVector(Value* Val, int StartIdx,
                                               bool Negate, int Stride) {
  // When unrolling and the VF is 1, we only need to add a simple scalar.
  Type *ITy = Val->getType();
  assert(!ITy->isVectorTy() && "Val must be a scalar");
  Constant *C = ConstantInt::get(ITy, StartIdx, Negate);
  return Builder.CreateAdd(Val, C, "induction");
}



LoopVectorizationCostModel::VectorizationFactor
LoopVectorizationCostModel::selectVectorizationFactor(bool OptForSize,
                                                      unsigned UserVF,
                                                      bool ForceVectorization) {
  // Width 1 means no vectorize
  VectorizationFactor Factor = { 1U, 0U };
  
  if (OptForSize && Legal->getRuntimePointerCheck()->Need) {
    DEBUG(dbgs() << "LV: Aborting. Runtime ptr check is required in Os.\n");
    return Factor;
  }

  if (!EnableCondStoresVectorization && Legal->NumPredStores) {
    DEBUG(dbgs() << "LV: No vectorization. There are conditional stores.\n");
    return Factor;
  }

  // Find the trip count.
  unsigned TC = SE->getSmallConstantTripCount(TheLoop, TheLoop->getLoopLatch());
  DEBUG(dbgs() << "LV: Found trip count: " << TC << '\n');

  unsigned WidestType = getWidestType();  
  unsigned WidestRegister = maxVecRegWidthInbit;
  unsigned MaxSafeDepDist = -1U;  
  if (Legal->getMaxSafeDepDistBytes() != -1U)
    MaxSafeDepDist = Legal->getMaxSafeDepDistBytes() * 8;
  WidestRegister = ((WidestRegister < MaxSafeDepDist) ?
                    WidestRegister : MaxSafeDepDist);
  unsigned MaxVectorSize = WidestRegister / WidestType;

  if (MaxVectorSize == 0) {
    MaxVectorSize = 1;
  }
  assert(MaxVectorSize <= (maxVecRegWidthInbit/8) && "Did not expect to pack so many elements into one vector!");
  
  unsigned VF = MaxVectorSize;

  // If we optimize the program for size, avoid creating the tail loop.
  if (OptForSize) {
    // If we are unable to calculate the trip count then don't try to vectorize.
    if (TC < 2) {
      DEBUG(dbgs() << "LV: Aborting. A tail loop is required in Os.\n");
      return Factor;
    }

    // Find the maximum SIMD width that can fit within the trip count.
    VF = TC % MaxVectorSize;

    if (VF == 0)
      VF = MaxVectorSize;

    // If the trip count that we found modulo the vectorization factor is not
    // zero then we require a tail.
    if (VF < 2) {
      return Factor;
    }
  }

  if (UserVF != 0) {
    assert(isPowerOf2_32(UserVF) && "VF needs to be a power of two");
    DEBUG(dbgs() << "LV: Using user VF " << UserVF << ".\n");

    Factor.Width = UserVF;
    return Factor;
  }

  float extra_threshold = VectorCostThreshold;

  float Cost = expectedCost(1);
//#ifndef NDEBUG
  float ScalarCost = Cost;
//#endif /* NDEBUG */
  unsigned Width = 1;
  
  // Ignore scalar width, because the user explicitly wants vectorization.
  if (ForceVectorization && VF > 1) {
    Width = 2;
    Cost = expectedCost(Width) / (float)Width;
  }  
  
  for (unsigned i=2; i <= VF; i*=2) {   
    // Notice that the vector loop needs to be executed less times, so
    // we need to divide the cost of the vector loops by the width of
    // the vector elements.
    float VectorCost = expectedCost(i) / (float)i;
    if (VectorCost < (Cost - extra_threshold)) 
	{
      Cost = VectorCost;
      Width = i;
    }
  }

  Factor.Width = Width;
  Factor.Cost = Width * Cost;
   
  return Factor;
  
}

unsigned LoopVectorizationCostModel::getWidestType() {
  unsigned MaxWidth = 8;

  // For each block.
  for (Loop::block_iterator bb = TheLoop->block_begin(),
       be = TheLoop->block_end(); bb != be; ++bb) {
    BasicBlock *BB = *bb;

    // For each instruction in the loop.
    for (BasicBlock::iterator it = BB->begin(), e = BB->end(); it != e; ++it) {
      Type *T = it->getType();

      // Only examine Loads, Stores and PHINodes.
      if (!isa<LoadInst>(it) && !isa<StoreInst>(it) && !isa<PHINode>(it))
        continue;

      // Examine PHI nodes that are reduction variables.
      if (PHINode *PN = dyn_cast<PHINode>(it))
        if (!Legal->getReductionVars()->count(PN))
          continue;

      // Examine the stored values.
      if (StoreInst *ST = dyn_cast<StoreInst>(it))
        T = ST->getValueOperand()->getType();

      // Ignore loaded pointer types and stored pointer types that are not
      // consecutive. However, we do want to take consecutive stores/loads of
      // pointer vectors into account.

	  ////update: as we also consider slp now, no matter the pointer is consecutive across iterations or not
	  ////we can not ignore them
      //if (T->isPointerTy() && !isConsecutiveLoadOrStore(it))
      //  continue;

      MaxWidth = std::max(MaxWidth,
                          (unsigned)DL->getTypeSizeInBits(T->getScalarType()));
    }
  }

  return MaxWidth;
}

unsigned
LoopVectorizationCostModel::selectUnrollFactor(bool OptForSize,
                                               unsigned UserUF,
                                               unsigned VF,
                                               unsigned LoopCost) {

  // -- The unroll heuristics --
  // We unroll the loop in order to expose ILP and reduce the loop overhead.
  // There are many micro-architectural considerations that we can't predict
  // at this level. For example frontend pressure (on decode or fetch) due to
  // code size, or the number and capabilities of the execution ports.
  //
  // We use the following heuristics to select the unroll factor:
  // 1. If the code has reductions the we unroll in order to break the cross
  // iteration dependency.
  // 2. If the loop is really small then we unroll in order to reduce the loop
  // overhead.
  // 3. We don't unroll if we think that we will spill registers to memory due
  // to the increased register pressure.

  // Use the user preference, unless 'auto' is selected.
  if (UserUF != 0)
    return UserUF;

  // When we optimize for size we don't unroll.
  if (OptForSize)
    return 1;

  // We used the distance for the unroll factor.
  if (Legal->getMaxSafeDepDistBytes() != -1U)
    return 1;

  // Do not unroll loops with a relatively small trip count.
  unsigned TC = SE->getSmallConstantTripCount(TheLoop,
                                              TheLoop->getLoopLatch());
  if (TC > 1 && TC < TinyTripCountUnrollThreshold)
    return 1;

  unsigned TargetNumRegisters = TTI.getNumberOfRegisters(VF > 1);
  DEBUG(dbgs() << "LV: The target has " << TargetNumRegisters <<
        " registers\n");

  if (VF == 1) {
    if (ForceTargetNumScalarRegs.getNumOccurrences() > 0)
      TargetNumRegisters = ForceTargetNumScalarRegs;
  } else {
    if (ForceTargetNumVectorRegs.getNumOccurrences() > 0)
      TargetNumRegisters = ForceTargetNumVectorRegs;
  }

  LoopVectorizationCostModel::RegisterUsage R = calculateRegisterUsage();
  // We divide by these constants so assume that we have at least one
  // instruction that uses at least one register.
  R.MaxLocalUsers = std::max(R.MaxLocalUsers, 1U);
  R.NumInstructions = std::max(R.NumInstructions, 1U);

  // We calculate the unroll factor using the following formula.
  // Subtract the number of loop invariants from the number of available
  // registers. These registers are used by all of the unrolled instances.
  // Next, divide the remaining registers by the number of registers that is
  // required by the loop, in order to estimate how many parallel instances
  // fit without causing spills. All of this is rounded down if necessary to be
  // a power of two. We want power of two unroll factors to simplify any
  // addressing operations or alignment considerations.
  unsigned UF = PowerOf2Floor((TargetNumRegisters - R.LoopInvariantRegs) /
                              R.MaxLocalUsers);

  // Don't count the induction variable as unrolled.
  if (EnableIndVarRegisterHeur)
    UF = PowerOf2Floor((TargetNumRegisters - R.LoopInvariantRegs - 1) /
                       std::max(1U, (R.MaxLocalUsers - 1)));

  // Clamp the unroll factor ranges to reasonable factors.
  unsigned MaxUnrollSize = TTI.getMaximumUnrollFactor();

  // Check if the user has overridden the unroll max.
  if (VF == 1) {
    if (ForceTargetMaxScalarUnrollFactor.getNumOccurrences() > 0)
      MaxUnrollSize = ForceTargetMaxScalarUnrollFactor;
  } else {
    if (ForceTargetMaxVectorUnrollFactor.getNumOccurrences() > 0)
      MaxUnrollSize = ForceTargetMaxVectorUnrollFactor;
  }

  // If we did not calculate the cost for VF (because the user selected the VF)
  // then we calculate the cost of VF here.
  if (LoopCost == 0)
    LoopCost = expectedCost(VF);

  // Clamp the calculated UF to be between the 1 and the max unroll factor
  // that the target allows.
  if (UF > MaxUnrollSize)
    UF = MaxUnrollSize;
  else if (UF < 1)
    UF = 1;

  // Unroll if we vectorized this loop and there is a reduction that could
  // benefit from unrolling.
  if (VF > 1 && Legal->getReductionVars()->size()) {
    DEBUG(dbgs() << "LV: Unrolling because of reductions.\n");
    return UF;
  }

  // Note that if we've already vectorized the loop we will have done the
  // runtime check and so unrolling won't require further checks.
  bool UnrollingRequiresRuntimePointerCheck =
      (VF == 1 && Legal->getRuntimePointerCheck()->Need);

  // We want to unroll small loops in order to reduce the loop overhead and
  // potentially expose ILP opportunities.
  DEBUG(dbgs() << "LV: Loop cost is " << LoopCost << '\n');
  if (!UnrollingRequiresRuntimePointerCheck &&
      LoopCost < SmallLoopCost) {
    // We assume that the cost overhead is 1 and we use the cost model
    // to estimate the cost of the loop and unroll until the cost of the
    // loop overhead is about 5% of the cost of the loop.
    unsigned SmallUF = std::min(UF, (unsigned)PowerOf2Floor(SmallLoopCost / LoopCost));

    // Unroll until store/load ports (estimated by max unroll factor) are
    // saturated.
    unsigned StoresUF = UF / (Legal->NumStores ? Legal->NumStores : 1);
    unsigned LoadsUF = UF /  (Legal->NumLoads ? Legal->NumLoads : 1);

    if (EnableLoadStoreRuntimeUnroll && std::max(StoresUF, LoadsUF) > SmallUF) {
      DEBUG(dbgs() << "LV: Unrolling to saturate store or load ports.\n");
      return std::max(StoresUF, LoadsUF);
    }

    DEBUG(dbgs() << "LV: Unrolling to reduce branch cost.\n");
    return SmallUF;
  }

  DEBUG(dbgs() << "LV: Not Unrolling.\n");
  return 1;
}

LoopVectorizationCostModel::RegisterUsage
LoopVectorizationCostModel::calculateRegisterUsage() {
  // This function calculates the register usage by measuring the highest number
  // of values that are alive at a single location. Obviously, this is a very
  // rough estimation. We scan the loop in a topological order in order and
  // assign a number to each instruction. We use RPO to ensure that defs are
  // met before their users. We assume that each instruction that has in-loop
  // users starts an interval. We record every time that an in-loop value is
  // used, so we have a list of the first and last occurrences of each
  // instruction. Next, we transpose this data structure into a multi map that
  // holds the list of intervals that *end* at a specific location. This multi
  // map allows us to perform a linear search. We scan the instructions linearly
  // and record each time that a new interval starts, by placing it in a set.
  // If we find this value in the multi-map then we remove it from the set.
  // The max register usage is the maximum size of the set.
  // We also search for instructions that are defined outside the loop, but are
  // used inside the loop. We need this number separately from the max-interval
  // usage number because when we unroll, loop-invariant values do not take
  // more register.
  LoopBlocksDFS DFS(TheLoop);
  DFS.perform(LI);

  RegisterUsage R;
  R.NumInstructions = 0;

  // Each 'key' in the map opens a new interval. The values
  // of the map are the index of the 'last seen' usage of the
  // instruction that is the key.
  typedef DenseMap<Instruction*, unsigned> IntervalMap;
  // Maps instruction to its index.
  DenseMap<unsigned, Instruction*> IdxToInstr;
  // Marks the end of each interval.
  IntervalMap EndPoint;
  // Saves the list of instruction indices that are used in the loop.
  SmallSet<Instruction*, 8> Ends;
  // Saves the list of values that are used in the loop but are
  // defined outside the loop, such as arguments and constants.
  SmallPtrSet<Value*, 8> LoopInvariants;

  unsigned Index = 0;
  for (LoopBlocksDFS::RPOIterator bb = DFS.beginRPO(),
       be = DFS.endRPO(); bb != be; ++bb) {
    R.NumInstructions += (*bb)->size();
    for (BasicBlock::iterator it = (*bb)->begin(), e = (*bb)->end(); it != e;
         ++it) {
      Instruction *I = it;
      IdxToInstr[Index++] = I;

      // Save the end location of each USE.
      for (unsigned i = 0; i < I->getNumOperands(); ++i) {
        Value *U = I->getOperand(i);
        Instruction *Instr = dyn_cast<Instruction>(U);

        // Ignore non-instruction values such as arguments, constants, etc.
        if (!Instr) continue;

        // If this instruction is outside the loop then record it and continue.
        if (!TheLoop->contains(Instr)) {
          LoopInvariants.insert(Instr);
          continue;
        }

        // Overwrite previous end points.
        EndPoint[Instr] = Index;
        Ends.insert(Instr);
      }
    }
  }

  // Saves the list of intervals that end with the index in 'key'.
  typedef SmallVector<Instruction*, 2> InstrList;
  DenseMap<unsigned, InstrList> TransposeEnds;

  // Transpose the EndPoints to a list of values that end at each index.
  for (IntervalMap::iterator it = EndPoint.begin(), e = EndPoint.end();
       it != e; ++it)
    TransposeEnds[it->second].push_back(it->first);

  SmallSet<Instruction*, 8> OpenIntervals;
  unsigned MaxUsage = 0;


  DEBUG(dbgs() << "LV(REG): Calculating max register usage:\n");
  for (unsigned int i = 0; i < Index; ++i) {
    Instruction *I = IdxToInstr[i];
    // Ignore instructions that are never used within the loop.
    if (!Ends.count(I)) continue;

    // Remove all of the instructions that end at this location.
    InstrList &List = TransposeEnds[i];
    for (unsigned int j=0, e = List.size(); j < e; ++j)
      OpenIntervals.erase(List[j]);

    // Count the number of live interals.
    MaxUsage = std::max(MaxUsage, OpenIntervals.size());

    DEBUG(dbgs() << "LV(REG): At #" << i << " Interval # " <<
          OpenIntervals.size() << '\n');

    // Add the current instruction to the list of open intervals.
    OpenIntervals.insert(I);
  }

  unsigned Invariant = LoopInvariants.size();
  DEBUG(dbgs() << "LV(REG): Found max usage: " << MaxUsage << '\n');
  DEBUG(dbgs() << "LV(REG): Found invariant usage: " << Invariant << '\n');
  DEBUG(dbgs() << "LV(REG): LoopSize: " << R.NumInstructions << '\n');

  R.LoopInvariantRegs = Invariant;
  R.MaxLocalUsers = MaxUsage;
  return R;
}

unsigned LoopVectorizationCostModel::expectedCost(unsigned VF) {
  unsigned Cost = 0;

  // For each block.
  for (Loop::block_iterator bb = TheLoop->block_begin(),
       be = TheLoop->block_end(); bb != be; ++bb) {
    unsigned BlockCost = 0;
    BasicBlock *BB = *bb;

    // For each instruction in the old loop.
    for (BasicBlock::iterator it = BB->begin(), e = BB->end(); it != e; ++it) {
      // Skip dbg intrinsics.
      if (isa<DbgInfoIntrinsic>(it))
        continue;

      unsigned C = getInstructionCost(it, VF);

      // Check if we should override the cost.
      if (ForceTargetInstructionCost.getNumOccurrences() > 0)
        C = ForceTargetInstructionCost;

      BlockCost += C;
    }

	////here we calculate all the cost related to SLP vars
	if(VF > 1){
		unsigned C = getAllSLPInstructionCost(BB, VF);
		BlockCost += C;	
	}

    // We assume that if-converted blocks have a 50% chance of being executed.
    // When the code is scalar then some of the blocks are avoided due to CF.
    // When the code is vectorized we execute all code paths.
    if (VF == 1 && Legal->blockNeedsPredication(*bb))
      BlockCost /= 2;

    Cost += BlockCost;
  }

  return Cost;
}



unsigned LoopVectorizationCostModel::getInstructionCost(Instruction *I, unsigned VF) {

  //////check if this instruction will be vectorized in SLP manner, if so, call the other function to estimate the cost
  if(VF > 1){
  	if(Legal->isSLPVar(I))
  		return 0;//this instrucion will costed later bu getSLPInstructionCost after checked all the insts in the BB;
  }

  // If we know that this instruction will remain uniform, check the cost of
  // the scalar version.
  if (Legal->isUniformAfterVectorization(I))
    VF = 1;

  Type *RetTy = I->getType();
  Type *VectorTy = ToVectorTy(RetTy, VF);

  // TODO: We need to estimate the cost of intrinsic calls.
  switch (I->getOpcode()) {
  case Instruction::GetElementPtr:
    // We mark this instruction as zero-cost because the cost of GEPs in
    // vectorized code depends on whether the corresponding memory instruction
    // is scalarized or not. Therefore, we handle GEPs with the memory
    // instruction cost.
    return 0;
  case Instruction::Br: {
    return TTI.getCFInstrCost(I->getOpcode());
  }
  case Instruction::PHI:
    //TODO: IF-converted IFs become selects.
    return 0;
  case Instruction::Add:
  case Instruction::FAdd:
  case Instruction::Sub:
  case Instruction::FSub:
  case Instruction::Mul:
  case Instruction::FMul:
  case Instruction::UDiv:
  case Instruction::SDiv:
  case Instruction::FDiv:
  case Instruction::URem:
  case Instruction::SRem:
  case Instruction::FRem:
  case Instruction::Shl:
  case Instruction::LShr:
  case Instruction::AShr:
  case Instruction::And:
  case Instruction::Or:
  case Instruction::Xor: {
    // Since we will replace the stride by 1 the multiplication should go away.
    if (I->getOpcode() == Instruction::Mul && isStrideMul(I, Legal))
      return 0;
    // Certain instructions can be cheaper to vectorize if they have a constant
    // second vector operand. One example of this are shifts on x86.
    TargetTransformInfo::OperandValueKind Op1VK =
      TargetTransformInfo::OK_AnyValue;
    TargetTransformInfo::OperandValueKind Op2VK =
      TargetTransformInfo::OK_AnyValue;
    Value *Op2 = I->getOperand(1);

    // Check for a splat of a constant or for a non uniform vector of constants.
    if (isa<ConstantInt>(Op2))
      Op2VK = TargetTransformInfo::OK_UniformConstantValue;
    else if (isa<ConstantVector>(Op2) || isa<ConstantDataVector>(Op2)) {
      Op2VK = TargetTransformInfo::OK_NonUniformConstantValue;
      if (cast<Constant>(Op2)->getSplatValue() != nullptr)
        Op2VK = TargetTransformInfo::OK_UniformConstantValue;
    }

    return TTI.getArithmeticInstrCost(I->getOpcode(), VectorTy, Op1VK, Op2VK);
  }
  case Instruction::Select: {
    SelectInst *SI = cast<SelectInst>(I);
    const SCEV *CondSCEV = SE->getSCEV(SI->getCondition());
    bool ScalarCond = (SE->isLoopInvariant(CondSCEV, TheLoop));
    Type *CondTy = SI->getCondition()->getType();
    if (!ScalarCond)
      CondTy = VectorType::get(CondTy, VF);

    return TTI.getCmpSelInstrCost(I->getOpcode(), VectorTy, CondTy);
  }
  case Instruction::ICmp:
  case Instruction::FCmp: {
    Type *ValTy = I->getOperand(0)->getType();
    VectorTy = ToVectorTy(ValTy, VF);
    return TTI.getCmpSelInstrCost(I->getOpcode(), VectorTy);
  }
  case Instruction::Store:
  case Instruction::Load: {
    StoreInst *SI = dyn_cast<StoreInst>(I);
    LoadInst *LI = dyn_cast<LoadInst>(I);
    Type *ValTy = (SI ? SI->getValueOperand()->getType() :
                   LI->getType());
    VectorTy = ToVectorTy(ValTy, VF);

    unsigned Alignment = SI ? SI->getAlignment() : LI->getAlignment();
    unsigned AS = SI ? SI->getPointerAddressSpace() :
      LI->getPointerAddressSpace();
    Value *Ptr = SI ? SI->getPointerOperand() : LI->getPointerOperand();
    // We add the cost of address computation here instead of with the gep
    // instruction because only here we know whether the operation is
    // scalarized.
    if (VF == 1)
      return TTI.getAddressComputationCost(VectorTy) +
        TTI.getMemoryOpCost(I->getOpcode(), VectorTy, Alignment, AS);

    // Scalarized loads/stores.
    int ConsecutiveStride = Legal->isConsecutivePtr(Ptr);
    bool Reverse = ConsecutiveStride < 0;
    unsigned ScalarAllocatedSize = DL->getTypeAllocSize(ValTy);
    unsigned VectorElementSize = DL->getTypeStoreSize(VectorTy)/VF;
    if (!ConsecutiveStride || ScalarAllocatedSize != VectorElementSize) {
      bool IsComplexComputation =
        isLikelyComplexAddressComputation(Ptr, Legal, SE, TheLoop);
      unsigned Cost = 0;
      // The cost of extracting from the value vector and pointer vector.
      Type *PtrTy = ToVectorTy(Ptr->getType(), VF);
      for (unsigned i = 0; i < VF; ++i) {
        //  The cost of extracting the pointer operand.
        Cost += TTI.getVectorInstrCost(Instruction::ExtractElement, PtrTy, i);
        // In case of STORE, the cost of ExtractElement from the vector.
        // In case of LOAD, the cost of InsertElement into the returned
        // vector.
        Cost += TTI.getVectorInstrCost(SI ? Instruction::ExtractElement :
                                            Instruction::InsertElement,
                                            VectorTy, i);
      }

      // The cost of the scalar loads/stores.
      Cost += VF * TTI.getAddressComputationCost(PtrTy, IsComplexComputation);
      Cost += VF * TTI.getMemoryOpCost(I->getOpcode(), ValTy->getScalarType(),
                                       Alignment, AS);
      return Cost;
    }

    // Wide load/stores.
    unsigned Cost = TTI.getAddressComputationCost(VectorTy);
    Cost += TTI.getMemoryOpCost(I->getOpcode(), VectorTy, Alignment, AS);

    if (Reverse)
      Cost += TTI.getShuffleCost(TargetTransformInfo::SK_Reverse,
                                  VectorTy, 0);
    return Cost;
  }
  case Instruction::ZExt:
  case Instruction::SExt:
  case Instruction::FPToUI:
  case Instruction::FPToSI:
  case Instruction::FPExt:
  case Instruction::PtrToInt:
  case Instruction::IntToPtr:
  case Instruction::SIToFP:
  case Instruction::UIToFP:
  case Instruction::Trunc:
  case Instruction::FPTrunc:
  case Instruction::BitCast: {
    // We optimize the truncation of induction variable.
    // The cost of these is the same as the scalar operation.
    if (I->getOpcode() == Instruction::Trunc &&
        Legal->isInductionVariable(I->getOperand(0)))
      return TTI.getCastInstrCost(I->getOpcode(), I->getType(),
                                  I->getOperand(0)->getType());

    Type *SrcVecTy = ToVectorTy(I->getOperand(0)->getType(), VF);
    return TTI.getCastInstrCost(I->getOpcode(), VectorTy, SrcVecTy);
  }
  case Instruction::Call: {
    CallInst *CI = cast<CallInst>(I);
    Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
    assert(ID && "Not an intrinsic call!");
    Type *RetTy = ToVectorTy(CI->getType(), VF);
    SmallVector<Type*, 4> Tys;
    for (unsigned i = 0, ie = CI->getNumArgOperands(); i != ie; ++i)
      Tys.push_back(ToVectorTy(CI->getArgOperand(i)->getType(), VF));
    return TTI.getIntrinsicInstrCost(ID, RetTy, Tys);
  }
  default: {
    // We are scalarizing the instruction. Return the cost of the scalar
    // instruction, plus the cost of insert and extract into vector
    // elements, times the vector width.
    unsigned Cost = 0;

    if (!RetTy->isVoidTy() && VF != 1) {
      unsigned InsCost = TTI.getVectorInstrCost(Instruction::InsertElement,
                                                VectorTy);
      unsigned ExtCost = TTI.getVectorInstrCost(Instruction::ExtractElement,
                                                VectorTy);

      // The cost of inserting the results plus extracting each one of the
      // operands.
      Cost += VF * (InsCost + ExtCost * I->getNumOperands());
    }

    // The cost of executing VF copies of the scalar instruction. This opcode
    // is unknown. Assume that it is the same as 'mul'.
    Cost += VF * TTI.getArithmeticInstrCost(Instruction::Mul, VectorTy);
    return Cost;
  }
  }// end of switch.
}



unsigned LoopVectorizationCostModel::getAllSLPInstructionCost(BasicBlock *BB, unsigned VF) {

	MemAccessChainList& memChains = Legal->getConsecutiveStoreMemAccChainList();
	MemAccessChainList::iterator I, IE; 

	unsigned cost_slp = 0;
	////get the consecutiveness of loads/stores
	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I){
		
		ValuePack root = *I;
		////we handle stores first
		if(dyn_cast<LoadInst>(root[0]))
			continue;
		StoreInst * SI = cast<StoreInst>(root[0]);
		if(SI->getParent() != BB)
			continue;
		
		unsigned bundle_sz = root.size(); 
		
		bool isPredicateStore = false;	
	    if (Legal->blockNeedsPredication(BB))
	    	isPredicateStore = true;

		unsigned vec_reg_sz = maxVecRegWidthInbit;
		unsigned ty_sz = (unsigned)DL->getTypeSizeInBits(SI->getValueOperand()->getType());
		unsigned max_VF = vec_reg_sz/ty_sz; 
		assert(bundle_sz <= max_VF && "slp store bundle size is not valid!\n");
		
		unsigned VF_t = Legal->getBestVectorWidth(SI->getValueOperand()->getType(), bundle_sz);
		if(!isPredicateStore){	
			VF_t = (bundle_sz*VF >= max_VF)? max_VF : VF_t;
		}

		ValuePack opr_bundle;
		for(unsigned ii=0; ii < bundle_sz; ii++)
			opr_bundle.push_back(cast<StoreInst>(root[ii])->getValueOperand());	
		
		unsigned cost_slp_t = getSLPInstructionCost(opr_bundle, VF_t, VF);

		Type * VectorTy = ToVectorTy(SI->getValueOperand()->getType(), VF_t);			
		unsigned Alignment = SI->getAlignment();
		unsigned AS = SI->getPointerAddressSpace();
        unsigned C = TTI.getAddressComputationCost(VectorTy) +
			TTI.getMemoryOpCost(SI->getOpcode(), VectorTy, Alignment, AS);	 ////single consectuive store		
			
		cost_slp_t += C;
		
	    ////we estimiate the cost by multiply the number of the subgroups with the cost of a single group
		unsigned cost_factor = ((bundle_sz*VF)%VF_t)? bundle_sz*VF/VF_t + 1 : bundle_sz*VF/VF_t;
		if(isPredicateStore)
			cost_factor = VF;

	    cost_slp += cost_slp_t*cost_factor;		
	}


	for(I = memChains.begin(), IE = memChains.end(); I != IE; ++I){
		
		ValuePack root = *I;
		////we then handle load
		if(dyn_cast<StoreInst>(root[0]))
			continue;
		LoadInst * LDI = cast<LoadInst>(root[0]);
		if(LDI->getParent() != BB)
			continue;
		
		unsigned bundle_sz = root.size(); 
		unsigned slp_cost_ld = 0;

		bool isConsecutiveLoad = true;		
		unsigned vec_reg_sz = maxVecRegWidthInbit;
		unsigned ty_sz = (unsigned)DL->getTypeSizeInBits(LDI->getType());
		unsigned max_VF = vec_reg_sz/ty_sz; 
		assert(bundle_sz <= max_VF && "slp load bundle size is not valid!\n");	
		 
		unsigned VF_t = Legal->getBestVectorWidth(LDI->getType(), bundle_sz); 
		int dis = Legal->getMemLocationDistanceAccrossInteration(root[0]);
		if(dis == bundle_sz){
			isConsecutiveLoad = true;
			VF_t = (bundle_sz*VF >= max_VF)? max_VF : VF_t;
		}
		else{
			isConsecutiveLoad = false;
		}

		Type * VectorTy = ToVectorTy(LDI->getType(), VF_t);			
		unsigned Alignment = LDI->getAlignment();
		unsigned AS = LDI->getPointerAddressSpace();

        unsigned C = TTI.getAddressComputationCost(VectorTy) +
		   TTI.getMemoryOpCost(LDI->getOpcode(), VectorTy, Alignment, AS); ////single consectuive store
		
        slp_cost_ld += C;
		////we estimiate the cost by multiply the number of the subgroups with the cost of a single group
		unsigned cost_factor = (isConsecutiveLoad)? (((bundle_sz*VF)%VF_t)? bundle_sz*VF/VF_t + 1 : bundle_sz*VF/VF_t) : VF;
		
		cost_slp += slp_cost_ld*cost_factor;		
	}

	return cost_slp;
}



unsigned LoopVectorizationCostModel::getSLPInstructionCost(ArrayRef<Value *> bundle, unsigned SLPVF, unsigned LoopVF){

  Instruction *I = dyn_cast<Instruction>(bundle[0]);  
  Type *RetTy = bundle[0]->getType();
  Type *VectorTy = ToVectorTy(RetTy, SLPVF);  
  unsigned inst_cost = 0;

  ////handle loop-->slp conflicts
  if(!(Legal->isExistingSlpBundle(bundle) || Legal->isReversedExistingSlpBundle(bundle))){
  	if(allConstant(bundle))
		inst_cost = 0;  ////only insert cost
	else if(isSplat(bundle)){
		inst_cost = TTI.getShuffleCost(TargetTransformInfo::SK_Broadcast, VectorTy, 0);
	}		
	else{
		inst_cost = TTI.getVectorInstrCost(Instruction::InsertElement, VectorTy)*SLPVF;  ////only insert cost		
		for(unsigned ii=0, jj=bundle.size(); ii < jj; ii++){
			if(dyn_cast<Instruction>(bundle[ii]) && !TheLoop->isLoopInvariant(bundle[ii]))
				inst_cost += TTI.getVectorInstrCost(Instruction::ExtractElement, ToVectorTy(RetTy, LoopVF));
		}
	}
	return inst_cost;
  }
  
  assert(I && "slp var is not an instrucion!!!");
  assert(getSameOpcode(bundle) && "slp vars with different opcode!!!");
  
  // TODO: We need to estimate the cost of intrinsic calls.
  switch (I->getOpcode()){
  	
	  case Instruction::Load:
	    break; ////loads will be costed somewhere else			

	  case Instruction::ZExt:
	  case Instruction::SExt:
	  case Instruction::FPToUI:
	  case Instruction::FPToSI:
	  case Instruction::FPExt:
	  case Instruction::PtrToInt:
	  case Instruction::IntToPtr:
	  case Instruction::SIToFP:
	  case Instruction::UIToFP:
	  case Instruction::Trunc:
	  case Instruction::FPTrunc:
	  case Instruction::BitCast: {
	    Type *SrcVecTy = ToVectorTy(I->getOperand(0)->getType(), SLPVF);
		ValuePack opr_bundle;
		for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
			Instruction * I = dyn_cast<Instruction>(bundle[ii]);
			opr_bundle.push_back(I->getOperand(0));
		}
		
	    inst_cost = TTI.getCastInstrCost(I->getOpcode(), VectorTy, SrcVecTy) 
			      + getSLPInstructionCost(opr_bundle, SLPVF, LoopVF) ;
		break;
	  }

	  case Instruction::Add:
	  case Instruction::FAdd:
	  case Instruction::Sub:
	  case Instruction::FSub:
	  case Instruction::Mul:
	  case Instruction::FMul:
	  case Instruction::UDiv:
	  case Instruction::SDiv:
	  case Instruction::FDiv:
	  case Instruction::URem:
	  case Instruction::SRem:
	  case Instruction::FRem:
	  case Instruction::Shl:
	  case Instruction::LShr:
	  case Instruction::AShr:
	  case Instruction::And:
	  case Instruction::Or:
	  case Instruction::Xor: {
	    TargetTransformInfo::OperandValueKind Op1VK =
	      TargetTransformInfo::OK_AnyValue;
	    TargetTransformInfo::OperandValueKind Op2VK =
	      TargetTransformInfo::OK_AnyValue;
	    Value *Op2 = I->getOperand(1);
	    // Check for a splat of a constant or for a non uniform vector of constants.
	    if (isa<ConstantInt>(Op2))
	      Op2VK = TargetTransformInfo::OK_UniformConstantValue;
	    else if (isa<ConstantVector>(Op2) || isa<ConstantDataVector>(Op2)) {
	      Op2VK = TargetTransformInfo::OK_NonUniformConstantValue;
	      if (cast<Constant>(Op2)->getSplatValue() != nullptr)
	        Op2VK = TargetTransformInfo::OK_UniformConstantValue;
	    }

		ValuePack oprA_bundle, oprB_bundle;
		for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
			Instruction * I = dyn_cast<Instruction>(bundle[ii]);
			oprA_bundle.push_back(I->getOperand(0));
			oprB_bundle.push_back(I->getOperand(1));
		}		

	    inst_cost = TTI.getArithmeticInstrCost(I->getOpcode(), VectorTy, Op1VK, Op2VK)
			      + getSLPInstructionCost(oprA_bundle, SLPVF, LoopVF)
			      + getSLPInstructionCost(oprB_bundle, SLPVF, LoopVF);
		break;
	  }
	  
	  case Instruction::ShuffleVector: {
	    TargetTransformInfo::OperandValueKind Op1VK =
	        TargetTransformInfo::OK_AnyValue;
	    TargetTransformInfo::OperandValueKind Op2VK =
	        TargetTransformInfo::OK_AnyValue;
	    ValuePack oprA_bundle, oprB_bundle;
	    for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
		  Instruction * I = dyn_cast<Instruction>(bundle[ii]);
		  oprA_bundle.push_back(I->getOperand(0));
		  oprB_bundle.push_back(I->getOperand(1));
		}
		inst_cost = TTI.getArithmeticInstrCost(cast<Instruction>(bundle[0])->getOpcode(), VectorTy, Op1VK, Op2VK)
	  	          + TTI.getArithmeticInstrCost(cast<Instruction>(bundle[1])->getOpcode(), VectorTy, Op1VK, Op2VK)
			  	  + TTI.getShuffleCost(TargetTransformInfo::SK_Alternate, VectorTy, 0)
		  		  + getSLPInstructionCost(oprA_bundle, SLPVF, LoopVF)
				  + getSLPInstructionCost(oprB_bundle, SLPVF, LoopVF);
		break;
	  }	
		
	  case Instruction::ICmp:
	  case Instruction::FCmp: {
	    Type *ValTy = I->getOperand(0)->getType();
	    VectorTy = ToVectorTy(ValTy, SLPVF);
		ValuePack oprA_bundle, oprB_bundle;
		for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
			Instruction * I = dyn_cast<Instruction>(bundle[ii]);
			oprA_bundle.push_back(I->getOperand(0));
			oprB_bundle.push_back(I->getOperand(1));
		}		
	    inst_cost = TTI.getCmpSelInstrCost(I->getOpcode(), VectorTy)
			      + getSLPInstructionCost(oprA_bundle, SLPVF, LoopVF)
			      + getSLPInstructionCost(oprB_bundle, SLPVF, LoopVF);
		break;
	  }

	  case Instruction::Call: {
	    CallInst *CI = cast<CallInst>(I);
	    Intrinsic::ID ID = getIntrinsicIDForCall(CI, TLI);
	    assert(ID && "Not an intrinsic call!");
	    Type *RetTy = ToVectorTy(CI->getType(), SLPVF);
	    SmallVector<Type*, 4> Tys;
		unsigned cost_t = 0;
	    for (unsigned i = 0, ie = CI->getNumArgOperands(); i != ie; ++i){
	      Tys.push_back(ToVectorTy(CI->getArgOperand(i)->getType(), SLPVF));
		  ValuePack opr_bundle;
		  for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
				Instruction * I = dyn_cast<Instruction>(bundle[ii]);
				opr_bundle.push_back(I->getOperand(i));
		  }		  
		  cost_t += getSLPInstructionCost(opr_bundle, SLPVF, LoopVF);
		}
	    inst_cost = cost_t + TTI.getIntrinsicInstrCost(ID, RetTy, Tys);
		break;
	  }
	  
	  default: 
	  	assert(false && "should not costing this instruction!!!");
  }// end of switch.	

  ////handle slp-->loop conflicts
  for(unsigned ii=0, jj = bundle.size(); ii < jj; ii++){
	  if(!Legal->onlyHasSlpVarUsers(bundle[ii])){
		  inst_cost += TTI.getVectorInstrCost(Instruction::ExtractElement, VectorTy);
	  }
  }

  if(Legal->isReversedExistingSlpBundle(bundle))
  	inst_cost += TTI.getShuffleCost(TargetTransformInfo::SK_Alternate, VectorTy, 0);
  
  return inst_cost;

}

Type* LoopVectorizationCostModel::ToVectorTy(Type *Scalar, unsigned VF) {
  if (Scalar->isVoidTy() || VF == 1)
    return Scalar;
  return VectorType::get(Scalar, VF);
}

bool LoopVectorizationCostModel::isConsecutiveLoadOrStore(Instruction *Inst) {
  // Check for a store.
  if (StoreInst *ST = dyn_cast<StoreInst>(Inst))
    return Legal->isConsecutivePtr(ST->getPointerOperand()) != 0;

  // Check for a load.
  if (LoadInst *LI = dyn_cast<LoadInst>(Inst))
    return Legal->isConsecutivePtr(LI->getPointerOperand()) != 0;

  return false;
}


char LoopVectorize::ID = 0;
static const char lv_name[] = "Loop Vectorization";
INITIALIZE_PASS_BEGIN(LoopVectorize, LV_NAME, lv_name, false, false)
INITIALIZE_AG_DEPENDENCY(TargetTransformInfo)
INITIALIZE_AG_DEPENDENCY(AliasAnalysis)
INITIALIZE_PASS_DEPENDENCY(BlockFrequencyInfo)
INITIALIZE_PASS_DEPENDENCY(DominatorTreeWrapperPass)
INITIALIZE_PASS_DEPENDENCY(ScalarEvolution)
INITIALIZE_PASS_DEPENDENCY(LCSSA)
INITIALIZE_PASS_DEPENDENCY(LoopInfo)
INITIALIZE_PASS_DEPENDENCY(LoopSimplify)
INITIALIZE_PASS_END(LoopVectorize, LV_NAME, lv_name, false, false)

namespace llvm {
  Pass *createLoopVectorizePass(bool NoUnrolling, bool AlwaysVectorize) {
    return new LoopVectorize(NoUnrolling, AlwaysVectorize);
  }
}

